@@ -1,30 +1,33 @@
-#include<qapplication.h>
-#include<qtextedit.h>
-#include<qgroupbox.h>
-#include<qlineedit.h>
-#include<qlabel.h>
-#include<qcheckbox.h>
-#include<qtextedit.h>
-#include<qcombobox.h>
-#include<qpushbutton.h>
-#include<qmessagebox.h>
-#include<qinputdialog.h>
-#include<qspinbox.h>
-#include<qtimer.h>
-#include<qmenubar.h>
-#include<qpopupmenu.h>
-#include<qtabwidget.h>
-#include<qca.h>
-//#include<iris/xmpp.h>
-#include"xmpp.h"
-#include"im.h"
+#include <qapplication.h>
+#include <q3textedit.h>
+#include <q3groupbox.h>
+#include <qlineedit.h>
+#include <qlabel.h>
+#include <qcheckbox.h>
+#include <q3textedit.h>
+#include <qcombobox.h>
+#include <qpushbutton.h>
+#include <qmessagebox.h>
+#include <qinputdialog.h>
+#include <qspinbox.h>
+#include <qtimer.h>
+#include <qmenubar.h>
+#include <q3popupmenu.h>
+#include <qtabwidget.h>
+#include <qca.h>
+//Added by qt3to4:
+#include <Q3PtrList>
+#include <QList>
+//#include <iris/xmpp.h>
+#include "xmpp.h"
+#include "im.h"
 
-#include<stdlib.h>
-#include<time.h>
+#include <stdlib.h>
+#include <time.h>
 
-#include"ui_test.h"
+#include "ui_ui_test.h"
 
-#include<stdio.h>
+#include <stdio.h>
 
 #define AppName "xmpptest"
 
@@ -83,7 +86,7 @@ static QString resultToString(int result)
 {
 	QString s;
 	switch(result) {
-		case QCA::TLS::NoCert:
+		case QCA::TLS::NoCertificate:
 			s = QObject::tr("No certificate presented.");
 			break;
 		case QCA::TLS::Valid:
@@ -91,34 +94,35 @@ static QString resultToString(int result)
 		case QCA::TLS::HostMismatch:
 			s = QObject::tr("Hostname mismatch.");
 			break;
-		case QCA::TLS::Rejected:
-			s = QObject::tr("Root CA rejects the specified purpose.");
+		case QCA::TLS::InvalidCertificate:
+			s = QObject::tr("Invalid Certificate.");
 			break;
-		case QCA::TLS::Untrusted:
-			s = QObject::tr("Not trusted for the specified purpose.");
-			break;
-		case QCA::TLS::SignatureFailed:
-			s = QObject::tr("Invalid signature.");
-			break;
-		case QCA::TLS::InvalidCA:
-			s = QObject::tr("Invalid CA certificate.");
-			break;
-		case QCA::TLS::InvalidPurpose:
-			s = QObject::tr("Invalid certificate purpose.");
-			break;
-		case QCA::TLS::SelfSigned:
-			s = QObject::tr("Certificate is self-signed.");
-			break;
-		case QCA::TLS::Revoked:
-			s = QObject::tr("Certificate has been revoked.");
-			break;
-		case QCA::TLS::PathLengthExceeded:
-			s = QObject::tr("Maximum cert chain length exceeded.");
-			break;
-		case QCA::TLS::Expired:
-			s = QObject::tr("Certificate has expired.");
-			break;
-		case QCA::TLS::Unknown:
+			// TODO: Inspect why
+//			case QCA::TLS::Untrusted:
+//				s = QObject::tr("Not trusted for the specified purpose.");
+//				break;
+//			case QCA::TLS::SignatureFailed:
+//				s = QObject::tr("Invalid signature.");
+//				break;
+//			case QCA::TLS::InvalidCA:
+//				s = QObject::tr("Invalid CA certificate.");
+//				break;
+//			case QCA::TLS::InvalidPurpose:
+//				s = QObject::tr("Invalid certificate purpose.");
+//				break;
+//			case QCA::TLS::SelfSigned:
+//				s = QObject::tr("Certificate is self-signed.");
+//				break;
+//			case QCA::TLS::Revoked:
+//				s = QObject::tr("Certificate has been revoked.");
+//				break;
+//			case QCA::TLS::PathLengthExceeded:
+//				s = QObject::tr("Maximum cert chain length exceeded.");
+//				break;
+//			case QCA::TLS::Expired:
+//				s = QObject::tr("Certificate has expired.");
+//				break;
+//			case QCA::TLS::Unknown:
 		default:
 			s = QObject::tr("General validation error.");
 			break;
@@ -136,7 +140,7 @@ public:
 	void incomingXml(const QDomElement &);
 };
 
-class TestDlg : public TestUI
+class TestDlg : public QDialog, public Ui::TestUI
 {
 	Q_OBJECT
 public:
@@ -147,9 +151,10 @@ public:
 	XMPP::ClientStream *stream;
 	XMPP::Jid jid;
 
-	TestDlg(QWidget *parent=0, const char *name=0) : TestUI(parent, name)
+	TestDlg(QWidget *parent=0) : QDialog(parent)
 	{
-		setCaption(tr("XMPP Test"));
+		setupUi(this);
+		setWindowTitle(tr("XMPP Test"));
 
 		connect(ck_probe, SIGNAL(toggled(bool)), SLOT(probe_toggled(bool)));
 		connect(cb_proxy, SIGNAL(activated(int)), SLOT(proxy_activated(int)));
@@ -168,7 +173,7 @@ public:
 		pb_send->setEnabled(false);
 		proxy_activated(0);
 		ck_probe->setChecked(true);
-		ck_mutual->setChecked(true);
+		ck_mutual->setChecked(false);
 		pb_go->setText(tr("&Connect"));
 
 		//le_jid->setText("psitest@jabberd.jabberstudio.org/Test");
@@ -188,9 +193,10 @@ public:
 		connect(conn, SIGNAL(httpSyncStarted()), SLOT(conn_httpSyncStarted()));
 		connect(conn, SIGNAL(httpSyncFinished()), SLOT(conn_httpSyncFinished()));
 
-		if(QCA::isSupported(QCA::CAP_TLS)) {
+		if(QCA::isSupported("tls")) {
 			tls = new QCA::TLS;
 			tlsHandler = new XMPP::QCATLSHandler(tls);
+			tlsHandler->setXMPPCertCheck(true);
 			connect(tlsHandler, SIGNAL(tlsHandshaken()), SLOT(tls_handshaken()));
 		}
 		else {
@@ -199,6 +205,7 @@ public:
 		}
 
 		stream = new XMPP::ClientStream(conn, tlsHandler);
+		//stream->setOldOnly(true);
 		connect(stream, SIGNAL(connected()), SLOT(cs_connected()));
 		connect(stream, SIGNAL(securityLayerActivated(int)), SLOT(cs_securityLayerActivated(int)));
 		connect(stream, SIGNAL(needAuthParams(bool, bool, bool)), SLOT(cs_needAuthParams(bool, bool, bool)));
@@ -356,15 +363,16 @@ private slots:
 		conn->setOptSSL(ssl);
 
 		if(tls) {
-			QPtrList<QCA::Cert> certStore;
-			tls->setCertificateStore(certStore);
+			tls->setTrustedCertificates(QCA::systemStore());
 		}
 
 		stream->setNoopTime(55000); // every 55 seconds
-		stream->setAllowPlain(ck_plain->isChecked());
+		stream->setAllowPlain(ck_plain->isChecked() ? XMPP::ClientStream::AllowPlain : XMPP::ClientStream::NoAllowPlain);
 		stream->setRequireMutualAuth(ck_mutual->isChecked());
 		stream->setSSFRange(sb_ssfmin->value(), sb_ssfmax->value());
-
+		//stream->setOldOnly(true);
+		stream->setCompress(true);
+		
 		gb_server->setEnabled(false);
 		pb_go->setText(tr("&Disconnect"));
 		pb_go->setFocus();
@@ -428,7 +436,7 @@ private slots:
 
 		int num = 0;
 		QDomNodeList nl = e.childNodes();
-		QValueList<XMPP::Stanza> stanzaList;
+		QList<XMPP::Stanza> stanzaList;
 		for(uint x = 0; x < nl.count(); ++x) {
 			QDomNode n = nl.item(x);
 			if(n.isElement()) {
@@ -448,7 +456,7 @@ private slots:
 		}
 
 		// out the door
-		for(QValueList<XMPP::Stanza>::ConstIterator it = stanzaList.begin(); it != stanzaList.end(); ++it) {
+		for(QList<XMPP::Stanza>::ConstIterator it = stanzaList.begin(); it != stanzaList.end(); ++it) {
 			appendXmlOut(XMPP::Stream::xmlToString((*it).element(), true));
 			stream->write(*it);
 		}
@@ -545,8 +553,9 @@ private slots:
 
 	void tls_handshaken()
 	{
-		QCA::Cert cert = tls->peerCertificate();
-		int vr = tls->certificateValidityResult();
+		//QCA::Certificate cert = tls->peerCertificate();
+		int vr = tls->peerIdentityResult();
+		if (vr == QCA::TLS::Valid && !tlsHandler->certMatchesHostname()) vr = QCA::TLS::HostMismatch;
 
 		appendSysMsg("Successful TLS handshake.");
 		if(vr == QCA::TLS::Valid)
@@ -727,7 +736,7 @@ private slots:
 				s = "server rejected STARTTLS";
 			else if(x == XMPP::ClientStream::TLSFail) {
 				int t = tlsHandler->tlsError();
-				if(t == QCA::TLS::ErrHandshake)
+				if(t == QCA::TLS::ErrorHandshake)
 					s = "TLS handshake error";
 				else
 					s = "broken security layer (TLS)";
@@ -813,7 +822,7 @@ public:
 		for(QStringList::ConstIterator it = lines.begin(); it != lines.end(); ++it) {
 			if(!first)
 				str += "<br>";
-			str += QString("<font color=\"%1\">%2</font>").arg(Qt::darkGreen.name()).arg(plain2rich(*it));
+			str += QString("<font color=\"%1\">%2</font>").arg(QColor(Qt::darkGreen).name()).arg(plain2rich(*it));
 			first = false;
 		}
 		te_log->append(str);
@@ -827,7 +836,7 @@ public:
 		for(QStringList::ConstIterator it = lines.begin(); it != lines.end(); ++it) {
 			if(!first)
 				str += "<br>";
-			str += QString("<font color=\"%1\">%2</font>").arg(Qt::darkBlue.name()).arg(plain2rich(*it));
+			str += QString("<font color=\"%1\">%2</font>").arg(QColor(Qt::darkBlue).name()).arg(plain2rich(*it));
 			first = false;
 		}
 		te_log->append(str);
@@ -868,10 +877,12 @@ void TestDebug::incomingXml(const QDomElement &e)
 		td_glob->appendXmlIn(out);
 }
 
-#include"xmpptest.moc"
+#include "xmpptest.moc"
 
 int main(int argc, char **argv)
 {
+	QCA::Initializer init;
+
 #ifdef Q_OS_WIN32
 	QApplication::addLibraryPath(".");
 	putenv("SASL_PATH=.\\sasl");
@@ -896,3 +907,10 @@ int main(int argc, char **argv)
 
 	return 0;
 }
+
+#ifdef QCA_STATIC
+#include <QtPlugin>
+#ifdef HAVE_OPENSSL
+Q_IMPORT_PLUGIN(qca_openssl)
+#endif
+#endif
