@@ -18,15 +18,18 @@
  *
  */
 
-#include"s5b.h"
+#include "s5b.h"
 
-#include<qtimer.h>
-#include<qguardedptr.h>
-#include<stdlib.h>
-#include<qca.h>
-#include"xmpp_xmlcommon.h"
-#include"socks.h"
-#include"safedelete.h"
+#include <qtimer.h>
+#include <qpointer.h>
+#include <Q3PtrList>
+#include <QByteArray>
+#include <stdlib.h>
+#include <qca.h>
+#include "xmpp_xmlcommon.h"
+#include "im.h"
+#include "socks.h"
+#include "safedelete.h"
 
 #ifdef Q_OS_WIN
 # include <windows.h>
@@ -43,7 +46,7 @@ namespace XMPP {
 static QString makeKey(const QString &sid, const Jid &initiator, const Jid &target)
 {
 	QString str = sid + initiator.full() + target.full();
-	return QCA::SHA1::hashToString(str.utf8());
+	return QCA::Hash("sha1").hashToString(str.utf8());
 }
 
 static bool haveHost(const StreamHostList &list, const Jid &j)
@@ -172,7 +175,7 @@ public:
 	S5BRequest req;
 	Jid proxy;
 	Mode mode;
-	QPtrList<S5BDatagram> dglist;
+	Q3PtrList<S5BDatagram> dglist;
 };
 
 static int id_conn = 0;
@@ -551,7 +554,7 @@ public:
 	QString sid;
 	JT_S5B *query;
 	StreamHost proxyInfo;
-	QGuardedPtr<S5BServer> relatedServer;
+	QPointer<S5BServer> relatedServer;
 
 	bool udp_init;
 	QHostAddress udp_addr;
@@ -563,7 +566,7 @@ class S5BManager::Private
 public:
 	Client *client;
 	S5BServer *serv;
-	QPtrList<Entry> activeList;
+	Q3PtrList<Entry> activeList;
 	S5BConnectionList incomingConns;
 	JT_PushS5B *ps;
 };
@@ -571,6 +574,10 @@ public:
 S5BManager::S5BManager(Client *parent)
 :QObject(parent)
 {
+	// S5B needs SHA1
+	//if(!QCA::isSupported(QCA::CAP_SHA1))
+	//	QCA::insertProvider(createProviderHash());
+
 	d = new Private;
 	d->client = parent;
 	d->serv = 0;
@@ -757,7 +764,7 @@ bool S5BManager::isAcceptableSID(const Jid &peer, const QString &sid) const
 
 S5BConnection *S5BManager::findIncoming(const Jid &from, const QString &sid) const
 {
-	QPtrListIterator<S5BConnection> it(d->incomingConns);
+	Q3PtrListIterator<S5BConnection> it(d->incomingConns);
 	for(S5BConnection *c; (c = it.current()); ++it) {
 		if(c->d->peer.compare(from) && c->d->sid == sid)
 			return c;
@@ -767,7 +774,7 @@ S5BConnection *S5BManager::findIncoming(const Jid &from, const QString &sid) con
 
 S5BManager::Entry *S5BManager::findEntry(S5BConnection *c) const
 {
-	QPtrListIterator<Entry> it(d->activeList);
+	Q3PtrListIterator<Entry> it(d->activeList);
 	for(Entry *e; (e = it.current()); ++it) {
 		if(e->c == c)
 			return e;
@@ -777,7 +784,7 @@ S5BManager::Entry *S5BManager::findEntry(S5BConnection *c) const
 
 S5BManager::Entry *S5BManager::findEntry(Item *i) const
 {
-	QPtrListIterator<Entry> it(d->activeList);
+	Q3PtrListIterator<Entry> it(d->activeList);
 	for(Entry *e; (e = it.current()); ++it) {
 		if(e->i == i)
 			return e;
@@ -787,7 +794,7 @@ S5BManager::Entry *S5BManager::findEntry(Item *i) const
 
 S5BManager::Entry *S5BManager::findEntryByHash(const QString &key) const
 {
-	QPtrListIterator<Entry> it(d->activeList);
+	Q3PtrListIterator<Entry> it(d->activeList);
 	for(Entry *e; (e = it.current()); ++it) {
 		if(e->i && e->i->key == key)
 			return e;
@@ -797,7 +804,7 @@ S5BManager::Entry *S5BManager::findEntryByHash(const QString &key) const
 
 S5BManager::Entry *S5BManager::findEntryBySID(const Jid &peer, const QString &sid) const
 {
-	QPtrListIterator<Entry> it(d->activeList);
+	Q3PtrListIterator<Entry> it(d->activeList);
 	for(Entry *e; (e = it.current()); ++it) {
 		if(e->i && e->i->peer.compare(peer) && e->sid == sid)
 			return e;
@@ -807,8 +814,8 @@ S5BManager::Entry *S5BManager::findEntryBySID(const Jid &peer, const QString &si
 
 S5BManager::Entry *S5BManager::findServerEntryByHash(const QString &key) const
 {
-	const QPtrList<S5BManager> &manList = d->serv->managerList();
-	QPtrListIterator<S5BManager> it(manList);
+	const Q3PtrList<S5BManager> &manList = d->serv->managerList();
+	Q3PtrListIterator<S5BManager> it(manList);
 	for(S5BManager *m; (m = it.current()); ++it) {
 		Entry *e = m->findEntryByHash(key);
 		if(e)
@@ -1016,7 +1023,7 @@ void S5BManager::entryContinue(Entry *e)
 
 void S5BManager::queryProxy(Entry *e)
 {
-	QGuardedPtr<QObject> self = this;
+	QPointer<QObject> self = this;
 	e->c->proxyQuery(); // signal
 	if(!self)
 		return;
@@ -1035,7 +1042,7 @@ void S5BManager::query_finished()
 	JT_S5B *query = (JT_S5B *)sender();
 	Entry *e;
 	bool found = false;
-	QPtrListIterator<Entry> it(d->activeList);
+	Q3PtrListIterator<Entry> it(d->activeList);
 	for(; (e = it.current()); ++it) {
 		if(e->query == query) {
 			found = true;
@@ -1061,7 +1068,7 @@ void S5BManager::query_finished()
 #endif
 	}
 
-	QGuardedPtr<QObject> self = this;
+	QPointer<QObject> self = this;
 	e->c->proxyResult(query->success()); // signal
 	if(!self)
 		return;
@@ -1191,7 +1198,7 @@ void S5BManager::Item::handleFast(const StreamHostList &hosts, const QString &iq
 {
 	targetMode = Fast;
 
-	QGuardedPtr<QObject> self = this;
+	QPointer<QObject> self = this;
 	accepted();
 	if(!self)
 		return;
@@ -1283,7 +1290,7 @@ void S5BManager::Item::doIncoming()
 	conn = new S5BConnector;
 	connect(conn, SIGNAL(result(bool)), SLOT(conn_result(bool)));
 
-	QGuardedPtr<QObject> self = this;
+	QPointer<QObject> self = this;
 	tryingHosts(list);
 	if(!self)
 		return;
@@ -1325,7 +1332,7 @@ void S5BManager::Item::jt_finished()
 	if(state == Initiator) {
 		if(targetMode == Unknown) {
 			targetMode = NotFast;
-			QGuardedPtr<QObject> self = this;
+			QPointer<QObject> self = this;
 			accepted();
 			if(!self)
 				return;
@@ -1381,7 +1388,7 @@ void S5BManager::Item::jt_finished()
 			StreamHostList list;
 			list += proxy;
 
-			QGuardedPtr<QObject> self = this;
+			QPointer<QObject> self = this;
 			proxyConnect();
 			if(!self)
 				return;
@@ -1607,12 +1614,12 @@ void S5BManager::Item::tryActivation()
 
 void S5BManager::Item::checkForActivation()
 {
-	QPtrList<SocksClient> clientList;
+	Q3PtrList<SocksClient> clientList;
 	if(client)
 		clientList.append(client);
 	if(client_out)
 		clientList.append(client_out);
-	QPtrListIterator<SocksClient> it(clientList);
+	Q3PtrListIterator<SocksClient> it(clientList);
 	for(SocksClient *sc; (sc = it.current()); ++it) {
 #ifdef S5B_DEBUG
 		printf("checking for activation\n");
@@ -1816,9 +1823,7 @@ private slots:
 		}
 
 		// send initialization with our JID
-		QCString cs = jid.full().utf8();
-		QByteArray a(cs.length());
-		memcpy(a.data(), cs.data(), a.size());
+		QByteArray a(jid.full().utf8());
 		client_udp->write(a);
 		++udp_tries;
 	}
@@ -1847,7 +1852,7 @@ class S5BConnector::Private
 public:
 	SocksClient *active;
 	SocksUDP *active_udp;
-	QPtrList<Item> itemList;
+	Q3PtrList<Item> itemList;
 	QString key;
 	StreamHost activeHost;
 	QTimer t;
@@ -1954,7 +1959,7 @@ void S5BConnector::t_timeout()
 void S5BConnector::man_udpSuccess(const Jid &streamHost)
 {
 	// was anyone sending to this streamhost?
-	QPtrListIterator<Item> it(d->itemList);
+	Q3PtrListIterator<Item> it(d->itemList);
 	for(Item *i; (i = it.current()); ++it) {
 		if(i->host.jid().compare(streamHost) && i->client_udp) {
 			i->udpSuccess();
@@ -2037,8 +2042,8 @@ class S5BServer::Private
 public:
 	SocksServer serv;
 	QStringList hostList;
-	QPtrList<S5BManager> manList;
-	QPtrList<Item> itemList;
+	Q3PtrList<S5BManager> manList;
+	Q3PtrList<Item> itemList;
 };
 
 S5BServer::S5BServer(QObject *parent)
@@ -2102,7 +2107,7 @@ void S5BServer::ss_incomingUDP(const QString &host, int port, const QHostAddress
 	if(port != 0 || port != 1)
 		return;
 
-	QPtrListIterator<S5BManager> it(d->manList);
+	Q3PtrListIterator<S5BManager> it(d->manList);
 	for(S5BManager *m; (m = it.current()); ++it) {
 		if(m->srv_ownsHash(host)) {
 			m->srv_incomingUDP(port == 1 ? true : false, addr, sourcePort, host, data);
@@ -2128,7 +2133,7 @@ void S5BServer::item_result(bool b)
 	d->itemList.removeRef(i);
 
 	// find the appropriate manager for this incoming connection
-	QPtrListIterator<S5BManager> it(d->manList);
+	Q3PtrListIterator<S5BManager> it(d->manList);
 	for(S5BManager *m; (m = it.current()); ++it) {
 		if(m->srv_ownsHash(key)) {
 			m->srv_incomingReady(c, key);
@@ -2152,13 +2157,13 @@ void S5BServer::unlink(S5BManager *m)
 
 void S5BServer::unlinkAll()
 {
-	QPtrListIterator<S5BManager> it(d->manList);
+	Q3PtrListIterator<S5BManager> it(d->manList);
 	for(S5BManager *m; (m = it.current()); ++it)
 		m->srv_unlink();
 	d->manList.clear();
 }
 
-const QPtrList<S5BManager> & S5BServer::managerList() const
+const Q3PtrList<S5BManager> & S5BServer::managerList() const
 {
 	return d->manList;
 }
@@ -2388,7 +2393,7 @@ bool JT_PushS5B::take(const QDomElement &e)
 
 	StreamHostList hosts;
 	QDomNodeList nl = q.elementsByTagName("streamhost");
-	for(uint n = 0; n < nl.count(); ++n) {
+	for(int n = 0; n < nl.count(); ++n) {
 		QDomElement shost = nl.item(n).toElement();
 		if(hosts.count() < MAXSTREAMHOSTS) {
 			Jid j = shost.attribute("jid");
@@ -2525,4 +2530,4 @@ void StreamHost::setIsProxy(bool b)
 
 }
 
-#include"s5b.moc"
+#include "s5b.moc"
