@@ -18,17 +18,20 @@
  *
  */
 
-#include"xmpp.h"
+#include "xmpp_jid.h"
+
+#include <QApplication>
+#include <QByteArray>
+#include <QHash>
+#include <libidn/stringprep.h>
 
-#include<qdict.h>
-#include<stringprep.h>
 
 using namespace XMPP;
 
 //----------------------------------------------------------------------------
 // StringPrepCache
 //----------------------------------------------------------------------------
-class StringPrepCache
+class StringPrepCache : public QObject
 {
 public:
 	static bool nameprep(const QString &in, int maxbytes, QString *out)
@@ -42,7 +45,7 @@ public:
 
 		StringPrepCache *that = get_instance();
 
-		Result *r = that->nameprep_table.find(in);
+		Result *r = that->nameprep_table[in];
 		if(r)
 		{
 			if(!r->norm)
@@ -52,7 +55,7 @@ public:
 			return true;
 		}
 
-		QCString cs = in.utf8();
+		QByteArray cs = in.toUtf8();
 		cs.resize(maxbytes);
 		if(stringprep(cs.data(), maxbytes, (Stringprep_profile_flags)0, stringprep_nameprep) != 0)
 		{
@@ -78,7 +81,7 @@ public:
 
 		StringPrepCache *that = get_instance();
 
-		Result *r = that->nodeprep_table.find(in);
+		Result *r = that->nodeprep_table[in];
 		if(r)
 		{
 			if(!r->norm)
@@ -88,7 +91,7 @@ public:
 			return true;
 		}
 
-		QCString cs = in.utf8();
+		QByteArray cs = in.toUtf8();
 		cs.resize(maxbytes);
 		if(stringprep(cs.data(), maxbytes, (Stringprep_profile_flags)0, stringprep_xmpp_nodeprep) != 0)
 		{
@@ -114,7 +117,7 @@ public:
 
 		StringPrepCache *that = get_instance();
 
-		Result *r = that->resourceprep_table.find(in);
+		Result *r = that->resourceprep_table[in];
 		if(r)
 		{
 			if(!r->norm)
@@ -124,7 +127,7 @@ public:
 			return true;
 		}
 
-		QCString cs = in.utf8();
+		QByteArray cs = in.toUtf8();
 		cs.resize(maxbytes);
 		if(stringprep(cs.data(), maxbytes, (Stringprep_profile_flags)0, stringprep_xmpp_resourceprep) != 0)
 		{
@@ -159,9 +162,9 @@ private:
 		}
 	};
 
-	QDict<Result> nameprep_table;
-	QDict<Result> nodeprep_table;
-	QDict<Result> resourceprep_table;
+	QHash<QString,Result*> nameprep_table;
+	QHash<QString,Result*> nodeprep_table;
+	QHash<QString,Result*> resourceprep_table;
 
 	static StringPrepCache *instance;
 
@@ -173,10 +176,24 @@ private:
 	}
 
 	StringPrepCache()
+		: QObject(qApp)
 	{
-		nameprep_table.setAutoDelete(true);
-		nodeprep_table.setAutoDelete(true);
-		resourceprep_table.setAutoDelete(true);
+	}
+
+	~StringPrepCache()
+	{
+		foreach(Result* r, nameprep_table) {
+			delete r;
+		}
+		nameprep_table.clear();
+		foreach(Result* r, nodeprep_table) {
+			delete r;
+		}
+		nodeprep_table.clear();
+		foreach(Result* r, resourceprep_table) {
+			delete r;
+		}
+		resourceprep_table.clear();
 	}
 };
 
@@ -188,6 +205,7 @@ StringPrepCache *StringPrepCache::instance = 0;
 Jid::Jid()
 {
 	valid = false;
+	null = true;
 }
 
 Jid::~Jid()
@@ -224,6 +242,7 @@ void Jid::reset()
 	n = QString();
 	r = QString();
 	valid = false;
+	null = true;
 }
 
 void Jid::update()
@@ -239,13 +258,14 @@ void Jid::update()
 		f = b + '/' + r;
 	if(f.isEmpty())
 		valid = false;
+	null = f.isEmpty() && r.isEmpty();
 }
 
 void Jid::set(const QString &s)
 {
 	QString rest, domain, node, resource;
 	QString norm_domain, norm_node, norm_resource;
-	int x = s.find('/');
+	int x = s.indexOf('/');
 	if(x != -1) {
 		rest = s.mid(0, x);
 		resource = s.mid(x+1);
@@ -259,7 +279,7 @@ void Jid::set(const QString &s)
 		return;
 	}
 
-	x = rest.find('@');
+	x = rest.indexOf('@');
 	if(x != -1) {
 		node = rest.mid(0, x);
 		domain = rest.mid(x+1);
@@ -274,6 +294,7 @@ void Jid::set(const QString &s)
 	}
 
 	valid = true;
+	null = false;
 	d = norm_domain;
 	n = norm_node;
 	r = norm_resource;
@@ -288,6 +309,7 @@ void Jid::set(const QString &domain, const QString &node, const QString &resourc
 		return;
 	}
 	valid = true;
+	null = false;
 	d = norm_domain;
 	n = norm_node;
 	r = norm_resource;
@@ -359,6 +381,9 @@ bool Jid::isEmpty() const
 
 bool Jid::compare(const Jid &a, bool compareRes) const
 {
+	if(null && a.null)
+		return true;
+
 	// only compare valid jids
 	if(!valid || !a.valid)
 		return false;
