@@ -42,22 +42,24 @@
     - sasl anonymous
 */
 
-#include"xmpp.h"
+#include "xmpp.h"
 
-#include<qtextstream.h>
-#include<qguardedptr.h>
-#include<qtimer.h>
-#include<qca.h>
-#include<stdlib.h>
-#include"bytestream.h"
-#include"base64.h"
-#include"hash.h"
-#include"simplesasl.h"
-#include"securestream.h"
-#include"protocol.h"
+#include <qtextstream.h>
+#include <qpointer.h>
+#include <qtimer.h>
+#include <QList>
+#include <Q3CString>
+#include <Q3PtrList>
+#include <stdlib.h>
+#include "bytestream.h"
+#include <QtCrypto>
+#include <QUrl>
+#include "simplesasl.h"
+#include "securestream.h"
+#include "protocol.h"
 
 #ifdef XMPP_TEST
-#include"td.h"
+#include "td.h"
 #endif
 
 //#define XMPP_DEBUG
@@ -81,387 +83,10 @@ static QByteArray randomArray(int size)
 static QString genId()
 {
 	// need SHA1 here
-	if(!QCA::isSupported(QCA::CAP_SHA1))
-		QCA::insertProvider(createProviderHash());
+	//if(!QCA::isSupported(QCA::CAP_SHA1))
+	//	QCA::insertProvider(createProviderHash());
 
-	return QCA::SHA1::hashToString(randomArray(128));
-}
-
-//----------------------------------------------------------------------------
-// Stanza
-//----------------------------------------------------------------------------
-Stanza::Error::Error(int _type, int _condition, const QString &_text, const QDomElement &_appSpec)
-{
-	type = _type;
-	condition = _condition;
-	text = _text;
-	appSpec = _appSpec;
-}
-
-class Stanza::Private
-{
-public:
-	struct ErrorTypeEntry
-	{
-		const char *str;
-		int type;
-	};
-	static ErrorTypeEntry errorTypeTable[];
-
-	struct ErrorCondEntry
-	{
-		const char *str;
-		int cond;
-	};
-	static ErrorCondEntry errorCondTable[];
-
-	static int stringToKind(const QString &s)
-	{
-		if(s == "message")
-			return Message;
-		else if(s == "presence")
-			return Presence;
-		else if(s == "iq")
-			return IQ;
-		else
-			return -1;
-	}
-
-	static QString kindToString(Kind k)
-	{
-		if(k == Message)
-			return "message";
-		else if(k == Presence)
-			return "presence";
-		else
-			return "iq";
-	}
-
-	static int stringToErrorType(const QString &s)
-	{
-		for(int n = 0; errorTypeTable[n].str; ++n) {
-			if(s == errorTypeTable[n].str)
-				return errorTypeTable[n].type;
-		}
-		return -1;
-	}
-
-	static QString errorTypeToString(int x)
-	{
-		for(int n = 0; errorTypeTable[n].str; ++n) {
-			if(x == errorTypeTable[n].type)
-				return errorTypeTable[n].str;
-		}
-		return QString();
-	}
-
-	static int stringToErrorCond(const QString &s)
-	{
-		for(int n = 0; errorCondTable[n].str; ++n) {
-			if(s == errorCondTable[n].str)
-				return errorCondTable[n].cond;
-		}
-		return -1;
-	}
-
-	static QString errorCondToString(int x)
-	{
-		for(int n = 0; errorCondTable[n].str; ++n) {
-			if(x == errorCondTable[n].cond)
-				return errorCondTable[n].str;
-		}
-		return QString();
-	}
-
-	Stream *s;
-	QDomElement e;
-};
-
-Stanza::Private::ErrorTypeEntry Stanza::Private::errorTypeTable[] =
-{
-	{ "cancel",   Cancel },
-	{ "continue", Continue },
-	{ "modify",   Modify },
-	{ "auth",     Auth },
-	{ "wait",     Wait },
-	{ 0, 0 },
-};
-
-Stanza::Private::ErrorCondEntry Stanza::Private::errorCondTable[] =
-{
-	{ "bad-request",             BadRequest },
-	{ "conflict",                Conflict },
-	{ "feature-not-implemented", FeatureNotImplemented },
-	{ "forbidden",               Forbidden },
-	{ "internal-server-error",   InternalServerError },
-	{ "item-not-found",          ItemNotFound },
-	{ "jid-malformed",           JidMalformed },
-	{ "not-allowed",             NotAllowed },
-	{ "payment-required",        PaymentRequired },
-	{ "recipient-unavailable",   RecipientUnavailable },
-	{ "registration-required",   RegistrationRequired },
-	{ "remote-server-not-found", ServerNotFound },
-	{ "remote-server-timeout",   ServerTimeout },
-	{ "resource-constraint",     ResourceConstraint },
-	{ "service-unavailable",     ServiceUnavailable },
-	{ "subscription-required",   SubscriptionRequired },
-	{ "undefined-condition",     UndefinedCondition },
-	{ "unexpected-request",      UnexpectedRequest },
-	{ 0, 0 },
-};
-
-Stanza::Stanza()
-{
-	d = 0;
-}
-
-Stanza::Stanza(Stream *s, Kind k, const Jid &to, const QString &type, const QString &id)
-{
-	d = new Private;
-
-	Kind kind;
-	if(k == Message || k == Presence || k == IQ)
-		kind = k;
-	else
-		kind = Message;
-
-	d->s = s;
-	d->e = d->s->doc().createElementNS(s->baseNS(), Private::kindToString(kind));
-	if(to.isValid())
-		setTo(to);
-	if(!type.isEmpty())
-		setType(type);
-	if(!id.isEmpty())
-		setId(id);
-}
-
-Stanza::Stanza(Stream *s, const QDomElement &e)
-{
-	d = 0;
-	if(e.namespaceURI() != s->baseNS())
-		return;
-	int x = Private::stringToKind(e.tagName());
-	if(x == -1)
-		return;
-	d = new Private;
-	d->s = s;
-	d->e = e;
-}
-
-Stanza::Stanza(const Stanza &from)
-{
-	d = 0;
-	*this = from;
-}
-
-Stanza & Stanza::operator=(const Stanza &from)
-{
-	delete d;
-	d = 0;
-	if(from.d)
-		d = new Private(*from.d);
-	return *this;
-}
-
-Stanza::~Stanza()
-{
-	delete d;
-}
-
-bool Stanza::isNull() const
-{
-	return (d ? false: true);
-}
-
-QDomElement Stanza::element() const
-{
-	return d->e;
-}
-
-QString Stanza::toString() const
-{
-	return Stream::xmlToString(d->e);
-}
-
-QDomDocument & Stanza::doc() const
-{
-	return d->s->doc();
-}
-
-QString Stanza::baseNS() const
-{
-	return d->s->baseNS();
-}
-
-QDomElement Stanza::createElement(const QString &ns, const QString &tagName)
-{
-	return d->s->doc().createElementNS(ns, tagName);
-}
-
-QDomElement Stanza::createTextElement(const QString &ns, const QString &tagName, const QString &text)
-{
-	QDomElement e = d->s->doc().createElementNS(ns, tagName);
-	e.appendChild(d->s->doc().createTextNode(text));
-	return e;
-}
-
-void Stanza::appendChild(const QDomElement &e)
-{
-	d->e.appendChild(e);
-}
-
-Stanza::Kind Stanza::kind() const
-{
-	return (Kind)Private::stringToKind(d->e.tagName());
-}
-
-void Stanza::setKind(Kind k)
-{
-	d->e.setTagName(Private::kindToString(k));
-}
-
-Jid Stanza::to() const
-{
-	return Jid(d->e.attribute("to"));
-}
-
-Jid Stanza::from() const
-{
-	return Jid(d->e.attribute("from"));
-}
-
-QString Stanza::id() const
-{
-	return d->e.attribute("id");
-}
-
-QString Stanza::type() const
-{
-	return d->e.attribute("type");
-}
-
-QString Stanza::lang() const
-{
-	return d->e.attributeNS(NS_XML, "lang", QString());
-}
-
-void Stanza::setTo(const Jid &j)
-{
-	d->e.setAttribute("to", j.full());
-}
-
-void Stanza::setFrom(const Jid &j)
-{
-	d->e.setAttribute("from", j.full());
-}
-
-void Stanza::setId(const QString &id)
-{
-	d->e.setAttribute("id", id);
-}
-
-void Stanza::setType(const QString &type)
-{
-	d->e.setAttribute("type", type);
-}
-
-void Stanza::setLang(const QString &lang)
-{
-	d->e.setAttribute("xml:lang", lang);
-}
-
-Stanza::Error Stanza::error() const
-{
-	Error err;
-	QDomElement e = d->e.elementsByTagNameNS(d->s->baseNS(), "error").item(0).toElement();
-	if(e.isNull())
-		return err;
-
-	// type
-	int x = Private::stringToErrorType(e.attribute("type"));
-	if(x != -1)
-		err.type = x;
-
-	// condition: find first element
-	QDomNodeList nl = e.childNodes();
-	QDomElement t;
-	uint n;
-	for(n = 0; n < nl.count(); ++n) {
-		QDomNode i = nl.item(n);
-		if(i.isElement()) {
-			t = i.toElement();
-			break;
-		}
-	}
-	if(!t.isNull() && t.namespaceURI() == NS_STANZAS) {
-		x = Private::stringToErrorCond(t.tagName());
-		if(x != -1)
-			err.condition = x;
-	}
-
-	// text
-	t = e.elementsByTagNameNS(NS_STANZAS, "text").item(0).toElement();
-	if(!t.isNull())
-		err.text = t.text();
-	else
-		err.text = e.text();
-
-	// appspec: find first non-standard namespaced element
-	nl = e.childNodes();
-	for(n = 0; n < nl.count(); ++n) {
-		QDomNode i = nl.item(n);
-		if(i.isElement() && i.namespaceURI() != NS_STANZAS) {
-			err.appSpec = i.toElement();
-			break;
-		}
-	}
-	return err;
-}
-
-void Stanza::setError(const Error &err)
-{
-	// create the element if necessary
-	QDomElement errElem = d->e.elementsByTagNameNS(d->s->baseNS(), "error").item(0).toElement();
-	if(errElem.isNull()) {
-		errElem = d->e.ownerDocument().createElementNS(d->s->baseNS(), "error");
-		d->e.appendChild(errElem);
-	}
-
-	// error type/condition
-	if(d->s->old()) {
-		errElem.setAttribute("code", QString::number(err.condition));
-	}
-	else {
-		QString stype = Private::errorTypeToString(err.type);
-		if(stype.isEmpty())
-			return;
-		QString scond = Private::errorCondToString(err.condition);
-		if(scond.isEmpty())
-			return;
-
-		errElem.setAttribute("type", stype);
-		errElem.appendChild(d->e.ownerDocument().createElementNS(d->s->baseNS(), scond));
-	}
-
-	// text
-	if(d->s->old()) {
-		errElem.appendChild(d->e.ownerDocument().createTextNode(err.text));
-	}
-	else {
-		QDomElement te = d->e.ownerDocument().createElementNS(d->s->baseNS(), "text");
-		te.appendChild(d->e.ownerDocument().createTextNode(err.text));
-		errElem.appendChild(te);
-	}
-
-	// application specific
-	errElem.appendChild(err.appSpec);
-}
-
-void Stanza::clearError()
-{
-	QDomElement errElem = d->e.elementsByTagNameNS(d->s->baseNS(), "error").item(0).toElement();
-	if(!errElem.isNull())
-		d->e.removeChild(errElem);
+	return QCA::Hash("sha1").hashToString(randomArray(128));
 }
 
 //----------------------------------------------------------------------------
@@ -526,12 +151,13 @@ public:
 		in.setAutoDelete(true);
 
 		oldOnly = false;
-		allowPlain = false;
+		allowPlain = NoAllowPlain;
 		mutualAuth = false;
 		haveLocalAddr = false;
 		minimumSSF = 0;
 		maximumSSF = 0;
 		doBinding = true;
+		lang = "";
 
 		in_rrsig = false;
 
@@ -551,7 +177,8 @@ public:
 	Jid jid;
 	QString server;
 	bool oldOnly;
-	bool allowPlain, mutualAuth;
+	bool mutualAuth;
+	AllowPlainType allowPlain;
 	bool haveLocalAddr;
 	QHostAddress localAddr;
 	Q_UINT16 localPort;
@@ -569,6 +196,7 @@ public:
 	SecureStream *ss;
 	CoreProtocol client;
 	CoreProtocol srv;
+	QString lang;
 
 	QString defRealm;
 
@@ -579,6 +207,7 @@ public:
 	int sasl_ssf;
 	bool tls_warned, using_tls;
 	bool doAuth;
+	bool doCompress;
 
 	QStringList sasl_mechlist;
 
@@ -586,7 +215,7 @@ public:
 	QString errText;
 	QDomElement errAppSpec;
 
-	QPtrList<Stanza> in;
+	Q3PtrList<Stanza> in;
 
 	QTimer noopTimer;
 	int noop_time;
@@ -660,8 +289,9 @@ void ClientStream::reset(bool all)
 	// client
 	if(d->mode == Client) {
 		// reset tls
-		if(d->tlsHandler)
-			d->tlsHandler->reset();
+		// FIXME: Temporarily disabled
+		//if(d->tlsHandler)
+		//	d->tlsHandler->reset();
 
 		// reset connector
 		if(d->bs) {
@@ -754,7 +384,7 @@ void ClientStream::setPassword(const QString &s)
 	}
 	else {
 		if(d->sasl)
-			d->sasl->setPassword(s);
+			d->sasl->setPassword(QCA::SecureArray(s.utf8()));
 	}
 }
 
@@ -764,6 +394,12 @@ void ClientStream::setRealm(const QString &s)
 		d->sasl->setRealm(s);
 }
 
+void ClientStream::setAuthzid(const QString &s)
+{
+	if(d->sasl)
+		d->sasl->setAuthzid(s);
+}
+
 void ClientStream::continueAfterParams()
 {
 	if(d->state == NeedParams) {
@@ -783,6 +419,11 @@ void ClientStream::setResourceBinding(bool b)
 	d->doBinding = b;
 }
 
+void ClientStream::setLang(const QString& lang)
+{
+	d->lang = lang;
+}
+
 void ClientStream::setNoopTime(int mills)
 {
 	d->noop_time = mills;
@@ -819,6 +460,11 @@ void ClientStream::setLocalAddr(const QHostAddress &addr, Q_UINT16 port)
 	d->localPort = port;
 }
 
+void ClientStream::setCompress(bool compress) 
+{
+	d->doCompress = compress;
+}
+
 int ClientStream::errorCondition() const
 {
 	return d->errCond;
@@ -861,9 +507,9 @@ QString ClientStream::baseNS() const
 	return NS_CLIENT;
 }
 
-void ClientStream::setAllowPlain(bool b)
+void ClientStream::setAllowPlain(AllowPlainType a)
 {
-	d->allowPlain = b;
+	d->allowPlain = a;
 }
 
 void ClientStream::setRequireMutualAuth(bool b)
@@ -925,10 +571,11 @@ void ClientStream::cr_connected()
 	//d->client.startDialbackOut("andbit.net", "im.pyxa.org");
 	//d->client.startServerOut(d->server);
 
-	d->client.startClientOut(d->jid, d->oldOnly, d->conn->useSSL(), d->doAuth);
+	d->client.startClientOut(d->jid, d->oldOnly, d->conn->useSSL(), d->doAuth, d->doCompress);
 	d->client.setAllowTLS(d->tlsHandler ? true: false);
 	d->client.setAllowBind(d->doBinding);
-	d->client.setAllowPlain(d->allowPlain);
+	d->client.setAllowPlain(d->allowPlain == AllowPlain || (d->allowPlain == AllowPlainOverTLS && d->conn->useSSL()));
+	d->client.setLang(d->lang);
 
 	/*d->client.jid = d->jid;
 	d->client.server = d->server;
@@ -938,7 +585,7 @@ void ClientStream::cr_connected()
 	d->client.doTLS = d->tlsHandler ? true: false;
 	d->client.doBinding = d->doBinding;*/
 
-	QGuardedPtr<QObject> self = this;
+	QPointer<QObject> self = this;
 	connected();
 	if(!self)
 		return;
@@ -981,7 +628,7 @@ void ClientStream::ss_readyRead()
 	QByteArray a = d->ss->read();
 
 #ifdef XMPP_DEBUG
-	QCString cs(a.data(), a.size()+1);
+	Q3CString cs(a.data(), a.size()+1);
 	fprintf(stderr, "ClientStream: recv: %d [%s]\n", a.size(), cs.data());
 #endif
 
@@ -1014,7 +661,7 @@ void ClientStream::ss_bytesWritten(int bytes)
 
 void ClientStream::ss_tlsHandshaken()
 {
-	QGuardedPtr<QObject> self = this;
+	QPointer<QObject> self = this;
 	securityLayerActivated(LayerTLS);
 	if(!self)
 		return;
@@ -1040,9 +687,9 @@ void ClientStream::ss_error(int x)
 	}
 }
 
-void ClientStream::sasl_clientFirstStep(const QString &mech, const QByteArray *stepData)
+void ClientStream::sasl_clientFirstStep(bool, const QByteArray& ba)
 {
-	d->client.setSASLFirst(mech, stepData ? *stepData : QByteArray());
+	d->client.setSASLFirst(d->sasl->mechanism(), ba);
 	//d->client.sasl_mech = mech;
 	//d->client.sasl_firstStep = stepData ? true : false;
 	//d->client.sasl_step = stepData ? *stepData : QByteArray();
@@ -1062,18 +709,18 @@ void ClientStream::sasl_nextStep(const QByteArray &stepData)
 	processNext();
 }
 
-void ClientStream::sasl_needParams(bool user, bool authzid, bool pass, bool realm)
+void ClientStream::sasl_needParams(const QCA::SASL::Params& p) 
 {
 #ifdef XMPP_DEBUG
-	printf("need params: %d,%d,%d,%d\n", user, authzid, pass, realm);
+	printf("need params: %d,%d,%d,%d\n", p.user, p.authzid, p.pass, p.realm);
 #endif
-	if(authzid) {
+	/*if(p.authzid && !p.user) {
 		d->sasl->setAuthzid(d->jid.bare());
 		//d->sasl->setAuthzid("infiniti.homelesshackers.org");
-	}
-	if(user || pass || realm) {
+	}*/
+	if(p.needUsername() || p.needPassword() || p.canSendRealm()) {
 		d->state = NeedParams;
-		needAuthParams(user, pass, realm);
+		needAuthParams(p.needUsername(), p.needPassword(), p.canSendRealm());
 	}
 	else
 		d->sasl->continueAfterParams();
@@ -1105,11 +752,11 @@ void ClientStream::sasl_authenticated()
 	}
 }
 
-void ClientStream::sasl_error(int)
+void ClientStream::sasl_error()
 {
-//#ifdef XMPP_DEBUG
-//	printf("sasl error: %d\n", c);
-//#endif
+#ifdef XMPP_DEBUG
+	printf("sasl error: %d\n", d->sasl->authCondition());
+#endif
 	// has to be auth error
 	int x = convertedSASLCond();
 	reset();
@@ -1136,21 +783,17 @@ void ClientStream::srvProcessNext()
 					connect(d->sasl, SIGNAL(authCheck(const QString &, const QString &)), SLOT(sasl_authCheck(const QString &, const QString &)));
 					connect(d->sasl, SIGNAL(nextStep(const QByteArray &)), SLOT(sasl_nextStep(const QByteArray &)));
 					connect(d->sasl, SIGNAL(authenticated()), SLOT(sasl_authenticated()));
-					connect(d->sasl, SIGNAL(error(int)), SLOT(sasl_error(int)));
+					connect(d->sasl, SIGNAL(error()), SLOT(sasl_error()));
 
 					//d->sasl->setAllowAnonymous(false);
 					//d->sasl->setRequirePassCredentials(true);
 					//d->sasl->setExternalAuthID("localhost");
-
-					d->sasl->setMinimumSSF(0);
-					d->sasl->setMaximumSSF(256);
+					QCA::SASL::AuthFlags auth_flags = (QCA::SASL::AuthFlags) 0;
+					d->sasl->setConstraints(auth_flags,0,256);
 
 					QStringList list;
 					// TODO: d->server is probably wrong here
-					if(!d->sasl->startServer("xmpp", d->server, d->defRealm, &list)) {
-						printf("Error initializing SASL\n");
-						return;
-					}
+					d->sasl->startServer("xmpp", d->server, d->defRealm, QCA::SASL::AllowServerSendLast);
 					d->sasl_mechlist = list;
 				}
 				d->srv.setSASLMechList(d->sasl_mechlist);
@@ -1158,11 +801,8 @@ void ClientStream::srvProcessNext()
 			}
 			else if(need == CoreProtocol::NStartTLS) {
 				printf("Need StartTLS\n");
-				if(!d->tls->startServer()) {
-					printf("unable to start server!\n");
-					// TODO
-					return;
-				}
+				//if(!d->tls->startServer()) {
+				d->tls->startServer();
 				QByteArray a = d->srv.spare;
 				d->ss->startTLSServer(d->tls, a);
 			}
@@ -1174,7 +814,7 @@ void ClientStream::srvProcessNext()
 			else if(need == CoreProtocol::NSASLNext) {
 				printf("Need SASL Next Step\n");
 				QByteArray a = d->srv.saslStep();
-				QCString cs(a.data(), a.size()+1);
+				Q3CString cs(a.data(), a.size()+1);
 				printf("[%s]\n", cs.data());
 				d->sasl->putStep(a);
 			}
@@ -1201,7 +841,7 @@ void ClientStream::srvProcessNext()
 			}
 			case CoreProtocol::ESend: {
 				QByteArray a = d->srv.takeOutgoingData();
-				QCString cs(a.size()+1);
+				Q3CString cs(a.size()+1);
 				memcpy(cs.data(), a.data(), a.size());
 				printf("Need Send: {%s}\n", cs.data());
 				d->ss->write(a);
@@ -1211,9 +851,9 @@ void ClientStream::srvProcessNext()
 				printf("Break (RecvOpen)\n");
 
 				// calculate key
-				QCString str = QCA::SHA1::hashToString("secret").utf8();
-				str = QCA::SHA1::hashToString(str + "im.pyxa.org").utf8();
-				str = QCA::SHA1::hashToString(str + d->srv.id.utf8()).utf8();
+				Q3CString str = QCA::Hash("sha1").hashToString("secret").utf8();
+				str = QCA::Hash("sha1").hashToString(str + "im.pyxa.org").utf8();
+				str = QCA::Hash("sha1").hashToString(str + d->srv.id.utf8()).utf8();
 				d->srv.setDialbackKey(str);
 
 				//d->srv.setDialbackKey("3c5d721ea2fcc45b163a11420e4e358f87e3142a");
@@ -1228,7 +868,7 @@ void ClientStream::srvProcessNext()
 			}
 			case CoreProtocol::ESASLSuccess: {
 				printf("Break SASL Success\n");
-				disconnect(d->sasl, SIGNAL(error(int)), this, SLOT(sasl_error(int)));
+				disconnect(d->sasl, SIGNAL(error()), this, SLOT(sasl_error()));
 				QByteArray a = d->srv.spare;
 				d->ss->setLayerSASL(d->sasl, a);
 				break;
@@ -1260,7 +900,7 @@ void ClientStream::processNext()
 		return;
 	}
 
-	QGuardedPtr<QObject> self = this;
+	QPointer<QObject> self = this;
 
 	while(1) {
 #ifdef XMPP_DEBUG
@@ -1268,7 +908,7 @@ void ClientStream::processNext()
 #endif
 		bool ok = d->client.processStep();
 		// deal with send/received items
-		for(QValueList<XmlProtocol::TransferItem>::ConstIterator it = d->client.transferItemList.begin(); it != d->client.transferItemList.end(); ++it) {
+		for(QList<XmlProtocol::TransferItem>::ConstIterator it = d->client.transferItemList.begin(); it != d->client.transferItemList.end(); ++it) {
 			const XmlProtocol::TransferItem &i = *it;
 			if(i.isExternal)
 				continue;
@@ -1315,7 +955,7 @@ void ClientStream::processNext()
 			case CoreProtocol::ESend: {
 				QByteArray a = d->client.takeOutgoingData();
 #ifdef XMPP_DEBUG
-				QCString cs(a.size()+1);
+				Q3CString cs(a.size()+1);
 				memcpy(cs.data(), a.data(), a.size());
 				printf("Need Send: {%s}\n", cs.data());
 #endif
@@ -1365,7 +1005,7 @@ void ClientStream::processNext()
 				printf("Done!\n");
 #endif
 				// grab the JID, in case it changed
-				// TODO: d->jid = d->client.jid;
+				d->jid = d->client.jid();
 				d->state = Active;
 				setNoopTime(d->noop_time);
 				authenticated();
@@ -1437,44 +1077,46 @@ bool ClientStream::handleNeed()
 			d->ss->startTLSClient(d->tlsHandler, d->server, d->client.spare);
 			return false;
 		}
+		case CoreProtocol::NCompress: {
+#ifdef XMPP_DEBUG
+			printf("Need compress\n");
+#endif
+			d->ss->setLayerCompress(d->client.spare);
+			return true;
+		}
 		case CoreProtocol::NSASLFirst: {
 #ifdef XMPP_DEBUG
 			printf("Need SASL First Step\n");
 #endif
 			// no SASL plugin?  fall back to Simple SASL
-			if(!QCA::isSupported(QCA::CAP_SASL)) {
-				// Simple SASL needs MD5.  do we have that either?
-				if(!QCA::isSupported(QCA::CAP_MD5))
-					QCA::insertProvider(createProviderHash());
+			if(!QCA::isSupported("sasl")) {
 				QCA::insertProvider(createProviderSimpleSASL());
 			}
 
-			d->sasl = new QCA::SASL;
-			connect(d->sasl, SIGNAL(clientFirstStep(const QString &, const QByteArray *)), SLOT(sasl_clientFirstStep(const QString &, const QByteArray *)));
+			d->sasl = new QCA::SASL();
+			connect(d->sasl, SIGNAL(clientStarted(bool,const QByteArray&)), SLOT(sasl_clientFirstStep(bool, const QByteArray&)));
 			connect(d->sasl, SIGNAL(nextStep(const QByteArray &)), SLOT(sasl_nextStep(const QByteArray &)));
-			connect(d->sasl, SIGNAL(needParams(bool, bool, bool, bool)), SLOT(sasl_needParams(bool, bool, bool, bool)));
+			connect(d->sasl, SIGNAL(needParams(const QCA::SASL::Params&)), SLOT(sasl_needParams(const QCA::SASL::Params&)));
 			connect(d->sasl, SIGNAL(authenticated()), SLOT(sasl_authenticated()));
-			connect(d->sasl, SIGNAL(error(int)), SLOT(sasl_error(int)));
+			connect(d->sasl, SIGNAL(error()), SLOT(sasl_error()));
 
 			if(d->haveLocalAddr)
-				d->sasl->setLocalAddr(d->localAddr, d->localPort);
+				d->sasl->setLocalAddress(d->localAddr.toString(), d->localPort);
 			if(d->conn->havePeerAddress())
-				d->sasl->setRemoteAddr(d->conn->peerAddress(), d->conn->peerPort());
-
-			d->sasl->setAllowAnonymous(false);
+				d->sasl->setRemoteAddress(d->conn->peerAddress().toString(), d->conn->peerPort());
 
 			//d->sasl_mech = "ANONYMOUS";
 			//d->sasl->setRequirePassCredentials(true);
-
 			//d->sasl->setExternalAuthID("localhost");
 			//d->sasl->setExternalSSF(64);
 			//d->sasl_mech = "EXTERNAL";
 
-			d->sasl->setAllowPlain(d->allowPlain);
-			d->sasl->setRequireMutualAuth(d->mutualAuth);
-
-			d->sasl->setMinimumSSF(d->minimumSSF);
-			d->sasl->setMaximumSSF(d->maximumSSF);
+			QCA::SASL::AuthFlags auth_flags = (QCA::SASL::AuthFlags) 0;
+			if (d->allowPlain == AllowPlain || (d->allowPlain == AllowPlainOverTLS && d->using_tls))
+				auth_flags = (QCA::SASL::AuthFlags) (auth_flags | QCA::SASL::AllowPlain);
+			if (d->mutualAuth)
+				auth_flags = (QCA::SASL::AuthFlags) (auth_flags | QCA::SASL::RequireMutualAuth);
+			d->sasl->setConstraints(auth_flags,d->minimumSSF,d->maximumSSF);
 
 			QStringList ml;
 			if(!d->sasl_mech.isEmpty())
@@ -1482,13 +1124,7 @@ bool ClientStream::handleNeed()
 			else
 				ml = d->client.features.sasl_mechs;
 
-			if(!d->sasl->startClient("xmpp", d->server, ml, true)) {
-				int x = convertedSASLCond();
-				reset();
-				d->errCond = x;
-				error(ErrAuth);
-				return false;
-			}
+			d->sasl->startClient("xmpp", QUrl::toAce(d->server), ml, QCA::SASL::AllowClientSendFirst);
 			return false;
 		}
 		case CoreProtocol::NSASLNext: {
@@ -1501,10 +1137,10 @@ bool ClientStream::handleNeed()
 		}
 		case CoreProtocol::NSASLLayer: {
 			// SecureStream will handle the errors from this point
-			disconnect(d->sasl, SIGNAL(error(int)), this, SLOT(sasl_error(int)));
+			disconnect(d->sasl, SIGNAL(error()), this, SLOT(sasl_error()));
 			d->ss->setLayerSASL(d->sasl, d->client.spare);
 			if(d->sasl_ssf > 0) {
-				QGuardedPtr<QObject> self = this;
+				QPointer<QObject> self = this;
 				securityLayerActivated(LayerSASL);
 				if(!self)
 					return false;
@@ -1526,17 +1162,18 @@ bool ClientStream::handleNeed()
 
 int ClientStream::convertedSASLCond() const
 {
-	int x = d->sasl->errorCondition();
-	if(x == QCA::SASL::NoMech)
+	int x = d->sasl->authCondition();
+	if(x == QCA::SASL::NoMechanism)
 		return NoMech;
-	else if(x == QCA::SASL::BadProto)
+	else if(x == QCA::SASL::BadProtocol)
 		return BadProto;
-	else if(x == QCA::SASL::BadServ)
+	else if(x == QCA::SASL::BadServer)
 		return BadServ;
 	else if(x == QCA::SASL::TooWeak)
 		return MechTooWeak;
 	else
 		return GenericAuthError;
+	return 0;
 }
 
 void ClientStream::doNoop()
