@@ -18,9 +18,10 @@
  *
  */
 
-#include"im.h"
+#include "im.h"
+#include "safedelete.h"
 
-//! \class Client client.h
+//! \class XMPP::Client client.h
 //! \brief Communicates with the Jabber network.  Start here.
 //!
 //!  Client controls an active Jabber connection.  It allows you to connect,
@@ -68,29 +69,31 @@
 //!  }
 //!  \endcode
 
-#include<stdarg.h>
-#include<qobjectlist.h>
-#include<qtimer.h>
-#include<qguardedptr.h>
-#include"xmpp_tasks.h"
-#include"xmpp_xmlcommon.h"
-#include"s5b.h"
-#include"xmpp_ibb.h"
-#include"xmpp_jidlink.h"
-#include"filetransfer.h"
+#include <stdarg.h>
+#include <qobject.h>
+#include <QMap>
+#include <qtimer.h>
+#include <qpointer.h>
+//Added by qt3to4:
+#include <QList>
+#include "xmpp_tasks.h"
+#include "xmpp_xmlcommon.h"
+#include "s5b.h"
+#include "xmpp_ibb.h"
+#include "filetransfer.h"
 
-/*#include<stdio.h>
-#include<stdarg.h>
-#include<qstring.h>
-#include<qdom.h>
-#include<qobjectlist.h>
-#include<qtimer.h>
-#include"xmpp_stream.h"
-#include"xmpp_tasks.h"
-#include"xmpp_xmlcommon.h"
-#include"xmpp_dtcp.h"
-#include"xmpp_ibb.h"
-#include"xmpp_jidlink.h"
+/*#include <stdio.h>
+#include <stdarg.h>
+#include <qstring.h>
+#include <qdom.h>
+#include <qobjectlist.h>
+#include <qtimer.h>
+#include "xmpp_stream.h"
+#include "xmpp_tasks.h"
+#include "xmpp_xmlcommon.h"
+#include "xmpp_dtcp.h"
+#include "xmpp_ibb.h"
+#include "xmpp_jidlink.h"
 
 using namespace Jabber;*/
 
@@ -112,6 +115,7 @@ public:
 
 	Jid j;
 	int status;
+	QString password;
 };
 
 class Client::ClientPrivate
@@ -124,7 +128,10 @@ public:
 	int id_seed;
 	Task *root;
 	QString host, user, pass, resource;
-	QString osname, tzname, clientName, clientVersion;
+	QString osname, tzname, clientName, clientVersion, capsNode, capsVersion, capsExt;
+	DiscoItem::Identity identity;
+	Features features;
+	QMap<QString,Features> extension_features;
 	int tzoffset;
 	bool active;
 
@@ -132,10 +139,9 @@ public:
 	ResourceList resourceList;
 	S5BManager *s5bman;
 	IBBManager *ibbman;
-	JidLinkManager *jlman;
 	FileTransferManager *ftman;
 	bool ftEnabled;
-	QValueList<GroupChat> groupChatList;
+	QList<GroupChat> groupChatList;
 };
 
 
@@ -148,6 +154,9 @@ Client::Client(QObject *par)
 	d->osname = "N/A";
 	d->clientName = "N/A";
 	d->clientVersion = "0.0";
+	d->capsNode = "";
+	d->capsVersion = "";
+	d->capsExt = "";
 
 	d->id_seed = 0xaaaa;
 	d->root = new Task(this, true);
@@ -160,8 +169,6 @@ Client::Client(QObject *par)
 	d->ibbman = new IBBManager(this);
 	connect(d->ibbman, SIGNAL(incomingReady()), SLOT(ibb_incomingReady()));
 
-	d->jlman = new JidLinkManager(this);
-
 	d->ftman = 0;
 }
 
@@ -170,7 +177,6 @@ Client::~Client()
 	close(true);
 
 	delete d->ftman;
-	delete d->jlman;
 	delete d->ibbman;
 	delete d->s5bman;
 	delete d->root;
@@ -206,7 +212,7 @@ void Client::start(const QString &host, const QString &user, const QString &pass
 	d->resourceList += Resource(resource(), stat);
 
 	JT_PushPresence *pp = new JT_PushPresence(rootTask());
-	connect(pp, SIGNAL(subscription(const Jid &, const QString &)), SLOT(ppSubscription(const Jid &, const QString &)));
+	connect(pp, SIGNAL(subscription(const Jid &, const QString &, const QString&)), SLOT(ppSubscription(const Jid &, const QString &, const QString&)));
 	connect(pp, SIGNAL(presence(const Jid &, const Status &)), SLOT(ppPresence(const Jid &, const Status &)));
 
 	JT_PushMessage *pm = new JT_PushMessage(rootTask());
@@ -239,11 +245,6 @@ FileTransferManager *Client::fileTransferManager() const
 	return d->ftman;
 }
 
-JidLinkManager *Client::jidLinkManager() const
-{
-	return d->jlman;
-}
-
 S5BManager *Client::s5bManager() const
 {
 	return d->s5bman;
@@ -259,12 +260,25 @@ bool Client::isActive() const
 	return d->active;
 }
 
+QString Client::groupChatPassword(const QString& host, const QString& room) const
+{
+	Jid jid(room + "@" + host);
+	foreach(GroupChat i, d->groupChatList) {
+		if(i.j.compare(jid, false)) {
+			return i.password;
+		}
+	}
+	return QString();
+}
+
 void Client::groupChatChangeNick(const QString &host, const QString &room, const QString &nick, const Status &_s)
 {
 	Jid jid(room + "@" + host + "/" + nick);
-	for(QValueList<GroupChat>::ConstIterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
-		const GroupChat &i = *it;
+	for(QList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
+		GroupChat &i = *it;
 		if(i.j.compare(jid, false)) {
+			i.j = jid;
+
 			Status s = _s;
 			s.setIsAvailable(true);
 
@@ -277,10 +291,10 @@ void Client::groupChatChangeNick(const QString &host, const QString &room, const
 	}
 }
 
-bool Client::groupChatJoin(const QString &host, const QString &room, const QString &nick)
+bool Client::groupChatJoin(const QString &host, const QString &room, const QString &nick, const QString& password, int maxchars, int maxstanzas, int seconds, const Status& _s)
 {
 	Jid jid(room + "@" + host + "/" + nick);
-	for(QValueList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end();) {
+	for(QList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end();) {
 		GroupChat &i = *it;
 		if(i.j.compare(jid, false)) {
 			// if this room is shutting down, then free it up
@@ -297,10 +311,17 @@ bool Client::groupChatJoin(const QString &host, const QString &room, const QStri
 	GroupChat i;
 	i.j = jid;
 	i.status = GroupChat::Connecting;
+	i.password = password;
 	d->groupChatList += i;
 
 	JT_Presence *j = new JT_Presence(rootTask());
-	j->pres(jid, Status());
+	Status s = _s;
+	s.setMUC();
+	s.setMUCHistory(maxchars,maxstanzas,seconds);
+	if (!password.isEmpty()) {
+		s.setMUCPassword(password);
+	}
+	j->pres(jid,s);
 	j->go(true);
 
 	return true;
@@ -310,7 +331,7 @@ void Client::groupChatSetStatus(const QString &host, const QString &room, const
 {
 	Jid jid(room + "@" + host);
 	bool found = false;
-	for(QValueList<GroupChat>::ConstIterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
+	for(QList<GroupChat>::ConstIterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
 		const GroupChat &i = *it;
 		if(i.j.compare(jid, false)) {
 			found = true;
@@ -332,7 +353,7 @@ void Client::groupChatSetStatus(const QString &host, const QString &room, const
 void Client::groupChatLeave(const QString &host, const QString &room)
 {
 	Jid jid(room + "@" + host);
-	for(QValueList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
+	for(QList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
 		GroupChat &i = *it;
 
 		if(!i.j.compare(jid, false))
@@ -366,7 +387,7 @@ void Client::close(bool)
 {
 	if(d->stream) {
 		if(d->active) {
-			for(QValueList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
+			for(QList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
 				GroupChat &i = *it;
 				i.status = GroupChat::Closing;
 
@@ -440,7 +461,7 @@ static QDomElement oldStyleNS(const QDomElement &e)
 		noShowNS = true;
 
 	QDomElement i;
-	uint x;
+	int x;
 	//if(noShowNS)
 		i = e.ownerDocument().createElement(e.tagName());
 	//else
@@ -469,7 +490,7 @@ static QDomElement oldStyleNS(const QDomElement &e)
 void Client::streamReadyRead()
 {
 	// HACK HACK HACK
-	QGuardedPtr<ClientStream> pstream = d->stream;
+	QPointer<ClientStream> pstream = d->stream;
 
 	while(pstream && d->stream->stanzaAvailable()) {
 		Stanza s = d->stream->read();
@@ -532,56 +553,30 @@ void Client::distribute(const QDomElement &x)
 		}
 	}
 
-	if(!rootTask()->take(x)) {
-		debug("Client: packet was ignored.\n");
+	if(!rootTask()->take(x) && (x.attribute("type") == "get" || x.attribute("type") == "set") ) {
+		debug("Client: Unrecognized IQ.\n");
+
+		// Create reply element
+		QDomElement reply = createIQ(doc(), "error", x.attribute("from"), x.attribute("id"));
+
+		// Copy children
+		for (QDomNode n = x.firstChild(); !n.isNull(); n = n.nextSibling()) {
+			reply.appendChild(n.cloneNode());
+		}
+		
+		// Add error
+		QDomElement error = doc()->createElement("error");
+		error.setAttribute("type","cancel");
+		reply.appendChild(error);
+
+		QDomElement error_type = doc()->createElement("feature-not-implemented");
+		error_type.setAttribute("xmlns","urn:ietf:params:xml:ns:xmpp-stanzas");
+		error.appendChild(error_type);
+
+		send(reply);
 	}
 }
 
-static QDomElement addCorrectNS(const QDomElement &e)
-{
-	uint x;
-
-	// grab child nodes
-	/*QDomDocumentFragment frag = e.ownerDocument().createDocumentFragment();
-	QDomNodeList nl = e.childNodes();
-	for(x = 0; x < nl.count(); ++x)
-		frag.appendChild(nl.item(x).cloneNode());*/
-
-	// find closest xmlns
-	QDomNode n = e;
-	while(!n.isNull() && !n.toElement().hasAttribute("xmlns"))
-		n = n.parentNode();
-	QString ns;
-	if(n.isNull() || !n.toElement().hasAttribute("xmlns"))
-		ns = "jabber:client";
-	else
-		ns = n.toElement().attribute("xmlns");
-
-	// make a new node
-	QDomElement i = e.ownerDocument().createElementNS(ns, e.tagName());
-
-	// copy attributes
-	QDomNamedNodeMap al = e.attributes();
-	for(x = 0; x < al.count(); ++x) {
-		QDomAttr a = al.item(x).toAttr();
-		if(a.name() != "xmlns")
-			i.setAttributeNodeNS(a.cloneNode().toAttr());
-	}
-
-	// copy children
-	QDomNodeList nl = e.childNodes();
-	for(x = 0; x < nl.count(); ++x) {
-		QDomNode n = nl.item(x);
-		if(n.isElement())
-			i.appendChild(addCorrectNS(n.toElement()));
-		else
-			i.appendChild(n.cloneNode());
-	}
-
-	//i.appendChild(frag);
-	return i;
-}
-
 void Client::send(const QDomElement &x)
 {
 	if(!d->stream)
@@ -625,6 +620,11 @@ Stream & Client::stream()
 	return *d->stream;
 }
 
+QString Client::streamBaseNS() const
+{
+	return d->stream->baseNS();
+}
+
 const LiveRoster & Client::roster() const
 {
 	return d->roster;
@@ -669,9 +669,9 @@ Jid Client::jid() const
 	return Jid(s);
 }
 
-void Client::ppSubscription(const Jid &j, const QString &s)
+void Client::ppSubscription(const Jid &j, const QString &s, const QString& n)
 {
-	subscription(j, s);
+	subscription(j, s, n);
 }
 
 void Client::ppPresence(const Jid &j, const Status &s)
@@ -681,7 +681,7 @@ void Client::ppPresence(const Jid &j, const Status &s)
 	else
 		debug(QString("Client: %1 is unavailable.\n").arg(j.full()));
 
-	for(QValueList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
+	for(QList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
 		GroupChat &i = *it;
 
 		if(i.j.compare(j, false)) {
@@ -820,7 +820,7 @@ void Client::pmMessage(const Message &m)
 	debug(QString("Client: Message from %1\n").arg(m.from().full()));
 
 	if(m.type() == "groupchat") {
-		for(QValueList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
+		for(QList<GroupChat>::Iterator it = d->groupChatList.begin(); it != d->groupChatList.end(); it++) {
 			const GroupChat &i = *it;
 
 			if(!i.j.compare(m.from(), false))
@@ -948,10 +948,10 @@ void Client::sendMessage(const Message &m)
 	j->go(true);
 }
 
-void Client::sendSubscription(const Jid &jid, const QString &type)
+void Client::sendSubscription(const Jid &jid, const QString &type, const QString& nick)
 {
 	JT_Presence *j = new JT_Presence(rootTask());
-	j->sub(jid, type);
+	j->sub(jid, type, nick);
 	j->go(true);
 }
 
@@ -993,6 +993,21 @@ QString Client::clientVersion() const
 	return d->clientVersion;
 }
 
+QString Client::capsNode() const
+{
+	return d->capsNode;
+}
+
+QString Client::capsVersion() const
+{
+	return d->capsVersion;
+}
+
+QString Client::capsExt() const
+{
+	return d->capsExt;
+}
+
 void Client::setOSName(const QString &name)
 {
 	d->osname = name;
@@ -1014,6 +1029,62 @@ void Client::setClientVersion(const QString &s)
 	d->clientVersion = s;
 }
 
+void Client::setCapsNode(const QString &s)
+{
+	d->capsNode = s;
+}
+
+void Client::setCapsVersion(const QString &s)
+{
+	d->capsVersion = s;
+}
+
+DiscoItem::Identity Client::identity()
+{
+	return d->identity;
+}
+
+void Client::setIdentity(DiscoItem::Identity identity)
+{
+	d->identity = identity;
+}
+
+void Client::setFeatures(const Features& f)
+{
+	d->features = f;
+}
+
+const Features& Client::features() const
+{
+	return d->features;
+}
+
+void Client::addExtension(const QString& ext, const Features& features)
+{
+	if (!ext.isEmpty()) {
+		d->extension_features[ext] = features;
+		d->capsExt = extensions().join(" ");
+	}
+}
+
+void Client::removeExtension(const QString& ext)
+{
+	if (d->extension_features.contains(ext)) {
+		d->extension_features.remove(ext);
+		d->capsExt = extensions().join(" ");
+	}
+}
+
+QStringList Client::extensions() const
+{
+	return d->extension_features.keys();
+}
+
+const Features& Client::extension(const QString& ext) const
+{
+	return d->extension_features[ext];
+}
+
 void Client::s5b_incomingReady()
 {
 	S5BConnection *c = d->s5bman->takeIncoming();
@@ -1025,8 +1096,6 @@ void Client::s5b_incomingReady()
 		return;
 	}
 	d->ftman->s5b_incomingReady(c);
-	//d->jlman->insertStream(c);
-	//incomingJidLink();
 }
 
 void Client::ibb_incomingReady()
@@ -1035,275 +1104,6 @@ void Client::ibb_incomingReady()
 	if(!c)
 		return;
 	c->deleteLater();
-	//d->jlman->insertStream(c);
-	//incomingJidLink();
-}
-
-//----------------------------------------------------------------------------
-// Task
-//----------------------------------------------------------------------------
-class Task::TaskPrivate
-{
-public:
-	TaskPrivate() {}
-
-	QString id;
-	bool success;
-	int statusCode;
-	QString statusString;
-	Client *client;
-	bool insig, deleteme, autoDelete;
-	bool done;
-};
-
-Task::Task(Task *parent)
-:QObject(parent)
-{
-	init();
-
-	d->client = parent->client();
-	d->id = client()->genUniqueId();
-	connect(d->client, SIGNAL(disconnected()), SLOT(clientDisconnected()));
-}
-
-Task::Task(Client *parent, bool)
-:QObject(0)
-{
-	init();
-
-	d->client = parent;
-	connect(d->client, SIGNAL(disconnected()), SLOT(clientDisconnected()));
-}
-
-Task::~Task()
-{
-	delete d;
-}
-
-void Task::init()
-{
-	d = new TaskPrivate;
-	d->success = false;
-	d->insig = false;
-	d->deleteme = false;
-	d->autoDelete = false;
-	d->done = false;
-}
-
-Task *Task::parent() const
-{
-	return (Task *)QObject::parent();
-}
-
-Client *Task::client() const
-{
-	return d->client;
-}
-
-QDomDocument *Task::doc() const
-{
-	return client()->doc();
-}
-
-QString Task::id() const
-{
-	return d->id;
-}
-
-bool Task::success() const
-{
-	return d->success;
-}
-
-int Task::statusCode() const
-{
-	return d->statusCode;
-}
-
-const QString & Task::statusString() const
-{
-	return d->statusString;
-}
-
-void Task::go(bool autoDelete)
-{
-	d->autoDelete = autoDelete;
-
-	onGo();
-}
-
-bool Task::take(const QDomElement &x)
-{
-	const QObjectList *p = children();
-	if(!p)
-		return false;
-
-	// pass along the xml
-	QObjectListIt it(*p);
-	Task *t;
-	for(; it.current(); ++it) {
-		QObject *obj = it.current();
-		if(!obj->inherits("XMPP::Task"))
-			continue;
-
-		t = static_cast<Task*>(obj);
-		if(t->take(x))
-			return true;
-	}
-
-	return false;
-}
-
-void Task::safeDelete()
-{
-	if(d->deleteme)
-		return;
-
-	d->deleteme = true;
-	if(!d->insig)
-		deleteLater();
-}
-
-void Task::onGo()
-{
-}
-
-void Task::onDisconnect()
-{
-	if(!d->done) {
-		d->success = false;
-		d->statusCode = ErrDisc;
-		d->statusString = tr("Disconnected");
-
-		// delay this so that tasks that react don't block the shutdown
-		QTimer::singleShot(0, this, SLOT(done()));
-	}
-}
-
-void Task::send(const QDomElement &x)
-{
-	client()->send(x);
-}
-
-void Task::setSuccess(int code, const QString &str)
-{
-	if(!d->done) {
-		d->success = true;
-		d->statusCode = code;
-		d->statusString = str;
-		done();
-	}
-}
-
-void Task::setError(const QDomElement &e)
-{
-	if(!d->done) {
-		d->success = false;
-		getErrorFromElement(e, &d->statusCode, &d->statusString);
-		done();
-	}
-}
-
-void Task::setError(int code, const QString &str)
-{
-	if(!d->done) {
-		d->success = false;
-		d->statusCode = code;
-		d->statusString = str;
-		done();
-	}
-}
-
-void Task::done()
-{
-	if(d->done || d->insig)
-		return;
-	d->done = true;
-
-	if(d->deleteme || d->autoDelete)
-		d->deleteme = true;
-
-	d->insig = true;
-	finished();
-	d->insig = false;
-
-	if(d->deleteme)
-		deleteLater();
-}
-
-void Task::clientDisconnected()
-{
-	onDisconnect();
-}
-
-void Task::debug(const char *fmt, ...)
-{
-	char *buf;
-	QString str;
-	int size = 1024;
-	int r;
-
-	do {
-		buf = new char[size];
-		va_list ap;
-		va_start(ap, fmt);
-		r = vsnprintf(buf, size, fmt, ap);
-		va_end(ap);
-
-		if(r != -1)
-			str = QString(buf);
-
-		delete [] buf;
-
-		size *= 2;
-	} while(r == -1);
-
-	debug(str);
-}
-
-void Task::debug(const QString &str)
-{
-	client()->debug(QString("%1: ").arg(className()) + str);
-}
-
-bool Task::iqVerify(const QDomElement &x, const Jid &to, const QString &id, const QString &xmlns)
-{
-	if(x.tagName() != "iq")
-		return false;
-
-	Jid from(x.attribute("from"));
-	Jid local = client()->jid();
-	Jid server = client()->host();
-
-	// empty 'from' ?
-	if(from.isEmpty()) {
-		// allowed if we are querying the server
-		if(!to.isEmpty() && !to.compare(server))
-			return false;
-	}
-	// from ourself?
-	else if(from.compare(local, false)) {
-		// allowed if we are querying ourself or the server
-		if(!to.isEmpty() && !to.compare(local, false) && !to.compare(server))
-			return false;
-	}
-	// from anywhere else?
-	else {
-		if(!from.compare(to))
-			return false;
-	}
-
-	if(!id.isEmpty()) {
-		if(x.attribute("id") != id)
-			return false;
-	}
-
-	if(!xmlns.isEmpty()) {
-		if(queryNS(x) != xmlns)
-			return false;
-	}
-
-	return true;
 }
 
 //---------------------------------------------------------------------------
@@ -1386,7 +1186,7 @@ void LiveRosterItem::setFlagForDelete(bool b)
 // LiveRoster
 //---------------------------------------------------------------------------
 LiveRoster::LiveRoster()
-:QValueList<LiveRosterItem>()
+:QList<LiveRosterItem>()
 {
 }
 
