@@ -18,17 +18,17 @@
  *
  */
 
-#include"xmpp_tasks.h"
+#include "xmpp_tasks.h"
 
-#include"base64.h"
-//#include"sha1.h"
-#include"xmpp_xmlcommon.h"
-//#include"xmpp_stream.h"
-//#include"xmpp_types.h"
-#include"xmpp_vcard.h"
+//#include <QtCrypto>
+//#include "sha1.h"
+#include "xmpp_xmlcommon.h"
+//#include "xmpp_stream.h"
+//#include "xmpp_types.h"
+#include "xmpp_vcard.h"
 
-#include<qregexp.h>
-#include<qvaluelist.h>
+#include <qregexp.h>
+#include <QList>
 
 using namespace XMPP;
 
@@ -45,7 +45,7 @@ static QString lineDecode(const QString &str)
 {
 	QString ret;
 
-	for(unsigned int n = 0; n < str.length(); ++n) {
+	for(int n = 0; n < str.length(); ++n) {
 		if(str.at(n) == '\\') {
 			++n;
 			if(n >= str.length())
@@ -89,6 +89,38 @@ static Roster xmlReadRoster(const QDomElement &q, bool push)
 	return r;
 }
 
+//----------------------------------------------------------------------------
+// JT_Session
+//----------------------------------------------------------------------------
+//
+#include "protocol.h"
+
+JT_Session::JT_Session(Task *parent) : Task(parent)
+{
+}
+
+void JT_Session::onGo() 
+{
+	QDomElement iq = createIQ(doc(), "set", "", id());
+	QDomElement session = doc()->createElement("session");
+	session.setAttribute("xmlns",NS_SESSION);
+	iq.appendChild(session);
+	send(iq);
+}
+
+bool JT_Session::take(const QDomElement& x) 
+{
+	if(!iqVerify(x, "", id()))
+		return false;
+
+	if(x.attribute("type") == "result") {
+		setSuccess();
+	}
+	else {
+		setError(x);
+	}
+	return true;
+}
 
 //----------------------------------------------------------------------------
 // JT_Register
@@ -99,6 +131,8 @@ public:
 	Private() {}
 
 	Form form;
+	XData xdata;
+	bool hasXData;
 	Jid jid;
 	int type;
 };
@@ -108,6 +142,7 @@ JT_Register::JT_Register(Task *parent)
 {
 	d = new Private;
 	d->type = -1;
+	d->hasXData = false;
 }
 
 JT_Register::~JT_Register()
@@ -185,11 +220,31 @@ void JT_Register::setForm(const Form &form)
 	}
 }
 
+void JT_Register::setForm(const Jid& to, const XData& xdata)
+{
+	d->type = 4;
+	iq = createIQ(doc(), "set", to.full(), id());
+	QDomElement query = doc()->createElement("query");
+	query.setAttribute("xmlns", "jabber:iq:register");
+	iq.appendChild(query);
+	query.appendChild(xdata.toXml(doc(), true));
+}
+
 const Form & JT_Register::form() const
 {
 	return d->form;
 }
 
+bool JT_Register::hasXData() const
+{
+	return d->hasXData;
+}
+
+const XData& JT_Register::xdata() const
+{
+	return d->xdata;
+}
+
 void JT_Register::onGo()
 {
 	send(iq);
@@ -216,6 +271,10 @@ bool JT_Register::take(const QDomElement &x)
 					d->form.setInstructions(tagContent(i));
 				else if(i.tagName() == "key")
 					d->form.setKey(tagContent(i));
+				else if(i.tagName() == "x" && i.attribute("xmlns") == "jabber:x:data") {
+					d->xdata.fromXml(i);
+					d->hasXData = true;
+				}
 				else {
 					FormField f;
 					if(f.setType(i.tagName())) {
@@ -303,7 +362,7 @@ public:
 	Private() {}
 
 	Roster roster;
-	QValueList<QDomElement> itemList;
+	QList<QDomElement> itemList;
 };
 
 JT_Roster::JT_Roster(Task *parent)
@@ -361,7 +420,7 @@ void JT_Roster::onGo()
 		QDomElement query = doc()->createElement("query");
 		query.setAttribute("xmlns", "jabber:iq:roster");
 		iq.appendChild(query);
-		for(QValueList<QDomElement>::ConstIterator it = d->itemList.begin(); it != d->itemList.end(); ++it)
+		for(QList<QDomElement>::ConstIterator it = d->itemList.begin(); it != d->itemList.end(); ++it)
 			query.appendChild(*it);
 		send(iq);
 	}
@@ -379,7 +438,7 @@ QString JT_Roster::toString() const
 
 	QDomElement i = doc()->createElement("request");
 	i.setAttribute("type", "JT_Roster");
-	for(QValueList<QDomElement>::ConstIterator it = d->itemList.begin(); it != d->itemList.end(); ++it)
+	for(QList<QDomElement>::ConstIterator it = d->itemList.begin(); it != d->itemList.end(); ++it)
 		i.appendChild(*it);
 	return lineEncode(Stream::xmlToString(i));
 	return "";
@@ -467,6 +526,7 @@ bool JT_PushRoster::take(const QDomElement &e)
 		return false;
 
 	roster(xmlReadRoster(queryTag(e), true));
+	send(createIQ(doc(), "result", e.attribute("from"), e.attribute("id")));
 
 	return true;
 }
@@ -516,6 +576,35 @@ void JT_Presence::pres(const Status &s)
 			x.setAttribute("xmlns", "jabber:x:signed");
 			tag.appendChild(x);
 		}
+
+		if(!s.capsNode().isEmpty() && !s.capsVersion().isEmpty()) {
+			QDomElement c = doc()->createElement("c");
+			c.setAttribute("xmlns","http://jabber.org/protocol/caps");
+			c.setAttribute("node",s.capsNode());
+			c.setAttribute("ver",s.capsVersion());
+			if (!s.capsExt().isEmpty()) 
+				c.setAttribute("ext",s.capsExt());
+			tag.appendChild(c);
+		}
+
+		if(s.isMUC()) {
+			QDomElement m = doc()->createElement("x");
+			m.setAttribute("xmlns","http://jabber.org/protocol/muc");
+			if (!s.mucPassword().isEmpty()) {
+				m.appendChild(textTag(doc(),"password",s.mucPassword()));
+			}
+			if (s.hasMUCHistory()) {
+				QDomElement h = doc()->createElement("history");
+				if (s.mucHistoryMaxChars() >= 0)
+					h.setAttribute("maxchars",s.mucHistoryMaxChars());
+				if (s.mucHistoryMaxStanzas() >= 0)
+					h.setAttribute("maxstanzas",s.mucHistoryMaxStanzas());
+				if (s.mucHistorySeconds() >= 0)
+					h.setAttribute("seconds",s.mucHistorySeconds());
+				m.appendChild(h);
+			}
+			tag.appendChild(m);
+		}
 	}
 }
 
@@ -525,13 +614,18 @@ void JT_Presence::pres(const Jid &to, const Status &s)
 	tag.setAttribute("to", to.full());
 }
 
-void JT_Presence::sub(const Jid &to, const QString &subType)
+void JT_Presence::sub(const Jid &to, const QString &subType, const QString& nick)
 {
 	type = 1;
 
 	tag = doc()->createElement("presence");
 	tag.setAttribute("to", to.full());
 	tag.setAttribute("type", subType);
+	if (!nick.isEmpty()) {
+		QDomElement nick_tag = textTag(doc(),"nick",nick);
+		nick_tag.setAttribute("xmlns","http://jabber.org/protocol/nick");
+		tag.appendChild(nick_tag);
+	}
 }
 
 void JT_Presence::onGo()
@@ -569,11 +663,17 @@ bool JT_PushPresence::take(const QDomElement &e)
 		else if(type == "error") {
 			QString str = "";
 			int code = 0;
-			getErrorFromElement(e, &code, &str);
+			getErrorFromElement(e, client()->stream().baseNS(), &code, &str);
 			p.setError(code, str);
 		}
-		else {
-			subscription(j, type);
+		else if(type == "subscribe" || type == "subscribed" || type == "unsubscribe" || type == "unsubscribed") {
+			QString nick;
+			bool found;
+			QDomElement tag = findSubTag(e, "nick", &found);
+			if (found && tag.attribute("xmlns") == "http://jabber.org/protocol/nick") {
+				nick = tagContent(tag);
+			}
+			subscription(j, type, nick);
 			return true;
 		}
 	}
@@ -625,6 +725,34 @@ bool JT_PushPresence::take(const QDomElement &e)
 		else if(i.tagName() == "x" && i.attribute("xmlns") == "http://jabber.org/protocol/e2e") {
 			p.setKeyID(tagContent(i));
 		}
+ 		else if(i.tagName() == "c" && i.attribute("xmlns") == "http://jabber.org/protocol/caps") {
+ 			p.setCapsNode(i.attribute("node"));
+ 			p.setCapsVersion(i.attribute("ver"));
+ 			p.setCapsExt(i.attribute("ext"));
+  		}
+		else if(i.tagName() == "x" && i.attribute("xmlns") == "vcard-temp:x:update") {
+			QDomElement t;
+			bool found;
+			t = findSubTag(i, "photo", &found);
+			if (found)
+				p.setPhotoHash(tagContent(t));
+			else
+				p.setPhotoHash("");
+		}
+		else if(i.tagName() == "x" && i.attribute("xmlns") == "http://jabber.org/protocol/muc#user") {
+			for(QDomNode muc_n = i.firstChild(); !muc_n.isNull(); muc_n = muc_n.nextSibling()) {
+				QDomElement muc_e = muc_n.toElement();
+				if(muc_e.isNull())
+					continue;
+
+				if (muc_e.tagName() == "item") 
+					p.setMUCItem(MUCItem(muc_e));
+				else if (muc_e.tagName() == "status") 
+					p.setMUCStatus(muc_e.attribute("code").toInt());
+				else if (muc_e.tagName() == "destroy")
+					p.setMUCDestroy(MUCDestroy(muc_e));
+			}
+		}
 	}
 
 	presence(j, p);
@@ -647,7 +775,7 @@ static QDomElement oldStyleNS(const QDomElement &e)
 		noShowNS = true;
 
 	QDomElement i;
-	uint x;
+	int x;
 	//if(noShowNS)
 		i = e.ownerDocument().createElement(e.tagName());
 	//else
@@ -677,6 +805,8 @@ JT_Message::JT_Message(Task *parent, const Message &msg)
 :Task(parent)
 {
 	m = msg;
+	if (m.id().isEmpty())
+		m.setId(id());
 }
 
 JT_Message::~JT_Message()
@@ -695,51 +825,6 @@ void JT_Message::onGo()
 //----------------------------------------------------------------------------
 // JT_PushMessage
 //----------------------------------------------------------------------------
-static QDomElement addCorrectNS(const QDomElement &e)
-{
-	uint x;
-
-	// grab child nodes
-	/*QDomDocumentFragment frag = e.ownerDocument().createDocumentFragment();
-	QDomNodeList nl = e.childNodes();
-	for(x = 0; x < nl.count(); ++x)
-		frag.appendChild(nl.item(x).cloneNode());*/
-
-	// find closest xmlns
-	QDomNode n = e;
-	while(!n.isNull() && !n.toElement().hasAttribute("xmlns"))
-		n = n.parentNode();
-	QString ns;
-	if(n.isNull() || !n.toElement().hasAttribute("xmlns"))
-		ns = "jabber:client";
-	else
-		ns = n.toElement().attribute("xmlns");
-
-	// make a new node
-	QDomElement i = e.ownerDocument().createElementNS(ns, e.tagName());
-
-	// copy attributes
-	QDomNamedNodeMap al = e.attributes();
-	for(x = 0; x < al.count(); ++x) {
-		QDomAttr a = al.item(x).toAttr();
-		if(a.name() != "xmlns")
-			i.setAttributeNodeNS(al.item(x).cloneNode().toAttr());
-	}
-
-	// copy children
-	QDomNodeList nl = e.childNodes();
-	for(x = 0; x < nl.count(); ++x) {
-		QDomNode n = nl.item(x);
-		if(n.isElement())
-			i.appendChild(addCorrectNS(n.toElement()));
-		else
-			i.appendChild(n.cloneNode());
-	}
-
-	//i.appendChild(frag);
-	return i;
-}
-
 JT_PushMessage::JT_PushMessage(Task *parent)
 :Task(parent)
 {
@@ -968,7 +1053,7 @@ public:
 
 	Jid jid;
 	Form form;
-	QValueList<SearchResult> resultList;
+	QList<SearchResult> resultList;
 };
 
 JT_Search::JT_Search(Task *parent)
@@ -1018,7 +1103,7 @@ const Form & JT_Search::form() const
 	return d->form;
 }
 
-const QValueList<SearchResult> & JT_Search::results() const
+const QList<SearchResult> & JT_Search::results() const
 {
 	return d->resultList;
 }
@@ -1264,25 +1349,117 @@ bool JT_ServInfo::take(const QDomElement &e)
 	//	return TRUE;
 	//}
 	else if(ns == "http://jabber.org/protocol/disco#info") {
+		// Find out the node
+		bool invalid_node = false;
+		QString node;
+		bool found;
+		QDomElement q = findSubTag(e, "query", &found);
+		if(found) // NOTE: Should always be true, since a NS was found above
+			node = q.attribute("node");
+		
 		QDomElement iq = createIQ(doc(), "result", e.attribute("from"), e.attribute("id"));
 		QDomElement query = doc()->createElement("query");
 		query.setAttribute("xmlns", "http://jabber.org/protocol/disco#info");
+		if (!node.isEmpty())
+			query.setAttribute("node", node);
 		iq.appendChild(query);
+
+		// Identity
+		DiscoItem::Identity identity = client()->identity();
+		QDomElement id = doc()->createElement("identity");
+		if (!identity.category.isEmpty() && !identity.type.isEmpty()) {
+			id.setAttribute("category",identity.category);
+			id.setAttribute("type",identity.type);
+			if (!identity.name.isEmpty()) {
+				id.setAttribute("name",identity.name);
+			}
+		}
+		else {
+			// Default values
+			id.setAttribute("category","client");
+			id.setAttribute("type","pc");
+		}
+		query.appendChild(id);
+
 		QDomElement feature;
+		if (node.isEmpty() || node == client()->capsNode() + "#" + client()->capsVersion()) {
+			// Standard features
+			feature = doc()->createElement("feature");
+			feature.setAttribute("var", "http://jabber.org/protocol/bytestreams");
+			query.appendChild(feature);
 
-		feature = doc()->createElement("feature");
-		feature.setAttribute("var", "http://jabber.org/protocol/bytestreams");
-		query.appendChild(feature);
+			feature = doc()->createElement("feature");
+			feature.setAttribute("var", "http://jabber.org/protocol/si");
+			query.appendChild(feature);
 
-		feature = doc()->createElement("feature");
-		feature.setAttribute("var", "http://jabber.org/protocol/si");
-		query.appendChild(feature);
+			feature = doc()->createElement("feature");
+			feature.setAttribute("var", "http://jabber.org/protocol/si/profile/file-transfer");
+			query.appendChild(feature);
+			
+			feature = doc()->createElement("feature");
+			feature.setAttribute("var", "http://jabber.org/protocol/disco#info");
+			query.appendChild(feature);
 
-		feature = doc()->createElement("feature");
-		feature.setAttribute("var", "http://jabber.org/protocol/si/profile/file-transfer");
-		query.appendChild(feature);
+			// Client-specific features
+			QStringList clientFeatures = client()->features().list();
+			for (QStringList::ConstIterator i = clientFeatures.begin(); i != clientFeatures.end(); ++i) {
+				feature = doc()->createElement("feature");
+				feature.setAttribute("var", *i);
+				query.appendChild(feature);
+			}
 
-		send(iq);
+			if (node.isEmpty()) {
+				// Extended features
+				QStringList exts = client()->extensions();
+				for (QStringList::ConstIterator i = exts.begin(); i != exts.end(); ++i) {
+					const QStringList& l = client()->extension(*i).list();
+					for ( QStringList::ConstIterator j = l.begin(); j != l.end(); ++j ) {
+						feature = doc()->createElement("feature");
+						feature.setAttribute("var", *j);
+						query.appendChild(feature);
+					}
+				}
+			}
+		}
+		else if (node.startsWith(client()->capsNode() + "#")) {
+			QString ext = node.right(node.length()-client()->capsNode().length()-1);
+			if (client()->extensions().contains(ext)) {
+				const QStringList& l = client()->extension(ext).list();
+				for ( QStringList::ConstIterator it = l.begin(); it != l.end(); ++it ) {
+					feature = doc()->createElement("feature");
+					feature.setAttribute("var", *it);
+					query.appendChild(feature);
+				}
+			}
+			else {
+				invalid_node = true;
+			}
+		}
+		else {
+			invalid_node = true;
+		}
+		
+		if (!invalid_node) {
+			send(iq);
+		}
+		else {
+			// Create error reply
+			QDomElement error_reply = createIQ(doc(), "result", e.attribute("from"), e.attribute("id"));
+			
+			// Copy children
+			for (QDomNode n = e.firstChild(); !n.isNull(); n = n.nextSibling()) {
+				error_reply.appendChild(n.cloneNode());
+			}
+			
+			// Add error
+			QDomElement error = doc()->createElement("error");
+			error.setAttribute("type","cancel");
+			error_reply.appendChild(error);
+			QDomElement error_type = doc()->createElement("item-not-found");
+			error_type.setAttribute("xmlns","urn:ietf:params:xml:ns:xmpp-stanzas");
+			error.appendChild(error_type);
+			send(error_reply);
+		}
 		return true;
 	}
 
@@ -1575,7 +1752,7 @@ bool JT_DiscoItems::take(const QDomElement &x)
 				item.setNode( e.attribute("node") );
 				item.setAction( DiscoItem::string2action(e.attribute("action")) );
 
-				d->items.push_back( item );
+				d->items.append( item );
 			}
 		}
 
@@ -1588,124 +1765,6 @@ bool JT_DiscoItems::take(const QDomElement &x)
 	return true;
 }
 
-//----------------------------------------------------------------------------
-// JT_DiscoInfo
-//----------------------------------------------------------------------------
-class JT_DiscoInfo::Private
-{
-public:
-	Private() { }
-
-	QDomElement iq;
-	Jid jid;
-	DiscoItem item;
-};
-
-JT_DiscoInfo::JT_DiscoInfo(Task *parent)
-: Task(parent)
-{
-	d = new Private;
-}
-
-JT_DiscoInfo::~JT_DiscoInfo()
-{
-	delete d;
-}
-
-void JT_DiscoInfo::get(const DiscoItem &item)
-{
-	DiscoItem::Identity id;
-	if ( item.identities().count() == 1 )
-		id = item.identities().first();
-	get(item.jid(), item.node(), id);
-}
-
-void JT_DiscoInfo::get (const Jid &j, const QString &node, DiscoItem::Identity ident)
-{
-	d->item = DiscoItem(); // clear item
-
-	d->jid = j;
-	d->iq = createIQ(doc(), "get", d->jid.full(), id());
-	QDomElement query = doc()->createElement("query");
-	query.setAttribute("xmlns", "http://jabber.org/protocol/disco#info");
-
-	if ( !node.isEmpty() )
-		query.setAttribute("node", node);
-
-	if ( !ident.category.isEmpty() && !ident.type.isEmpty() ) {
-		QDomElement i = doc()->createElement("item");
-
-		i.setAttribute("category", ident.category);
-		i.setAttribute("type", ident.type);
-		if ( !ident.name.isEmpty() )
-			i.setAttribute("name", ident.name);
-
-		query.appendChild( i );
-
-	}
-
-	d->iq.appendChild(query);
-}
-
-const DiscoItem &JT_DiscoInfo::item() const
-{
-	return d->item;
-}
-
-void JT_DiscoInfo::onGo ()
-{
-	send(d->iq);
-}
-
-bool JT_DiscoInfo::take(const QDomElement &x)
-{
-	if(!iqVerify(x, d->jid, id()))
-		return false;
-
-	if(x.attribute("type") == "result") {
-		QDomElement q = queryTag(x);
-
-		DiscoItem item;
-
-		item.setJid( d->jid );
-		item.setNode( q.attribute("node") );
-
-		QStringList features;
-		DiscoItem::Identities identities;
-
-		for(QDomNode n = q.firstChild(); !n.isNull(); n = n.nextSibling()) {
-			QDomElement e = n.toElement();
-			if( e.isNull() )
-				continue;
-
-			if ( e.tagName() == "feature" ) {
-				features << e.attribute("var");
-			}
-			else if ( e.tagName() == "identity" ) {
-				DiscoItem::Identity id;
-
-				id.category = e.attribute("category");
-				id.name     = e.attribute("name");
-				id.type     = e.attribute("type");
-
-				identities.push_back( id );
-			}
-		}
-
-		item.setFeatures( features );
-		item.setIdentities( identities );
-
-		d->item = item;
-
-		setSuccess(true);
-	}
-	else {
-		setError(x);
-	}
-
-	return true;
-}
-
 //----------------------------------------------------------------------------
 // JT_DiscoPublish
 //----------------------------------------------------------------------------
