@@ -18,27 +18,27 @@
  *
  */
 
-#include"adduserdlg.h"
+#include "adduserdlg.h"
 
-#include<qlabel.h>
-#include<qlayout.h>
-#include<qpushbutton.h>
-#include<qmessagebox.h>
-#include<qcombobox.h>
-#include<qgroupbox.h>
-#include<qstringlist.h>
-#include<qlineedit.h>
-#include<qcheckbox.h>
-#include"xmpp_tasks.h"
-#include"psiaccount.h"
-#include"contactview.h"
-#include"busywidget.h"
-#include"common.h"
-#include"iconwidget.h"
-#include"tasklist.h"
-#include"xmpp_vcard.h"
-#include"vcardfactory.h"
-#include"infodlg.h"
+#include <qlabel.h>
+#include <qlayout.h>
+#include <qpushbutton.h>
+#include <qmessagebox.h>
+#include <qcombobox.h>
+#include <qstringlist.h>
+#include <qlineedit.h>
+#include <qcheckbox.h>
+#include "xmpp_tasks.h"
+#include "psiaccount.h"
+#include "psiiconset.h"
+#include "contactview.h"
+#include "busywidget.h"
+#include "common.h"
+#include "iconwidget.h"
+#include "tasklist.h"
+#include "xmpp_vcard.h"
+#include "vcardfactory.h"
+#include "infodlg.h"
 
 class AddUserDlg::Private
 {
@@ -53,8 +53,10 @@ public:
 };
 
 AddUserDlg::AddUserDlg(const QStringList &services, const QStringList &names, const QStringList &groups, PsiAccount *pa)
-:AddUserUI(0, 0, false, WDestructiveClose)
+:QDialog(0)
 {
+  	setupUi(this);
+	setModal(false);
 	d = new Private;
 	d->pa = pa;
 	d->pa->dialogRegister(this);
@@ -65,15 +67,15 @@ AddUserDlg::AddUserDlg(const QStringList &services, const QStringList &names, co
 	connect(d->tasks, SIGNAL(started()),  busy, SLOT(start()));
 	connect(d->tasks, SIGNAL(finished()), busy, SLOT(stop()));
 
-	setCaption(CAP(caption()));
-	setIcon(IconsetFactory::icon("psi/addContact"));
+	setWindowTitle(CAP(caption()));
+	setWindowIcon(IconsetFactory::icon("psi/addContact").icon());
 
 	d->busy = busy;
 
 	QStringList::ConstIterator it1 = services.begin();
 	QStringList::ConstIterator it2 = names.begin();
 	for(; it1 != services.end(); ++it1, ++it2)
-		cb_service->insertItem(is->status(*it1, STATUS_ONLINE), *it2);
+		cb_service->insertItem(PsiIconset::instance()->status(*it1, STATUS_ONLINE).impix(), *it2);
 	connect(cb_service, SIGNAL(activated(int)), SLOT(serviceActivated(int)));
 
 	connect(le_transPrompt, SIGNAL(textChanged(const QString &)), SLOT(le_transPromptChanged(const QString &)));
@@ -95,6 +97,8 @@ AddUserDlg::AddUserDlg(const QStringList &services, const QStringList &names, co
 	connect(tb_vCard, SIGNAL(clicked()), SLOT(getVCardActivated()));
 	connect(tb_resolveNick, SIGNAL(clicked()), SLOT(resolveNickActivated()));
 
+	connect(le_jid, SIGNAL(textChanged(QString)), SLOT(jid_Changed()));
+
 	ck_authreq->setChecked(true);
 	ck_close->setChecked(true);
 
@@ -114,14 +118,18 @@ void AddUserDlg::pa_updatedActivity()
 		close();
 }
 
+Jid AddUserDlg::jid() const
+{
+	return Jid(le_jid->text().stripWhiteSpace());
+}
+
 void AddUserDlg::ok()
 {
 	if(le_jid->text().isEmpty()) {
 		QMessageBox::information(this, tr("Add User: Error"), tr("Please fill in the Jabber ID of the person you wish to add."));
 		return;
 	}
-	Jid j(le_jid->text().stripWhiteSpace());
-	if(!j.isValid()) {
+	if(!jid().isValid()) {
 		QMessageBox::information(this, tr("Add User: Error"), tr("The Jabber ID you entered is not valid!\nMake sure you enter a fully qualified Jabber ID."));
 		return;
 	}
@@ -131,15 +139,14 @@ void AddUserDlg::ok()
 	if(gname != tr("<None>"))
 		list += gname;
 
-	add(j, le_nick->text(), list, ck_authreq->isChecked());
-
-	QMessageBox::information(this, tr("Add User: Success"), tr("Added %1 to your roster.").arg(j.full()));
+	add(jid(), le_nick->text(), list, ck_authreq->isChecked());
 
+	QMessageBox::information(this, tr("Add User: Success"), tr("Added %1 to your roster.").arg(jid().full()));
+	le_jid->setText("");
+	le_nick->setText("");
 	if(ck_close->isChecked())
 		accept();
 	else {
-		le_jid->setText("");
-		le_nick->setText("");
 		le_jid->setFocus();
 	}
 }
@@ -241,20 +248,18 @@ void AddUserDlg::errorGateway(const QString &str, const QString &err)
 		"&nbsp;&nbsp;joe%hotmail.com@[Jabber ID of MSN Transport]<br>"
 		"&nbsp;&nbsp;yahooUser@[Jabber ID of Yahoo Transport]<br>"
 		"</qt>"
-		).arg(str).arg(err));
+		).arg(str).arg(QString(err).replace('\n', "<br>")));
 }
 
 void AddUserDlg::getVCardActivated()
 {
-	Jid j(le_jid->text().stripWhiteSpace());
-
-	const VCard *vcard = VCardFactory::vcard(j);
+	const VCard *vcard = VCardFactory::instance()->vcard(jid());
 
 	VCard tmp;
 	if ( vcard )
 		tmp = *vcard;
 
-	InfoDlg *w = new InfoDlg(InfoDlg::Contact, j, tmp, d->pa, 0, 0, false);
+	InfoDlg *w = new InfoDlg(InfoDlg::Contact, jid(), tmp, d->pa, 0, false);
 	w->show();
 
 	// automatically retrieve info if it doesn't exist
@@ -264,8 +269,7 @@ void AddUserDlg::getVCardActivated()
 
 void AddUserDlg::resolveNickActivated()
 {
-	Jid j(le_jid->text().stripWhiteSpace());
-	JT_VCard *jt = VCardFactory::getVCard(j, d->pa->client()->rootTask(), this, SLOT(resolveNickFinished()), false);
+	JT_VCard *jt = VCardFactory::instance()->getVCard(jid(), d->pa->client()->rootTask(), this, SLOT(resolveNickFinished()), false);
 	d->tasks->append( jt );
 }
 
@@ -278,3 +282,13 @@ void AddUserDlg::resolveNickFinished()
 			le_nick->setText( jt->vcard().nickName() );
 	}
 }
+
+/**
+ * Called when the Jid changes to enable the vcard and nick resolution buttons.
+ */
+void AddUserDlg::jid_Changed()
+{
+	bool enableVCardButtons = jid().isValid();
+	tb_vCard->setEnabled(enableVCardButtons);
+	tb_resolveNick->setEnabled(enableVCardButtons);
+}
