@@ -22,6 +22,8 @@
 #include "common.h"
 #include <qtimer.h>
 #include <qapplication.h>
+//Added by qt3to4:
+#include <QPixmap>
 
 //----------------------------------------------------------------------------
 // MetaAlertIcon
@@ -64,9 +66,8 @@ MetaAlertIcon::MetaAlertIcon()
 	frame = 0;
 
 	// blank icon
-	QImage blankImg(16, 16, 32);
-	blankImg.fill(0);
-	blankImg.setAlphaBuffer(true);
+	QImage blankImg(16, 16, QImage::Format_ARGB32);
+	blankImg.fill(0x00000000);
 	_blank16.setImage(blankImg);
 }
 
@@ -114,11 +115,11 @@ public slots:
 	void stop();
 
 	void updateFrame(int frame);
-	void pixmapChanged(const QPixmap &);
+	void pixmapChanged();
 
 public:
 	AlertIcon *ai;
-	Icon *real;
+	PsiIcon *real;
 	bool isActivated;
 	Impix impix;
 };
@@ -145,7 +146,7 @@ AlertIcon::Private::~Private()
 void AlertIcon::Private::init()
 {
 	connect(metaAlertIcon, SIGNAL(update()), SLOT(update()));
-	connect(real, SIGNAL(iconModified(const QPixmap &)), SLOT(pixmapChanged(const QPixmap &)));
+	connect(real, SIGNAL(iconModified()), SLOT(pixmapChanged()));
 
 	if ( option.alertStyle == 2 && real->isAnimated() )
 		impix = real->frameImpix();
@@ -163,7 +164,7 @@ void AlertIcon::Private::activated(bool playSound)
 {
 	if ( option.alertStyle == 2 && real->isAnimated() ) {
 		if ( !isActivated ) {
-			connect(real, SIGNAL(pixmapChanged(const QPixmap &)), SLOT(pixmapChanged(const QPixmap &)));
+			connect(real, SIGNAL(pixmapChanged()), SLOT(pixmapChanged()));
 			real->activated(playSound);
 			isActivated = true;
 		}
@@ -173,7 +174,7 @@ void AlertIcon::Private::activated(bool playSound)
 	}
 	else {
 		impix = real->impix();
-		emit ai->pixmapChanged(impix.pixmap());
+		emit ai->pixmapChanged();
 	}
 }
 
@@ -182,7 +183,7 @@ void AlertIcon::Private::stop()
 	disconnect(metaAlertIcon, SIGNAL(updateFrame(int)), this, SLOT(updateFrame(int)));
 
 	if ( isActivated ) {
-		disconnect(real, SIGNAL(pixmapChanged(const QPixmap &)), this, SLOT(pixmapChanged(const QPixmap &)));
+		disconnect(real, SIGNAL(pixmapChanged()), this, SLOT(pixmapChanged()));
 		real->stop();
 		isActivated = false;
 	}
@@ -198,27 +199,27 @@ void AlertIcon::Private::updateFrame(int frame)
 	else
 		impix = metaAlertIcon->blank16();
 
-	emit ai->pixmapChanged(impix.pixmap());
+	emit ai->pixmapChanged();
 }
 
-void AlertIcon::Private::pixmapChanged(const QPixmap &p)
+void AlertIcon::Private::pixmapChanged()
 {
 	impix = real->frameImpix();
 
-	emit ai->pixmapChanged(p);
+	emit ai->pixmapChanged();
 }
 
 //----------------------------------------------------------------------------
 // AlertIcon
 //----------------------------------------------------------------------------
 
-AlertIcon::AlertIcon(const Icon *icon)
+AlertIcon::AlertIcon(const PsiIcon *icon)
 {
 	d = new Private(this);
 	if ( icon )
-		d->real = new Icon(*icon);
+		d->real = new PsiIcon(*icon);
 	else
-		d->real = new Icon();
+		d->real = new PsiIcon();
 
 	d->init();
 }
@@ -253,9 +254,9 @@ void AlertIcon::stop()
 	d->stop();
 }
 
-const QIconSet &AlertIcon::iconSet() const
+const QIcon &AlertIcon::icon() const
 {
-	return d->real->iconSet();
+	return d->real->icon();
 }
 
 const Impix &AlertIcon::impix() const
@@ -263,10 +264,10 @@ const Impix &AlertIcon::impix() const
 	return d->impix;
 }
 
-int AlertIcon::framenumber() const
+int AlertIcon::frameNumber() const
 {
 	if ( option.alertStyle == 2 && d->real->isAnimated() ) {
-		return d->real->framenumber();
+		return d->real->frameNumber();
 	}
 	else if ( option.alertStyle == 1 || (option.alertStyle == 2 && !d->real->isAnimated()) ) {
 		return metaAlertIcon->framenumber();
@@ -280,6 +281,11 @@ const QString &AlertIcon::name() const
 	return d->real->name();
 }
 
+PsiIcon *AlertIcon::copy() const
+{
+	return new AlertIcon(d->real);
+}
+
 //----------------------------------------------------------------------------
 
 void alertIconUpdateAlertStyle()
