@@ -21,15 +21,15 @@
 #ifndef CHANGEPWDLG_H
 #define CHANGEPWDLG_H
 
-#include"ui_changepw.h"
+#include "ui_changepw.h"
 
 class PsiAccount;
 
-class ChangePasswordDlg : public ChangePasswordUI
+class ChangePasswordDlg : public QDialog, public Ui::ChangePassword
 {
 	Q_OBJECT
 public:
-	ChangePasswordDlg(PsiAccount *, QWidget *parent=0, const char *name=0);
+	ChangePasswordDlg(PsiAccount *, QWidget *parent=0);
 	~ChangePasswordDlg();
 
 protected:
