@@ -21,9 +21,21 @@
 #ifndef CHATDLG_H
 #define CHATDLG_H
 
-#include "advwidget.h"
-#include "im.h"
+#include <QContextMenuEvent>
+#include <QDragEnterEvent>
+#include <QShowEvent>
+#include <QKeyEvent>
+#include <QResizeEvent>
+#include <QDropEvent>
+#include <QCloseEvent>
 
+#include "advwidget.h"
+#include "xmpp_chatstate.h" 
+
+namespace XMPP {
+	class Jid;
+	class Message;
+}
 using namespace XMPP;
 
 class PsiAccount;
@@ -31,6 +43,8 @@ class UserListItem;
 class QDropEvent;
 class QDragEnterEvent;
 
+#include "ui_chatdlg.h"
+
 class ChatDlg : public AdvancedWidget<QWidget>
 {
 	Q_OBJECT
@@ -43,26 +57,33 @@ public:
 	const QString & getDisplayNick();
 
 	static QSize defaultSize();
+	bool readyToHide();
 
 signals:
 	void aInfo(const Jid &);
 	void aHistory(const Jid &);
+	void aVoice(const Jid &);
 	void messagesRead(const Jid &);
 	void aSend(const Message &);
 	void aFile(const Jid &);
 	void captionChanged(ChatDlg*);
-	void contactIsComposing(ChatDlg*, bool);
+	void contactStateChanged( XMPP::ChatState );
 	void unreadMessageUpdate(ChatDlg*, int);
 
 protected:
+	void setShortcuts();
+
 	// reimplemented
 	void keyPressEvent(QKeyEvent *);
 	void closeEvent(QCloseEvent *);
 	void resizeEvent(QResizeEvent *);
+	void hideEvent(QHideEvent *);
 	void showEvent(QShowEvent *);
 	void windowActivationChange(bool);
 	void dropEvent(QDropEvent* event);
 	void dragEnterEvent(QDragEnterEvent* event);
+	
+	bool eventFilter(QObject *obj, QEvent *event);
 
 public slots:
 	void optionsUpdate();
@@ -73,28 +94,38 @@ public slots:
 	void updateAvatar(const Jid&);
 
 private slots:
+	void scrollUp();
+	void scrollDown();
 	void doInfo();
 	void doHistory();
 	void doClear();
 	void doClearButton();
 	void doSend();
+	void doVoice();
 	void doFile();
 	void setKeepOpenFalse();
 	void setWarnSendFalse();
 	void updatePGP();
-	void encryptedMessageSent(int, bool);
+	void encryptedMessageSent(int, bool, int);
 	void slotScroll();
+	void setChatState(XMPP::ChatState s);
 	void updateIsComposing(bool);
-	void updateContactIsComposing(bool);
+	void setContactChatState(ChatState s);
 	void toggleSmallChat();
 	void toggleEncryption();
 	void buildMenu();
 	void logSelectionChanged();
+	void capsChanged(const Jid&);
+	void updateIdentityVisibility();
+	void chatEditCreated();
+	void initComposing();
 
 public:
 	class Private;
 private:
 	Private *d;
+	Ui::ChatDlg ui_;
+	bool highlightersInstalled_;
 
 	void contextMenuEvent(QContextMenuEvent *);
 
