@@ -21,21 +21,27 @@
 #ifndef CONTACTVIEW_H
 #define CONTACTVIEW_H
 
-#include<qobject.h>
-#include<qlistview.h>
-#include<qtooltip.h>
-#include<qvaluelist.h>
-#include"xmpp.h"
-#include"common.h"
+#include <QObject>
+#include <Q3ListView>
+#include <QList>
+#include <QPixmap>
+#include <QKeyEvent>
+#include <QEvent>
+#include <QDropEvent>
+
+#include "xmpp_clientstream.h"
 
 class IconAction;
 class UserListItem;
 class ContactView;
 class ContactViewItem;
 class PsiAccount;
-class Icon;
+class PsiIcon;
 class QTimer;
 class QPixmap;
+namespace XMPP {
+	class Status;
+}
 
 using namespace XMPP;
 
@@ -65,12 +71,13 @@ public:
 	void updateSelf();
 	void addSelf();
 	void removeSelf();
-	void setAlert(const Jid &, const Icon *);
+	void setAlert(const Jid &, const PsiIcon *);
 	void clearAlert(const Jid &);
 	void animateNick(const Jid &);
 
 	void addAllNeededContactItems();
 	void removeAllUnneededContactItems();
+	void resetAllContactItemNames();
 
 	void ensureVisible(const Jid &);
 	void clear();
@@ -87,7 +94,7 @@ signals:
 	void actionDefault(const Jid &);
 	void actionRecvEvent(const Jid &);
 	void actionSendMessage(const Jid &);
-	void actionSendMessage(const JidList &);
+	void actionSendMessage(const QList<XMPP::Jid>&);
 	void actionSendUrl(const Jid &);
 	void actionRemove(const Jid &);
 	void actionRename(const Jid &, const QString &);
@@ -95,6 +102,10 @@ signals:
 	void actionHistory(const Jid &);
 	void actionOpenChat(const Jid &);
 	void actionOpenChatSpecific(const Jid &);
+#ifdef WHITEBOARDING
+	void actionOpenWhiteboard(const Jid &);
+	void actionOpenWhiteboardSpecific(const Jid &);
+#endif
 	void actionAgentSetStatus(const Jid &, Status &);
 	void actionInfo(const Jid &);
 	void actionAuth(const Jid &);
@@ -103,13 +114,18 @@ signals:
 	void actionAdd(const Jid &);
 	void actionGroupAdd(const Jid &, const QString &);
 	void actionGroupRemove(const Jid &, const QString &);
-	void actionTest(const Jid &);
 	void actionSendFile(const Jid &);
 	void actionSendFiles(const Jid &, const QStringList &);
+	void actionVoice(const Jid &);
+	void actionExecuteCommand(const Jid &, const QString&);
+	void actionExecuteCommandSpecific(const Jid &, const QString&);
 	void actionDisco(const Jid &, const QString &);
 	void actionInvite(const Jid &, const QString &);
 	void actionAssignKey(const Jid &);
 	void actionUnassignKey(const Jid &);
+	void actionSetMood();
+	void actionSetAvatar();
+	void actionUnsetAvatar();
 
 private slots:
 	void updateGroups();
@@ -140,10 +156,10 @@ private:
 	void ensureVisible(Entry *);
 
 	// useful functions to grab groups of users
-	JidList contactListFromCVGroup(ContactViewItem *) const;
+	QList<XMPP::Jid> contactListFromCVGroup(ContactViewItem *) const;
 	int contactSizeFromCVGroup(ContactViewItem *) const;
 	int contactsOnlineFromCVGroup(ContactViewItem *) const;
-	JidList contactListFromGroup(const QString &groupName) const;
+	QList<XMPP::Jid> contactListFromGroup(const QString &groupName) const;
 	int contactSizeFromGroup(const QString &groupName) const;
 
 	void updateGroupInfo(ContactViewItem *group);
@@ -159,6 +175,9 @@ private:
 	void scVCard(ContactViewItem *);
 	void scHistory(ContactViewItem *);
 	void scOpenChat(ContactViewItem *);
+#ifdef WHITEBOARDING
+	void scOpenWhiteboard(ContactViewItem *);
+#endif
 	void scAgentSetStatus(ContactViewItem *, Status &);
 	void scRemove(ContactViewItem *);
 	void doItemRenamed(ContactViewItem *, const QString &);
@@ -170,7 +189,7 @@ private:
 };
 
 // ContactView: the actual widget
-class ContactView : public QListView, public QToolTip
+class ContactView : public Q3ListView
 {
 	Q_OBJECT
 public:
@@ -182,6 +201,7 @@ public:
 	bool isShowAway() const { return v_showAway; }
 	bool isShowHidden() const { return v_showHidden; }
 	bool isShowSelf() const { return v_showSelf; }
+	bool isShowStatusMsg() const { return v_showStatusMsg; }
 
 	void clear();
 	void resetAnim();
@@ -189,16 +209,19 @@ public:
 
 	IconAction *qa_send, *qa_chat, *qa_ren, *qa_hist, *qa_logon, *qa_recv, *qa_rem, *qa_vcard;
 	IconAction *qa_assignAvatar, *qa_clearAvatar;
+#ifdef WHITEBOARDING
+	IconAction *qa_wb;
+#endif
 
 	QSize minimumSizeHint() const;
 	QSize sizeHint() const;
 
 protected:
+	void setShortcuts();
 	// reimplemented
-	void maybeTip(const QPoint &);
 	void keyPressEvent(QKeyEvent *);
 	bool eventFilter( QObject *, QEvent * );
-	QDragObject *dragObject();
+	Q3DragObject *dragObject();
 
 signals:
 	void showOffline(bool);
@@ -206,6 +229,7 @@ signals:
 	void showHidden(bool);
 	void showAgents(bool);
 	void showSelf(bool);
+	void showStatusMsg(bool);
 
 public slots:
 	void setShowOffline(bool);
@@ -213,15 +237,16 @@ public slots:
 	void setShowAway(bool);
 	void setShowHidden(bool);
 	void setShowSelf(bool);
+	void setShowStatusMsg(bool);
 	void optionsUpdate();
 	void recalculateSize();
 
 private slots:
-	void qlv_singleclick(int, QListViewItem *, const QPoint &, int);
-	void qlv_doubleclick(QListViewItem *);
-	void qlv_contextPopup(QListViewItem *, const QPoint &, int);
-	void qlv_contextMenuRequested(QListViewItem *, const QPoint &, int);
-	void qlv_itemRenamed(QListViewItem *, int, const QString &);
+	void qlv_singleclick(int, Q3ListViewItem *, const QPoint &, int);
+	void qlv_doubleclick(Q3ListViewItem *);
+	void qlv_contextPopup(Q3ListViewItem *, const QPoint &, int);
+	void qlv_contextMenuRequested(Q3ListViewItem *, const QPoint &, int);
+	void qlv_itemRenamed(Q3ListViewItem *, int, const QString &);
 	void leftClickTimeOut();
 
 	void doRecvEvent();
@@ -230,6 +255,9 @@ private slots:
 	void doContext();
 	void doSendMessage();
 	void doOpenChat();
+#ifdef WHITEBOARDING
+	void doOpenWhiteboard();
+#endif
 	void doHistory();
 	void doVCard();
 	void doLogon();
@@ -245,10 +273,10 @@ private:
 	Private *d;
 
 	QPoint mousePressPos; // store pressed position, idea taken from Licq
-	bool v_showOffline, v_showAgents, v_showAway, v_showHidden, v_showSelf;
+	bool v_showOffline, v_showAgents, v_showAway, v_showHidden, v_showSelf, v_showStatusMsg;
 	bool lcto_active; // double click active?
 	QPoint lcto_pos;
-	QListViewItem *lcto_item;
+	Q3ListViewItem *lcto_item;
 	QSize lastSize;
 
 	friend class ContactProfile;
@@ -257,8 +285,48 @@ private:
 	bool allowResize() const;
 };
 
+
+
+//------------------------------------------------------------------------------
+// RichListViewItem: A RichText listview item
+//------------------------------------------------------------------------------
+
+#include <Q3StyleSheet>
+
+class RichListViewStyleSheet : public Q3StyleSheet
+{
+public:
+	static RichListViewStyleSheet* instance();
+	virtual void scaleFont(QFont& font, int logicalSize) const;
+
+private:
+	RichListViewStyleSheet(QObject* parent=0, const char * name=0);
+	static RichListViewStyleSheet* instance_;
+};
+
+class Q3SimpleRichText;
+class RichListViewItem : public Q3ListViewItem
+{
+public:
+	RichListViewItem( Q3ListView * parent );
+	RichListViewItem( Q3ListViewItem * parent );
+	virtual void setText(int column, const QString& text);
+	virtual void setup();
+	virtual ~RichListViewItem();
+	int widthUsed();
+
+protected:
+	virtual void paintCell( QPainter * p, const QColorGroup & cg, int column
+, int width, int align );
+private:
+	int v_widthUsed;
+	bool v_selected, v_active;
+	bool v_rich;
+	Q3SimpleRichText* v_rt;
+};
+
 // ContactViewItem: an entry in the ContactView (profile, group, or contact)
-class ContactViewItem : public QObject, public QListViewItem
+class ContactViewItem : public QObject, public RichListViewItem
 {
 	Q_OBJECT
 public:
@@ -290,12 +358,12 @@ public:
 	void setGroupName(const QString &);
 	void setGroupInfo(const QString &);
 	void setAnimateNick();
-	void setAlert(const Icon *);
+	void setAlert(const PsiIcon *);
 	void clearAlert();
-	void setIcon(const Icon *, bool alert = false);
+	void setIcon(const PsiIcon *, bool alert = false);
 
 	void resetStatus();
-	void resetName(); // use this to cancel a rename
+	void resetName(bool forceNoStatusMsg = false); // use this to cancel a rename
 	void resetGroupName();
 
 	void updatePosition();
@@ -307,14 +375,14 @@ public:
 	void paintBranches(QPainter *, const QColorGroup &, int, int, int);
 	void paintCell(QPainter *, const QColorGroup & cg, int column, int width, int alignment);
 	void setOpen(bool o);
-	void insertItem(QListViewItem * newChild);
-	void takeItem(QListViewItem * item);
-	int compare(QListViewItem *, int, bool) const;
+	void insertItem(Q3ListViewItem * newChild);
+	void takeItem(Q3ListViewItem * item);
+	int compare(Q3ListViewItem *, int, bool) const;
 	bool acceptDrop(const QMimeSource *) const;
 
 public slots:
 	void resetAnim();
-	void iconUpdated(const QPixmap &);
+	void iconUpdated();
 	void animateNick();
 	void stopAnimateNick();
 
@@ -322,14 +390,15 @@ protected:
 	void dragEntered();
 	void dragLeft();
 	void dropped(QDropEvent *);
+	void cancelRename(int);
 
 private:
+	int type_;
 	class Private;
 	Private *d;
 
 	void cacheValues();
 	int rankGroup(int groupType) const;
-	int rankStatus(int status) const;
 	void drawGroupIcon();
 };
 
