@@ -20,29 +20,121 @@
 
 #include "discodlg.h"
 
-#include <qlistview.h>
-#include <qcombobox.h>
-#include <qcheckbox.h>
-#include <qmessagebox.h>
-#include <qaction.h>
-#include <qstatusbar.h>
-#include <qpopupmenu.h>
-#include <qsignalmapper.h>
-#include <qpushbutton.h>
-#include <qdragobject.h>
-#include <qtooltip.h>
-#include <qtoolbutton.h>
+#include <q3listview.h>
+#include <q3popupmenu.h>
+#include <Q3PtrList>
+
+#include <QComboBox>
+#include <QCheckBox>
+#include <QMessageBox>
+#include <QAction>
+#include <QSignalMapper>
+#include <QPushButton>
+#include <QToolButton>
+#include <QToolBar>
+#include <QScrollBar>
+
+#include <QActionGroup>
+#include <QEvent>
+#include <QList>
+#include <QContextMenuEvent>
 
-#include "im.h"
 #include "xmpp_tasks.h"
 
-#include "psitoolbar.h"
 #include "tasklist.h"
 #include "psiaccount.h"
 #include "psicon.h"
 #include "busywidget.h"
 #include "common.h"
 #include "iconaction.h"
+#include "psiiconset.h"
+#include "psitooltip.h"
+#include "stretchwidget.h"
+#include "psioptions.h"
+#include "accountlabel.h"
+
+//----------------------------------------------------------------------------
+
+PsiIcon category2icon(const QString &category, const QString &type, int status=STATUS_ONLINE)
+{
+	// TODO: update this to http://www.jabber.org/registrar/disco-categories.html#gateway
+
+	// still have to add more options...
+	if ( category == "service" || category == "gateway" ) {
+		QString trans;
+
+		if (type == "aim")
+			trans = "aim";
+		else if (type == "icq")
+			trans = "icq";
+		else if (type == "msn")
+			trans = "msn";
+		else if (type == "yahoo")
+			trans = "yahoo";
+		else if (type == "gadu-gadu" || type == "x-gadugadu")
+			trans = "gadugadu";
+		else if (type == "sms")
+			trans = "sms";
+		else
+			trans = "transport";
+
+		return PsiIconset::instance()->transportStatus(trans, status);
+
+		// irc
+		// jud
+		// pager
+		// jabber
+		// serverlist
+		// smtp
+	}
+	else if ( category == "conference" ) {
+		if (type == "public" || type == "private" || type == "text" || type == "irc")
+			return IconsetFactory::icon("psi/groupChat");
+		else if (type == "url")
+			return IconsetFactory::icon("psi/www");
+		// irc
+		// list
+		// topic
+	}
+	else if ( category == "validate" ) {
+		if (type == "xml")
+			return IconsetFactory::icon("psi/xml");
+		// grammar
+		// spell
+	}
+	else if ( category == "user" || category == "client" ) {
+		// client
+		// forward
+		// inbox
+		// portable
+		// voice
+		return PsiIconset::instance()->status(STATUS_ONLINE);
+	}
+	// application
+	   // bot
+	   // calendar
+	   // editor
+	   // fileserver
+	   // game
+	   // whiteboard
+	// headline
+	   // logger
+	   // notice
+	   // rss
+	   // stock
+	// keyword
+	   // dictionary
+	   // dns
+	   // software
+	   // thesaurus
+	   // web
+	   // whois
+	// render
+	   // en2ru
+	   // ??2??
+	   // tts
+	return PsiIcon();
+}
 
 //----------------------------------------------------------------------------
 // DiscoData -- a shared data struct
@@ -57,7 +149,7 @@ public:
 	: QObject(parent) {}
 
 signals:
-	void itemUpdated(QListViewItem *);
+	void itemUpdated(Q3ListViewItem *);
 
 private:
 	friend class DiscoListItem;
@@ -78,79 +170,16 @@ struct DiscoData {
 	Protocol protocol;
 };
 
-//----------------------------------------------------------------------------
-// ProtocolAction
-//----------------------------------------------------------------------------
-
-class ProtocolAction : public QAction
-{
-	Q_OBJECT
-public:
-	ProtocolAction(QString text, QString toolTip, QObject *parent, QSignalMapper *sm, int parm);
-
-	bool addTo(QWidget *w);
-
-public slots:
-	void setOn(bool);
-private:
-	QToolButton *btn;
-};
-
-ProtocolAction::ProtocolAction(QString text, QString toolTip, QObject *parent, QSignalMapper *sm, int parm)
-: QAction(text, text, 0, parent)
-{
-	setText( text );
-	setToggleAction(true);
-	setToolTip(toolTip);
-	connect(this, SIGNAL(activated()), sm, SLOT(map()));
-	sm->setMapping(this, parm);
-	btn = 0;
-}
-
-bool ProtocolAction::addTo(QWidget *w)
-{
-	if ( w->inherits("QToolBar") ) {
-		if ( btn )
-			delete btn;
-		QCString bname = name() + QCString("_action_button");
-		btn = new QToolButton ( w, bname );
-
-		btn->setToggleButton ( isToggleAction() );
-		btn->setOn( isOn() );
-		btn->setTextLabel ( text() );
-		btn->setEnabled ( isEnabled() );
-		btn->setUsesTextLabel( true );
-		QToolTip::add(btn, toolTip());
-
-		connect(btn, SIGNAL(toggled(bool)), this, SLOT(setOn(bool)));
-
-		return true;
-	}
-
-	return false;
-}
-
-void ProtocolAction::setOn(bool b)
-{
-	if ( btn ) {
-		if ( b )
-			emit activated();
-		btn->setOn(b);
-	}
-
-	QAction::setOn(b);
-}
-
 //----------------------------------------------------------------------------
 // DiscoListItem
 //----------------------------------------------------------------------------
 
-class DiscoListItem : public QObject, public QListViewItem
+class DiscoListItem : public QObject, public Q3ListViewItem
 {
 	Q_OBJECT
 public:
-	DiscoListItem(DiscoItem it, DiscoData *d, QListView *parent);
-	DiscoListItem(DiscoItem it, DiscoData *d, QListViewItem *parent);
+	DiscoListItem(DiscoItem it, DiscoData *d, Q3ListView *parent);
+	DiscoListItem(DiscoItem it, DiscoData *d, Q3ListViewItem *parent);
 	~DiscoListItem();
 
 	QString text(int columns) const;
@@ -162,6 +191,7 @@ public:
 public slots: // the two are used internally by class, and also called by DiscoDlg::Private::refresh()
 	void updateInfo();
 	void updateItems(bool parentAutoItems = false);
+	QString getErrorInfo() const;
 
 private slots:
 	void discoItemsFinished();
@@ -179,6 +209,7 @@ private:
 	bool alreadyItems, alreadyInfo;
 	bool autoItems; // used in updateItemsFinished
 	bool autoInfo;
+	QString errorInfo;
 
 	void copyItem(const DiscoItem &);
 	void updateInfo(const DiscoItem &);
@@ -195,16 +226,16 @@ private:
 	DiscoDlg *dlg() const;
 };
 
-DiscoListItem::DiscoListItem(DiscoItem it, DiscoData *_d, QListView *parent)
-: QListViewItem (parent)
+DiscoListItem::DiscoListItem(DiscoItem it, DiscoData *_d, Q3ListView *parent)
+: Q3ListViewItem (parent)
 {
 	isRoot = true;
 
 	init(it, _d);
 }
 
-DiscoListItem::DiscoListItem(DiscoItem it, DiscoData *_d, QListViewItem *parent)
-: QListViewItem (parent)
+DiscoListItem::DiscoListItem(DiscoItem it, DiscoData *_d, Q3ListViewItem *parent)
+: Q3ListViewItem (parent)
 {
 	isRoot = false;
 
@@ -232,8 +263,6 @@ void DiscoListItem::init(DiscoItem _item, DiscoData *_d)
 		if ( !isRoot )
 			autoInfo = true;
 	}
-
-	//setDragEnabled(true); // EXPERIMENTAL
 }
 
 void DiscoListItem::copyItem(const DiscoItem &it)
@@ -244,6 +273,9 @@ void DiscoListItem::copyItem(const DiscoItem &it)
 		di.setNode ( it.node() );
 	if ( !(!di.name().isEmpty() && it.name().isEmpty()) )
 		di.setName ( it.name() );
+	
+	if ( di.name().isEmpty() && !di.jid().full().isEmpty() ) // use JID in the Name column
+		di.setName ( di.jid().full() );
 
 	di.setAction ( it.action() );
 
@@ -283,17 +315,17 @@ void DiscoListItem::copyItem(const DiscoItem &it)
 		DiscoItem::Identity id = di.identities().first();
 
 		if ( !id.category.isEmpty() ) {
-			Icon ic = category2icon(id.category, id.type);
+			PsiIcon ic = category2icon(id.category, id.type);
 
 			if ( !ic.impix().isNull() ) {
-				setPixmap (0, ic.impix().pixmap());
+				setPixmap (0, ic.impix());
 				pixmapOk = true;
 			}
 		}
 	}
 
 	if ( !pixmapOk )
-		setPixmap (0, is->status(di.jid(), STATUS_ONLINE));
+		setPixmap(0, PsiIconset::instance()->status(di.jid(), STATUS_ONLINE).impix());
 
 	repaint();
 
@@ -304,14 +336,19 @@ void DiscoListItem::copyItem(const DiscoItem &it)
 QString DiscoListItem::text (int c) const
 {
 	if (c == 0)
-		return di.name();
+		return di.name().simplified();
 	else if (c == 1)
 		return di.jid().full();
 	else if (c == 2)
-		return di.node();
+		return di.node().simplified();
 	return "";
 }
 
+QString DiscoListItem::getErrorInfo() const
+{
+	return errorInfo;
+}
+
 const DiscoItem &DiscoListItem::item() const
 {
 	return di;
@@ -319,7 +356,7 @@ const DiscoItem &DiscoListItem::item() const
 
 DiscoDlg *DiscoListItem::dlg() const
 {
-	return (DiscoDlg *)listView()->parent()->parent();
+	return (DiscoDlg *)listView()->parent();
 }
 
 bool DiscoListItem::autoItemsEnabled() const
@@ -341,7 +378,7 @@ void DiscoListItem::setOpen (bool o)
 			autoItemsChildren();
 	}
 
-	QListViewItem::setOpen(o);
+	Q3ListViewItem::setOpen(o);
 }
 
 void DiscoListItem::itemSelected()
@@ -399,7 +436,7 @@ void DiscoListItem::discoItemsFinished()
 	}
 	else if ( !autoItems ) {
 		QString error = jt->statusString();
-		QMessageBox::critical(dlg(), tr("Error"), tr("There was an error getting items for <b>%1</b>.\nReason: %2").arg(di.jid().full()).arg(error));
+		QMessageBox::critical(dlg(), tr("Error"), tr("There was an error getting items for <b>%1</b>.<br>Reason: %2").arg(di.jid().full()).arg(QString(error).replace('\n', "<br>")));
 	}
 
 	alreadyItems = true;
@@ -461,7 +498,7 @@ void DiscoListItem::browseFinished()
 	}
 	else if ( !autoItems ) {
 		QString error = jt->statusString();
-		QMessageBox::critical(dlg(), tr("Error"), tr("There was an error browsing items for <b>%1</b>.\nReason: %2").arg(di.jid().full()).arg(error));
+		QMessageBox::critical(dlg(), tr("Error"), tr("There was an error browsing items for <b>%1</b>.<br>Reason: %2").arg(di.jid().full()).arg(QString(error).replace('\n', "<br>")));
 	}
 
 	alreadyItems = true;
@@ -525,7 +562,7 @@ void DiscoListItem::agentsFinished()
 	}
 	else if ( !autoItems ) {
 		QString error = jt->statusString();
-		QMessageBox::critical(dlg(), tr("Error"), tr("There was an error getting agents for <b>%1</b>.\nReason: %2").arg(di.jid().full()).arg(error));
+		QMessageBox::critical(dlg(), tr("Error"), tr("There was an error getting agents for <b>%1</b>.<br>Reason: %2").arg(di.jid().full()).arg(QString(error).replace('\n', "<br>")));
 	}
 
 	alreadyItems = true;
@@ -541,7 +578,7 @@ QString DiscoListItem::computeHash( QString jid, QString node )
 
 void DiscoListItem::updateItemsFinished(const DiscoList &list)
 {
-	QDict<DiscoListItem> children;
+	Q3Dict<DiscoListItem> children;
 	DiscoListItem *child = (DiscoListItem *)firstChild();
 	while ( child ) {
 		children.insert( child->hash(), child );
@@ -614,9 +651,40 @@ void DiscoListItem::discoInfoFinished()
 	if ( jt->success() ) {
 		updateInfo( jt->item() );
 	}
-	else if ( !autoInfo && d->protocol != DiscoData::Auto ) {
-		QString error = jt->statusString();
-		QMessageBox::critical(dlg(), tr("Error"), tr("There was an error getting item's info for <b>%1</b>.\nReason: %2").arg(di.jid().full()).arg(error));
+	else {
+		QString error_str = jt->statusString();
+		int error_code = jt->statusCode();
+		setExpandable(false);
+
+		// we change the icon for the items with disco#info returning type=="cancel" || type=="wait" error codes
+		// based on http://www.jabber.org/jeps/jep-0086.html
+
+		// FIXME: use another method for checking XMPP error-types when Iris will provide one
+		if ( error_code==400 || error_code==404 || error_code==405 || error_code==409 ||
+		     error_code==500 || error_code==501 || error_code==503 || error_code==504 ) {
+			bool pixmapOk = false;
+			if ( !di.identities().isEmpty() ) {
+				DiscoItem::Identity id = di.identities().first();
+				if ( !id.category.isEmpty() ) {
+					PsiIcon ic = category2icon(id.category, id.type, STATUS_ERROR);
+
+					if ( !ic.impix().isNull() ) {
+						setPixmap (0, ic.impix());
+						pixmapOk = true;
+					}
+				}
+			}
+			if ( !pixmapOk )
+				setPixmap(0, PsiIconset::instance()->status(di.jid(), STATUS_ERROR).impix());
+		} else {
+			repaint(); //for setExpandable() called earlier
+		}
+
+		errorInfo=QString("%1").arg(QString(error_str).replace('\n', "<br>"));
+
+		if ( !autoInfo && d->protocol != DiscoData::Auto ) {	
+			QMessageBox::critical(dlg(), tr("Error"), tr("There was an error getting item's info for <b>%1</b>.<br>Reason: %2").arg(di.jid().full()).arg(QString(error_str).replace('\n', "<br>")));
+		}
 	}
 
 	alreadyInfo = true;
@@ -635,30 +703,50 @@ void DiscoListItem::updateInfo(const DiscoItem &item)
 // DiscoList
 //----------------------------------------------------------------------------
 
-class DiscoListView : public QListView, public QToolTip
+class DiscoListView : public Q3ListView
 {
 	Q_OBJECT
 public:
 	DiscoListView(QWidget *parent);
 
 protected:
-	void maybeTip(const QPoint &);
-	QDragObject *dragObject();
+	bool maybeTip(const QPoint &);
+
+	// reimplemented
+	bool eventFilter(QObject* o, QEvent* e);
+	void resizeEvent(QResizeEvent*);
 };
 
 DiscoListView::DiscoListView(QWidget *parent)
-: QListView(parent), QToolTip(viewport())
+: Q3ListView(parent)
 {
 	addColumn( tr( "Name" ) );
 	addColumn( tr( "JID" ) );
 	addColumn( tr( "Node" ) );
+	for (int i = 0; i < 3; i++)
+		setColumnWidthMode(i, Q3ListView::Manual);
+	header()->setStretchEnabled(true, 0);
 }
 
-void DiscoListView::maybeTip(const QPoint &pos)
+void DiscoListView::resizeEvent(QResizeEvent* e)
 {
-	DiscoListItem *i = (DiscoListItem *)itemAt(pos);
+	Q3ListView::resizeEvent(e);
+
+	setColumnWidth(2, header()->fontMetrics().width(columnText(2)) * 2);
+	float remainingWidth = visibleWidth() - columnWidth(2);
+	setColumnWidth(1, int(remainingWidth * 0.3));
+
+	header()->adjustHeaderSize();
+}
+
+/**
+ * \param pos should be in global coordinate system.
+ */
+bool DiscoListView::maybeTip(const QPoint &pos)
+{
+	DiscoListItem* i = (DiscoListItem*)itemAt(viewport()->mapFromGlobal(pos));
 	if(!i)
-		return;
+		return false;
 
 	// NAME <JID> (Node "NODE")
 	//
@@ -674,7 +762,7 @@ void DiscoListView::maybeTip(const QPoint &pos)
 	QString text = "<qt><nobr>";
 	DiscoItem item = i->item();
 
-	if ( !item.name().isEmpty() )
+	if ( item.name()!=item.jid().full() )
 		text += item.name() + " ";
 
 	text += "&lt;" + item.jid().full() + "&gt;";
@@ -682,7 +770,7 @@ void DiscoListView::maybeTip(const QPoint &pos)
 	if ( !item.node().isEmpty() )
 		text += " (" + tr("Node") + " \"" + item.node() + "\")";
 
-	text += "<nobr>";
+	text += "</nobr>";
 
 	if ( !item.identities().isEmpty() || !item.features().list().isEmpty() )
 		text += "<br>\n";
@@ -694,7 +782,7 @@ void DiscoListView::maybeTip(const QPoint &pos)
 		DiscoItem::Identities::ConstIterator it = item.identities().begin();
 		for ( ; it != item.identities().end(); ++it) {
 			text += "<br>";
-			Icon icon( category2icon((*it).category, (*it).type) );
+			PsiIcon icon( category2icon((*it).category, (*it).type) );
 			if ( !icon.name().isEmpty() )
 				text += "<icon name=\"" + icon.name() + "\"> ";
 			text += (*it).name;
@@ -722,20 +810,27 @@ void DiscoListView::maybeTip(const QPoint &pos)
 		}
 	}
 
+	QString errorInfo=i->getErrorInfo();
+	if ( !errorInfo.isEmpty() ) {
+		text += "<br>\n<br>\n<b>" + tr("Error:") + "</b>\n";
+		text += errorInfo;
+	}
+
 	text += "</qt>";
 	QRect r( itemRect(i) );
-	tip(r, text);
+	PsiToolTip::showText(pos, text, this);
+	return true;
 }
 
-QDragObject *DiscoListView::dragObject()
+bool DiscoListView::eventFilter(QObject* o, QEvent* e)
 {
-	DiscoListItem *i = (DiscoListItem *)selectedItem();
-	if(!i)
-		return 0;
-
-	QDragObject *d = new QTextDrag(i->item().jid().full(), this);
-	d->setPixmap(IconsetFactory::icon("status/online"), QPoint(8,8));
-	return d;
+	if (e->type() == QEvent::ToolTip && o->isWidgetType()) {
+		QWidget*    w  = static_cast<QWidget*>(o);
+		QHelpEvent* he = static_cast<QHelpEvent*>(e);
+		maybeTip(w->mapToGlobal(he->pos()));
+		return true;
+	}
+	return Q3ListView::eventFilter(o, e);
 }
 
 //----------------------------------------------------------------------------
@@ -747,23 +842,28 @@ class DiscoDlg::Private : public QObject
 	Q_OBJECT
 
 private:
-	// helper class for use in toolbar
-	class StretchWidget : public QWidget
+	class ProtocolAction : public QAction
 	{
 	public:
-		StretchWidget(QWidget *parent)
-		: QWidget(parent)
+		ProtocolAction(QString text, QString toolTip, QObject *parent, QSignalMapper *sm, int parm)
+			: QAction(text, parent)
 		{
-			setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+			setText(text);
+			setIconText(text);
+	
+			setCheckable(true);
+			setToolTip(toolTip);
+			connect(this, SIGNAL(activated()), sm, SLOT(map()));
+			sm->setMapping(this, parm);
 		}
 	};
 
 	// helper class to store browser history
 	class History {
 	private:
-		QListViewItem *item;
+		Q3ListViewItem *item;
 	public:
-		History(QListViewItem *it) {
+		History(Q3ListViewItem *it) {
 			item = it;
 		}
 
@@ -772,8 +872,8 @@ private:
 				delete item;
 		}
 
-		QListViewItem *takeItem() {
-			QListViewItem *i = item;
+		Q3ListViewItem *takeItem() {
+			Q3ListViewItem *i = item;
 			item = 0;
 			return i;
 		}
@@ -786,13 +886,13 @@ public: // data
 
 	DiscoData data;
 
-	PsiToolBar *toolBar;
+	QToolBar *toolBar;
 	IconAction *actBrowse, *actBack, *actForward, *actRefresh, *actStop;
 
 	// custom actions, that will be added to toolbar and context menu
-	IconAction *actRegister, *actSearch, *actJoin, *actVCard, *actAdd;
+	IconAction *actRegister, *actSearch, *actJoin, *actAHCommand, *actVCard, *actAdd;
 
-	typedef QPtrList<History> HistoryList;
+	typedef Q3PtrList<History> HistoryList;
 	HistoryList backHistory, forwardHistory;
 
 	BusyWidget *busy;
@@ -811,7 +911,7 @@ public slots:
 	void actionBack();
 	void actionForward();
 	void updateBackForward();
-	void backForwardHelper(QListViewItem *);
+	void backForwardHelper(Q3ListViewItem *);
 
 	void updateComboBoxes(Jid j, QString node);
 
@@ -821,8 +921,8 @@ public slots:
 	void disableButtons();
 	void enableButtons(const DiscoItem &);
 
-	void itemSelected (QListViewItem *);
-	void itemDoubleclicked (QListViewItem *);
+	void itemSelected (Q3ListViewItem *);
+	void itemDoubleclicked (Q3ListViewItem *);
 	bool eventFilter (QObject *, QEvent *);
 
 	void setProtocol(int);
@@ -844,7 +944,7 @@ DiscoDlg::Private::Private(DiscoDlg *parent, PsiAccount *pa)
 	connect(data.tasks, SIGNAL(started()),  SLOT(itemUpdateStarted()));
 	connect(data.tasks, SIGNAL(finished()), SLOT(itemUpdateFinished()));
 	data.d = new DiscoConnector(this);
-	connect(data.d, SIGNAL(itemUpdated(QListViewItem *)), SLOT(itemSelected (QListViewItem *)));
+	connect(data.d, SIGNAL(itemUpdated(Q3ListViewItem *)), SLOT(itemSelected (Q3ListViewItem *)));
 	data.protocol = DiscoData::Auto;
 
 	backHistory.setAutoDelete(true);
@@ -854,13 +954,13 @@ DiscoDlg::Private::Private(DiscoDlg *parent, PsiAccount *pa)
 	busy = parent->busy;
 	connect(busy, SIGNAL(destroyed(QObject *)), SLOT(objectDestroyed(QObject *)));
 
-	QListView *lv_discoOld = dlg->lv_disco;
-	dlg->lv_disco = new DiscoListView(dlg->centralWidget());
+	Q3ListView *lv_discoOld = dlg->lv_disco;
+	dlg->lv_disco = new DiscoListView(dlg);
 	replaceWidget(lv_discoOld, dlg->lv_disco);
 
 	dlg->lv_disco->installEventFilter (this);
-	connect(dlg->lv_disco, SIGNAL(selectionChanged (QListViewItem *)), SLOT(itemSelected (QListViewItem *)));;
-	connect(dlg->lv_disco, SIGNAL(doubleClicked (QListViewItem *)),    SLOT(itemDoubleclicked (QListViewItem *)));;
+	connect(dlg->lv_disco, SIGNAL(selectionChanged (Q3ListViewItem *)), SLOT(itemSelected (Q3ListViewItem *)));;
+	connect(dlg->lv_disco, SIGNAL(doubleClicked (Q3ListViewItem *)),    SLOT(itemDoubleclicked (Q3ListViewItem *)));;
 
 	// protocol actions
 	QSignalMapper *pm = new QSignalMapper(this);
@@ -868,11 +968,14 @@ DiscoDlg::Private::Private(DiscoDlg *parent, PsiAccount *pa)
 	QActionGroup *protocolActions = new QActionGroup (this);
 	protocolActions->setExclusive(true);
 
-	ProtocolAction *autoProtocol = new ProtocolAction (tr("Auto"), tr("Automatically determine protocol"), protocolActions, pm, DiscoData::Auto);
-	ProtocolAction *discoProtocol = new ProtocolAction ("D", tr("Service Discovery"), protocolActions, pm, DiscoData::Disco);
-	ProtocolAction *browseProtocol = new ProtocolAction ("B", tr("Browse Services"), protocolActions, pm, DiscoData::Browse);
-	ProtocolAction *agentsProtocol = new ProtocolAction ("A", tr("Browse Agents"), protocolActions, pm, DiscoData::Agents);
-	autoProtocol->setOn(true);
+	ProtocolAction *autoProtocol = NULL, *discoProtocol = NULL, *browseProtocol = NULL, *agentsProtocol = NULL;
+	if (PsiOptions::instance()->getOption("options.ui.show-deprecated.service-discovery.protocol-selector").toBool()) {
+		autoProtocol = new ProtocolAction (tr("Auto"), tr("Automatically determine protocol"), protocolActions, pm, DiscoData::Auto);
+		discoProtocol = new ProtocolAction ("D", tr("Service Discovery"), protocolActions, pm, DiscoData::Disco);
+		browseProtocol = new ProtocolAction ("B", tr("Browse Services"), protocolActions, pm, DiscoData::Browse);
+		agentsProtocol = new ProtocolAction ("A", tr("Browse Agents"), protocolActions, pm, DiscoData::Agents);
+		autoProtocol->setChecked(true);
+	}
 
 	// create actions
 	actBrowse = new IconAction (tr("Browse"), "psi/jabber", tr("&Browse"), 0, dlg);
@@ -898,6 +1001,9 @@ DiscoDlg::Private::Private(DiscoDlg *parent, PsiAccount *pa)
 	actJoin = new IconAction (tr("Join"), "psi/groupChat", tr("&Join"), 0, dlg);
 	connect (actJoin, SIGNAL(activated()), sm, SLOT(map()));
 	sm->setMapping(actJoin, Features::FID_Groupchat);
+	actAHCommand = new IconAction (tr("Execute command"), "psi/command", tr("&Execute command"), 0, dlg);
+	connect (actAHCommand, SIGNAL(activated()), sm, SLOT(map()));
+	sm->setMapping(actAHCommand, Features::FID_AHCommand);
 	actVCard = new IconAction (tr("vCard"), "psi/vCard", tr("&vCard"), 0, dlg);
 	connect (actVCard, SIGNAL(activated()), sm, SLOT(map()));
 	sm->setMapping(actVCard, Features::FID_VCard);
@@ -906,36 +1012,45 @@ DiscoDlg::Private::Private(DiscoDlg *parent, PsiAccount *pa)
 	sm->setMapping(actAdd, Features::FID_Add);
 
 	// create toolbar
-	toolBar = new PsiToolBar(dlg);
-	toolBar->setCustomizeable( false );
+	toolBar = new QToolBar(tr("Service Discovery toolbar"), dlg);
+	toolBar->setIconSize(QSize(16,16));
+	toolBar->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+	replaceWidget(dlg->toolBarPlaceholder, toolBar);
 
-	actBack->addTo(toolBar);
-	actBrowse->addTo(toolBar);
-	actForward->addTo(toolBar);
+	toolBar->addAction(actBack);
+	toolBar->addAction(actBrowse);
+	toolBar->addAction(actForward);
 
 	toolBar->addSeparator();
-	actRefresh->addTo(toolBar);
-	actStop->addTo(toolBar);
+	toolBar->addAction(actRefresh);
+	toolBar->addAction(actStop);
 
 	// custom actions
 	toolBar->addSeparator();
-	actRegister->addTo(toolBar);
-	actSearch->addTo(toolBar);
-	actJoin->addTo(toolBar);
+	toolBar->addAction(actRegister);
+	toolBar->addAction(actSearch);
+	toolBar->addAction(actJoin);
+	toolBar->addAction(actAHCommand);
 
 	toolBar->addSeparator();
-	actAdd->addTo(toolBar);
-	actVCard->addTo(toolBar);
+	toolBar->addAction(actAdd);
+	toolBar->addAction(actVCard);
+	toolBar->addAction(actAHCommand);
 
 	// select protocol
-	toolBar->addSeparator();
-	autoProtocol->addTo(toolBar);
-	discoProtocol->addTo(toolBar);
-	browseProtocol->addTo(toolBar);
-	agentsProtocol->addTo(toolBar);
+	if (PsiOptions::instance()->getOption("options.ui.show-deprecated.service-discovery.protocol-selector").toBool()) {
+		toolBar->addSeparator();
+		toolBar->addAction(autoProtocol);
+		toolBar->addAction(discoProtocol);
+		toolBar->addAction(browseProtocol);
+		toolBar->addAction(agentsProtocol);
+	}
 
-	toolBar->setStretchableWidget(new StretchWidget(toolBar));
-	pa->accountLabel(toolBar, true);
+	toolBar->addWidget(new StretchWidget(toolBar));
+	AccountLabel* lb_ident = new AccountLabel(toolBar);
+	lb_ident->setAccount(pa);
+	lb_ident->setShowJid(false);
+	toolBar->addWidget(lb_ident);
 
 	// misc stuff
 	disableButtons();
@@ -969,7 +1084,7 @@ void DiscoDlg::Private::doDisco(QString _host, QString _node)
 
 	// check, whether we need to update history
 	if ( (jid.full() != j.full()) || (node != n) ) {
-		QListViewItem *item = dlg->lv_disco->firstChild(); // get the root item
+		Q3ListViewItem *item = dlg->lv_disco->firstChild(); // get the root item
 
 		if ( item ) {
 			dlg->lv_disco->takeItem( item );
@@ -1039,7 +1154,7 @@ void DiscoDlg::Private::actionBrowse()
 void DiscoDlg::Private::actionBack()
 {
 	// add current selection to forward history
-	QListViewItem *item = dlg->lv_disco->firstChild();
+	Q3ListViewItem *item = dlg->lv_disco->firstChild();
 	if ( item ) {
 		dlg->lv_disco->takeItem( item );
 
@@ -1047,7 +1162,7 @@ void DiscoDlg::Private::actionBack()
 	}
 
 	// now, take info from back history...
-	QListViewItem *i = backHistory.last()->takeItem();
+	Q3ListViewItem *i = backHistory.last()->takeItem();
 	backHistory.removeLast();
 
 	// and restore view
@@ -1057,7 +1172,7 @@ void DiscoDlg::Private::actionBack()
 void DiscoDlg::Private::actionForward()
 {
 	// add current selection to back history
-	QListViewItem *item = dlg->lv_disco->firstChild();
+	Q3ListViewItem *item = dlg->lv_disco->firstChild();
 	if ( item ) {
 		dlg->lv_disco->takeItem( item );
 
@@ -1065,14 +1180,14 @@ void DiscoDlg::Private::actionForward()
 	}
 
 	// now, take info from forward history...
-	QListViewItem *i = forwardHistory.last()->takeItem();
+	Q3ListViewItem *i = forwardHistory.last()->takeItem();
 	forwardHistory.removeLast();
 
 	// and restore view
 	backForwardHelper(i);
 }
 
-void DiscoDlg::Private::backForwardHelper(QListViewItem *root)
+void DiscoDlg::Private::backForwardHelper(Q3ListViewItem *root)
 {
 	DiscoListItem *i = (DiscoListItem *)root;
 
@@ -1088,9 +1203,9 @@ void DiscoDlg::Private::backForwardHelper(QListViewItem *root)
 	dlg->lv_disco->insertItem( root );
 
 	// fixes multiple selection bug
-	QListViewItemIterator it( dlg->lv_disco );
+	Q3ListViewItemIterator it( dlg->lv_disco );
 	while ( it.current() ) {
-		QListViewItem *item = it.current();
+		Q3ListViewItem *item = it.current();
 		++it;
 
 		if ( item->isSelected() )
@@ -1138,9 +1253,10 @@ void DiscoDlg::Private::enableButtons(const DiscoItem &it)
 	actJoin->setEnabled( f.canGroupchat() );
 	actAdd->setEnabled( itemSelected );
 	actVCard->setEnabled( f.haveVCard() );
+	actAHCommand->setEnabled( f.canCommand() );
 }
 
-void DiscoDlg::Private::itemSelected (QListViewItem *item)
+void DiscoDlg::Private::itemSelected (Q3ListViewItem *item)
 {
 	DiscoListItem *it = (DiscoListItem *)item;
 	if ( !it ) {
@@ -1154,7 +1270,7 @@ void DiscoDlg::Private::itemSelected (QListViewItem *item)
 	enableButtons ( di );
 }
 
-void DiscoDlg::Private::itemDoubleclicked (QListViewItem *item)
+void DiscoDlg::Private::itemDoubleclicked (Q3ListViewItem *item)
 {
 	DiscoListItem *it = (DiscoListItem *)item;
 	if ( !it )
@@ -1165,23 +1281,13 @@ void DiscoDlg::Private::itemDoubleclicked (QListViewItem *item)
 
 	// set the prior state of item
 	// FIXME: causes minor flickering
-	if ( f.canGroupchat() || f.canRegister() || f.canSearch() ) {
-		if ( !it->isOpen() ) {
-			if ( it->isExpandable() || it->childCount() )
-				it->setOpen( true );
-		}
-		else {
-			it->setOpen( false );
-		}
-	}
-
 	long id = 0;
 
 	// trigger default action
-	if ( f.canGroupchat() ) {
-		id = Features::FID_Groupchat;
+	if (f.canCommand() ) {
+		id = Features::FID_AHCommand;
 	}
-	else {
+	if (!d.identities().isEmpty()) {
 		// FIXME: check the category and type for JUD!
 		DiscoItem::Identity ident = d.identities().first();
 		bool searchFirst = ident.category == "service" && ident.type == "jud";
@@ -1189,14 +1295,24 @@ void DiscoDlg::Private::itemDoubleclicked (QListViewItem *item)
 		if ( searchFirst && f.canSearch() ) {
 			id = Features::FID_Search;
 		}
-		else {
-			if ( f.canRegister() )
+		else if ( ident.category == "conference" &&  f.canGroupchat() ) {
+			id = Features::FID_Groupchat;
+		}
+		else if ( f.canRegister() ) {
 				id = Features::FID_Register;
 		}
 	}
 
-	if ( id > 0 )
+	if ( id > 0 ) {
+		if ( !it->isOpen() ) {
+			if ( it->isExpandable() || it->childCount() )
+				it->setOpen( true );
+		}
+		else {
+			it->setOpen( false );
+		}
 		emit dlg->featureActivated( Features::feature(id), d.jid(), d.node() );
+	}
 }
 
 bool DiscoDlg::Private::eventFilter (QObject *object, QEvent *event)
@@ -1210,7 +1326,7 @@ bool DiscoDlg::Private::eventFilter (QObject *object, QEvent *event)
 				return true;
 
 			// prepare features list
-			QValueList<long> idFeatures;
+			QList<long> idFeatures;
 			QStringList features = it->item().features().list();
 			{	// convert all features to their IDs
 				QStringList::Iterator it = features.begin();
@@ -1222,10 +1338,10 @@ bool DiscoDlg::Private::eventFilter (QObject *object, QEvent *event)
 				//qHeapSort(idFeatures);
 			}
 
-			QValueList<long> ids;
+			QList<long> ids;
 			{	// ensure, that there's in no duplicated IDs inside. FIXME: optimize this, anyone?
 				long id = 0, count = 0;
-				QValueList<long>::Iterator it;
+				QList<long>::Iterator it;
 				while ( count < (long)idFeatures.count() ) {
 					bool found = false;
 
@@ -1243,7 +1359,7 @@ bool DiscoDlg::Private::eventFilter (QObject *object, QEvent *event)
 			}
 
 			// prepare popup menu
-			QPopupMenu p;
+			Q3PopupMenu p;
 
 			actBrowse->addTo (&p);
 			actRefresh->addTo (&p);
@@ -1258,11 +1374,12 @@ bool DiscoDlg::Private::eventFilter (QObject *object, QEvent *event)
 			p.insertSeparator();
 			actAdd->addTo(&p);
 			actVCard->addTo(&p);
+			actAHCommand->addTo(&p);
 
 			// popup with all available features
-			QPopupMenu *fm = new QPopupMenu(&p);
+			Q3PopupMenu *fm = new Q3PopupMenu(&p);
 			{
-				QValueList<long>::Iterator it = ids.begin();
+				QList<long>::Iterator it = ids.begin();
 				for ( ; it != ids.end(); ++it)
 					fm->insertItem(Features::name(*it), *it + 10000); // TODO: add pixmap
 			}
@@ -1310,8 +1427,10 @@ void DiscoDlg::Private::setProtocol(int p)
 //----------------------------------------------------------------------------
 
 DiscoDlg::DiscoDlg(PsiAccount *pa, const Jid &jid, const QString &node)
-: DiscoUI (0, 0, WDestructiveClose)
+	: QDialog(0)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
+  	setupUi(this);
 	// restore options
 	ck_autoItems->setChecked(option.discoItems);
 	ck_autoInfo->setChecked(option.discoInfo);
@@ -1321,23 +1440,23 @@ DiscoDlg::DiscoDlg(PsiAccount *pa, const Jid &jid, const QString &node)
 	d->jid  = jid;
 	d->node = node;
 
-	setCaption(CAP(caption()));
-	setIcon(is->transportStatus("transport", STATUS_ONLINE));
+	setWindowTitle(CAP(caption()));
+	setWindowIcon(PsiIconset::instance()->transportStatus("transport", STATUS_ONLINE).icon());
 	X11WM_CLASS("disco");
 
-	statusBar()->hide();
-
-	pb_browse->setDefault(true);
 	connect (pb_browse, SIGNAL(clicked()), d, SLOT(doDisco()));
+	pb_browse->setDefault(false);
+	pb_browse->setAutoDefault(false);
 
+	connect(pb_close, SIGNAL(clicked()), this, SLOT(close()));
+	pb_close->setDefault(false);
+	pb_close->setAutoDefault(false);
 
-	cb_address->setInsertionPolicy(QComboBox::NoInsertion);
 	cb_address->insertStringList(pa->psi()->recentBrowseList()); // FIXME
 	cb_address->setFocus();
 	connect(cb_address, SIGNAL(activated(const QString &)), d, SLOT(doDisco()));
 	cb_address->setCurrentText(d->jid.full());
 
-	cb_node->setInsertionPolicy(QComboBox::NoInsertion);
 	cb_node->insertStringList(pa->psi()->recentNodeList());
 	connect(cb_node, SIGNAL(activated(const QString &)), d, SLOT(doDisco()));
 	cb_node->setCurrentText(node);
