@@ -18,18 +18,23 @@
  *
  */
 
-#include"eventdb.h"
+#include "eventdb.h"
 
 #define FAKEDELAY 0
 
-#include<qfile.h>
-#include<qptrlist.h>
-#include<qtimer.h>
-#include<qtextstream.h>
-#include<qvaluevector.h>
-#include"common.h"
-#include"profiles.h"
+#include <QVector>
+#include <QFileInfo>
+#include <QDir>
+#include <q3ptrlist.h>
+#include <qtimer.h>
+#include <qtextstream.h>
 
+#include "profiles.h"
+#include "applicationinfo.h"
+#include "psievent.h"
+#include "jidutil.h"
+
+using namespace XMPP;
 
 //----------------------------------------------------------------------------
 // EDBItem
@@ -141,6 +146,13 @@ void EDBHandle::append(const Jid &j, PsiEvent *e)
 	d->listeningFor = d->edb->op_append(j, e);
 }
 
+void EDBHandle::erase(const Jid &j)
+{
+	d->busy = true;
+	d->lastRequestType = Erase;
+	d->listeningFor = d->edb->op_erase(j);
+}
+
 bool EDBHandle::busy() const
 {
 	return d->busy;
@@ -197,7 +209,7 @@ class EDB::Private
 public:
 	Private() {}
 
-	QPtrList<EDBHandle> list;
+	Q3PtrList<EDBHandle> list;
 	int reqid_base;
 };
 
@@ -254,10 +266,15 @@ int EDB::op_append(const Jid &j, PsiEvent *e)
 	return append(j, e);
 }
 
+int EDB::op_erase(const Jid &j)
+{
+	return erase(j);
+}
+
 void EDB::resultReady(int req, EDBResult *r)
 {
 	// deliver
-	QPtrListIterator<EDBHandle> it(d->list);
+	Q3PtrListIterator<EDBHandle> it(d->list);
 	for(EDBHandle *h; (h = it.current()); ++it) {
 		if(h->listeningFor() == req) {
 			h->edb_resultReady(r);
@@ -270,7 +287,7 @@ void EDB::resultReady(int req, EDBResult *r)
 void EDB::writeFinished(int req, bool b)
 {
 	// deliver
-	QPtrListIterator<EDBHandle> it(d->list);
+	Q3PtrListIterator<EDBHandle> it(d->list);
 	for(EDBHandle *h; (h = it.current()); ++it) {
 		if(h->listeningFor() == req) {
 			h->edb_writeFinished(b);
@@ -299,7 +316,8 @@ struct item_file_req
 		Type_getOldest,
 		Type_get,
 		Type_append,
-		Type_find
+		Type_find,
+		Type_erase
 	};
 };
 
@@ -308,8 +326,8 @@ class EDBFlatFile::Private
 public:
 	Private() {}
 
-	QPtrList<File> flist;
-	QPtrList<item_file_req> rlist;
+	Q3PtrList<File> flist;
+	Q3PtrList<item_file_req> rlist;
 };
 
 EDBFlatFile::EDBFlatFile()
@@ -402,9 +420,22 @@ int EDBFlatFile::append(const Jid &j, PsiEvent *e)
 	return r->id;
 }
 
+int EDBFlatFile::erase(const Jid &j)
+{
+	item_file_req *r = new item_file_req;
+	r->j = j;
+	r->type = item_file_req::Type_erase;
+	r->event = 0;
+	r->id = genUniqueId();
+	d->rlist.append(r);
+
+	QTimer::singleShot(FAKEDELAY, this, SLOT(performRequests()));
+	return r->id;
+}
+
 EDBFlatFile::File *EDBFlatFile::findFile(const Jid &j) const
 {
-	QPtrListIterator<File> it(d->flist);
+	Q3PtrListIterator<File> it(d->flist);
 	for(File *i; (i = it.current()); ++it) {
 		if(i->j.compare(j, false))
 			return i;
@@ -423,6 +454,30 @@ EDBFlatFile::File *EDBFlatFile::ensureFile(const Jid &j)
 	return i;
 }
 
+bool EDBFlatFile::deleteFile(const Jid &j)
+{
+	File *i = findFile(j);
+
+	QString fname;
+
+	if (i) {
+		fname = i->fname;
+		d->flist.remove(i);
+		delete i;
+	}
+	else {
+		fname = File::jidToFileName(j);
+	}
+
+	QFileInfo fi(fname);
+	if(fi.exists()) {
+		QDir dir = fi.dir();
+		return dir.remove(fi.fileName());
+	}
+	else
+		return true;
+}
+
 void EDBFlatFile::performRequests()
 {
 	if(d->rlist.isEmpty())
@@ -524,6 +579,9 @@ void EDBFlatFile::performRequests()
 		}
 		resultReady(r->id, result);
 	}
+	else if(type == item_file_req::Type_erase) {
+		writeFinished(r->id, deleteFile(f->j));
+	}
 
 	delete r;
 }
@@ -544,7 +602,7 @@ class EDBFlatFile::File::Private
 public:
 	Private() {}
 
-	QValueVector<int> index;
+	QVector<quint64> index;
 	bool indexed;
 };
 
@@ -559,10 +617,9 @@ EDBFlatFile::File::File(const Jid &_j)
 	connect(t, SIGNAL(timeout()), SLOT(timer_timeout()));
 
 	//printf("[EDB opening -- %s]\n", j.full().latin1());
-	QString s = j.userHost();
-	fname = getHistoryDir() + "/" + qstrlower(jidEncode(s)) + ".history";
+	fname = jidToFileName(_j);
 	f.setName(fname);
-	valid = f.open(IO_ReadWrite);
+	valid = f.open(QIODevice::ReadWrite);
 
 	touch();
 }
@@ -576,26 +633,36 @@ EDBFlatFile::File::~File()
 	delete d;
 }
 
+QString EDBFlatFile::File::jidToFileName(const XMPP::Jid &j)
+{
+	return ApplicationInfo::historyDir() + "/" + JIDUtil::encode(j.userHost()).toLower() + ".history";
+}
+
 void EDBFlatFile::File::ensureIndex()
 {
 	if ( valid && !d->indexed ) {
+		if (f.isSequential()) {
+			qWarning("EDBFlatFile::File::ensureIndex(): Can't index sequential files.");
+			return;
+		}
+
 		f.reset(); // go to beginning
 		d->index.clear();
 
 		//printf(" file: %s\n", fname.latin1());
 		// build index
 		while(1) {
-			int at = f.at();
+			quint64 at = f.pos();
 
 			// locate a newline
 			bool found = false;
-			do {
-				int c = f.getch();
-				if(c == '\n') {
+			char c;
+			while (f.getChar(&c)) {
+				if (c == '\n') {
 					found = true;
 					break;
 				}
-			} while(!f.atEnd());
+			}
 
 			if(!found)
 				break;
@@ -641,13 +708,12 @@ PsiEvent *EDBFlatFile::File::get(int id)
 	if(id < 0 || id > (int)d->index.size())
 		return 0;
 
-	f.at(d->index[id]);
+	f.seek(d->index[id]);
 
 	QTextStream t;
 	t.setDevice(&f);
 	t.setEncoding(QTextStream::UnicodeUTF8);
 	QString line = t.readLine();
-	t.unsetDevice();
 
 	return lineToEvent(line);
 }
@@ -663,14 +729,13 @@ bool EDBFlatFile::File::append(PsiEvent *e)
 	if(line.isEmpty())
 		return false;
 
-	f.at(f.size());
-	int at = f.at();
+	f.seek(f.size());
+	quint64 at = f.pos();
 
 	QTextStream t;
 	t.setDevice(&f);
 	t.setEncoding(QTextStream::UnicodeUTF8);
 	t << line << endl;
-	t.unsetDevice();
 	f.flush();
 
 	if ( d->indexed ) {
@@ -707,7 +772,7 @@ PsiEvent *EDBFlatFile::File::lineToEvent(const QString &line)
 
 	// check for extra fields
 	if(sFlags[1] != '-') {
-		int subflags = hexChar2int(sFlags[1].latin1());
+		int subflags = QString(sFlags[1]).toInt(NULL,16);
 
 		// have subject?
 		if(subflags & 1) {
@@ -785,8 +850,8 @@ PsiEvent *EDBFlatFile::File::lineToEvent(const QString &line)
 		ae->setTimeStamp(QDateTime::fromString(sTime, Qt::ISODate));
 		return ae;
 	}
-	else
-		return 0;
+
+	return NULL;
 }
 
 QString EDBFlatFile::File::eventToLine(PsiEvent *e)
@@ -817,7 +882,7 @@ QString EDBFlatFile::File::eventToLine(PsiEvent *e)
 		sFlags = "N---";
 
 		if(subflags != 0)
-			sFlags[1] = int2hexChar(subflags);
+			sFlags[1] = QString::number(subflags,16)[0];
 
 		//  | date | type | To/from | flags | text
 		QString line = "|" + sTime + "|" + sType + "|" + sOrigin + "|" + sFlags + "|";
@@ -857,6 +922,6 @@ QString EDBFlatFile::File::eventToLine(PsiEvent *e)
 
 		return line;
 	}
-	else
-		return "";
+	
+	return "";
 }
