@@ -21,12 +21,14 @@
 #ifndef EVENTDB_H
 #define EVENTDB_H
 
-#include<qobject.h>
-#include<qtimer.h>
-#include<qfile.h>
-#include<qptrlist.h>
-#include"psievent.h"
+#include <qobject.h>
+#include <qtimer.h>
+#include <qfile.h>
+#include <q3ptrlist.h>
 
+#include "xmpp_jid.h"
+
+class PsiEvent;
 
 class EDBItem
 {
@@ -44,7 +46,7 @@ private:
 	PsiEvent *e;
 };
 
-class EDBResult : public QPtrList<EDBItem>
+class EDBResult : public Q3PtrList<EDBItem>
 {
 public:
 	EDBResult() {}
@@ -55,16 +57,17 @@ class EDBHandle : public QObject
 {
 	Q_OBJECT
 public:
-	enum { Read, Write };
+	enum { Read, Write, Erase };
 	EDBHandle(EDB *);
 	~EDBHandle();
 
 	// operations
-	void getLatest(const Jid &, int len);
-	void getOldest(const Jid &, int len);
-	void get(const Jid &jid, const QString &id, int direction, int len);
-	void find(const QString &, const Jid &, const QString &id, int direction);
-	void append(const Jid &, PsiEvent *);
+	void getLatest(const XMPP::Jid &, int len);
+	void getOldest(const XMPP::Jid &, int len);
+	void get(const XMPP::Jid &jid, const QString &id, int direction, int len);
+	void find(const QString &, const XMPP::Jid &, const QString &id, int direction);
+	void append(const XMPP::Jid &, PsiEvent *);
+	void erase(const XMPP::Jid &);
 
 	bool busy() const;
 	const EDBResult *result() const;
@@ -94,11 +97,12 @@ public:
 
 protected:
 	int genUniqueId() const;
-	virtual int getLatest(const Jid &, int len)=0;
-	virtual int getOldest(const Jid &, int len)=0;
-	virtual int get(const Jid &jid, const QString &id, int direction, int len)=0;
-	virtual int append(const Jid &, PsiEvent *)=0;
-	virtual int find(const QString &, const Jid &, const QString &id, int direction)=0;
+	virtual int getLatest(const XMPP::Jid &, int len)=0;
+	virtual int getOldest(const XMPP::Jid &, int len)=0;
+	virtual int get(const XMPP::Jid &jid, const QString &id, int direction, int len)=0;
+	virtual int append(const XMPP::Jid &, PsiEvent *)=0;
+	virtual int find(const QString &, const XMPP::Jid &, const QString &id, int direction)=0;
+	virtual int erase(const XMPP::Jid &)=0;
 	void resultReady(int, EDBResult *);
 	void writeFinished(int, bool);
 
@@ -110,11 +114,12 @@ private:
 	void reg(EDBHandle *);
 	void unreg(EDBHandle *);
 
-	int op_getLatest(const Jid &, int len);
-	int op_getOldest(const Jid &, int len);
-	int op_get(const Jid &, const QString &id, int direction, int len);
-	int op_find(const QString &, const Jid &, const QString &id, int direction);
-	int op_append(const Jid &, PsiEvent *);
+	int op_getLatest(const XMPP::Jid &, int len);
+	int op_getOldest(const XMPP::Jid &, int len);
+	int op_get(const XMPP::Jid &, const QString &id, int direction, int len);
+	int op_find(const QString &, const XMPP::Jid &, const QString &id, int direction);
+	int op_append(const XMPP::Jid &, PsiEvent *);
+	int op_erase(const XMPP::Jid &);
 };
 
 class EDBFlatFile : public EDB
@@ -124,11 +129,12 @@ public:
 	EDBFlatFile();
 	~EDBFlatFile();
 
-	int getLatest(const Jid &, int len);
-	int getOldest(const Jid &, int len);
-	int get(const Jid &jid, const QString &id, int direction, int len);
-	int find(const QString &, const Jid &, const QString &id, int direction);
-	int append(const Jid &, PsiEvent *);
+	int getLatest(const XMPP::Jid &, int len);
+	int getOldest(const XMPP::Jid &, int len);
+	int get(const XMPP::Jid &jid, const QString &id, int direction, int len);
+	int find(const QString &, const XMPP::Jid &, const QString &id, int direction);
+	int append(const XMPP::Jid &, PsiEvent *);
+	int erase(const XMPP::Jid &);
 
 	class File;
 
@@ -140,15 +146,16 @@ private:
 	class Private;
 	Private *d;
 
-	File *findFile(const Jid &) const;
-	File *ensureFile(const Jid &);
+	File *findFile(const XMPP::Jid &) const;
+	File *ensureFile(const XMPP::Jid &);
+	bool deleteFile(const XMPP::Jid &);
 };
 
 class EDBFlatFile::File : public QObject
 {
 	Q_OBJECT
 public:
-	File(const Jid &_j);
+	File(const XMPP::Jid &_j);
 	~File();
 
 	int total() const;
@@ -156,6 +163,8 @@ public:
 	PsiEvent *get(int);
 	bool append(PsiEvent *);
 
+	static QString jidToFileName(const XMPP::Jid &);
+
 signals:
 	void timeout();
 
@@ -163,7 +172,7 @@ private slots:
 	void timer_timeout();
 
 public:
-	Jid j;
+	XMPP::Jid j;
 	QString fname;
 	QFile f;
 	bool valid;
