@@ -1,42 +1,50 @@
-#include"filetransdlg.h"
+#include "filetransdlg.h"
 
-#include<qlabel.h>
-#include<qlineedit.h>
-#include<qpushbutton.h>
-#include<qtimer.h>
-#include<qfiledialog.h>
-#include<qfile.h>
-#include<qprogressbar.h>
-#include<qdir.h>
-#include<qlistview.h>
-#include<qlayout.h>
-#include<qhbox.h>
-#include<qdatetime.h>
-#include<qpainter.h>
-#include<qheader.h>
-#include<qtooltip.h>
-#include<qpopupmenu.h>
-#include<qtextedit.h>
-#include"psicon.h"
-#include"psiaccount.h"
-#include"userlist.h"
-#include"common.h"
-#include"iconwidget.h"
-#include"busywidget.h"
-#include"filetransfer.h"
-#include"profiles.h"
-#include"psiiconset.h"
-#include"msgmle.h"
+#include <QFileDialog>
+#include <qlabel.h>
+#include <qlineedit.h>
+#include <QMessageBox>
+#include <qpushbutton.h>
+#include <qtimer.h>
+#include <qfile.h>
+#include <q3progressbar.h>
+#include <qdir.h>
+#include <q3listview.h>
+#include <qlayout.h>
+#include <q3hbox.h>
+#include <qdatetime.h>
+#include <qpainter.h>
+#include <q3header.h>
+#include <q3popupmenu.h>
+#include <QResizeEvent>
+#include <QKeyEvent>
+#include <QVBoxLayout>
+#include <Q3PtrList>
+#include <QHBoxLayout>
+#include <QPixmap>
+
+#include "psicon.h"
+#include "psiaccount.h"
+#include "userlist.h"
+#include "common.h"
+#include "iconwidget.h"
+#include "s5b.h"
+#include "busywidget.h"
+#include "filetransfer.h"
+#include "accountscombobox.h"
+#include "profiles.h"
+#include "psiiconset.h"
+#include "msgmle.h"
+#include "jidutil.h"
+#include "psitooltip.h"
+#include "psicontactlist.h"
+#include "accountlabel.h"
 
-#if QT_VERSION >= 0x030200
 typedef Q_UINT64 LARGE_TYPE;
-#else
-typedef Q_UINT32 LARGE_TYPE;
-#endif
 
 #define CSMAX (sizeof(LARGE_TYPE)*8)
 #define CSMIN 16
-static int calcShift(Q_LLONG big)
+static int calcShift(qlonglong big)
 {
 	LARGE_TYPE val = 1;
 	val <<= CSMAX - 1;
@@ -48,24 +56,24 @@ static int calcShift(Q_LLONG big)
 	return 0;
 }
 
-static int calcComplement(Q_LLONG big, int shift)
+static int calcComplement(qlonglong big, int shift)
 {
 	int block = 1 << shift;
-	Q_LLONG rem = big % block;
+	qlonglong rem = big % block;
 	if(rem == 0)
 		return 0;
 	else
 		return (block - (int)rem);
 }
 
-static int calcTotalSteps(Q_LLONG big, int shift)
+static int calcTotalSteps(qlonglong big, int shift)
 {
 	if(big < 1)
 		return 0;
 	return ((big - 1) >> shift) + 1;
 }
 
-static int calcProgressStep(Q_LLONG big, int complement, int shift)
+static int calcProgressStep(qlonglong big, int complement, int shift)
 {
 	return ((big + complement) >> shift);
 }
@@ -100,9 +108,9 @@ static QString clean_filename(const QString &s)
 //#ifdef Q_OS_WIN
 	QString badchars = "\\/|?*:\"<>";
 	QString str;
-	for(uint n = 0; n < s.length(); ++n) {
+	for(int n = 0; n < s.length(); ++n) {
 		bool found = false;
-		for(uint b = 0; b < badchars.length(); ++b) {
+		for(int b = 0; b < badchars.length(); ++b) {
 			if(s.at(n) == badchars.at(b)) {
 				found = true;
 				break;
@@ -130,7 +138,7 @@ public:
 	S5BConnection *c;
 	Jid peer;
 	QString fileName, saveName;
-	Q_LLONG fileSize, sent, offset, length;
+	qlonglong fileSize, sent, offset, length;
 	QString desc;
 	bool sending;
 	QFile f;
@@ -196,7 +204,7 @@ void FileTransferHandler::send(const XMPP::Jid &to, const QString &fname, const
 		d->ft->setProxy(proxy);
 	mapSignals();
 
-	d->f.setName(fname);
+	d->f.setFileName(fname);
 	d->ft->sendFile(d->peer, d->fileName, d->fileSize, desc);
 }
 
@@ -220,7 +228,7 @@ QString FileTransferHandler::fileName() const
 	return d->fileName;
 }
 
-Q_LLONG FileTransferHandler::fileSize() const
+qlonglong FileTransferHandler::fileSize() const
 {
 	return d->fileSize;
 }
@@ -230,7 +238,7 @@ QString FileTransferHandler::description() const
 	return d->desc;
 }
 
-Q_LLONG FileTransferHandler::offset() const
+qlonglong FileTransferHandler::offset() const
 {
 	return d->offset;
 }
@@ -253,7 +261,7 @@ QString FileTransferHandler::saveName() const
 	return d->saveName;
 }
 
-void FileTransferHandler::accept(const QString &saveName, const QString &fileName, Q_LLONG offset)
+void FileTransferHandler::accept(const QString &saveName, const QString &fileName, qlonglong offset)
 {
 	if(d->sending)
 		return;
@@ -261,7 +269,7 @@ void FileTransferHandler::accept(const QString &saveName, const QString &fileNam
 	d->saveName = saveName;
 	d->offset = offset;
 	d->length = d->fileSize;
-	d->f.setName(saveName);
+	d->f.setFileName(saveName);
 	d->ft->accept(offset);
 }
 
@@ -319,6 +327,8 @@ void FileTransferHandler::ft_accepted()
 
 	if(d->sending)
 		accepted();
+	else
+		statusMessage(QString());
 }
 
 void FileTransferHandler::ft_connected()
@@ -328,7 +338,7 @@ void FileTransferHandler::ft_connected()
 	if(d->sending) {
 		// open the file, and set the correct offset
 		bool ok = false;
-		if(d->f.open(IO_ReadOnly)) {
+		if(d->f.open(QIODevice::ReadOnly)) {
 			if(d->offset == 0) {
 				ok = true;
 			}
@@ -340,7 +350,7 @@ void FileTransferHandler::ft_connected()
 		if(!ok) {
 			delete d->ft;
 			d->ft = 0;
-			error(ErrFile, 0, "");
+			error(ErrFile, 0, d->f.errorString());
 			return;
 		}
 
@@ -351,9 +361,9 @@ void FileTransferHandler::ft_connected()
 	}
 	else {
 		// open the file, truncating if offset is zero, otherwise set the correct offset
-		int m = IO_ReadWrite;
+		QIODevice::OpenMode m = QIODevice::ReadWrite;
 		if(d->offset == 0)
-			m |= IO_Truncate;
+			m |= QIODevice::Truncate;
 		bool ok = false;
 		if(d->f.open(m)) {
 			if(d->offset == 0) {
@@ -367,7 +377,7 @@ void FileTransferHandler::ft_connected()
 		if(!ok) {
 			delete d->ft;
 			d->ft = 0;
-			error(ErrFile, 0, "");
+			error(ErrFile, 0, d->f.errorString());
 			return;
 		}
 
@@ -391,7 +401,7 @@ void FileTransferHandler::ft_readyRead(const QByteArray &a)
 			d->f.close();
 			delete d->ft;
 			d->ft = 0;
-			error(ErrFile, 0, "");
+			error(ErrFile, 0, d->f.errorString());
 			return;
 		}
 		d->sent += a.size();
@@ -437,13 +447,15 @@ void FileTransferHandler::ft_error(int x)
 void FileTransferHandler::trySend()
 {
 	int blockSize = d->ft->dataSizeNeeded();
-	QByteArray a(blockSize);
-	int r = d->f.readBlock(a.data(), a.size());
+	QByteArray a(blockSize, 0);
+	int r = 0;
+	if(blockSize > 0)
+		r = d->f.read(a.data(), a.size());
 	if(r < 0) {
 		d->f.close();
 		delete d->ft;
 		d->ft = 0;
-		error(ErrFile, 0, "");
+		error(ErrFile, 0, d->f.errorString());
 		return;
 	}
 	if(r < (int)a.size())
@@ -467,7 +479,7 @@ void FileTransferHandler::mapSignals()
 	connect(d->ft, SIGNAL(connected()), SLOT(ft_connected()));
 	connect(d->ft, SIGNAL(readyRead(const QByteArray &)), SLOT(ft_readyRead(const QByteArray &)));
 	connect(d->ft, SIGNAL(bytesWritten(int)), SLOT(ft_bytesWritten(int)));
-	connect(d->ft, SIGNAL(error(int)), SLOT(ft_error(int)));
+	connect(d->ft, SIGNAL(error(int)), SLOT(ft_error(int)),Qt::QueuedConnection);
 }
 
 //----------------------------------------------------------------------------
@@ -480,54 +492,65 @@ public:
 	PsiCon *psi;
 	PsiAccount *pa;
 	AccountsComboBox *cb_ident;
-	QLabel *lb_ident, *lb_time;
+	QLabel* lb_identity;
+	AccountLabel* lb_ident;
+	QLabel* lb_time;
 	ChatView *te;
 	Jid jid;
 	FileTransferHandler *ft;
 	QString fileName;
-	Q_LLONG fileSize;
+	qlonglong fileSize;
 	bool sending;
 	QTimer t;
 };
 
 
 FileRequestDlg::FileRequestDlg(const Jid &j, PsiCon *psi, PsiAccount *pa) 
-:FileTransUI(0, 0, false, psi_dialog_flags | WDestructiveClose)
+	: QDialog(0, psi_dialog_flags)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
+	setModal(false);
+	setupUi(this);
 	QStringList l;
 	FileRequestDlg(j, psi, pa, l);
 }
 
 
-FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa, const QStringList& files)
-:FileTransUI(0, 0, false, psi_dialog_flags | WDestructiveClose)
+FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa, const QStringList& files, bool direct)
+	: QDialog(0, psi_dialog_flags)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
 	d = new Private;
+	setModal(false);
+	setupUi(this);
 	d->psi = psi;
 	d->pa = 0;
 	d->jid = jid;
 	d->ft = 0;
 	d->sending = true;
+	d->lb_ident = 0;
 	updateIdentity(pa);
 
-	QHBox *hb = new QHBox(this);
-	new QLabel(tr("Identity: "), hb);
+	Q3HBox *hb = new Q3HBox(this);
+	d->lb_identity = new QLabel(tr("Identity: "), hb);
 	d->cb_ident = d->psi->accountsComboBox(hb);
 	connect(d->cb_ident, SIGNAL(activated(PsiAccount *)), SLOT(updateIdentity(PsiAccount *)));
 	d->cb_ident->setAccount(pa);
 	replaceWidget(lb_accountlabel, hb);
 	setTabOrder(d->cb_ident, le_to);
+	connect(d->psi, SIGNAL(accountCountChanged()), this, SLOT(updateIdentityVisibility()));
+	updateIdentityVisibility();
 
 	d->te = new ChatView(this);
 	d->te->setReadOnly(false);
-	d->te->setTextFormat(PlainText);
+	d->te->setTextFormat(Qt::PlainText);
 	replaceWidget(te_desc, d->te);
 	setTabOrder(le_fname, d->te);
 	setTabOrder(d->te, pb_stop);
 
-	setCaption(tr("Send File"));
+	setWindowTitle(tr("Send File"));
 #ifndef Q_WS_MAC
-	setIcon(IconsetFactory::icon("psi/upload"));
+	setWindowIcon(IconsetFactory::icon("psi/upload").icon());
 #endif
 
 	le_to->setText(d->jid.full());
@@ -567,12 +590,19 @@ FileRequestDlg::FileRequestDlg(const Jid &jid, PsiCon *psi, PsiAccount *pa, cons
 		le_fname->setText(QDir::convertSeparators(fi.filePath()));
 		lb_size->setText(tr("%1 byte(s)").arg(fi.size())); // TODO: large file support
 	}
+
+	if (direct) {
+		doStart();
+	}
 }
 
 FileRequestDlg::FileRequestDlg(const QDateTime &ts, FileTransfer *ft, PsiAccount *pa)
-:FileTransUI(0, 0, false, psi_dialog_flags | WDestructiveClose)
+	: QDialog(0, psi_dialog_flags)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
 	d = new Private;
+	setModal(false);
+	setupUi(this);
 	d->psi = 0;
 	d->pa = 0;
 	d->jid = ft->peer();
@@ -584,26 +614,29 @@ FileRequestDlg::FileRequestDlg(const QDateTime &ts, FileTransfer *ft, PsiAccount
 	d->fileSize = ft->fileSize();
 
 	d->cb_ident = 0;
-	QHBox *hb = new QHBox(this);
-	new QLabel(tr("Identity: "), hb);
-	d->lb_ident = d->pa->accountLabel(hb);
+	Q3HBox *hb = new Q3HBox(this);
+	d->lb_identity = new QLabel(tr("Identity: "), hb);
+	d->lb_ident = new AccountLabel(hb);
+	d->lb_ident->setAccount(d->pa);
 	d->lb_ident->setSizePolicy(QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::Fixed ));
 	new QLabel(tr("Time:"), hb);
-	d->lb_time = new QLabel(ts.time().toString(LocalDate), hb);
+	d->lb_time = new QLabel(ts.time().toString(Qt::LocalDate), hb);
 	d->lb_time->setFrameStyle( QFrame::Panel | QFrame::Sunken );
+	connect(d->pa->psi(), SIGNAL(accountCountChanged()), this, SLOT(updateIdentityVisibility()));
+	updateIdentityVisibility();
 
 	replaceWidget(lb_accountlabel, hb);
 
 	d->te = new ChatView(this);
-	d->te->setTextFormat(PlainText);
+	d->te->setTextFormat(Qt::PlainText);
 	replaceWidget(te_desc, d->te);
 	setTabOrder(le_fname, d->te);
 	setTabOrder(d->te, pb_stop);
 
 	lb_to->setText(tr("From:"));
-	setCaption(tr("Receive File"));
+	setWindowTitle(tr("Receive File"));
 #ifndef Q_WS_MAC
-	setIcon(IconsetFactory::icon("psi/download"));
+	setWindowIcon(IconsetFactory::icon("psi/download").icon());
 #endif
 
 	le_to->setText(d->jid.full());
@@ -656,7 +689,7 @@ void FileRequestDlg::done(int r)
 
 void FileRequestDlg::keyPressEvent(QKeyEvent *e)
 {
-	if(e->key() == Key_Return && ((e->state() & ControlButton) || (e->state() & AltButton)) ) {
+	if(e->key() == Qt::Key_Return && ((e->modifiers() & Qt::ControlModifier) || (e->modifiers() & Qt::AltModifier)) ) {
 		if(pb_start->isEnabled())
 			doStart();
 	}
@@ -678,6 +711,16 @@ void FileRequestDlg::updateIdentity(PsiAccount *pa)
 	connect(d->pa, SIGNAL(disconnected()), this, SLOT(pa_disconnected()));
 }
 
+void FileRequestDlg::updateIdentityVisibility()
+{
+	bool visible = d->pa->psi()->contactList()->enabledAccounts().count() > 1;
+	if (d->cb_ident) 
+		d->cb_ident->setVisible(visible);
+	if (d->lb_ident)
+		d->lb_ident->setVisible(visible);
+	d->lb_identity->setVisible(visible);
+}
+
 void FileRequestDlg::pa_disconnected()
 {
 	//if(busy->isActive()) {
@@ -713,7 +756,7 @@ void FileRequestDlg::chooseFile()
 	while(1) {
 		if(option.lastPath.isEmpty())
 			option.lastPath = QDir::homeDirPath();
-		QString str = QFileDialog::getOpenFileName(option.lastPath, tr("All files (*)"), this, 0, tr("Choose a file"));
+		QString str = QFileDialog::getOpenFileName(this, tr("Choose a file"), option.lastPath, tr("All files (*)"));
 		if(!str.isEmpty()) {
 			QFileInfo fi(str);
 			if(!fi.exists()) {
@@ -765,36 +808,23 @@ void FileRequestDlg::doStart()
 	}
 	else {
 		QString fname, savename;
-		bool overwrite = false;
-		while(1) {
-			if(option.lastSavePath.isEmpty())
-				option.lastSavePath = QDir::homeDirPath();
-			fname = QFileDialog::getSaveFileName(QDir(option.lastSavePath).filePath(d->fileName), tr("All files (*)"), this, 0, tr("Save As"));
-			if(!fname.isEmpty()) {
-				QFileInfo fi(fname);
-				if(fi.exists()) {
-					int x = QMessageBox::information(this, tr("Error"), tr("File already exists.  Overwrite?"), tr("&Yes"), tr("&No"));
-					if(x != 0)
-						continue;
-					overwrite = true;
-				}
-				option.lastSavePath = fi.dirPath();
-				savename = fname + ".part";
-				fname = fi.fileName();
-			}
-			else
-				return;
-
-			break;
-		}
+		if(option.lastSavePath.isEmpty())
+			option.lastSavePath = QDir::homeDirPath();
+		fname = QFileDialog::getSaveFileName(this, tr("Save As"), QDir(option.lastSavePath).filePath(d->fileName), tr("All files (*)"));
+		if(fname.isEmpty())
+			return;
+		QFileInfo fi(fname);
+		option.lastSavePath = fi.dirPath();
+		savename = fname + ".part";
+		fname = fi.fileName();
 
 		if(active_file_check(savename)) {
 			QMessageBox::information(this, tr("Error"), tr("This file is being transferred already!"));
 			return;
 		}
 
-		Q_LLONG resume_offset = 0;
-		if(!overwrite) {
+		qlonglong resume_offset = 0;
+		if(!fi.exists()) {
 			// supports resume?  check for a .part
 			if(d->ft->resumeSupported()) {
 				QFileInfo fi(savename);
@@ -828,7 +858,9 @@ void FileRequestDlg::ft_accepted()
 
 void FileRequestDlg::ft_statusMessage(const QString &s)
 {
-	lb_status->setText(s);
+	if(!s.isEmpty()) {
+		lb_status->setText(s);
+	}
 
 	// stop the timer at first activity
 	if(d->t.isActive())
@@ -914,17 +946,17 @@ void FileRequestDlg::t_timeout()
 //----------------------------------------------------------------------------
 // FileTransDlg
 //----------------------------------------------------------------------------
-class FileTransItem : public QListViewItem
+class FileTransItem : public Q3ListViewItem
 {
 public:
 	QPixmap icon;
 	bool sending;
 	QString name;
-	Q_LLONG size;
+	qlonglong size;
 	QString peer;
 	QString rate;
 	int progress;
-	Q_LLONG sent;
+	qlonglong sent;
 	int bps;
 	int timeRemaining;
 	int id;
@@ -932,8 +964,8 @@ public:
 	bool done;
 	QString error;
 
-	FileTransItem(QListView *parent, const QString &_name, Q_LLONG _size, const QString &_peer, bool _sending)
-	:QListViewItem(parent)
+	FileTransItem(Q3ListView *parent, const QString &_name, qlonglong _size, const QString &_peer, bool _sending)
+	:Q3ListViewItem(parent)
 	{
 		done = false;
 		sending = _sending;
@@ -946,11 +978,11 @@ public:
 		dist = -1;
 	}
 
-	void niceUnit(Q_LLONG n, Q_LLONG *div, QString *unit)
+	void niceUnit(qlonglong n, qlonglong *div, QString *unit)
 	{
-		Q_LLONG gb = 1024 * 1024 * 1024;
-		Q_LLONG mb = 1024 * 1024;
-		Q_LLONG kb = 1024;
+		qlonglong gb = 1024 * 1024 * 1024;
+		qlonglong mb = 1024 * 1024;
+		qlonglong kb = 1024;
 		if(n >= gb) {
 			*div = gb;
 			*unit = QString("GB");
@@ -969,14 +1001,14 @@ public:
 		}
 	}
 
-	QString roundedNumber(Q_LLONG n, Q_LLONG div)
+	QString roundedNumber(qlonglong n, qlonglong div)
 	{
 		bool decimal = false;
 		if(div >= 1024) {
 			div /= 10;
 			decimal = true;
 		}
-		Q_LLONG x_long = n / div;
+		qlonglong x_long = n / div;
 		int x = (int)x_long;
 		if(decimal) {
 			double f = (double)x;
@@ -987,14 +1019,14 @@ public:
 			return QString::number(x);
 	}
 
-	bool setProgress(int _progress, Q_LLONG _sent, int _bps)
+	bool setProgress(int _progress, qlonglong _sent, int _bps)
 	{
 		progress = _progress;
 		sent = _sent;
 		bps = _bps;
 
 		if(bps > 0) {
-			Q_LLONG rest_long = size - sent;
+			qlonglong rest_long = size - sent;
 			rest_long /= bps;
 			int maxtime = (23 * 60 * 60) + (59 * 60) + (59); // 23h59m59s
 			if(rest_long > maxtime)
@@ -1014,7 +1046,7 @@ public:
 	{
 		QString s;
 		{
-			Q_LLONG div;
+			qlonglong div;
 			QString unit;
 			niceUnit(size, &div, &unit);
 
@@ -1084,13 +1116,13 @@ public:
 
 		p->save();
 		p->setPen(cg.highlightedText());
-		p->setClipRect(xoff, yoff, dist, ysize, QPainter::CoordPainter);
+		p->setClipRect(xoff, yoff, dist, ysize);
 		p->drawText(center - (textwidth / 2), ty, s);
 		p->restore();
 
 		p->save();
 		p->setPen(cg.text());
-		p->setClipRect(xoff + dist, yoff, width - 2 - dist, ysize, QPainter::CoordPainter);
+		p->setClipRect(xoff + dist, yoff, width - 2 - dist, ysize);
 		p->drawText(center - (textwidth / 2), ty, s);
 		p->restore();
 		p->restore();
@@ -1099,7 +1131,7 @@ public:
 	void setup()
 	{
 		widthChanged();
-		QListView *lv = listView();
+		Q3ListView *lv = listView();
 
 		QFontMetrics fm = lv->fontMetrics();
 		int m = 4;
@@ -1258,32 +1290,43 @@ public:
 	}
 };
 
-class FileTransView : public QListView, public QToolTip
+class FileTransView : public Q3ListView
 {
 	Q_OBJECT
 public:
 	FileTransView(QWidget *parent=0, const char *name=0)
-	:QListView(parent, name), QToolTip(viewport())
+	:Q3ListView(parent, name)
 	{
-		connect(this, SIGNAL(contextMenuRequested(QListViewItem *, const QPoint &, int)), this, SLOT(qlv_contextMenuRequested(QListViewItem *, const QPoint &, int)));
+		connect(this, SIGNAL(contextMenuRequested(Q3ListViewItem *, const QPoint &, int)), this, SLOT(qlv_contextMenuRequested(Q3ListViewItem *, const QPoint &, int)));
 	}
 
-	void maybeTip(const QPoint &pos)
+	bool maybeTip(const QPoint &pos)
 	{
 		FileTransItem *i = static_cast<FileTransItem*>(itemAt(pos));
 		if(!i)
-			return;
+			return false;
 		QRect r(itemRect(i));
-		tip(r, i->makeTip());
+		PsiToolTip::showText(mapToGlobal(pos), i->makeTip(), this);
+		return true;
 	}
 
 	void resizeEvent(QResizeEvent *e)
 	{
-		QListView::resizeEvent(e);
+		Q3ListView::resizeEvent(e);
 
 		if(e->oldSize().width() != e->size().width())
 			doResize();
 	}
+	
+	bool event(QEvent* e)
+	{
+		if (e->type() == QEvent::ToolTip) {
+			QPoint pos = ((QHelpEvent*) e)->pos();
+			e->setAccepted(maybeTip(pos));
+			return true;
+		}
+		return Q3ListView::event(e);
+	}
 
 signals:
 	void itemCancel(int id);
@@ -1291,14 +1334,14 @@ signals:
 	void itemClear(int id);
 
 private slots:
-	void qlv_contextMenuRequested(QListViewItem *item, const QPoint &pos, int)
+	void qlv_contextMenuRequested(Q3ListViewItem *item, const QPoint &pos, int)
 	{
 		if(!item)
 			return;
 
 		FileTransItem *i = static_cast<FileTransItem*>(item);
 
-		QPopupMenu p;
+		Q3PopupMenu p;
 		p.insertItem(tr("&Cancel"), 0);
 		p.insertSeparator();
 		//p.insertItem(tr("&Open Destination Folder"), 1);
@@ -1329,8 +1372,8 @@ private slots:
 private:
 	void doResize()
 	{
-		QListViewItemIterator it(this);
-		for(QListViewItem *i; (i = it.current()); ++it)
+		Q3ListViewItemIterator it(this);
+		for(Q3ListViewItem *i; (i = it.current()); ++it)
 			i->setup();
 	}
 };
@@ -1341,10 +1384,10 @@ public:
 	FileTransferHandler *h;
 	int id;
 	int p;
-	Q_LLONG sent;
+	qlonglong sent;
 
 	int at;
-	Q_LLONG last[10];
+	qlonglong last[10];
 
 	TransferMapping()
 	{
@@ -1375,7 +1418,7 @@ public:
 	FileTransDlg *parent;
 	PsiCon *psi;
 	FileTransView *lv;
-	QPtrList<TransferMapping> transferList;
+	Q3PtrList<TransferMapping> transferList;
 	QTimer t;
 
 	Private(FileTransDlg *_parent)
@@ -1389,8 +1432,8 @@ public:
 		int n = 0;
 		while(1) {
 			bool found = false;
-			QListViewItemIterator it(lv);
-			for(QListViewItem *i; (i = it.current()); ++it) {
+			Q3ListViewItemIterator it(lv);
+			for(Q3ListViewItem *i; (i = it.current()); ++it) {
 				FileTransItem *fi = static_cast<FileTransItem*>(i);
 				if(fi->id == n) {
 					found = true;
@@ -1406,8 +1449,8 @@ public:
 
 	FileTransItem *findItem(int id)
 	{
-		QListViewItemIterator it(lv);
-		for(QListViewItem *i; (i = it.current()); ++it) {
+		Q3ListViewItemIterator it(lv);
+		for(Q3ListViewItem *i; (i = it.current()); ++it) {
 			FileTransItem *fi = static_cast<FileTransItem*>(i);
 			if(fi->id == id)
 				return fi;
@@ -1415,11 +1458,11 @@ public:
 		return 0;
 	}
 
-	QPtrList<FileTransItem> getFinished()
+	Q3PtrList<FileTransItem> getFinished()
 	{
-		QPtrList<FileTransItem> list;
-		QListViewItemIterator it(lv);
-		for(QListViewItem *i; (i = it.current()); ++it) {
+		Q3PtrList<FileTransItem> list;
+		Q3ListViewItemIterator it(lv);
+		for(Q3ListViewItem *i; (i = it.current()); ++it) {
 			FileTransItem *fi = static_cast<FileTransItem*>(i);
 			if(fi->done)
 				list.append(fi);
@@ -1429,7 +1472,7 @@ public:
 
 	TransferMapping *findMapping(FileTransferHandler *h)
 	{
-		QPtrListIterator<TransferMapping> it(transferList);
+		Q3PtrListIterator<TransferMapping> it(transferList);
 		for(TransferMapping *i; (i = it.current()); ++it) {
 			if(i->h == h)
 				return i;
@@ -1439,7 +1482,7 @@ public:
 
 	TransferMapping *findMapping(int id)
 	{
-		QPtrListIterator<TransferMapping> it(transferList);
+		Q3PtrListIterator<TransferMapping> it(transferList);
 		for(TransferMapping *i; (i = it.current()); ++it) {
 			if(i->id == id)
 				return i;
@@ -1455,7 +1498,7 @@ public:
 		int bps = -1;
 		if(!done && i->at >= 2) {
 			int seconds = i->at - 1;
-			Q_LLONG average = i->last[i->at-1] - i->last[0];
+			qlonglong average = i->last[i->at-1] - i->last[0];
 			bps = ((int)average) / seconds;
 		}
 
@@ -1504,9 +1547,9 @@ FileTransDlg::FileTransDlg(PsiCon *psi)
 
 	connect(&d->t, SIGNAL(timeout()), SLOT(updateItems()));
 
-	setCaption(tr("Transfer Manager"));
+	setWindowTitle(tr("Transfer Manager"));
 #ifndef Q_WS_MAC
-	setIcon(IconsetFactory::icon("psi/filemanager"));
+	setWindowIcon(IconsetFactory::icon("psi/filemanager").icon());
 #endif
 
 	QVBoxLayout *vb = new QVBoxLayout(this, 11, 6);
@@ -1530,7 +1573,7 @@ FileTransDlg::FileTransDlg(PsiCon *psi)
 	d->lv->addColumn("");
 	d->lv->header()->hide();
 
-	d->lv->setResizeMode(QListView::LastColumn);
+	d->lv->setResizeMode(Q3ListView::LastColumn);
 	d->lv->setAllColumnsShowFocus(true);
 	d->lv->setSorting(-1);
 
@@ -1543,7 +1586,7 @@ FileTransDlg::~FileTransDlg()
 	delete d;
 }
 
-int FileTransDlg::addItem(const QString &filename, Q_LLONG size, const QString &peer, bool sending)
+int FileTransDlg::addItem(const QString &filename, qlonglong size, const QString &peer, bool sending)
 {
 	int id = d->findFreeId();
 	FileTransItem *i = new FileTransItem(d->lv, filename, size, peer, sending);
@@ -1556,7 +1599,7 @@ int FileTransDlg::addItem(const QString &filename, Q_LLONG size, const QString &
 	return id;
 }
 
-void FileTransDlg::setProgress(int id, int step, int total, Q_LLONG sent, int bytesPerSecond, bool updateAll)
+void FileTransDlg::setProgress(int id, int step, int total, qlonglong sent, int bytesPerSecond, bool updateAll)
 {
 	FileTransItem *i = d->findItem(id);
 	if(i) {
@@ -1600,12 +1643,12 @@ void FileTransDlg::setError(int id, const QString &reason)
 	}
 }
 
-void FileTransDlg::takeTransfer(FileTransferHandler *h, int p, Q_LLONG sent)
+void FileTransDlg::takeTransfer(FileTransferHandler *h, int p, qlonglong sent)
 {
 	QString peer;
-	UserListItem *u = h->account()->findFirstRelavent(h->peer());
+	UserListItem *u = h->account()->findFirstRelevant(h->peer());
 	if(u)
-		peer = jidnick(u->jid().full(), u->name());
+		peer = JIDUtil::nickOrJid(u->name(),u->jid().full());
 	else
 		peer = h->peer().full();
 
@@ -1623,17 +1666,17 @@ void FileTransDlg::takeTransfer(FileTransferHandler *h, int p, Q_LLONG sent)
 		d->updateProgress(i, true);
 	}
 	else {
-		connect(h, SIGNAL(progress(int, Q_LLONG)), SLOT(ft_progress(int, Q_LLONG)));
+		connect(h, SIGNAL(progress(int, qlonglong)), SLOT(ft_progress(int, qlonglong)));
 		connect(h, SIGNAL(error(int, int, const QString &)), SLOT(ft_error(int, int, const QString &)));
 	}
 }
 
 void FileTransDlg::clearFinished()
 {
-	QPtrList<FileTransItem> list = d->getFinished();
+	Q3PtrList<FileTransItem> list = d->getFinished();
 	{
 		// remove related transfer mappings
-		QPtrListIterator<FileTransItem> it(list);
+		Q3PtrListIterator<FileTransItem> it(list);
 		for(FileTransItem *fi; (fi = it.current()); ++it) {
 			TransferMapping *i = d->findMapping(fi->id);
 			d->transferList.removeRef(i);
@@ -1643,7 +1686,7 @@ void FileTransDlg::clearFinished()
 	list.clear();
 }
 
-void FileTransDlg::ft_progress(int p, Q_LLONG sent)
+void FileTransDlg::ft_progress(int p, qlonglong sent)
 {
 	TransferMapping *i = d->findMapping((FileTransferHandler *)sender());
 	i->p = p;
@@ -1666,15 +1709,15 @@ void FileTransDlg::ft_error(int x, int, const QString &s)
 	if(x == FileTransferHandler::ErrTransfer)
 		str = s;
 	else
-		str = tr("File I/O error");
+		str = tr("File I/O error (%1)").arg(s);
 	setError(id, str);
 }
 
 void FileTransDlg::updateItems()
 {
 	// operate on a copy so that we can delete items in updateProgress
-	QPtrList<TransferMapping> list = d->transferList;
-	QPtrListIterator<TransferMapping> it(list);
+	Q3PtrList<TransferMapping> list = d->transferList;
+	Q3PtrListIterator<TransferMapping> it(list);
 	for(TransferMapping *i; (i = it.current()); ++it) {
 		if(i->h) {
 			i->logSent();
@@ -1715,8 +1758,8 @@ void FileTransDlg::itemClear(int id)
 
 void FileTransDlg::killTransfers(PsiAccount *pa)
 {
-	QPtrList<TransferMapping> list = d->transferList;
-	QPtrListIterator<TransferMapping> it(list);
+	Q3PtrList<TransferMapping> list = d->transferList;
+	Q3PtrListIterator<TransferMapping> it(list);
 	for(TransferMapping *i; (i = it.current()); ++it) {
 		// this account?
 		if(i->h->account() == pa) {
@@ -1727,4 +1770,4 @@ void FileTransDlg::killTransfers(PsiAccount *pa)
 	}
 }
 
-#include"filetransdlg.moc"
+#include "filetransdlg.moc"
