@@ -1,19 +1,14 @@
 #ifndef FILETRANSDLG_H
 #define FILETRANSDLG_H
 
-#include"ui_filetrans.h"
-#include"im.h"
-#include"s5b.h"
-
-#if QT_VERSION < 0x030200
-typedef long int Q_LLONG;
-#endif
+#include "ui_filetrans.h"
+#include "s5b.h"
 
 class PsiCon;
 class PsiAccount;
-namespace XMPP
-{
+namespace XMPP {
 	class FileTransfer;
+	class Jid;
 }
 using namespace XMPP;
 
@@ -30,21 +25,21 @@ public:
 	int mode() const;
 	Jid peer() const;
 	QString fileName() const;
-	Q_LLONG fileSize() const;
+	qlonglong fileSize() const;
 	QString description() const;
-	Q_LLONG offset() const;
+	qlonglong offset() const;
 	int totalSteps() const;
 	bool resumeSupported() const;
 	QString saveName() const;
 
 	void send(const Jid &to, const QString &fname, const QString &desc);
-	void accept(const QString &saveName, const QString &fileName, Q_LLONG offset=0);
+	void accept(const QString &saveName, const QString &fileName, qlonglong offset=0);
 
 signals:
 	void accepted();
 	void statusMessage(const QString &s);
 	void connected();
-	void progress(int p, Q_LLONG sent);
+	void progress(int p, qlonglong sent);
 	void error(int, int, const QString &s);
 
 private slots:
@@ -73,12 +68,12 @@ private:
 	void mapSignals();
 };
 
-class FileRequestDlg : public FileTransUI
+class FileRequestDlg : public QDialog, public Ui::FileTrans
 {
 	Q_OBJECT
 public:
 	FileRequestDlg(const Jid &j, PsiCon *psi, PsiAccount *pa);
-	FileRequestDlg(const Jid &j, PsiCon *psi, PsiAccount *pa, const QStringList& files);
+	FileRequestDlg(const Jid &j, PsiCon *psi, PsiAccount *pa, const QStringList& files, bool direct = false);
 	FileRequestDlg(const QDateTime &ts, FileTransfer *ft, PsiAccount *pa);
 	~FileRequestDlg();
 
@@ -90,6 +85,7 @@ public slots:
 
 private slots:
 	void updateIdentity(PsiAccount *);
+	void updateIdentityVisibility();
 	void pa_disconnected();
 	void chooseFile();
 	void doStart();
@@ -114,17 +110,17 @@ public:
 	FileTransDlg(PsiCon *);
 	~FileTransDlg();
 
-	int addItem(const QString &filename, Q_LLONG size, const QString &peer, bool sending);
-	void setProgress(int id, int step, int total, Q_LLONG sent, int bytesPerSecond, bool updateAll=false);
+	int addItem(const QString &filename, qlonglong size, const QString &peer, bool sending);
+	void setProgress(int id, int step, int total, qlonglong sent, int bytesPerSecond, bool updateAll=false);
 	void setError(int id, const QString &reason);
 	void removeItem(int id);
 
-	void takeTransfer(FileTransferHandler *h, int p, Q_LLONG sent);
+	void takeTransfer(FileTransferHandler *h, int p, qlonglong sent);
 	void killTransfers(PsiAccount *pa);
 
 private slots:
 	void clearFinished();
-	void ft_progress(int p, Q_LLONG sent);
+	void ft_progress(int p, qlonglong sent);
 	void ft_error(int, int, const QString &s);
 	void updateItems();
 
