@@ -18,80 +18,29 @@
  *
  */
 
-#include<qlistview.h>
-#include<qlineedit.h>
-#include<qtooltip.h>
-#include"advwidget.h"
-#include"im.h"
-#include"ui_gcjoin.h"
+#ifndef GROUPCHATDLG_H
+#define GROUPCHATDLG_H
+
+#include <QWidget>
+#include <QDialog>
+
+#include "ui_groupchatdlg.h"
+#include "mucmanager.h"
+#include "advwidget.h"
 
 using namespace XMPP;
 
 class PsiCon;
 class PsiAccount;
-
-class GCJoinDlg : public GCJoinUI
-{
-	Q_OBJECT
-public:
-	GCJoinDlg(PsiCon *, PsiAccount *);
-	~GCJoinDlg();
-
-	void joined();
-	void error(int, const QString &);
-
-protected:
-	//void closeEvent(QCloseEvent *);
-
-public slots:
-	void done(int);
-
-private slots:
-	void doJoin();
-	void updateIdentity(PsiAccount *);
-	void pa_disconnected();
-	void recent_activated(int);
-
-private:
-	class Private;
-	Private *d;
-
-	void disableWidgets();
-	void enableWidgets();
-};
-
-class GCUserViewItem : public QListViewItem
-{
-public:
-	GCUserViewItem(QListView *);
-
-	Status s;
-};
-
-class GCUserView : public QListView, public QToolTip
-{
-	Q_OBJECT
-public:
-	GCUserView(QWidget *parent=0, const char *name=0);
-	~GCUserView();
-
-	void updateAll();
-	QListViewItem *findEntry(const QString &);
-	void updateEntry(const QString &, const Status &);
-	void removeEntry(const QString &);
-	QStringList nickList() const;
-
-protected:
-	// reimplemented
-	void maybeTip(const QPoint &);
-
-signals:
-	void action(const QString &, const Status &, int);
-
-private slots:
-	void qlv_doubleClicked(QListViewItem *);
-	void qlv_contextMenuRequested(QListViewItem *, const QPoint &, int);
-};
+class PsiOptions;
+class QRect;
+class GCMainDlg;
+class QPainter;
+class QColorGroup;
+class Q3DragObject;
+namespace XMPP {
+	class Message;
+}
 
 /*class GCLineEdit : public QLineEdit
 {
@@ -115,16 +64,27 @@ public:
 	~GCMainDlg();
 
 	Jid jid() const;
+	PsiAccount* account() const;
 
 	void error(int, const QString &);
 	void presence(const QString &, const Status &);
 	void message(const Message &);
 	void joined();
+	void setPassword(const QString&);
+	const QString& nick() const;
 
 protected:
+	void setShortcuts();
+
+	// reimplemented
 	void keyPressEvent(QKeyEvent *);
+	void dragEnterEvent(QDragEnterEvent *);
+	void dropEvent(QDropEvent *);
 	void closeEvent(QCloseEvent *);
+	void resizeEvent(QResizeEvent*);
+
 	void windowActivationChange(bool);
+	void mucInfoDialog(const QString& title, const QString& message, const Jid& actor, const QString& reason);
 
 signals:
 	void aSend(const Message &);
@@ -133,10 +93,13 @@ public slots:
 	void optionsUpdate();
 
 private slots:
+	void scrollUp();
+	void scrollDown();
 	void mle_returnPressed();
 	void doTopic();
 	void openFind();
-	void doFind(int, int, const QString &);
+	void configureRoom();
+	void doFind(const QString &);
 	void flashAnimate();
 	void pa_updatedActivity();
 	void goDisc();
@@ -144,14 +107,23 @@ private slots:
 	void lv_action(const QString &, const Status &, int);
 	void doClear();
 	void doClearButton();
-	void toggleSmallChat();
 	void buildMenu();
 	void logSelectionChanged();
+	void setConnecting();
+	void unsetConnecting();
+	void action_error(MUCManager::Action, int, const QString&);
+	void updateIdentityVisibility();
+#ifdef WHITEBOARDING
+	void openWhiteboard();
+#endif
+	void chatEditCreated();
 
 public:
 	class Private;
+	friend class Private;
 private:
 	Private *d;
+	Ui::GroupChatDlg ui_;
 
 	void doFlash(bool);
 	void doAlert();
@@ -165,25 +137,27 @@ private:
 	QString getNickColor(QString);
 	QMap<QString,int> nicks;
 	int nicknumber;
+	PsiOptions* options_;
 };
 
 class GCFindDlg : public QDialog
 {
 	Q_OBJECT
 public:
-	GCFindDlg(int, int, const QString &, QWidget *parent=0, const char *name=0);
+	GCFindDlg(const QString &, QWidget *parent=0, const char *name=0);
 	~GCFindDlg();
 
-	void found(int, int);
+	void found();
 	void error(const QString &);
 
 signals:
-	void find(int, int, const QString &);
+	void find(const QString &);
 
 private slots:
 	void doFind();
 
 private:
-	int para, index;
 	QLineEdit *le_input;
 };
+
+#endif
