@@ -18,37 +18,45 @@
  *
  */
 
-#include"historydlg.h"
+#include "historydlg.h"
 
-#include<qpopupmenu.h>
-#include<qheader.h>
-#include<qlayout.h>
-#include<qlabel.h>
-#include<qlineedit.h>
-#include<qpushbutton.h>
-#include<qfiledialog.h>
-#include<qmessagebox.h>
-#include<qframe.h>
-#include<qapplication.h>
-#include<qclipboard.h>
-#include"psiaccount.h"
-#include"psievent.h"
-#include"common.h"
-#include"busywidget.h"
-#include"eventdb.h"
-#include"userlist.h"
+#include <q3popupmenu.h>
+#include <q3header.h>
+#include <qlayout.h>
+#include <qlabel.h>
+#include <qlineedit.h>
+#include <qpushbutton.h>
+#include <QFileDialog>
+#include <Q3SimpleRichText>
+#include <qmessagebox.h>
+#include <QFrame>
+#include <qapplication.h>
+#include <qclipboard.h>
+#include <QTextStream>
+#include <QCloseEvent>
+#include <QKeyEvent>
+#include <QHBoxLayout>
+#include <QVBoxLayout>
+#include <QResizeEvent>
+#include "psiaccount.h"
+#include "psievent.h"
+#include "busywidget.h"
+#include "applicationinfo.h"
+#include "common.h"
+#include "eventdb.h"
+#include "psiiconset.h"
+#include "textutil.h"
+#include "jidutil.h"
+#include "userlist.h"
 
 static QString getNext(QString *str)
 {
-	// are we in space?
 	int n = 0;
-	if(str->at(n).isSpace()) {
-		// get out of it
-		while(n < (int)str->length() && str->at(n).isSpace())
-			++n;
-		if(n == (int)str->length())
-			return QString::null;
-	}
+	// skip leading spaces (but *do* return them later!)
+	while(n < (int)str->length() && str->at(n).isSpace())
+		++n;
+	if(n == (int)str->length())
+		return QString::null;
 	// find end or next space
 	while(n < (int)str->length() && !str->at(n).isSpace())
 		++n;
@@ -115,17 +123,20 @@ public:
 };
 
 HistoryDlg::HistoryDlg(const Jid &jid, PsiAccount *pa)
-:QWidget(0, 0, WDestructiveClose)
+	: QWidget(0, 0)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
+  	if ( option.brushedMetal )
+		setAttribute(Qt::WA_MacMetalStyle);
 	d = new Private;
 	d->pa = pa;
 	d->jid = jid;
 	d->pa->dialogRegister(this, d->jid);
 	d->exp = 0;
 
-	setCaption(d->jid.full());
+	setWindowTitle(d->jid.full());
 #ifndef Q_WS_MAC
-	setIcon(IconsetFactory::icon("psi/history"));
+	setWindowIcon(IconsetFactory::icon("psi/history").icon());
 #endif
 
 	d->h = new EDBHandle(d->pa->edb());
@@ -133,7 +144,7 @@ HistoryDlg::HistoryDlg(const Jid &jid, PsiAccount *pa)
 
 	QVBoxLayout *vb1 = new QVBoxLayout(this, 8);
 	d->lv = new HistoryView(this);
-	d->lv->setVScrollBarMode(QScrollView::AlwaysOn);
+	d->lv->setVScrollBarMode(Q3ScrollView::AlwaysOn);
 	connect(d->lv, SIGNAL(aOpenEvent(PsiEvent *)), SLOT(actionOpenEvent(PsiEvent *)));
 	QSizePolicy sp = d->lv->sizePolicy();
 	sp.setVerStretch(1);
@@ -220,7 +231,7 @@ HistoryDlg::~HistoryDlg()
 
 void HistoryDlg::keyPressEvent(QKeyEvent *e)
 {
-	if(e->key() == Key_Escape)
+	if(e->key() == Qt::Key_Escape)
 		close();
 	else {
 		e->ignore();
@@ -275,40 +286,24 @@ void HistoryDlg::doSave()
 	if(option.lastSavePath.isEmpty())
 		option.lastSavePath = QDir::homeDirPath();
 
-	UserListItem *u = d->pa->findFirstRelavent(d->jid);
-	QString them = jidnick(u->jid().full(), u->name());
-	QString s = qstrlower(jidEncode(them));
+	UserListItem *u = d->pa->findFirstRelevant(d->jid);
+	QString them = JIDUtil::nickOrJid(u->name(), u->jid().full());
+	QString s = JIDUtil::encode(them).toLower();
 
 	QString str = option.lastSavePath + "/" + s + ".txt";
-	while(1) {
-		str = QFileDialog::getSaveFileName(str, tr("Text files (*.txt);;All files (*.*)"), this, 0, tr("Export message history"));
-		if(str.isEmpty())
-			break;
-
+	str = QFileDialog::getSaveFileName(this, tr("Export message history"), str, tr("Text files (*.txt);;All files (*.*)"));
+	if(!str.isEmpty()) {
 		QFileInfo fi(str);
-		if(fi.exists()) {
-			int x = QMessageBox::information(this, tr("Confirm overwrite"), tr("File already exists, overwrite?"), tr("&Yes"), tr("&No"));
-			if(x != 0)
-				continue;
-		}
-
 		option.lastSavePath = fi.dirPath();
 		exportHistory(str);
-		break;
 	}
 }
 
 void HistoryDlg::doErase()
 {
 	int x = QMessageBox::information(this, tr("Confirm erase all"), tr("This will erase all message history for this contact!\nAre you sure you want to do this?"), tr("&Yes"), tr("&No"), QString::null, 1);
-	if(x == 0) {
-		QString fname = getHistoryDir() + "/" + qstrlower(jidEncode(d->jid.userHost())) + ".history";
-		QFileInfo fi(fname);
-		if(fi.exists()) {
-			QDir dir = fi.dir();
-			dir.remove(fi.fileName());
-		}
-		d->lv->clear();
+	if (x == 0) {
+		d->h->erase(d->jid);
 	}
 }
 
@@ -338,7 +333,7 @@ void HistoryDlg::displayResult(const EDBResult *r, int direction, int max)
 {
 	//d->lv->setUpdatesEnabled(false);
 	d->lv->clear();
-	QPtrListIterator<EDBItem> it(*r);
+	Q3PtrListIterator<EDBItem> it(*r);
 	if(direction == EDB::Forward)
 		it.toLast();
 	int at = 0;
@@ -377,10 +372,10 @@ printf("\n");*/
 void HistoryDlg::edb_finished()
 {
 	const EDBResult *r = d->h->result();
-	if(r) {
+	if(d->h->lastRequestType() == EDBHandle::Read && r) {
 		//printf("EDB: retrieved %d events:\n", r->count());
 		if(r->count() > 0) {
-			QPtrListIterator<EDBItem> it(*r);
+			Q3PtrListIterator<EDBItem> it(*r);
 			if(d->reqtype == 0 || d->reqtype == 1) {
 				// events are in backward order
 				// first entry is the end event
@@ -422,6 +417,18 @@ void HistoryDlg::edb_finished()
 			}
 		}
 	}
+	else if (d->h->lastRequestType() == EDBHandle::Erase) {
+		if (d->h->writeSuccess()) {
+			d->lv->clear();
+			d->id_prev = "";
+			d->id_begin = "";
+			d->id_end = "";
+			d->id_next = "";
+		}
+		else {
+			QMessageBox::critical(this, tr("Error"), tr("Unable to delete history file."));
+		}
+	}
 	else {
 		//printf("EDB: error\n");
 	}
@@ -466,15 +473,15 @@ void HistoryDlg::doFind()
 void HistoryDlg::exportHistory(const QString &fname)
 {
 	QFile f(fname);
-	if(!f.open(IO_WriteOnly | IO_Truncate | IO_Translate)) {
+	if(!f.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Text)) {
 		QMessageBox::information(this, tr("Error"), tr("Error writing to file."));
 		return;
 	}
 	QTextStream stream(&f);
 
 	QString us = d->pa->nick();
-	UserListItem *u = d->pa->findFirstRelavent(d->jid);
-	QString them = jidnick(u->jid().full(), u->name());
+	UserListItem *u = d->pa->findFirstRelevant(d->jid);
+	QString them = JIDUtil::nickOrJid(u->name(), u->jid().full());
 
 	d->exp = new EDBHandle(d->pa->edb());
 	QString id;
@@ -493,7 +500,7 @@ void HistoryDlg::exportHistory(const QString &fname)
 			break;
 
 		// events are in forward order
-		QPtrListIterator<EDBItem> it(*r);
+		Q3PtrListIterator<EDBItem> it(*r);
 		for(EDBItem *i; (i = it.current()); ++it) {
 			id = it.current()->nextId();
 			PsiEvent *e = i->event();
@@ -502,7 +509,7 @@ void HistoryDlg::exportHistory(const QString &fname)
 			QDateTime dt = e->timeStamp();
 			QString ts;
 			//ts.sprintf("%04d/%02d/%02d %02d:%02d:%02d", dt.date().year(), dt.date().month(), dt.date().day(), dt.time().hour(), dt.time().minute(), dt.time().second());
-			ts = dt.toString(LocalDate);
+			ts = dt.toString(Qt::LocalDate);
 
 			QString nick;
 			if(e->originLocal())
@@ -541,11 +548,11 @@ void HistoryDlg::exportHistory(const QString &fname)
 // HistoryView
 //----------------------------------------------------------------------------
 HistoryView::HistoryView(QWidget *parent, const char *name)
-:QListView(parent, name)
+:Q3ListView(parent, name)
 {
 	at_id = 0;
-	connect(this, SIGNAL(doubleClicked(QListViewItem *)), SLOT(qlv_doubleclick(QListViewItem *)));
-	connect(this, SIGNAL(rightButtonPressed(QListViewItem *, const QPoint &, int)), SLOT(qlv_contextPopup(QListViewItem *, const QPoint &, int)));
+	connect(this, SIGNAL(doubleClicked(Q3ListViewItem *)), SLOT(qlv_doubleclick(Q3ListViewItem *)));
+	connect(this, SIGNAL(rightButtonPressed(Q3ListViewItem *, const QPoint &, int)), SLOT(qlv_contextPopup(Q3ListViewItem *, const QPoint &, int)));
 
 	setAllColumnsShowFocus(true);
 	addColumn(tr("Type"));
@@ -553,7 +560,7 @@ HistoryView::HistoryView(QWidget *parent, const char *name)
 	addColumn(tr("Date"));
 	addColumn(tr("Text"));
 	setSorting(2);
-	setResizeMode(QListView::LastColumn);
+	setResizeMode(Q3ListView::LastColumn);
 	setShowToolTips(false);
 	header()->setClickEnabled(false);
 	header()->setMovingEnabled(false);
@@ -562,7 +569,7 @@ HistoryView::HistoryView(QWidget *parent, const char *name)
 
 void HistoryView::resizeEvent(QResizeEvent *e)
 {
-	QListView::resizeEvent(e);
+	Q3ListView::resizeEvent(e);
 
 	if(e->oldSize().width() != e->size().width())
 		doResize();
@@ -570,15 +577,15 @@ void HistoryView::resizeEvent(QResizeEvent *e)
 
 void HistoryView::keyPressEvent(QKeyEvent *e)
 {
-	if(e->key() == Key_Enter || e->key() == Key_Return)
+	if(e->key() == Qt::Key_Enter || e->key() == Qt::Key_Return)
 		doOpenEvent();
 	else
-		QListView::keyPressEvent(e);
+		Q3ListView::keyPressEvent(e);
 }
 
 void HistoryView::doResize()
 {
-	QListViewItemIterator it(this);
+	Q3ListViewItemIterator it(this);
 	HistoryViewItem *item;
 	for(; it.current() ; ++it) {
 		item = (HistoryViewItem *)it.current();
@@ -599,7 +606,7 @@ void HistoryView::doOpenEvent()
 	aOpenEvent(i->e);
 }
 
-void HistoryView::qlv_doubleclick(QListViewItem *xi)
+void HistoryView::qlv_doubleclick(Q3ListViewItem *xi)
 {
 	HistoryViewItem *i = (HistoryViewItem *)xi;
 
@@ -607,13 +614,13 @@ void HistoryView::qlv_doubleclick(QListViewItem *xi)
 	doOpenEvent();
 }
 
-void HistoryView::qlv_contextPopup(QListViewItem *ix, const QPoint &pos, int)
+void HistoryView::qlv_contextPopup(Q3ListViewItem *ix, const QPoint &pos, int)
 {
 	HistoryViewItem *i = (HistoryViewItem *)ix;
 	if(!i)
 		return;
 
-	QPopupMenu popup;
+	Q3PopupMenu popup;
 	popup.insertItem(tr("Open"), 1);
 	popup.insertSeparator();
 	popup.insertItem(tr("Copy"), 2);
@@ -641,8 +648,8 @@ void HistoryView::qlv_contextPopup(QListViewItem *ix, const QPoint &pos, int)
 //----------------------------------------------------------------------------
 // HistoryViewItem
 //----------------------------------------------------------------------------
-HistoryViewItem::HistoryViewItem(PsiEvent *_e, const QString &eid, int xid, QListView *parent)
-:QListViewItem(parent)
+HistoryViewItem::HistoryViewItem(PsiEvent *_e, const QString &eid, int xid, Q3ListView *parent)
+:Q3ListViewItem(parent)
 {
 	rt = 0;
 	id = xid;
@@ -657,23 +664,24 @@ HistoryViewItem::HistoryViewItem(PsiEvent *_e, const QString &eid, int xid, QLis
 		e = new AuthEvent(*ae);
 	}
 
-	Icon *a = is->event2icon(e);
+	PsiIcon *a = PsiIconset::instance()->event2icon(e);
 	if(e->type() == PsiEvent::Message) {
 		MessageEvent *me = (MessageEvent *)e;
 		const Message &m = me->message();
-		text = plain2rich(m.body());
+		text = TextUtil::plain2rich(m.body());
 
 		if(!m.urlList().isEmpty())
-			setPixmap(0, IconsetFactory::icon("psi/www"));
+			setPixmap(0, IconsetFactory::icon("psi/www").impix());
 		else if(e->originLocal())
-			setPixmap(0, IconsetFactory::icon("psi/sendMessage"));
-		else
+			setPixmap(0, IconsetFactory::icon("psi/sendMessage").impix());
+		else if(a)
 			setPixmap(0, a->impix());
 	}
 	else if(e->type() == PsiEvent::Auth) {
 		AuthEvent *ae = (AuthEvent *)e;
 		text = ae->authType();
-		setPixmap(0, a->impix());
+		if (a)
+			setPixmap(0, a->impix());
 	}
 
 	if(e->originLocal())
@@ -690,11 +698,11 @@ HistoryViewItem::HistoryViewItem(PsiEvent *_e, const QString &eid, int xid, QLis
 		ts.time().hour(),
 		ts.time().minute(),
 		ts.time().second());*/
-	date = ts.toString(LocalDate);
+	date = ts.toString(Qt::LocalDate);
 
 	setText(2, date);
 
-	rt = new QSimpleRichText(text, listView()->font());
+	rt = new Q3SimpleRichText(text, listView()->font());
 }
 
 HistoryViewItem::~HistoryViewItem()
@@ -708,7 +716,7 @@ void HistoryViewItem::setup()
 {
 	widthChanged();
 
-	QListView *lv = listView();
+	Q3ListView *lv = listView();
 
 	if(rt) {
 		int w = lv->columnWidth(3);
@@ -751,19 +759,19 @@ void HistoryViewItem::paintCell(QPainter *p, const QColorGroup & cg, int column,
 
 		int h = height();
 		if(rt) {
-			QSimpleRichText tmp(QString("<qt><font color=\"%1\">" + text + "</font></qt>").arg(mycg.text().name()), listView()->font());
+			Q3SimpleRichText tmp(QString("<qt><font color=\"%1\">" + text + "</font></qt>").arg(mycg.text().name()), listView()->font());
 			tmp.setWidth(rt->width());
 			tmp.draw(p, 0, 0, QRect(0, 0, width, h), mycg, &br);
 		}
 	}
 	else {
-		alignment = AlignTop;
+		alignment = Qt::AlignTop;
 
-		QListViewItem::paintCell(p, mycg, column, width, alignment);
+		Q3ListViewItem::paintCell(p, mycg, column, width, alignment);
 	}
 }
 
-int HistoryViewItem::compare(QListViewItem *xi, int, bool) const
+int HistoryViewItem::compare(Q3ListViewItem *xi, int, bool) const
 {
 	HistoryViewItem *i = (HistoryViewItem *)xi;
 	return id - i->id;
