@@ -18,49 +18,65 @@
  *
  */
 
-#include"mainwin.h"
+#include "mainwin.h"
 
-#include<qmessagebox.h>
-#include<qiconset.h>
-#include<qtooltip.h>
-#include<qstyle.h>
-#include<qapplication.h>
-#include<qptrlist.h>
-#include<qtimer.h>
-#include<qobjectlist.h>
-#include<qpainter.h>
-#include<qsignalmapper.h>
-#include<qstatusbar.h>
-#include<qmenubar.h>
-#include"im.h"
-#include"common.h"
-#include"showtextdlg.h"
-#include"psicon.h"
-#include"psiaccount.h"
-#include"psitoolbar.h"
-#include"ui_about.h"
-#include"ui_tip.h"
-#include"fancylabel.h"
-#include"psitoolbar.h"
-#include"psipopup.h"
+#include <QDesktopServices>
+#include <qmessagebox.h>
+#include <qicon.h>
+#include <qapplication.h>
+#include <qtimer.h>
+#include <qobject.h>
+#include <qpainter.h>
+#include <qsignalmapper.h>
+#include <qmenubar.h>
+#include <QPixmap>
+#include <QCloseEvent>
+#include <QKeyEvent>
+#include <QEvent>
+#include <QVBoxLayout>
+#include <QMenu>
+#include <QMenuItem>
 
-#include"mainwin_p.h"
+#ifdef Q_WS_WIN
+#include <windows.h>
+#endif
+
+#include "common.h"
+#include "showtextdlg.h"
+#include "psicon.h"
+#include "contactview.h"
+#include "psiiconset.h"
+#include "serverinfomanager.h"
+#include "applicationinfo.h"
+#include "psiaccount.h"
+#include "psitrayicon.h"
+#include "psitoolbar.h"
+#include "aboutdlg.h"
+#include "psitoolbar.h"
+#include "psipopup.h"
+#include "psioptions.h"
+#include "tipdlg.h"
+#include "mucjoindlg.h"
+#include "psicontactlist.h"
+
+#include "mainwin_p.h"
 
 using namespace XMPP;
 
 // deletes submenus in a popupmenu
-void qpopupmenuclear(QPopupMenu *p)
+/*void qpopupmenuclear(QMenu *p)
 {
 	while(p->count()) {
 		QMenuItem *item = p->findItem(p->idAt(0));
-		QPopupMenu *popup = item->popup();
+		QMenu *popup = item->menu();
 		p->removeItemAt(0);
 
 		if(popup)
 			delete popup;
 	}
-}
+}*/
 
+	
 //----------------------------------------------------------------------------
 // MainWin::Private
 //----------------------------------------------------------------------------
@@ -73,11 +89,11 @@ public:
 
 	QVBoxLayout *vb_main;
 	bool onTop, asTool;
-	QPopupMenu *mainMenu, *statusMenu, *optionsMenu, *toolsMenu;
+	QMenu *mainMenu, *statusMenu, *optionsMenu, *toolsMenu;
 	int sbState;
 	QString nickname;
-	MTray *tray;
-	QPopupMenu *trayMenu;
+	PsiTrayIcon *tray;
+	QMenu *trayMenu;
 	QString statusTip;
 
 	PopupAction *optionsButton, *statusButton;
@@ -88,18 +104,17 @@ public:
 
 	QSignalMapper *statusMapper;
 
-	Icon *nextAnim;
+	PsiIcon *nextAnim;
 	int nextAmount;
 
 	QMap<QAction *, int> statusActions;
 
 	int lastStatus;
-	QMenuBar *gm;
-
-	QString infoString;
+	bool old_trayicon;
 
 	void registerActions();
 	IconAction *getAction( QString name );
+	void updateMenu(QStringList actions, QMenu *menu);
 };
 
 MainWin::Private::Private(PsiCon *_psi, MainWin *_mainWin)
@@ -173,6 +188,24 @@ IconAction *MainWin::Private::getAction( QString name )
 	return action;
 }
 
+void MainWin::Private::updateMenu(QStringList actions, QMenu *menu)
+{
+	menu->clear();
+
+	IconAction *action;
+	foreach (QString name, actions) {
+		// workind around Qt/X11 bug, which displays
+		// actions's text and the separator bar in Qt 4.1.1
+		if ( name == "separator" ) {
+			menu->insertSeparator();
+			continue;
+		}
+		
+		if ( (action = getAction(name)) )
+			action->addTo(menu);
+	}
+}
+
 //----------------------------------------------------------------------------
 // MainWin
 //----------------------------------------------------------------------------
@@ -180,15 +213,27 @@ IconAction *MainWin::Private::getAction( QString name )
 //#ifdef Q_WS_X11
 //#define TOOLW_FLAGS WStyle_Customize
 //#else
-#define TOOLW_FLAGS 0
+//#define TOOLW_FLAGS ((Qt::WFlags) 0)
 //#endif
 
+#ifdef Q_WS_WIN
+#define TOOLW_FLAGS (Qt::WindowMinimizeButtonHint)
+#else
+#define TOOLW_FLAGS ((Qt::WFlags) 0)
+#endif
+
 MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
-:AdvancedWidget<QMainWindow>(0, name, 0 | (_onTop ? WStyle_StaysOnTop: 0) | (_asTool ? WStyle_Tool | TOOLW_FLAGS : 0))
+:AdvancedWidget<Q3MainWindow>(0, (_onTop ? Qt::WStyle_StaysOnTop : Qt::Widget) | (_asTool ? (Qt::WStyle_Tool |TOOLW_FLAGS) : Qt::Widget))
+//: Q3MainWindow(0,name,(_onTop ? Qt::WStyle_StaysOnTop : Qt::Widget) | (_asTool ? (Qt::WStyle_Tool |TOOLW_FLAGS) : Qt::Widget))
 {
+	setObjectName(name);
+	setAttribute(Qt::WA_AlwaysShowToolTips);
+  	if ( option.brushedMetal ) {
+		setAttribute(Qt::WA_MacMetalStyle);
+	}
 	d = new Private(psi, this);
 
-	setIcon(is->status(STATUS_OFFLINE));
+	setWindowIcon(PsiIconset::instance()->status(STATUS_OFFLINE).impix());
 
 	d->onTop = _onTop;
 	d->asTool = _asTool;
@@ -204,30 +249,51 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 	d->tray = 0;
 	d->trayMenu = 0;
 	d->statusTip = "";
-	d->infoString = "";
 	d->nickname = "";
+#ifdef Q_WS_MAC
+	d->old_trayicon = false;
+#else
+	d->old_trayicon = PsiOptions::instance()->getOption("options.ui.systemtray.use-old").toBool();
+#endif
 
 	QWidget *center = new QWidget (this, "Central widget");
 	setCentralWidget ( center );
 
-	d->vb_main = new QVBoxLayout(center, 2);
+	d->vb_main = new QVBoxLayout(center);
 	cvlist = new ContactView(center);
-	d->vb_main->addWidget(cvlist, 1);
+
+	int layoutMargin = 2;
 #ifdef Q_WS_MAC
-	// If the space isn't there, MacOS draws an empty vertical scrollbar :(
-	d->vb_main->addSpacing(1);
+	layoutMargin = 0;
+	cvlist->setFrameShape(QFrame::NoFrame);
+#endif
+	d->vb_main->setMargin(layoutMargin);
+	d->vb_main->setSpacing(layoutMargin);
+
+	d->vb_main->addWidget(cvlist);
+
+#ifdef Q_WS_MAC
+	// Disable the empty vertical scrollbar:
+	// it's here because of weird code in q3scrollview.cpp
+	// Q3ScrollView::updateScrollBars() around line 877
+	d->vb_main->addSpacing(4);
+#endif
+
+	d->statusMenu = new QMenu(this);
+	d->optionsMenu = new QMenu(this);
+#ifdef Q_WS_MAC
+	d->trayMenu = d->statusMenu;
+#else
+	d->trayMenu = new QMenu(this);
+	buildTrayMenu();
+	connect(d->trayMenu, SIGNAL(aboutToShow()), SLOT(buildTrayMenu()));
 #endif
 
-	d->statusMenu = new QPopupMenu(this);
-	d->optionsMenu = new QPopupMenu(this);
 
 	buildStatusMenu();
 	buildOptionsMenu();
-	connect(d->statusMenu, SIGNAL(aboutToShow()), SLOT(buildStatusMenu()));
 	connect(d->optionsMenu, SIGNAL(aboutToShow()), SLOT(buildOptionsMenu()));
 
-	d->optionsButton->setPopup( d->optionsMenu );
-	d->statusButton->setPopup( d->statusMenu );
 
 	X11WM_CLASS("main");
 
@@ -237,77 +303,73 @@ MainWin::MainWin(bool _onTop, bool _asTool, PsiCon *psi, const char *name)
 	updateCaption();
 
 	d->registerActions();
-	buildToolbars();
+	
+	connect(d->psi->contactList(), SIGNAL(accountFeaturesChanged()), SLOT(accountFeaturesChanged()));
+	accountFeaturesChanged();
 
 	decorateButton(STATUS_OFFLINE);
 
-	// show tip of the day
-	if ( option.showTips )
-		actTipActivated();
-
-#ifdef Q_WS_MAC
-	// Create a global menubar
-	d->gm = new QMenuBar(0);
-#else
-	// Create a menu for the current window
-	d->gm = new QMenuBar(this);
-#endif
-
 	// Mac-only menus
 #ifdef Q_WS_MAC
-	QPopupMenu *mainMenu = new QPopupMenu(this);
-	d->gm->insertItem(tr("Menu"), mainMenu);
+	QMenu *mainMenu = new QMenu(this);
+	mainMenuBar()->insertItem(tr("Menu"), mainMenu);
 	mainMenu->insertItem(tr("Preferences"), this, SIGNAL(doOptions()));
 	mainMenu->insertItem(tr("Quit"), this, SLOT(try2tryCloseProgram()));
 	d->getAction("help_about")->addTo(mainMenu);
 	d->getAction("help_about_qt")->addTo(mainMenu);
 
-	d->mainMenu = new QPopupMenu(this);
-	d->gm->insertItem(tr("General"), d->mainMenu);
+	d->mainMenu = new QMenu(this);
+	mainMenuBar()->insertItem(tr("General"), d->mainMenu);
 	connect(d->mainMenu, SIGNAL(aboutToShow()), SLOT(buildMainMenu()));
 #else
-	d->gm->insertItem(tr("General"), d->optionsMenu);
+	mainMenuBar()->insertItem(tr("General"), d->optionsMenu);
 #endif
 
-	d->gm->insertItem(tr("Status"), d->statusMenu);
+	mainMenuBar()->insertItem(tr("Status"), d->statusMenu);
 
-	QPopupMenu *viewMenu = new QPopupMenu(this);
-	d->gm->insertItem(tr("View"), viewMenu);
+	QMenu *viewMenu = new QMenu(this);
+	mainMenuBar()->insertItem(tr("View"), viewMenu);
 	d->getAction("show_offline")->addTo(viewMenu);
-	d->getAction("show_away")->addTo(viewMenu);
+	if (PsiOptions::instance()->getOption("options.ui.menu.view.show-away").toBool())
+		d->getAction("show_away")->addTo(viewMenu);
 	d->getAction("show_hidden")->addTo(viewMenu);
 	d->getAction("show_agents")->addTo(viewMenu);
 	d->getAction("show_self")->addTo(viewMenu);
+	viewMenu->insertSeparator();
+	d->getAction("show_statusmsg")->addTo(viewMenu);
 
 	// Mac-only menus
 #ifdef Q_WS_MAC
-	d->toolsMenu = new QPopupMenu(this);
-	d->gm->insertItem(tr("Tools"), d->toolsMenu);
+	d->toolsMenu = new QMenu(this);
+	mainMenuBar()->insertItem(tr("Tools"), d->toolsMenu);
 	connect(d->toolsMenu, SIGNAL(aboutToShow()), SLOT(buildToolsMenu()));
 
-	QPopupMenu *helpMenu = new QPopupMenu(this);
-	d->gm->insertItem(tr("Help"), helpMenu);
+	QMenu *helpMenu = new QMenu(this);
+	mainMenuBar()->insertItem(tr("Help"), helpMenu);
 	d->getAction("help_readme")->addTo (helpMenu);
 	d->getAction("help_tip")->addTo (helpMenu);
 	helpMenu->insertSeparator();
 	d->getAction("help_online_help")->addTo (helpMenu);
 	d->getAction("help_online_wiki")->addTo (helpMenu);
 	d->getAction("help_online_home")->addTo (helpMenu);
+	d->getAction("help_psi_muc")->addTo (helpMenu);
 	d->getAction("help_report_bug")->addTo (helpMenu);
-
-	d->gm->show();
 #else
 	if (option.hideMenubar) 
-		d->gm->hide();
-	else 
-		d->gm->show();
+		mainMenuBar()->hide();
+	//else 
+	//	mainMenuBar()->show();
 #endif
+	d->optionsButton->setMenu( d->optionsMenu );
+	d->statusButton->setMenu( d->statusMenu );
 	
-#if QT_VERSION >= 0x030300
-	setWindowOpacity(double(option.rosterOpacity)/100);
-#endif
+	buildToolbars();
+	
+	setWindowOpacity(double(qMax(MINIMUM_OPACITY,PsiOptions::instance()->getOption("options.ui.contactlist.opacity").toInt()))/100);
 
 	connect(qApp, SIGNAL(dockActivated()), SLOT(dockActivated()));
+
+	connect(PsiOptions::instance(), SIGNAL(optionChanged(const QString&)), SLOT(optionsUpdate()));
 }
 	
 
@@ -315,14 +377,9 @@ MainWin::~MainWin()
 {
 	PsiPopup::deleteAll();
 
-	if (d->gm)
-		delete d->gm;
-
 	if(d->tray) {
 		delete d->tray;
 		d->tray = 0;
-		delete d->trayMenu;
-		d->trayMenu = 0;
 	}
 
 	//saveToolbarsPositions();
@@ -337,7 +394,7 @@ void MainWin::registerAction( IconAction *action )
 {
 	char activated[] = SIGNAL( activated() );
 	char toggled[]   = SIGNAL( toggled(bool) );
-	char setOn[]     = SLOT( setOn(bool) );
+	char setChecked[]     = SLOT( setChecked(bool) );
 
 	struct {
 		const char *name;
@@ -350,6 +407,7 @@ void MainWin::registerAction( IconAction *action )
 		{ "show_hidden",  toggled, cvlist, SLOT( setShowHidden(bool) ) },
 		{ "show_agents",  toggled, cvlist, SLOT( setShowAgents(bool) ) },
 		{ "show_self",    toggled, cvlist, SLOT( setShowSelf(bool) ) },
+		{ "show_statusmsg", toggled, cvlist, SLOT( setShowStatusMsg(bool) ) },
 
 		{ "button_options", activated, this, SIGNAL( doOptions() ) },
 
@@ -366,6 +424,9 @@ void MainWin::registerAction( IconAction *action )
 		{ "menu_change_profile", activated, this, SIGNAL( changeProfile() ) },
 		{ "menu_quit",           activated, this, SLOT( try2tryCloseProgram() ) },
 		{ "menu_play_sounds",    toggled,   this, SLOT( actPlaySoundsActivated(bool) ) },
+#ifdef USE_PEP
+		{ "publish_tune",        toggled,   this, SLOT( actPublishTuneActivated(bool) ) },
+#endif
 
 		{ "event_notifier", SIGNAL( clicked(int) ), this, SLOT( statusClicked(int) ) },
 		{ "event_notifier", activated, this, SLOT( doRecvNextEvent() ) },
@@ -375,6 +436,7 @@ void MainWin::registerAction( IconAction *action )
 		{ "help_online_help", activated, this, SLOT( actOnlineHelpActivated() ) },
 		{ "help_online_wiki", activated, this, SLOT( actOnlineWikiActivated() ) },
 		{ "help_online_home", activated, this, SLOT( actOnlineHomeActivated() ) },
+		{ "help_psi_muc",     activated, this, SLOT( actJoinPsiMUCActivated() ) },
 		{ "help_report_bug",  activated, this, SLOT( actBugReportActivated() ) },
 		{ "help_about",       activated, this, SLOT( actAboutActivated() ) },
 		{ "help_about_qt",    activated, this, SLOT( actAboutQtActivated() ) },
@@ -386,12 +448,18 @@ void MainWin::registerAction( IconAction *action )
 	QString aName;
 	for ( i = 0; !(aName = QString(actionlist[i].name)).isEmpty(); i++ ) {
 		if ( aName == action->name() ) {
+#ifdef USE_PEP
+			// Check before connecting, otherwise we get a loop
+			if ( aName == "publish_tune")
+				action->setChecked( PsiOptions::instance()->getOption("options.extended-presence.tune.publish").toBool() );
+#endif
+
 			disconnect( action, actionlist[i].signal, actionlist[i].receiver, actionlist[i].slot ); // for safety
 			connect( action, actionlist[i].signal, actionlist[i].receiver, actionlist[i].slot );
 
 			// special cases
 			if ( aName == "menu_play_sounds" )
-				action->setOn( useSound );
+				action->setChecked( useSound );
 			//else if ( aName == "foobar" )
 			//	;
 		}
@@ -403,11 +471,12 @@ void MainWin::registerAction( IconAction *action )
 		const char *signal;
 		const char *slot;
 	} reverseactionlist[] = {
-		{ "show_away",    cvlist, SIGNAL( showAway(bool) ), setOn },
-		{ "show_hidden",  cvlist, SIGNAL( showHidden(bool) ), setOn },
-		{ "show_offline", cvlist, SIGNAL( showOffline(bool) ), setOn },
-		{ "show_self",    cvlist, SIGNAL( showSelf(bool) ), setOn },
-		{ "show_agents",  cvlist, SIGNAL( showAgents(bool) ), setOn },
+		{ "show_away",    cvlist, SIGNAL( showAway(bool) ), setChecked },
+		{ "show_hidden",  cvlist, SIGNAL( showHidden(bool) ), setChecked },
+		{ "show_offline", cvlist, SIGNAL( showOffline(bool) ), setChecked },
+		{ "show_self",    cvlist, SIGNAL( showSelf(bool) ), setChecked },
+		{ "show_agents",  cvlist, SIGNAL( showAgents(bool) ), setChecked },
+		{ "show_statusmsg", cvlist, SIGNAL( showStatusMsg(bool) ), setChecked },
 		{ "", 0, 0, 0 }
 	};
 
@@ -416,7 +485,11 @@ void MainWin::registerAction( IconAction *action )
 			disconnect( reverseactionlist[i].sender, reverseactionlist[i].signal, action, reverseactionlist[i].slot ); // for safety
 			connect( reverseactionlist[i].sender, reverseactionlist[i].signal, action, reverseactionlist[i].slot );
 
-			action->setOn( true );
+			if (aName == "show_statusmsg") {
+				action->setChecked( PsiOptions::instance()->getOption("options.ui.contactlist.status-messages.show").toBool() );
+			}
+			else
+				action->setChecked( true );
 		}
 	}
 }
@@ -434,11 +507,11 @@ void MainWin::setWindowOpts(bool _onTop, bool _asTool)
 	d->onTop = _onTop;
 	d->asTool = _asTool;
 
-	WFlags flags = 0;
+	Qt::WFlags flags = 0;
 	if(d->onTop)
-		flags |= WStyle_StaysOnTop;
+		flags |= Qt::WStyle_StaysOnTop;
 	if(d->asTool)
-		flags |= WStyle_Tool | TOOLW_FLAGS;
+		flags |= Qt::WStyle_Tool | TOOLW_FLAGS;
 
 	QPoint p = pos();
 	reparent(parentWidget(), flags, p, FALSE);
@@ -448,12 +521,10 @@ void MainWin::setWindowOpts(bool _onTop, bool _asTool)
 
 void MainWin::setUseDock(bool use)
 {
-	if(use == false || (d->tray && option.isWMDock != d->tray->isWMDock())) {
+	if(use == false || d->tray) {
 		if(d->tray) {
 			delete d->tray;
 			d->tray = 0;
-			delete d->trayMenu;
-			d->trayMenu = 0;
 		}
 
 		if (use == false)
@@ -463,64 +534,73 @@ void MainWin::setUseDock(bool use)
 	if(d->tray)
 		return;
 
-	d->trayMenu = new QPopupMenu;
-	connect(d->trayMenu, SIGNAL(aboutToShow()), SLOT(buildTrayMenu()));
-
-	d->tray = new MTray("Psi", d->trayMenu);
-	d->tray->setIcon( is->statusPtr( STATUS_OFFLINE ));
-	d->tray->setToolTip(PROG_NAME);
+	d->tray = new PsiTrayIcon("Psi", d->trayMenu, d->old_trayicon);
+	if (d->old_trayicon) {
+		connect(d->tray, SIGNAL(closed()), SLOT(dockActivated()));
+		connect(qApp, SIGNAL(trayOwnerDied()), SLOT(dockActivated()));
+	}
 	connect(d->tray, SIGNAL(clicked(const QPoint &, int)), SLOT(trayClicked(const QPoint &, int)));
 	connect(d->tray, SIGNAL(doubleClicked(const QPoint &)), SLOT(trayDoubleClicked()));
-	connect(d->tray, SIGNAL(closed()), SLOT(dockActivated()));
-	connect(qApp, SIGNAL(trayOwnerDied()), SLOT(dockActivated()));
+	d->tray->setIcon( PsiIconset::instance()->statusPtr( STATUS_OFFLINE ));
+	d->tray->setToolTip(ApplicationInfo::name());
 
 	updateReadNext(d->nextAnim, d->nextAmount);
 
 	d->tray->show();
 }
 
-void MainWin::setInfo(const QString &str)
-{
-	d->infoString = str;
-
-	if(d->nextAmount == 0)
-		d->eventNotifier->setText(d->infoString);
-}
-
 void MainWin::buildStatusMenu()
 {
-	//statusMenu->clear();
-	qpopupmenuclear(d->statusMenu);
-
-	d->statusGroup->addTo(d->statusMenu);
+	d->statusMenu->clear();
+	d->getAction("status_online")->addTo(d->statusMenu);
+	if (PsiOptions::instance()->getOption("options.ui.menu.status.chat").toBool())
+		d->getAction("status_chat")->addTo(d->statusMenu);
+	d->statusMenu->insertSeparator();
+	d->getAction("status_away")->addTo(d->statusMenu);
+	if (PsiOptions::instance()->getOption("options.ui.menu.status.xa").toBool())
+		d->getAction("status_xa")->addTo(d->statusMenu);
+	d->getAction("status_dnd")->addTo(d->statusMenu);
+	if (PsiOptions::instance()->getOption("options.ui.menu.status.invisible").toBool()) {
+		d->statusMenu->insertSeparator();
+		d->getAction("status_invisible")->addTo(d->statusMenu);
+	}
+	d->statusMenu->insertSeparator();
+	d->getAction("status_offline")->addTo(d->statusMenu);
+#ifdef USE_PEP
+	d->statusMenu->insertSeparator();
+	d->getAction("publish_tune")->addTo(d->statusMenu);
+#endif
 }
 
 void MainWin::activatedStatusAction(int id)
 {
-	QObjectList *l = d->statusGroup->queryList( "IconAction" );
-	QObjectListIt it( *l );
-	QObject *obj;
-	for ( ; (obj = it.current()); ++it) {
-		IconAction *action = (IconAction *)obj;
-		action->setOn ( d->statusActions[action] == id );
+	QObjectList l = d->statusGroup->queryList( "IconAction" );
+	for (QObjectList::Iterator it = l.begin() ; it != l.end(); ++it) {
+		IconAction *action = (IconAction *)(*it);
+		action->setChecked ( d->statusActions[action] == id );
 	}
-	delete l;
 
 	statusChanged(id);
 }
 
+QMenuBar* MainWin::mainMenuBar() const
+{
+#ifdef Q_WS_MAC
+	return psiCon()->defaultMenuBar();
+#else
+	return menuBar();
+#endif
+}
+
 void MainWin::buildToolbars()
 {
-	bool dockBottom = false;
-
-	while ( option.toolbars.count() < toolbars.count() && toolbars.count() ) {
+	while ( option.toolbars["mainWin"].count() < toolbars.count() && toolbars.count() ) {
 		PsiToolBar *tb = toolbars.last();
 		toolbars.removeLast();
 		delete tb;
 	}
 
-	uint i;
-	for (i = 0; i < option.toolbars["mainWin"].count(); i++) {
+	for (int i = 0; i < option.toolbars["mainWin"].count(); i++) {
 		PsiToolBar *tb = 0;
 		if ( i < toolbars.count() )
 			tb = toolbars.at(i);
@@ -532,36 +612,23 @@ void MainWin::buildToolbars()
 		if ( tb )
 			delete tb;
 
-		tb = new PsiToolBar (tbPref.name, this, this);
+		tb = new PsiToolBar(tbPref.name, this, d->psi);
 		moveDockWindow ( tb, tbPref.dock, tbPref.nl, tbPref.index, tbPref. extraOffset );
 
 		tb->setGroup( "mainWin", i );
-		tb->setPsiCon( d->psi );
 		tb->setType( PsiActionList::Actions_MainWin );
 		//connect( tb, SIGNAL( registerAction( IconAction * ) ), SLOT( registerAction( IconAction * ) ) );
 		tb->initialize( tbPref, false );
 
-		// MacOS stuff
-		dockBottom = dockBottom || (tbPref.dock == Qt::DockBottom && tbPref.on);
-
 		if ( i < toolbars.count() )
-			toolbars.remove(i);
+			toolbars.removeAt(i);
 		toolbars.insert(i, tb);
 	}
-
-#ifdef Q_WS_MAC
-	if (dockBottom) {
-		statusBar()->show();
-	}
-	else {
-		statusBar()->hide();
-	}
-#endif
 }
 
 void MainWin::saveToolbarsPositions()
 {
-	for (uint i = 0; i < toolbars.count(); i++) {
+	for (int i = 0; i < toolbars.count(); i++) {
 		Options::ToolbarPrefs &tbPref = option.toolbars["mainWin"][i];
 		getLocation ( toolbars.at(i), tbPref.dock, tbPref.index, tbPref.nl, tbPref.extraOffset );
 		tbPref.on = toolbars.at(i)->isVisible();
@@ -575,157 +642,127 @@ bool MainWin::showDockMenu(const QPoint &)
 
 void MainWin::buildOptionsMenu()
 {
-	d->optionsMenu->clear();
-
 	// help menu
-	QPopupMenu *helpMenu = new QPopupMenu(d->optionsMenu);
+	QMenu *helpMenu = new QMenu(d->optionsMenu);
 
-	QStringList actionNames;
-	actionNames << "help_readme";
-	actionNames << "help_tip";
-	actionNames << "separator";
-	actionNames << "help_online_help";
-	actionNames << "help_online_wiki";
-	actionNames << "help_online_home";
-	actionNames << "help_report_bug";
-	actionNames << "separator";
-	actionNames << "help_about";
-	actionNames << "help_about_qt";
+	QStringList actions;
+	actions << "help_readme"
+	        << "help_tip"
+	        << "separator"
+	        << "help_online_help"
+	        << "help_online_wiki"
+	        << "help_online_home"
+	        << "help_psi_muc"
+	        << "help_report_bug"
+	        << "separator"
+	        << "help_about"
+	        << "help_about_qt";
 
-	QStringList::Iterator it;
-	IconAction *action;
-	for ( it = actionNames.begin(); it != actionNames.end(); ++it )
-		if ( (action = d->getAction(*it)) )
-			action->addTo( helpMenu );
+	d->updateMenu(actions, helpMenu);
 
 	buildGeneralMenu( d->optionsMenu );
 
 	d->optionsMenu->insertSeparator();
-	d->optionsMenu->insertItem(IconsetFactory::icon("psi/help"), tr("&Help"), helpMenu);
+	d->optionsMenu->insertItem(IconsetFactory::icon("psi/help").icon(), tr("&Help"), helpMenu);
 	d->getAction("menu_quit")->addTo( d->optionsMenu );
 
 }
 
 void MainWin::buildMainMenu()
 {
-	d->mainMenu->clear();
+	// main menu
+	QStringList actions;
+	actions << "menu_add_contact";
+	if (PsiOptions::instance()->getOption("options.ui.message.enabled").toBool())
+		actions << "menu_new_message";
+	actions << "menu_disco"
+	        << "menu_join_groupchat"
+	        << "separator"
+	        << "menu_account_setup";
+	if (PsiOptions::instance()->getOption("options.ui.menu.main.change-profile").toBool())
+	        actions << "menu_change_profile";
+	actions << "menu_play_sounds";
 
-	QStringList actionNames;
-
-	// options menu
-	if(!option.lockdown.roster)
-		actionNames << "menu_add_contact";
-	actionNames << "menu_new_message";
-	if(!option.lockdown.services)
-		actionNames << "menu_disco";
-	actionNames << "menu_join_groupchat";
-	actionNames << "";
-	if(!option.lockdown.accounts)
-		actionNames << "menu_account_setup";
-	if(!option.lockdown.profiles)
-		actionNames << "menu_change_profile";
-	actionNames << "menu_toolbars";
-	actionNames << "menu_play_sounds";
-
-	QStringList::Iterator it;
-	IconAction *action;
-	for ( it = actionNames.begin(); it != actionNames.end(); ++it ) {
-		if ((*it).isEmpty())
-			d->mainMenu->insertSeparator();
-		else if ( (action = d->getAction(*it)) )
-			action->addTo( d->mainMenu );
-	}
+	d->updateMenu(actions, d->mainMenu);
 }
 
 void MainWin::buildToolsMenu()
 {
-	IconAction* action;
-
-	d->toolsMenu->clear();
-	if ( (action = d->getAction("menu_file_transfer")) )
-		action->addTo(d->toolsMenu);
-	d->toolsMenu->insertSeparator();
-	if ( (action = d->getAction("menu_xml_console")) )
-		action->addTo(d->toolsMenu);
+	QStringList actions;
+	actions << "menu_file_transfer"
+	        << "separator"
+	        << "menu_xml_console";
+	
+	d->updateMenu(actions, d->toolsMenu);
 }
 	
-void MainWin::buildGeneralMenu(QPopupMenu *menu)
+void MainWin::buildGeneralMenu(QMenu *menu)
 {
-	menu->clear();
-
-	QStringList actionNames;
-
 	// options menu
-	if(!option.lockdown.roster)
-		actionNames << "menu_add_contact";
-	actionNames << "menu_new_message";
-	if(!option.lockdown.services)
-		actionNames << "menu_disco";
-	actionNames << "menu_join_groupchat";
-	if(!option.lockdown.accounts)
-		actionNames << "menu_account_setup";
-	if(!option.lockdown.options)
-		actionNames << "menu_options";
-	actionNames << "menu_file_transfer"; // TODO: probably we want to lock down filetransfer, right?
-	if(!option.lockdown.profiles)
-		actionNames << "menu_change_profile";
-	actionNames << "menu_toolbars";
-	actionNames << "menu_play_sounds";
+	QStringList actions;
+	actions << "menu_add_contact";
+	if (PsiOptions::instance()->getOption("options.ui.message.enabled").toBool())
+		actions << "menu_new_message";
+	actions << "menu_disco"
+	        << "menu_join_groupchat"
+	        << "menu_account_setup"
+	        << "menu_options"
+	        << "menu_file_transfer";
+	if (PsiOptions::instance()->getOption("options.ui.menu.main.change-profile").toBool())
+	        actions << "menu_change_profile";
+	actions << "menu_play_sounds";
 
-	QStringList::Iterator it;
-	IconAction *action;
-	for ( it = actionNames.begin(); it != actionNames.end(); ++it )
-		if ( (action = d->getAction(*it)) )
-			action->addTo( menu );
-}
-
-// WTF??
-void MainWin::testme()
-{
-	d->optionsMenu->setItemEnabled(10, false);
+	d->updateMenu(actions, menu);
 }
 
 void MainWin::actReadmeActivated ()
 {
-#ifdef Q_WS_WIN
-	ShowTextDlg *w = new ShowTextDlg(g.pathBase + "/readme.txt");
-#else
-	ShowTextDlg *w = new ShowTextDlg(g.pathBase + "/README");
-#endif
-	w->setCaption(CAP(tr("ReadMe")));
+	ShowTextDlg *w = new ShowTextDlg(":/README");
+	w->setWindowTitle(CAP(tr("ReadMe")));
 	w->show();
 }
 
 void MainWin::actOnlineHelpActivated ()
 {
-	openURL("http://psi-im.org/wiki/User_Guide");
+	QDesktopServices::openUrl(QUrl("http://psi-im.org/wiki/User_Guide"));
 }
 
 void MainWin::actOnlineWikiActivated ()
 {
-	openURL("http://psi-im.org/wiki");
+	QDesktopServices::openUrl(QUrl("http://psi-im.org/wiki"));
 }
 
 void MainWin::actOnlineHomeActivated ()
 {
-	openURL("http://psi-im.org");
+	QDesktopServices::openUrl(QUrl("http://psi-im.org"));
+}
+
+void MainWin::actJoinPsiMUCActivated()
+{
+	PsiAccount *account = d->psi->contactList()->defaultAccount();
+	if(!account)
+		return;
+
+	MUCJoinDlg *w = new MUCJoinDlg(d->psi, account);
+	w->le_host->setText("conference.psi-im.org");
+	w->le_room->setText("psi");
+	w->show();
 }
 
 void MainWin::actBugReportActivated ()
 {
-	openURL("http://psi-im.org/forum/forum/2");
+	QDesktopServices::openUrl(QUrl("http://psi-im.org/forum/forum/2"));
 }
 
 void MainWin::actAboutActivated ()
 {
-	AboutDlg *about = new AboutDlg (this, "AboutDlg", false, WDestructiveClose);
+	AboutDlg *about = new AboutDlg();
 	about->show();
 }
 
 void MainWin::actTipActivated ()
 {
-	TipUI *tip = new TipUI (this, "TipDlg", false, WDestructiveClose);
-	tip->show();
+	TipDlg::show(d->psi);
 }
 
 void MainWin::actAboutQtActivated ()
@@ -738,6 +775,11 @@ void MainWin::actPlaySoundsActivated (bool state)
 	useSound = state;
 }
 
+void MainWin::actPublishTuneActivated (bool state)
+{
+	PsiOptions::instance()->setOption("options.extended-presence.tune.publish",state);
+}
+
 void MainWin::activatedAccOption(PsiAccount *pa, int x)
 {
 	if(x == 0)
@@ -750,8 +792,7 @@ void MainWin::activatedAccOption(PsiAccount *pa, int x)
 
 void MainWin::buildTrayMenu()
 {
-	if(!d->trayMenu)
-		return;
+#ifndef Q_WS_MAC
 	d->trayMenu->clear();
 
 	if(d->nextAmount > 0) {
@@ -764,10 +805,12 @@ void MainWin::buildTrayMenu()
 	else
 		d->trayMenu->insertItem(tr("&Hide"), this, SLOT(trayHide()));
 	d->optionsButton->addTo(d->trayMenu);
-	d->statusButton->addTo(d->trayMenu);
+	d->trayMenu->insertItem(tr("Status"), d->statusMenu);
+	
 	d->trayMenu->insertSeparator();
 	// TODO!
 	d->getAction("menu_quit")->addTo(d->trayMenu);
+#endif
 }
 
 void MainWin::setTrayToolTip(int status)
@@ -780,36 +823,35 @@ void MainWin::setTrayToolTip(int status)
 void MainWin::decorateButton(int status)
 {
 	// update the 'change status' buttons
-	QObjectList *l = d->statusGroup->queryList( "IconAction" );
-	QObjectListIt it( *l );
-	QObject *obj;
-	for ( ; (obj = it.current()); ++it) {
-		IconAction *action = (IconAction *)obj;
-		action->setOn ( d->statusActions[action] == status );
+	QObjectList l = d->statusGroup->queryList( "IconAction" );
+	for (QObjectList::Iterator it = l.begin() ; it != l.end(); ++it) {
+		IconAction *action = (IconAction *)(*it);
+		action->setChecked ( d->statusActions[action] == status );
 	}
-	delete l;
 
 	if(d->lastStatus == status)
 		return;
 	d->lastStatus = status;
 
-	QIconSet icon;
-	icon.setPixmap(is->status(status), QIconSet::Small);
-
 	if(status == -1) {
 		d->statusButton->setText(tr("Connecting"));
-		if (option.alertStyle != 0)
+		if (option.alertStyle != 0) {
 			d->statusButton->setAlert(IconsetFactory::iconPtr("psi/connect"));
-		else
-			d->statusButton->setIcon(is->statusPtr(STATUS_OFFLINE));
+			d->statusGroup->setPsiIcon(IconsetFactory::iconPtr("psi/connect"));
+		}
+		else {
+			d->statusButton->setIcon(PsiIconset::instance()->statusPtr(STATUS_OFFLINE));
+			d->statusGroup->setPsiIcon(PsiIconset::instance()->statusPtr(STATUS_OFFLINE));
+		}
 
-		setIcon(is->status(STATUS_OFFLINE));
+		setWindowIcon(PsiIconset::instance()->status(STATUS_OFFLINE).impix());
 	}
 	else {
 		d->statusButton->setText(status2txt(status));
-		d->statusButton->setIcon(is->statusPtr(status));
+		d->statusButton->setIcon(PsiIconset::instance()->statusPtr(status));
+		d->statusGroup->setPsiIcon(PsiIconset::instance()->statusPtr(status));
 
-		setIcon(is->status(status));
+		setWindowIcon(PsiIconset::instance()->status(status).impix());
 	}
 
 	updateTray();
@@ -846,7 +888,9 @@ void MainWin::closeEvent(QCloseEvent *e)
 	if(!askQuit())
 		return;
 
-        closeProgram();
+	emit geomChanged(saveableGeometry());
+	closeProgram();
+
 	e->accept();
 #endif
 }
@@ -860,10 +904,10 @@ void MainWin::keyPressEvent(QKeyEvent *e)
 #endif
 
 	bool closekey = false;
-	if(e->key() == Key_Escape)
+	if(e->key() == Qt::Key_Escape)
 		closekey = true;
 #ifdef Q_WS_MAC
-	else if(e->key() == Key_W && e->state() & ControlButton)
+	else if(e->key() == Qt::Key_W && e->modifiers() & Qt::ControlModifier)
 		closekey = true;
 #endif
 
@@ -876,6 +920,20 @@ void MainWin::keyPressEvent(QKeyEvent *e)
 	QWidget::keyPressEvent(e);
 }
 
+#ifdef Q_WS_WIN
+#include <windows.h>
+bool MainWin::winEvent(MSG *msg, long *result)
+{
+	if (d->asTool && msg->message == WM_SYSCOMMAND && msg->wParam == SC_MINIMIZE) {
+		hide();	// minimized toolwindows look bad on Windows, so let's just hide it instead
+			// plus we cannot do this in changeEvent(), because it's called too late
+		*result = 0;
+		return true;	// don't let Qt process this event
+	}
+	return false;
+}
+#endif
+
 void MainWin::updateCaption()
 {
 	QString str = "";
@@ -884,14 +942,14 @@ void MainWin::updateCaption()
 		str += "* ";
 
 	if(d->nickname.isEmpty())
-		str += PROG_NAME;
+		str += ApplicationInfo::name();
 	else
 		str += d->nickname;
 
 	if(str == caption())
 		return;
 
-	setCaption(str);
+	setWindowTitle(str);
 }
 
 void MainWin::optionsUpdate()
@@ -900,15 +958,17 @@ void MainWin::optionsUpdate()
 	d->lastStatus = -2;
 	decorateButton(status);
 
+#ifndef Q_WS_MAC
 	if (option.hideMenubar) 
-		d->gm->hide();
+		mainMenuBar()->hide();
 	else 
-		d->gm->show();
-	
-#if QT_VERSION >= 0x030300
-	setWindowOpacity(double(option.rosterOpacity)/100);
+		mainMenuBar()->show();
 #endif
 	
+	setWindowOpacity(double(qMax(MINIMUM_OPACITY,PsiOptions::instance()->getOption("options.ui.contactlist.opacity").toInt()))/100);
+
+	buildStatusMenu();
+	
 	updateTray();
 }
 
@@ -920,7 +980,7 @@ void MainWin::toggleVisible()
 		trayShow();
 }
 
-void MainWin::setTrayToolTip(const Status &status)
+void MainWin::setTrayToolTip(const Status &status, bool)
 {
 	if (!d->tray)
 		return;
@@ -944,7 +1004,7 @@ void MainWin::trayClicked(const QPoint &, int button)
 	if(option.dockDCstyle)
 		return;
 
-	if(button == MidButton) {
+	if(button == Qt::MidButton) {
 		doRecvNextEvent();
 		return;
 	}
@@ -979,11 +1039,11 @@ void MainWin::trayShow()
 
 void MainWin::trayHide()
 {
-	emit geomChanged(x(), y(), width(), height());
+	emit geomChanged(saveableGeometry());
 	hide();
 }
 
-void MainWin::updateReadNext(Icon *anim, int amount)
+void MainWin::updateReadNext(PsiIcon *anim, int amount)
 {
 	d->nextAnim = anim;
 	if(anim == 0)
@@ -1029,7 +1089,10 @@ void MainWin::updateTray()
 	else if ( d->lastStatus == -1 )
 		d->tray->setAlert(IconsetFactory::iconPtr("psi/connect"));
 	else
-		d->tray->setIcon(is->statusPtr(d->lastStatus));
+		d->tray->setIcon(PsiIconset::instance()->statusPtr(d->lastStatus));
+	
+	buildTrayMenu();
+	d->tray->setContextMenu(d->trayMenu);
 }
 
 void MainWin::doRecvNextEvent()
@@ -1039,14 +1102,28 @@ void MainWin::doRecvNextEvent()
 
 void MainWin::statusClicked(int x)
 {
-	if(x == MidButton)
+	if(x == Qt::MidButton)
 		recvNextEvent();
 }
 
 void MainWin::numAccountsChanged()
 {
-	bool enabled = d->psi->accountList(TRUE).isEmpty() ? false : true;
-	d->statusButton->setEnabled (enabled);
+	d->statusButton->setEnabled(d->psi->contactList()->haveEnabledAccounts());
+}
+
+void MainWin::accountFeaturesChanged()
+{
+	bool have_pep = false;
+	foreach(PsiAccount* account, d->psi->contactList()->enabledAccounts()) {
+		if (account->serverInfoManager()->hasPEP()) {
+			have_pep = true;
+			break;
+		}
+	}
+
+#ifdef USE_PEP
+	d->getAction("publish_tune")->setEnabled(have_pep);
+#endif
 }
 
 void MainWin::dockActivated()
@@ -1057,18 +1134,19 @@ void MainWin::dockActivated()
 
 
 #ifdef Q_WS_MAC
-void MainWin::setIcon(const QPixmap&)
+void MainWin::setWindowIcon(const QPixmap&)
 {
 }
 #else
-void MainWin::setIcon(const QPixmap& p)
+void MainWin::setWindowIcon(const QPixmap& p)
 {
-	QMainWindow::setIcon(p);
+	Q3MainWindow::setWindowIcon(p);
 }
 #endif
 
+#if 0
 #if defined(Q_WS_WIN)
-#include<windows.h>
+#include <windows.h>
 void MainWin::showNoFocus()
 {
 	clearWState( WState_ForceHide );
@@ -1114,9 +1192,9 @@ void MainWin::showNoFocus()
 
 	setWState( WState_Visible );
 
-	if ( testWFlags(WStyle_Tool) || isPopup() ) {
+	if ( testWFlags(Qt::WStyle_Tool) || isPopup() ) {
 		raise();
-	} else if ( testWFlags(WType_TopLevel) ) {
+	} else if ( testWFlags(Qt::WType_TopLevel) ) {
 		while ( QApplication::activePopupWidget() )
 		QApplication::activePopupWidget()->close();
 	}
@@ -1148,10 +1226,12 @@ void MainWin::showNoFocus()
 }
 
 #else
+#endif
+#endif
 
 void MainWin::showNoFocus()
 {
 	bringToFront(this);
 }
 
-#endif
+//#endif
