@@ -21,26 +21,32 @@
 #ifndef MAINWIN_H
 #define MAINWIN_H
 
-#include<qmainwindow.h>
-#include<qmap.h>
-#include"im.h"
-#include"contactview.h"
-#include"psitoolbar.h"
-#include"advwidget.h"
+#include <Q3MainWindow>
+#include <QMap>
+#include <QList>
+#include <QString>
+#include <QStringList>
+
+#include "advwidget.h"
+
+class QMenuBar;
+class QAction;
+class QPixmap;
+class QPoint;
+class QMenu;
 
 class PsiCon;
+class PsiToolBar;
 class PsiAccount;
-class QIconSet;
-class IconLabel;
-
-class QAction;
-class QActionGroup;
 class IconAction;
-class QLabel;
-class QVBoxLayout;
-class QMenuBar;
+class PsiIcon;
+class ContactView;
+namespace XMPP {
+	class Status;
+}
 
-class MainWin : public AdvancedWidget<QMainWindow>
+class MainWin : public AdvancedWidget<Q3MainWindow>
+//class MainWin : public Q3MainWindow
 {
 	Q_OBJECT
 public:
@@ -49,14 +55,13 @@ public:
 
 	void setWindowOpts(bool onTop, bool asTool);
 	void setUseDock(bool);
-	void setInfo(const QString &);
 
 	// evil stuff! remove ASAP!!
 	QStringList actionList;
 	QMap<QString, QAction*> actions;
 
 	ContactView *cvlist;
-	QPtrList<PsiToolBar> toolbars;
+	QList<PsiToolBar*> toolbars;
 
 	void buildToolbars();
 	void saveToolbarsPositions();
@@ -65,6 +70,10 @@ public:
 protected:
 	void closeEvent(QCloseEvent *);
 	void keyPressEvent(QKeyEvent *);
+	QMenuBar* mainMenuBar() const;
+#ifdef Q_WS_WIN
+	bool winEvent(MSG *, long *);
+#endif
 
 signals:
 	void statusChanged(int);
@@ -79,7 +88,7 @@ signals:
 	void accountInfo();
 	void recvNextEvent();
 
-	void geomChanged(int x, int y, int w, int h);
+	void geomChanged(QRect saveableGeometry);
 
 private slots:
 	void buildStatusMenu();
@@ -103,17 +112,19 @@ private slots:
 	void tryCloseProgram();
 
 	void numAccountsChanged();
-	void testme();
+	void accountFeaturesChanged();
 	void activatedAccOption(PsiAccount *, int);
 
 	void actReadmeActivated ();
 	void actOnlineHelpActivated ();
 	void actOnlineWikiActivated ();
 	void actOnlineHomeActivated ();
+	void actJoinPsiMUCActivated();
 	void actBugReportActivated ();
 	void actAboutActivated ();
 	void actAboutQtActivated ();
 	void actPlaySoundsActivated (bool);
+	void actPublishTuneActivated (bool);
 	void actTipActivated();
 
 	bool showDockMenu(const QPoint &);
@@ -122,19 +133,19 @@ private slots:
 	void registerAction( IconAction * );
 
 public slots:
-	void setIcon(const QPixmap&);
+	void setWindowIcon(const QPixmap&);
 	void showNoFocus();
 
 	void decorateButton(int);
-	void updateReadNext(Icon *nextAnim, int nextAmount);
+	void updateReadNext(PsiIcon *nextAnim, int nextAmount);
 
 	void optionsUpdate();
-	void setTrayToolTip(const Status &);
+	void setTrayToolTip(const XMPP::Status &, bool usePriority = false);
 
 	void toggleVisible();
 
 private:
-	void buildGeneralMenu(QPopupMenu *);
+	void buildGeneralMenu(QMenu *);
 	QString numEventsString(int) const;
 
 	bool askQuit();
