@@ -23,14 +23,24 @@
 
 #include <qapplication.h>
 #include <qstyle.h>
-#include <qtoolbar.h>
+#include <q3toolbar.h>
 #include <qtimer.h>
 #include <qsignalmapper.h>
-#include <qobjectlist.h>
+#include <qobject.h>
 #include <qpixmapcache.h>
+#include <QPixmap>
+#include <Q3Button>
+#include <Q3PtrList>
+#include <QFrame>
+#include <QLabel>
+#include <QMenu>
+#include <QMouseEvent>
 #include "psiaccount.h"
+#include "stretchwidget.h"
 #include "iconwidget.h"
+#include "icontoolbutton.h"
 #include "alerticon.h"
+#include "psicontactlist.h"
 
 //----------------------------------------------------------------------------
 // PopupActionButton
@@ -43,25 +53,25 @@ public:
 	PopupActionButton(QWidget *parent = 0, const char *name = 0);
 	~PopupActionButton();
 
-	void setIcon(Icon *, bool showText);
+	void setIcon(PsiIcon *, bool showText);
+	void setLabel(QString);
+	QSize sizeHint() const;
 
 private slots:
-	void pixmapUpdated(const QPixmap &);
+	void pixmapUpdated();
 
 private:
-	void drawButtonLabel(QPainter *p);
+	void update();
+	void paintEvent(QPaintEvent *);
 	bool hasToolTip;
-	Icon *icon;
+	PsiIcon *icon;
 	bool showText;
+	QString label;
 };
 
 PopupActionButton::PopupActionButton(QWidget *parent, const char *name)
-: QPushButton(parent, name)
+: QPushButton(parent, name), hasToolTip(false), icon(0), showText(true)
 {
-	setWFlags(getWFlags() | WRepaintNoErase);
-
-	hasToolTip = false;
-	icon = 0;
 }
 
 PopupActionButton::~PopupActionButton()
@@ -70,7 +80,16 @@ PopupActionButton::~PopupActionButton()
 		icon->stop();
 }
 
-void PopupActionButton::setIcon(Icon *i, bool st)
+QSize PopupActionButton::sizeHint() const
+{
+	// hack to prevent QToolBar's extender button from displaying
+	if (sizePolicy().horizontalPolicy() == QSizePolicy::Expanding)
+		return QSize(16, 16);
+
+	return QPushButton::sizeHint();
+}
+
+void PopupActionButton::setIcon(PsiIcon *i, bool st)
 {
 	if ( icon ) {
 		icon->stop();
@@ -82,85 +101,106 @@ void PopupActionButton::setIcon(Icon *i, bool st)
 	showText = st;
 
 	if ( icon ) {
-		pixmapUpdated(icon->pixmap());
+		pixmapUpdated();
 
-		connect(icon, SIGNAL(pixmapChanged(const QPixmap &)), SLOT(pixmapUpdated(const QPixmap &)));
+		connect(icon, SIGNAL(pixmapChanged()), SLOT(pixmapUpdated()));
 		icon->activated();
 	}
 }
 
-void PopupActionButton::pixmapUpdated(const QPixmap &pix)
+void PopupActionButton::setLabel(QString lbl)
 {
-	if ( showText )
-		setIconSet(pix);
-	else
-		setPixmap(pix);
+	label = lbl;
+	update();
 }
 
-void PopupActionButton::drawButtonLabel(QPainter *p)
+void PopupActionButton::update()
+{
+	if ( showText )
+		QPushButton::setText(label);
+	else
+		QPushButton::setText("");
+}
+
+void PopupActionButton::pixmapUpdated()
+{
+	QPixmap pix = icon ? icon->pixmap() : QPixmap();
+	QPushButton::setIcon(pix);
+	QPushButton::setIconSize(pix.size());
+	update();
+}
+
+void PopupActionButton::paintEvent(QPaintEvent *p)
 {
-	// code by Justin Karneges :-)
 	// crazy code ahead!  watch out for potholes and deer.
 
 	// this gets us the width of the "text area" on the button.
 	// adapted from qt/src/styles/qcommonstyle.cpp and qt/src/widgets/qpushbutton.cpp
-	QRect r = style().subRect(QStyle::SR_PushButtonContents, this);
-	if(isMenuButton())
-		r.setWidth(r.width() - style().pixelMetric(QStyle::PM_MenuButtonIndicator, this));
-	if(iconSet() && !iconSet()->isNull())
-		r.setWidth(r.width() - (iconSet()->pixmap(QIconSet::Small, QIconSet::Normal, QIconSet::Off).width() + 4));
 
-	// font metrics
-	QFontMetrics fm(font());
+	if (showText) {
+		QStyleOptionButton style_option;
+		style_option.init(this);
+		QRect r = style()->subElementRect(QStyle::SE_PushButtonContents, &style_option, this);
 
-	// w1 = width of button text, w2 = width of text area
-	int w1 = fm.width(text());
-	int w2 = r.width();
+		if(isMenuButton())
+			r.setWidth(r.width() - style()->pixelMetric(QStyle::PM_MenuButtonIndicator, &style_option, this));
+		if(iconSet() && !iconSet()->isNull())
+			r.setWidth(r.width() - (iconSet()->pixmap(QIcon::Small, QIcon::Normal, QIcon::Off).width()));
 
-	// backup original text
-	QString oldtext = text();
+		// font metrics
+		QFontMetrics fm(font());
 
-	// button text larger than what will fit?
-	if(w1 > w2) {
-		if( !hasToolTip ) {
-			QToolTip::add(this, text());
-			hasToolTip = TRUE;
-		}
+		// w1 = width of button text, w2 = width of text area
+		int w1 = fm.width(label);
+		int w2 = r.width();
 
-		// make a string that fits
-		bool found = FALSE;
-		QString newtext;
-		int n;
-		for(n = oldtext.length(); n > 0; --n) {
-			if(fm.width(oldtext, n) < w2) {
-				found = TRUE;
-				break;
+		// backup original text
+		QString oldtext = label;
+
+		// button text larger than what will fit?
+		if(w1 > w2) {
+			if( !hasToolTip ) {
+				setToolTip(label);
+				hasToolTip = TRUE;
+			}
+
+			// make a string that fits
+			bool found = FALSE;
+			QString newtext;
+			int n;
+			for(n = oldtext.length(); n > 0; --n) {
+				if(fm.width(oldtext, n) < w2) {
+					found = TRUE;
+					break;
+				}
+			}
+			if(found)
+				newtext = oldtext.mid(0, n);
+			else
+				newtext = "";
+
+			// set the new text that fits.  updates must be off, or we recurse.
+			if (newtext != text()) {
+				setUpdatesEnabled(false);
+				QPushButton::setText(newtext);
+				setUpdatesEnabled(true);
 			}
 		}
-		if(found)
-			newtext = oldtext.mid(0, n);
-		else
-			newtext = "";
+		else {
+			if (oldtext != text()) {
+				setUpdatesEnabled(false);
+				QPushButton::setText(oldtext);
+				setUpdatesEnabled(true);
+			}
 
-		// set the new text that fits.  updates must be off, or we recurse.
-		setUpdatesEnabled(FALSE);
-		QButton::setText(newtext);
-		setUpdatesEnabled(TRUE);
-	}
-	else {
-		if( hasToolTip ) {
-			QToolTip::remove(this);
-			hasToolTip = FALSE;
+			if( hasToolTip ) {
+				setToolTip("");
+				hasToolTip = false;
+			}
 		}
 	}
 
-	// draw!
-	QPushButton::drawButtonLabel(p);
-
-	// restore original button text now that we are done drawing.
-	setUpdatesEnabled(FALSE);
-	QButton::setText(oldtext);
-	setUpdatesEnabled(TRUE);
+	QPushButton::paintEvent(p);
 }
 
 //----------------------------------------------------------------------------
@@ -171,8 +211,8 @@ class PopupAction::Private : public QObject
 {
 public:
 	QSizePolicy size;
-	QPtrList<PopupActionButton> buttons;
-	Icon *icon;
+	Q3PtrList<PopupActionButton> buttons;
+	PsiIcon *icon;
 	bool showText;
 
 	Private (QObject *parent)
@@ -190,11 +230,11 @@ public:
 	}
 };
 
-PopupAction::PopupAction (const QString &label, QPopupMenu *_menu, QObject *parent, const char *name)
+PopupAction::PopupAction (const QString &label, QMenu *_menu, QObject *parent, const char *name)
 : IconAction (label, label, 0, parent, name)
 {
 	d = new Private (this);
-	setPopup( _menu );
+	setMenu( _menu );
 }
 
 void PopupAction::setSizePolicy (const QSizePolicy &p)
@@ -202,14 +242,14 @@ void PopupAction::setSizePolicy (const QSizePolicy &p)
 	d->size = p;
 }
 
-void PopupAction::setAlert (const Icon *icon)
+void PopupAction::setAlert (const PsiIcon *icon)
 {
 	setIcon(icon, d->showText, true);
 }
 
-void PopupAction::setIcon (const Icon *icon, bool showText, bool alert)
+void PopupAction::setIcon (const PsiIcon *icon, bool showText, bool alert)
 {
-	Icon *oldIcon = 0;
+	PsiIcon *oldIcon = 0;
 	if ( d->icon ) {
 		oldIcon = d->icon;
 		d->icon = 0;
@@ -219,18 +259,18 @@ void PopupAction::setIcon (const Icon *icon, bool showText, bool alert)
 
 	if ( icon ) {
 		if ( !alert )
-			d->icon = new Icon(*icon);
+			d->icon = new PsiIcon(*icon);
 		else
 			d->icon = new AlertIcon(icon);
 
-		IconAction::setIconSet(*icon);
+		IconAction::setIcon(icon->icon());
 	}
 	else {
 		d->icon = 0;
-		IconAction::setIconSet(QIconSet());
+		IconAction::setIcon(QIcon());
 	}
 
-	for ( QPtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
+	for ( Q3PtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
 		PopupActionButton *btn = it.current();
 		btn->setIcon (d->icon, showText);
 	}
@@ -243,28 +283,20 @@ void PopupAction::setIcon (const Icon *icon, bool showText, bool alert)
 void PopupAction::setText (const QString &text)
 {
 	IconAction::setText (text);
-	for ( QPtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
+	for ( Q3PtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
 		PopupActionButton *btn = it.current();
-		btn->setText (text);
+		btn->setLabel (text);
 	}
 }
 
 bool PopupAction::addTo (QWidget *w)
 {
-	if ( w->inherits("QPopupMenu") ) {
-		QPopupMenu *m = (QPopupMenu*)w;
-#ifndef Q_WS_MAC
-		m->insertItem (iconSet(), menuText(), popup());
-#else
-		m->insertItem (QIconSet(), menuText(), popup());
-#endif
-	}
-	else if ( w->inherits("QToolBar") ) {
-		QCString bname = name() + QCString("_action_button");
-		PopupActionButton *btn = new PopupActionButton ( (QToolBar*)w, bname );
+	if ( w->inherits("QToolBar") || w->inherits("Q3ToolBar") ) {
+		QByteArray bname((const char*) (QString(name()) + QString("_action_button")));
+		PopupActionButton *btn = new PopupActionButton ( (Q3ToolBar*)w, bname );
 		d->buttons.append ( btn );
-		btn->setPopup ( popup() );
-		btn->setText ( text() );
+		btn->setMenu ( menu() );
+		btn->setLabel ( text() );
 		btn->setIcon ( d->icon, d->showText );
 		btn->setSizePolicy ( d->size );
 		btn->setEnabled ( isEnabled() );
@@ -272,7 +304,7 @@ bool PopupAction::addTo (QWidget *w)
 		connect( btn, SIGNAL( destroyed() ), SLOT( objectDestroyed() ) );
 	}
 	else
-		return false;
+		return IconAction::addTo(w);
 
 	return true;
 }
@@ -286,7 +318,7 @@ void PopupAction::objectDestroyed ()
 void PopupAction::setEnabled (bool e)
 {
 	IconAction::setEnabled (e);
-	for ( QPtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
+	for ( Q3PtrListIterator<PopupActionButton> it(d->buttons); it.current(); ++it ) {
 		PopupActionButton *btn = it.current();
 		btn->setEnabled (e);
 	}
@@ -294,7 +326,7 @@ void PopupAction::setEnabled (bool e)
 
 IconAction *PopupAction::copy() const
 {
-	PopupAction *act = new PopupAction(text(), popup(), 0, name());
+	PopupAction *act = new PopupAction(text(), menu(), 0, name());
 
 	*act = *this;
 
@@ -332,207 +364,12 @@ void MLabel::mouseReleaseEvent(QMouseEvent *e)
 
 void MLabel::mouseDoubleClickEvent(QMouseEvent *e)
 {
-	if(e->button() == LeftButton)
+	if(e->button() == Qt::LeftButton)
 		emit doubleClicked();
 
 	e->ignore();
 }
 
-//----------------------------------------------------------------------------
-// MTray
-//----------------------------------------------------------------------------
-
-class MTray::Private
-{
-public:
-	Private() {
-		icon	= 0;
-		ti	= 0;
-	}
-
-	~Private() {
-		if ( icon )
-			delete icon;
-
-		delete ti;
-	}
-
-	TrayIcon *ti;
-	Icon *icon;
-
-	QPixmap makeIcon();
-	QRgb pixelBlend(QRgb p1, QRgb p2);
-};
-
-MTray::MTray(const QString &tip, QPopupMenu *popup, QObject *parent)
-:QObject(parent)
-{
-	d = new Private;
-
-	d->ti = new TrayIcon(d->makeIcon(), tip, popup);
-	d->ti->setWMDock(option.isWMDock);
-	connect(d->ti, SIGNAL(clicked(const QPoint &, int)), SIGNAL(clicked(const QPoint &, int)));
-	connect(d->ti, SIGNAL(doubleClicked(const QPoint &)), SIGNAL(doubleClicked(const QPoint &)));
-	connect(d->ti, SIGNAL(closed()), SIGNAL(closed()));
-	connect(qApp, SIGNAL(newTrayOwner()), d->ti, SLOT(newTrayOwner()));
-	connect(qApp, SIGNAL(trayOwnerDied()), d->ti, SLOT(hide()));
-	d->ti->show();
-}
-
-MTray::~MTray()
-{
-	delete d;
-}
-
-void MTray::setToolTip(const QString &str)
-{
-	d->ti->setToolTip(str);
-}
-
-void MTray::setIcon(const Icon *icon, bool alert)
-{
-	if ( d->icon ) {
-		disconnect(d->icon, 0, this, 0 );
-		d->icon->stop();
-
-		delete d->icon;
-		d->icon = 0;
-	}
-
-	if ( icon ) {
-		if ( !alert )
-			d->icon = new Icon(*icon);
-		else
-			d->icon = new AlertIcon(icon);
-
-		connect(d->icon, SIGNAL(pixmapChanged(const QPixmap &)), SLOT(animate()));
-		d->icon->activated();
-	}
-	else
-		d->icon = new Icon();
-
-	animate();
-}
-
-void MTray::setAlert(const Icon *icon)
-{
-	setIcon(icon, true);
-}
-
-bool MTray::isAnimating() const
-{
-	return d->icon->isAnimated();
-}
-
-bool MTray::isWMDock()
-{
-	return d->ti->isWMDock();
-}
-
-void MTray::show()
-{
-	d->ti->show();
-}
-
-void MTray::hide()
-{
-	d->ti->hide();
-}
-
-
-// a function to blend 2 pixels taking their alpha channels
-// into consideration
-// p1 is in the 1st layer, p2 is in the 2nd layer (over p1)
-QRgb MTray::Private::pixelBlend(QRgb p1, QRgb p2)
-{
-	int a2 = qAlpha(p2);
-	if (a2 == 255) return p2; // don't calculate anything if p2 is completely opaque
-	int a1 = qAlpha(p1);
-	double prop1 = double(a1*(255-a2))/double(255*255);
-	double prop2 = double(a2)/255.0;
-	int r = int( qRed(p1)*prop1 + qRed(p2)*prop2 );
-	int g = int( qGreen(p1)*prop1 + qGreen(p2)*prop2 );
-	int b = int( qBlue(p1)*prop1 + qBlue(p2)*prop2 );
-	return qRgba(r, g, b, (a1>a2) ? a1:a2);
-}
-
-
-QPixmap MTray::Private::makeIcon()
-{
-	if ( !icon )
-		return QPixmap();
-
-#ifdef Q_WS_X11
-	// on X11, the KDE dock is 22x22.  let's make our icon "seem" bigger.
-	QImage real(22,22,32);
-	QImage in = icon->image();
-	in.detach();
-	real.setAlphaBuffer(true);
-
-	// make sure it is no bigger than 16x16
-	if(in.width() > 16 || in.height() > 16)
-		in = in.smoothScale(16,16);
-
-	int xo = (real.width() - in.width()) / 2;
-	int yo = (real.height() - in.height()) / 2;
-
-	int n, n2;
-
-	// clear the output and make it transparent
-	// deprecates real.fill(0)
-	for(n2 = 0; n2 < real.height(); ++n2)
-		for(n = 0; n < real.width(); ++n)
-			real.setPixel(n, n2, qRgba(0,0,0,0));
-
-	// draw a dropshadow
-	for(n2 = 0; n2 < in.height(); ++n2) {
-		for(n = 0; n < in.width(); ++n) {
-			if(int a = qAlpha(in.pixel(n,n2))) {
-				int x = n + xo + 2;
-				int y = n2 + yo + 2;
-				real.setPixel(x, y, qRgba(0x80,0x80,0x80,a));
-			}
-		}
-	}
-
-	// draw the image
-	for(n2 = 0; n2 < in.height(); ++n2) {
-		for(n = 0; n < in.width(); ++n) {
-			if(qAlpha(in.pixel(n,n2))) {
-				QRgb pold = real.pixel(n + xo , n2 + yo);
-				QRgb pnew = in.pixel(n , n2);
-				real.setPixel(n + xo, n2 + yo, pixelBlend(pold, pnew));
-			}
-		}
-	}
-
-	QPixmap pixmap;
-	pixmap.convertFromImage(real);
-	return pixmap;
-#else
-	return icon->pixmap();
-#endif
-}
-
-void MTray::animate()
-{
-#ifdef Q_WS_X11
-	if ( !d->icon )
-		return;
-
-	QString cachedName = "PsiTray/" + option.defaultRosterIconset + "/" + d->icon->name() + "/" + QString::number( d->icon->framenumber() );
-
-	QPixmap p;
-	if ( !QPixmapCache::find(cachedName, p) ) {
-		p = d->makeIcon();
-		QPixmapCache::insert( cachedName, p );
-	}
-
-	d->ti->setIcon(p);
-#else
-	d->ti->setIcon( d->makeIcon() );
-#endif
-}
 
 //----------------------------------------------------------------------------
 // MAction
@@ -553,21 +390,34 @@ public:
 		sm = new QSignalMapper(this, "MAction::Private::SignalMapper");
 	}
 
-	QPopupMenu *subMenu(QWidget *p)
+	QMenu *subMenu(QWidget *p)
 	{
-		QPopupMenu *pm = new QPopupMenu (p);
+		QMenu *pm = new QMenu (p);
 		uint i = 0;
-		for ( PsiAccountListIt it(psi->accountList(TRUE)); it.current(); ++it, i++ )
-		{
-			PsiAccount *acc = it.current();
+		foreach(PsiAccount* acc, psi->contactList()->enabledAccounts()) {
 			pm->insertItem( acc->name(), parent(), SLOT(itemActivated(int)), 0, id*1000 + i );
 			pm->setItemParameter ( id*1000 + i, i );
+			i++;
 		}
 		return pm;
 	}
+
+	void updateToolButton(QToolButton *btn)
+	{
+		if (psi->contactList()->enabledAccounts().count() >= 2) {
+			btn->setMenu(subMenu(btn));
+			disconnect(btn, SIGNAL(clicked()), sm, SLOT(map()));
+		}
+		else {
+			btn->setMenu(0);
+			// connect exactly once:
+			disconnect(btn, SIGNAL(clicked()), sm, SLOT(map()));
+			connect(btn, SIGNAL(clicked()), sm, SLOT(map()));
+		}
+	}
 };
 
-MAction::MAction(Icon i, const QString &s, int id, PsiCon *psi, QObject *parent)
+MAction::MAction(PsiIcon i, const QString &s, int id, PsiCon *psi, QObject *parent)
 : IconAction(s, s, 0, parent)
 {
 	init (i, id, psi);
@@ -576,28 +426,28 @@ MAction::MAction(Icon i, const QString &s, int id, PsiCon *psi, QObject *parent)
 MAction::MAction(const QString &s, int id, PsiCon *psi, QObject *parent)
 : IconAction(s, s, 0, parent)
 {
-	init (Icon(), id, psi);
+	init (PsiIcon(), id, psi);
 }
 
-void MAction::init(Icon i, int id, PsiCon *psi)
+void MAction::init(PsiIcon i, int id, PsiCon *psi)
 {
 	d = new Private(id, psi, this);
-	setIcon (&i);
+	setPsiIcon (&i);
 	connect(psi, SIGNAL(accountCountChanged()), SLOT(numAccountsChanged()));
-	setEnabled ( !d->psi->accountList(TRUE).isEmpty() );
+	setEnabled ( d->psi->contactList()->haveEnabledAccounts() );
 	connect (d->sm, SIGNAL(mapped(int)), SLOT(itemActivated(int)));
 }
 
 bool MAction::addTo(QWidget *w)
 {
-	if ( w->inherits("QPopupMenu") )
+	if ( w->inherits("QPopupMenu") || w->inherits("QMenu") )
 	{
-		QPopupMenu *menu = (QPopupMenu*)w;
-        QIconSet iconset;
+		QMenu *menu = (QMenu*)w;
+        QIcon iconset;
 #ifndef Q_WS_MAC
 		iconset = iconSet();
 #endif
-		if ( d->psi->accountList(TRUE).count() < 2 ) {
+		if ( d->psi->contactList()->enabledAccounts().count() < 2 ) {
 			menu->insertItem ( iconset, menuText(), this, SLOT(itemActivated(int)), 0, d->id*1000 + 0 );
 			menu->setItemEnabled (d->id*1000 + 0, isEnabled());
 			menu->setItemParameter ( d->id*1000 + 0, 0 );
@@ -613,20 +463,16 @@ bool MAction::addTo(QWidget *w)
 
 void MAction::addingToolButton(IconToolButton *btn)
 {
-	connect (btn, SIGNAL(clicked()), d->sm, SLOT(map()));
-	d->sm->setMapping (btn, 0);
+	d->sm->setMapping(btn, 0);
 
-	if ( d->psi->accountList(TRUE).count() >= 2 )
-		btn->setPopup ( d->subMenu(btn) );
-	else
-		btn->setPopup (0);
+	d->updateToolButton(btn);
 }
 
 void MAction::itemActivated(int n)
 {
-	PsiAccountList list = (PsiAccountList)d->psi->accountList(TRUE);
+	QList<PsiAccount*> list = d->psi->contactList()->enabledAccounts();
 
-	if (n >= (int)list.count()) // just in case
+	if (n >= list.count()) // just in case
 		return;
 
 	emit activated((PsiAccount *)list.at(n), d->id);
@@ -634,18 +480,16 @@ void MAction::itemActivated(int n)
 
 void MAction::numAccountsChanged()
 {
-	setEnabled ( !d->psi->accountList(TRUE).isEmpty() );
+	setEnabled( d->psi->contactList()->haveEnabledAccounts() );
 
-	QPtrList<IconToolButton> btns = buttonList();
-	for ( QPtrListIterator<IconToolButton> it(btns); it.current(); ++it ) {
-		QToolButton *btn = it.current();
+	QList<IconToolButton*> btns = buttonList();
+	for ( QList<IconToolButton*>::Iterator it = btns.begin(); it != btns.end(); ++it ) {
+		QToolButton *btn = (*it);
 
-		if ( btn->popup() )
-			delete btn->popup();
-		btn->setPopup (0);
+		if (btn->menu())
+			delete btn->menu();
 
-		if ( d->psi->accountList(TRUE).count() >= 2 )
-			btn->setPopup ( d->subMenu(btn) );
+		d->updateToolButton(btn);
 	}
 }
 
@@ -683,7 +527,7 @@ SpacerAction::~SpacerAction()
 
 bool SpacerAction::addTo(QWidget *w)
 {
-	if ( w->inherits("QToolBar") ) {
+	if ( w->inherits("QToolBar") || w->inherits("Q3ToolBar") ) {
 		new StretchWidget(w);
 		return true;
 	}
@@ -700,36 +544,28 @@ IconAction *SpacerAction::copy() const
 // SeparatorAction
 //----------------------------------------------------------------------------
 
-class SeparatorAction::Private
-{
-public:
-	Private() { }
-
-	QAction *separator;
-};
-
 SeparatorAction::SeparatorAction( QObject *parent, const char *name )
 	: IconAction( tr("<Separator>"), tr("<Separator>"), 0, parent, name )
 {
-	d = new Private();
-	d->separator = new QAction(this, "qt_separator_action");
-
+	setSeparator(true);
 	setWhatsThis (tr("Separator"));
 }
 
 SeparatorAction::~SeparatorAction()
 {
-	delete d;
 }
 
-bool SeparatorAction::addTo (QWidget *w)
+// we don't want QToolButtons when adding this action
+// on toolbar
+bool SeparatorAction::addTo(QWidget *w)
 {
-	return d->separator->addTo( w );
+	w->addAction(this);
+	return true;
 }
 
 IconAction *SeparatorAction::copy() const
 {
-	return new SeparatorAction( 0 );
+	return new SeparatorAction(0);
 }
 
 //----------------------------------------------------------------------------
@@ -741,7 +577,7 @@ class EventNotifierAction::Private
 public:
 	Private() { }
 
-	QPtrList<MLabel> labels;
+	Q3PtrList<MLabel> labels;
 	bool hide;
 };
 
@@ -760,7 +596,7 @@ EventNotifierAction::~EventNotifierAction()
 
 bool EventNotifierAction::addTo(QWidget *w)
 {
-	if ( w->inherits("QToolBar") ) {
+	if ( w->inherits("QToolBar") || w->inherits("Q3ToolBar") ) {
 		MLabel *label = new MLabel(w, "EventNotifierAction::MLabel");
 		label->setText(text());
 		d->labels.append(label);
@@ -781,7 +617,7 @@ void EventNotifierAction::setText(const QString &t)
 {
 	IconAction::setText("<nobr>" + t + "</nobr>");
 
-	QPtrListIterator<MLabel> it ( d->labels );
+	Q3PtrListIterator<MLabel> it ( d->labels );
 	for ( ; it.current(); ++it) {
 		MLabel *label = it.current();
 		label->setText(text());
@@ -798,22 +634,19 @@ void EventNotifierAction::hide()
 {
 	d->hide = true;
 
-	QPtrListIterator<MLabel> it ( d->labels );
+	Q3PtrListIterator<MLabel> it ( d->labels );
 	for ( ; it.current(); ++it) {
 		MLabel *label = it.current();
 		label->hide();
-		QToolBar *toolBar = (QToolBar *)label->parent();
+		Q3ToolBar *toolBar = (Q3ToolBar *)label->parent();
 
-		QObjectList *l = toolBar->queryList( "QWidget" );
+		QObjectList l = toolBar->queryList( "QWidget" );
 		int found = 0;
 
-		QObjectListIt it( *l );
-		QObject *obj;
-		for ( ; (obj = it.current()); ++it) {
-			if ( QCString(obj->name()).left(3) != "qt_" ) // misc internal Qt objects
+		for ( QObjectList::ConstIterator it = l.begin(); it != l.end(); ++it) {
+			if ( QString((*it)->name()).left(3) != "qt_" ) // misc internal Qt objects
 				found++;
 		}
-		delete l;
 
 		if ( found == 1 ) // only MLabel is on ToolBar
 			toolBar->hide();
@@ -824,12 +657,12 @@ void EventNotifierAction::show()
 {
 	d->hide = false;
 
-	QPtrListIterator<MLabel> it ( d->labels );
+	Q3PtrListIterator<MLabel> it ( d->labels );
 	for ( ; it.current(); ++it) {
 		MLabel *label = it.current();
-		QToolBar *toolBar = (QToolBar *)label->parent();
-		toolBar->show();
+		Q3ToolBar *toolBar = (Q3ToolBar *)label->parent();
 		label->show();
+		toolBar->show();
 	}
 }
 
