@@ -8,7 +8,15 @@
 #include <qbuttongroup.h>
 #include <qradiobutton.h>
 
-#include "opt_advanced-ui.h"
+#include "ui_opt_advanced.h"
+#include "psioptions.h"
+#include "spellchecker.h"
+
+class OptAdvancedUI : public QWidget, public Ui::OptAdvanced
+{
+public:
+	OptAdvancedUI() : QWidget() { setupUi(this); }
+};
 
 //----------------------------------------------------------------------------
 // OptionsTabAdvanced
@@ -36,9 +44,20 @@ QWidget *OptionsTabAdvanced::widget()
 	d->ck_autocopy->hide();
 #endif
 
+	d->ck_spell->setEnabled(SpellChecker::instance()->available());
+
 	QWhatsThis::add(d->ck_messageevents,
 		tr("Enables the sending and requesting of message events such as "
 		"'Contact is Typing', ..."));
+	QWhatsThis::add(d->ck_inactiveevents,
+		tr("Enables the sending of events when you end or suspend a "
+		"conversation"));
+	QWhatsThis::add(d->ck_rc,
+		tr("Enables remote controlling your client from other locations"));
+	QWhatsThis::add(d->ck_spell,
+		tr("Check this option if you want your spelling to be checked"));
+	QWhatsThis::add(d->ck_contactsMessageFormatting,
+		tr("If enabled, Psi will display incoming messages formatted in the style specified by the contact"));
 	QWhatsThis::add(d->ck_autocopy,
 		tr("Check this option if you want the selected text in incoming messages and chat log to be automatically copied to clipboard"));
 	QWhatsThis::add(d->ck_singleclick,
@@ -57,8 +76,6 @@ QWidget *OptionsTabAdvanced::widget()
 		tr("Makes Psi show separate subject line in messages. Uncheck this if you want to save some screen space."));
 	QWhatsThis::add(d->ck_showCounter,
 		tr("Makes Psi show message length counter. Check this if you want to know how long is your message. Can be useful when you're using SMS transport."));
-	QWhatsThis::add(d->ck_autoVersion,
-		tr("Automatically sends iq:version query to contact when he becomes online. This allows you to determine what client he is using to get online. Note: results in increased traffic."));
 	QWhatsThis::add(d->ck_autoVCardOnLogin,
 		tr("By default, Psi always checks your vCard on login. If you want to save some traffic, you can uncheck this option."));
 	QWhatsThis::add(d->ck_rosterAnim,
@@ -69,6 +86,9 @@ QWidget *OptionsTabAdvanced::widget()
 		tr("Makes Psi ignore all incoming \"headline\" events,"
 		" like system-wide news on MSN, announcements, etc."));
 
+	connect(d->ck_messageevents,SIGNAL(toggled(bool)),d->ck_inactiveevents,SLOT(setEnabled(bool)));
+	d->ck_inactiveevents->setEnabled(d->ck_messageevents->isChecked());
+
 	return w;
 }
 
@@ -80,6 +100,11 @@ void OptionsTabAdvanced::applyOptions(Options *opt)
 	OptAdvancedUI *d = (OptAdvancedUI *)w;
 
 	opt->messageEvents = d->ck_messageevents->isChecked();
+	opt->inactiveEvents = d->ck_inactiveevents->isChecked();
+	opt->useRC = d->ck_rc->isChecked();
+	if ( SpellChecker::instance()->available() )
+		PsiOptions::instance()->setOption("options.ui.spell-check.enabled",d->ck_spell->isChecked());
+	PsiOptions::instance()->setOption("options.html.chat.render", d->ck_contactsMessageFormatting->isChecked());
 	opt->autoCopy = d->ck_autocopy->isChecked();
 	opt->singleclick = d->ck_singleclick->isChecked();
 	opt->jidComplete = d->ck_jidComplete->isChecked();
@@ -87,7 +112,6 @@ void OptionsTabAdvanced::applyOptions(Options *opt)
 	opt->incomingAs = d->cb_incomingAs->currentItem();
 	opt->showSubjects = d->ck_showSubjects->isChecked();
 	opt->showCounter = d->ck_showCounter->isChecked();
-	opt->autoVersion = d->ck_autoVersion->isChecked();
 	opt->autoVCardOnLogin = d->ck_autoVCardOnLogin->isChecked();
 	opt->rosterAnim = d->ck_rosterAnim->isChecked();
 	opt->scrollTo = d->ck_scrollTo->isChecked();
@@ -102,6 +126,13 @@ void OptionsTabAdvanced::restoreOptions(const Options *opt)
 	OptAdvancedUI *d = (OptAdvancedUI *)w;
 
 	d->ck_messageevents->setChecked( opt->messageEvents );
+	d->ck_inactiveevents->setChecked( opt->inactiveEvents );
+	d->ck_rc->setChecked( opt->useRC );
+	if ( !SpellChecker::instance()->available() )
+		d->ck_spell->setChecked(false);
+	else
+		d->ck_spell->setChecked(PsiOptions::instance()->getOption("options.ui.spell-check.enabled").toBool());
+	d->ck_contactsMessageFormatting->setChecked(PsiOptions::instance()->getOption("options.html.chat.render").toBool());
 	d->ck_autocopy->setChecked( opt->autoCopy );
 	d->ck_singleclick->setChecked( opt->singleclick );
 	d->ck_jidComplete->setChecked( opt->jidComplete );
@@ -109,7 +140,6 @@ void OptionsTabAdvanced::restoreOptions(const Options *opt)
 	d->cb_incomingAs->setCurrentItem( opt->incomingAs );
 	d->ck_showSubjects->setChecked( opt->showSubjects );
 	d->ck_showCounter->setChecked( opt->showCounter );
-	d->ck_autoVersion->setChecked( opt->autoVersion );
 	d->ck_autoVCardOnLogin->setChecked( opt->autoVCardOnLogin );
 	d->ck_rosterAnim->setChecked( opt->rosterAnim );
 	d->ck_scrollTo->setChecked( opt->scrollTo );
