@@ -4,6 +4,7 @@
 #include "optionstab.h"
 
 #include <qlineedit.h>
+#include <qabstractbutton.h>
 
 class FontLabel : public QLineEdit
 {
@@ -65,8 +66,8 @@ public:
 
 private slots:
 	void setData(PsiCon *, QWidget *);
-	void chooseColor(int);
-	void chooseFont(int);
+	void chooseColor(QAbstractButton* button);
+	void chooseFont(QAbstractButton* button);
 
 private:
 	QWidget *w, *parentWidget;
