@@ -18,10 +18,6 @@ public:
 	void applyOptions(Options *opt);
 	void restoreOptions(const Options *opt);
 
-private slots:
-	void selectBrowser(int);
-	void hideMenubarToggled(bool);
-
 private:
 	QWidget *w;
 };
