@@ -2,6 +2,7 @@
 #define OPT_EVENTS_H
 
 #include "optionstab.h"
+#include "qradiobutton.h"
 
 class QWidget;
 struct Options;
@@ -16,12 +17,9 @@ public:
 	void applyOptions(Options *opt);
 	void restoreOptions(const Options *opt);
 
-private slots:
-	void selectAlertStyle(int);
-
 private:
 	QWidget *w;
-	int alertStyle;
+	QList<QRadioButton*> list_alerts;
 };
 
 #endif
