@@ -1,34 +1,134 @@
 #include "opt_iconset.h"
 #include "common.h"
 #include "iconwidget.h"
+#include "applicationinfo.h"
+#include "psiiconset.h"
 #include "psicon.h"
 
-#include <qbuttongroup.h>
-#include <qwhatsthis.h>
+#include <QDir>
 #include <qcheckbox.h>
 #include <qradiobutton.h>
 #include <qcombobox.h>
 #include <qlineedit.h>
 #include <qfileinfo.h>
-#include <qlistview.h>
-#include <qheader.h>
 #include <qapplication.h>
 #include <qthread.h>
 #include <qmutex.h>
 #include <qevent.h>
-#include <qprogressbar.h>
 #include <qcursor.h>
 #include <qpalette.h>
 #include <qtabwidget.h>
+#include <QFont>
+#include <QTreeWidget>
 
-#include "opt_iconset_emo.h"
-#include "opt_iconset_system.h"
-#include "opt_iconset_roster.h"
-#include "ui_isdetails.h"
+#include "ui_opt_iconset_emo.h"
+#include "ui_opt_iconset_system.h"
+#include "ui_opt_iconset_roster.h"
+#include "ui_ui_isdetails.h"
+
+class IconsetEmoUI : public QWidget, public Ui::IconsetEmo
+{
+public:
+	IconsetEmoUI() : QWidget() { setupUi(this); }
+};
+
+class IconsetSystemUI : public QWidget, public Ui::IconsetSystem
+{
+public:
+	IconsetSystemUI() : QWidget() { setupUi(this); }
+};
+
+class IconsetRosterUI : public QWidget, public Ui::IconsetRoster
+{
+public:
+	IconsetRosterUI() : QWidget() { setupUi(this); }
+};
+
+class IconsetDetailsDlg : public QDialog, public Ui::IconsetDetailsDlg
+{
+public:
+	IconsetDetailsDlg(QWidget* parent, const char* name, bool modal)
+		: QDialog(parent)
+	{
+		setAttribute(Qt::WA_DeleteOnClose);
+		setupUi(this); 
+		
+		QStringList bold_labels;
+		bold_labels << "lb_name2";
+		bold_labels << "lb_version2";
+		bold_labels << "lb_date2";
+		bold_labels << "lb_home2";
+		bold_labels << "lb_desc2";
+		bold_labels << "lb_authors";
+		
+		QList<QLabel *> labels = findChildren<QLabel *>();
+		foreach (QLabel *l, labels) {
+			if (bold_labels.contains(l->objectName())) {
+				QFont font = l->font();
+				font.setBold(true);
+				l->setFont(font);
+			}
+		}
+		
+		setName(name); 
+		setModal(modal); 
+	}
+	
+	void setIconset( const Iconset &is ) {
+		setWindowTitle(caption().arg(is.name()));
+		
+		lb_name->setText(is.name());
+		lb_version->setText(is.version());
+		
+		if ( is.description().isEmpty() ) {
+			lb_desc->hide();
+			lb_desc2->hide();
+		}
+		else
+			lb_desc->setText(is.description());
+		
+		if ( !is.homeUrl().isEmpty() ) {
+			lb_home->setTitle(is.homeUrl());
+			lb_home->setUrl(is.homeUrl());
+		}
+		else {
+			lb_home->hide();
+			lb_home2->hide();
+		}
+			
+		if ( is.creation().isEmpty() ) {
+			lb_date->hide();
+			lb_date2->hide();
+		}
+		else {
+			QDate date = QDate::fromString(is.creation(), Qt::ISODate);	
+			lb_date->setText(date.toString(Qt::LocalDate));
+		}
+		
+		if ( is.authors().count() == 0 ) {
+			ptv_authors->hide();
+			lb_authors->hide();
+		}
+		else {
+			QString authors;
+			QStringList::ConstIterator it = is.authors().begin();
+			for ( ; it != is.authors().end(); ++it) {
+				if ( !authors.isEmpty() )
+					authors += "<br><br>";
+				authors += *it;
+			}
+			ptv_authors->setText( "<qt><nobr>" + authors + "</nobr></qt>" );
+		}
+		
+		isd_iconset->setIconset(is);
+		
+		resize(sizeHint());
+	}
+};
 
 static void isDetails(const Iconset &is, QWidget *parent)
 {
-	IconsetDetailsDlg *isd = new IconsetDetailsDlg(parent, "IconsetDetailsDlg", false, QWidget::WDestructiveClose);
+	IconsetDetailsDlg *isd = new IconsetDetailsDlg(parent, "IconsetDetailsDlg", false);
 	isd->setIconset(is);
 	isd->show();
 }
@@ -86,7 +186,7 @@ public:
 
 	IconsetLoadThread *thread() const { return p; }
 
-	// if iconset() is '0' then it means that iconset wans't loaded successfully
+	// if iconset() is '0' then it means that iconset wasn't loaded successfully
 	Iconset *iconset() const { return is; }
 
 private:
@@ -269,9 +369,10 @@ OptionsTabIconsetSystem::OptionsTabIconsetSystem(QObject *parent)
 	thread = 0;
 
 	if ( dirs.isEmpty() ) {
+		dirs << ":";
 		dirs << ".";
-		dirs << g.pathHome;
-		dirs << g.pathBase;
+		dirs << ApplicationInfo::homeDir();
+		dirs << ApplicationInfo::resourcesDir();
 	}
 }
 
@@ -318,7 +419,7 @@ void OptionsTabIconsetSystem::restoreOptions(const Options *opt)
 	d->iss_system->clear();
 	QStringList loaded;
 
-	d->setCursor(WaitCursor);
+	d->setCursor(Qt::WaitCursor);
 
 	QPalette customPal = d->palette();
 	customPal.setDisabled(customPal.inactive());
@@ -329,7 +430,7 @@ void OptionsTabIconsetSystem::restoreOptions(const Options *opt)
 	d->pb_sysDetails->setPalette(customPal);
 
 	d->progress->show();
-	d->progress->setProgress( 0 );
+	d->progress->setValue( 0 );
 
 	numIconsets = countIconsets("/system", loaded);
 	iconsetsLoaded = 0;
@@ -351,16 +452,16 @@ bool OptionsTabIconsetSystem::event(QEvent *e)
 
 		if ( !numIconsets )
 			numIconsets = 1;
-		d->progress->setProgress( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
+		d->progress->setValue( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
 
 		Iconset *i = le->iconset();
 		if ( i ) {
-			is->stripFirstAnimFrame(i);
+			PsiIconset::instance()->stripFirstAnimFrame(i);
 			d->iss_system->insert(*i);
 
 			QFileInfo fi( i->fileName() );
 			if ( fi.fileName() == o->systemIconset )
-				d->iss_system->setSelected(d->iss_system->count()-1, true);
+				d->iss_system->setItemSelected(d->iss_system->lastItem(), true);
 
 			delete i;
 		}
@@ -374,7 +475,7 @@ bool OptionsTabIconsetSystem::event(QEvent *e)
 		d->pb_sysDetails->setEnabled(true);
 		d->pb_sysDetails->unsetPalette();
 
-		connect(d->iss_system, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
+		connect(d->iss_system, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
 
 		d->progress->hide();
 
@@ -441,7 +542,7 @@ QWidget *OptionsTabIconsetEmoticons::widget()
 	connect(d->pb_emoDown, SIGNAL(clicked()), SIGNAL(dataChanged()));
 	connect(d->pb_emoDetails, SIGNAL(clicked()), SLOT(previewIconset()));
 
-	QWhatsThis::add(d->ck_useEmoticons,
+	d->ck_useEmoticons->setWhatsThis(
 		tr("<P>Emoticons are short sequences of characters that are used to convey an emotion or idea.</P>"
 		"<P>Enable this option if you want Psi to replace common emoticons with a graphical image.</P>"
 		"<P>For example, <B>:-)</B> would be replaced by <icon name=\"psi/smile\"></P>"));
@@ -460,19 +561,17 @@ void OptionsTabIconsetEmoticons::applyOptions(Options *opt)
 	opt->useEmoticons = d->ck_useEmoticons->isChecked();
 
 	opt->emoticons.clear();
-	IconWidgetItem *item = (IconWidgetItem *)d->iss_emoticons->firstItem();
-	while ( item ) {
-		if ( item->isSelected() ) {
+	for (int row = 0; row < d->iss_emoticons->count(); row++) {
+		IconWidgetItem *item = (IconWidgetItem *)d->iss_emoticons->item(row);
+
+		if ( d->iss_emoticons->isItemSelected(item) ) {
 			const Iconset *is = item->iconset();
 			if ( is ) {
 				QFileInfo fi( is->fileName() );
 				opt->emoticons << fi.fileName();
 			}
 		}
-
-		item = (IconWidgetItem *)item->next();
 	}
-
 }
 
 void OptionsTabIconsetEmoticons::restoreOptions(const Options *opt)
@@ -487,11 +586,11 @@ void OptionsTabIconsetEmoticons::restoreOptions(const Options *opt)
 	d->iss_emoticons->clear();
 
 	{
-		QPtrListIterator<Iconset> it ( is->emoticons );
+		Q3PtrListIterator<Iconset> it ( PsiIconset::instance()->emoticons );
 		Iconset *is;
 		for ( ; (is = it.current()); ++it) {
 			d->iss_emoticons->insert(*is);
-			d->iss_emoticons->setSelected(d->iss_emoticons->count()-1, true);
+			d->iss_emoticons->setItemSelected(d->iss_emoticons->lastItem(), true);
 		}
 	}
 
@@ -499,7 +598,7 @@ void OptionsTabIconsetEmoticons::restoreOptions(const Options *opt)
 	{
 		QStringList loaded;
 		{
-			QPtrListIterator<Iconset> it( is->emoticons );
+			Q3PtrListIterator<Iconset> it( PsiIconset::instance()->emoticons );
 			Iconset *tmp;
 			for ( ; (tmp = it.current()); ++it) {
 				QFileInfo fi ( tmp->fileName() );
@@ -507,7 +606,7 @@ void OptionsTabIconsetEmoticons::restoreOptions(const Options *opt)
 			}
 		}
 
-		d->setCursor(WaitCursor);
+		d->setCursor(Qt::WaitCursor);
 
 		QPalette customPal = d->palette();
 		customPal.setDisabled(customPal.inactive());
@@ -518,7 +617,7 @@ void OptionsTabIconsetEmoticons::restoreOptions(const Options *opt)
 		d->groupBox9->setPalette(customPal);
 
 		d->progress->show();
-		d->progress->setProgress( 0 );
+		d->progress->setValue( 0 );
 
 		numIconsets = countIconsets("/emoticons", loaded);
 		iconsetsLoaded = 0;
@@ -542,7 +641,7 @@ bool OptionsTabIconsetEmoticons::event(QEvent *e)
 
 		if ( !numIconsets )
 			numIconsets = 1;
-		d->progress->setProgress( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
+		d->progress->setValue( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
 
 		Iconset *is = le->iconset();
 		if ( is ) {
@@ -560,7 +659,7 @@ bool OptionsTabIconsetEmoticons::event(QEvent *e)
 		d->groupBox9->setEnabled(true);
 		d->groupBox9->unsetPalette();
 
-		connect(d->iss_emoticons, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
+		connect(d->iss_emoticons, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
 
 		bool checked = d->ck_useEmoticons->isChecked();
 		d->ck_useEmoticons->setChecked( true );
@@ -625,22 +724,48 @@ QWidget *OptionsTabIconsetRoster::widget()
 	w = new IconsetRosterUI;
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 
+	// Initialize transport iconsets
+	QList<QTreeWidgetItem *> items;
+	QTreeWidgetItem *ti = new QTreeWidgetItem(d->tw_isServices, QStringList(QString("AIM")));
+	ti->setData(0, Qt::UserRole, QVariant(QString("aim")));
+	items.append(ti);
+        ti = new QTreeWidgetItem(d->tw_isServices, QStringList(QString("Gadu-Gadu")));
+        ti->setData(0, Qt::UserRole, QVariant(QString("gadugadu")));
+        items.append(ti);
+        ti = new QTreeWidgetItem(d->tw_isServices, QStringList(QString("ICQ")));
+        ti->setData(0, Qt::UserRole, QVariant(QString("icq")));
+        items.append(ti);
+        ti = new QTreeWidgetItem(d->tw_isServices, QStringList(QString("MSN")));
+        ti->setData(0, Qt::UserRole, QVariant(QString("msn")));
+        items.append(ti);
+        ti = new QTreeWidgetItem(d->tw_isServices, QStringList(QString("SMS")));
+        ti->setData(0, Qt::UserRole, QVariant(QString("sms")));
+        items.append(ti);
+        ti = new QTreeWidgetItem(d->tw_isServices, QStringList(QString("Yahoo!")));
+        ti->setData(0, Qt::UserRole, QVariant(QString("yahoo")));
+        items.append(ti);
+	d->tw_isServices->insertTopLevelItems(0, items);
+	
+	d->tw_isServices->resizeColumnToContents(0);
+
 	connect(d->pb_defRosterDetails, SIGNAL(clicked()), SLOT(defaultDetails()));
 
-	connect(d->iss_servicesRoster, SIGNAL(selectionChanged(QListBoxItem *)), SLOT(isServices_iconsetSelected(QListBoxItem *)));
-	connect(d->lv_isServices, SIGNAL(selectionChanged(QListViewItem *)), SLOT(isServices_selectionChanged(QListViewItem *)));
+	connect(d->iss_servicesRoster, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), SLOT(isServices_iconsetSelected(QListWidgetItem *, QListWidgetItem *)));
+	connect(d->tw_isServices, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), SLOT(isServices_selectionChanged(QTreeWidgetItem *)));
 	connect(d->pb_servicesDetails, SIGNAL(clicked()), SLOT(servicesDetails()));
 	isServices_selectionChanged(0);
 
-	connect(d->iss_customRoster, SIGNAL(selectionChanged(QListBoxItem *)), SLOT(isCustom_iconsetSelected(QListBoxItem *)));
-	connect(d->lv_customRoster, SIGNAL(selectionChanged(QListViewItem *)), SLOT(isCustom_selectionChanged(QListViewItem *)));
+	connect(d->iss_customRoster, SIGNAL(currentItemChanged(QListWidgetItem *, QListWidgetItem *)), SLOT(isCustom_iconsetSelected(QListWidgetItem *, QListWidgetItem *)));
+	connect(d->tw_customRoster, SIGNAL(currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *)), SLOT(isCustom_selectionChanged(QTreeWidgetItem *)));
+	connect(d->tw_customRoster, SIGNAL(itemClicked(QTreeWidgetItem *, int )), SLOT(isCustom_selectionChanged(QTreeWidgetItem *)));
+
 	connect(d->pb_customRosterDetails, SIGNAL(clicked()), SLOT(customDetails()));
 	connect(d->le_customRoster, SIGNAL(textChanged(const QString &)), SLOT(isCustom_textChanged()));
 	connect(d->pb_customRosterAdd, SIGNAL(clicked()), SLOT(isCustom_add()));
 	connect(d->pb_customRosterDelete, SIGNAL(clicked()), SLOT(isCustom_delete()));
 	isCustom_selectionChanged(0);
 
-	QWhatsThis::add(d->ck_useTransportIconsForContacts,
+	d->ck_useTransportIconsForContacts->setWhatsThis(
 		tr("Toggles use of transport icons to the contacts, that use that transports."));
 
 	// TODO: add QWhatsThis
@@ -669,11 +794,10 @@ void OptionsTabIconsetRoster::applyOptions(Options *opt)
 	{
 		opt->serviceRosterIconset.clear();
 
-		QListViewItemIterator it( d->lv_isServices );
-		for ( ; it.current(); ++it) {
-			QListViewItem *item = it.current();
-
-			opt->serviceRosterIconset[item->text(2)] = item->text(3);
+		QTreeWidgetItemIterator it( d->tw_isServices );
+		while(*it) {
+			opt->serviceRosterIconset[(*it)->data(0, Qt::UserRole).toString()] = (*it)->data(1, Qt::UserRole).toString();
+			++it;
 		}
 	}
 
@@ -681,11 +805,10 @@ void OptionsTabIconsetRoster::applyOptions(Options *opt)
 	{
 		opt->customRosterIconset.clear();
 
-		QListViewItemIterator it( d->lv_customRoster );
-		for ( ; it.current(); ++it) {
-			QListViewItem *item = it.current();
-
-			opt->customRosterIconset[item->text(2)] = item->text(3);
+		QTreeWidgetItemIterator it( d->tw_customRoster );
+		while(*it) {
+			opt->customRosterIconset[(*it)->data(0, Qt::UserRole).toString()] = (*it)->data(1, Qt::UserRole).toString();
+			++it;
 		}
 	}
 }
@@ -705,7 +828,7 @@ void OptionsTabIconsetRoster::restoreOptions(const Options *opt)
 
 	QStringList loaded;
 
-	d->setCursor(WaitCursor);
+	d->setCursor(Qt::WaitCursor);
 
 	QPalette customPal = d->palette();
 	customPal.setDisabled(customPal.inactive());
@@ -713,7 +836,7 @@ void OptionsTabIconsetRoster::restoreOptions(const Options *opt)
 	d->tabWidget3->setPalette(customPal);
 
 	d->progress->show();
-	d->progress->setProgress( 0 );
+	d->progress->setValue( 0 );
 
 	numIconsets = countIconsets("/roster", loaded);
 	iconsetsLoaded = 0;
@@ -735,17 +858,17 @@ bool OptionsTabIconsetRoster::event(QEvent *e)
 
 		if ( !numIconsets )
 			numIconsets = 1;
-		d->progress->setProgress( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
+		d->progress->setValue( (int)((float)100 * ++iconsetsLoaded / numIconsets) );
 
 		Iconset *i = le->iconset();
 		if ( i ) {
-			is->stripFirstAnimFrame(i);
+			PsiIconset::instance()->stripFirstAnimFrame(i);
 			QFileInfo fi( i->fileName() );
 
 			// roster - default
 			d->iss_defRoster->insert(*i);
 			if ( fi.fileName() == o->defaultRosterIconset )
-				d->iss_defRoster->setSelected(d->iss_defRoster->count()-1, true);
+				d->iss_defRoster->setItemSelected(d->iss_defRoster->lastItem(), true);
 
 			// roster - service
 			d->iss_servicesRoster->insert(*i);
@@ -761,53 +884,50 @@ bool OptionsTabIconsetRoster::event(QEvent *e)
 	else if ( e->type() == QEvent::User + 1 ) { // finish event
 		// roster - service
 		{
-			// fill the QListView
-			QListViewItemIterator it( d->lv_isServices );
-			for ( ; it.current(); ++it) {
-				QListViewItem *i = it.current();
-				if ( !i->text(2).isEmpty() ) {
-					Iconset *iss = is->roster[o->serviceRosterIconset[i->text(2)]];
+			// fill the QTreeWidget
+			QTreeWidgetItemIterator it( d->tw_isServices );
+			while(*it) {
+				QTreeWidgetItem *i = *it;
+				if ( !i->data(0, Qt::UserRole).toString().isEmpty() ) {
+					Iconset *iss = PsiIconset::instance()->roster[o->serviceRosterIconset[i->data(0, Qt::UserRole).toString()]];
 					if ( iss ) {
 						i->setText(1, iss->name());
-
 						QFileInfo fi ( iss->fileName() );
-						i->setText(3, fi.fileName());
+						i->setData(1, Qt::UserRole, fi.fileName());
 					}
 				}
+				++it;
 			}
-
-			QHeader *head = d->lv_isServices->header();
-			head->removeLabel(2);
 		}
 
 		// roster - custom
 		{
 			// Then, fill the QListView
-			QListViewItem *last = 0;
+			QTreeWidgetItem *last = 0;
 			QMap<QString, QString>::ConstIterator it = o->customRosterIconset.begin();
 			for ( ; it != o->customRosterIconset.end(); ++it) {
-				QListViewItem *item = new QListViewItem(d->lv_customRoster, last);
+				QTreeWidgetItem *item = new QTreeWidgetItem(d->tw_customRoster, last);
 				last = item;
 
 				item->setText(0, clipCustomText(it.key())); // RegExp
-				item->setText(2, it.key());
+				item->setData(0, Qt::UserRole, it.key());
 
-				Iconset *iss = is->roster[it.data()];
+				Iconset *iss = PsiIconset::instance()->roster[it.data()];
 				if ( iss ) {
 					item->setText(1, iss->name());
-
 					QFileInfo fi ( iss->fileName() );
-					item->setText(3, fi.fileName());
+					item->setData(1, Qt::UserRole, fi.fileName());
 				}
 			}
+			d->tw_customRoster->resizeColumnToContents(0);
 		}
 
 		d->tabWidget3->setEnabled(true);
 		d->tabWidget3->unsetPalette();
 
-		connect(d->iss_defRoster, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
-		connect(d->iss_servicesRoster, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
-		connect(d->iss_customRoster, SIGNAL(selectionChanged()), SIGNAL(dataChanged()));
+		connect(d->iss_defRoster, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
+		connect(d->iss_servicesRoster, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
+		connect(d->iss_customRoster, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SIGNAL(dataChanged()));
 
 		d->progress->hide();
 
@@ -851,13 +971,13 @@ void OptionsTabIconsetRoster::customDetails()
 
 //------------------------------------------------------------
 
-void OptionsTabIconsetRoster::isServices_iconsetSelected(QListBoxItem *item)
+void OptionsTabIconsetRoster::isServices_iconsetSelected(QListWidgetItem *item, QListWidgetItem *)
 {
 	if ( !item )
 		return;
 
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *it = d->lv_isServices->selectedItem();
+	QTreeWidgetItem *it = d->tw_isServices->currentItem();
 	if ( !it )
 		return;
 
@@ -867,10 +987,10 @@ void OptionsTabIconsetRoster::isServices_iconsetSelected(QListBoxItem *item)
 
 	it->setText(1, is->name());
 	QFileInfo fi ( is->fileName() );
-	it->setText(3, fi.fileName());
+	it->setData(1, Qt::UserRole, fi.fileName());
 }
 
-void OptionsTabIconsetRoster::isServices_selectionChanged(QListViewItem *it)
+void OptionsTabIconsetRoster::isServices_selectionChanged(QTreeWidgetItem *it)
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 	d->iss_servicesRoster->setEnabled( it != 0 );
@@ -879,21 +999,21 @@ void OptionsTabIconsetRoster::isServices_selectionChanged(QListViewItem *it)
 	if ( !it )
 		return;
 
-	if ( it->text(3).isEmpty() )
+	if ( it->data(1, Qt::UserRole).toString().isEmpty() )
 		return;
 
-	QString name = it->text(3);
+	QString name = it->data(1, Qt::UserRole).toString();
 
 	emit noDirty(true);
-	IconWidgetItem *item = (IconWidgetItem *)d->iss_servicesRoster->firstItem();
-	for ( ; item; item = (IconWidgetItem *)item->next()) {
+	for (int row = 0; row < d->iss_servicesRoster->count(); row++) {
+		IconWidgetItem *item = (IconWidgetItem *)d->iss_servicesRoster->item(row);
 		const Iconset *is = item->iconset();
 		if ( is ) {
 			QFileInfo fi ( is->fileName() );
 			if ( fi.fileName() == name ) {
 				emit noDirty(true);
-				d->iss_servicesRoster->setSelected(item, true);
-				d->iss_servicesRoster->ensureCurrentVisible();
+				d->iss_servicesRoster->setItemSelected(item, true);
+				d->iss_servicesRoster->scrollToItem(item);
 				emit noDirty(false);
 				break;
 			}
@@ -905,13 +1025,13 @@ void OptionsTabIconsetRoster::isServices_selectionChanged(QListViewItem *it)
 
 //------------------------------------------------------------
 
-void OptionsTabIconsetRoster::isCustom_iconsetSelected(QListBoxItem *item)
+void OptionsTabIconsetRoster::isCustom_iconsetSelected(QListWidgetItem *item, QListWidgetItem *)
 {
 	if ( !item )
 		return;
 
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *it = d->lv_customRoster->selectedItem();
+	QTreeWidgetItem *it = d->tw_customRoster->currentItem();
 	if ( !it )
 		return;
 
@@ -921,10 +1041,10 @@ void OptionsTabIconsetRoster::isCustom_iconsetSelected(QListBoxItem *item)
 
 	it->setText(1, is->name());
 	QFileInfo fi ( is->fileName() );
-	it->setText(3, fi.fileName());
+	it->setData(1, Qt::UserRole,  fi.fileName());
 }
 
-void OptionsTabIconsetRoster::isCustom_selectionChanged(QListViewItem *it)
+void OptionsTabIconsetRoster::isCustom_selectionChanged(QTreeWidgetItem *it)
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
 	d->iss_customRoster->setEnabled( it != 0 );
@@ -936,21 +1056,21 @@ void OptionsTabIconsetRoster::isCustom_selectionChanged(QListViewItem *it)
 	if ( !it )
 		return;
 
-	if ( it->text(3).isEmpty() )
+	if ( it->data(1, Qt::UserRole).toString().isEmpty() )
 		return;
 
 	emit noDirty(true);
-	d->le_customRoster->setText(it->text(2));
-	QString name = it->text(3);
+	d->le_customRoster->setText(it->data(0, Qt::UserRole).toString());
+	QString name = it->data(1, Qt::UserRole).toString();
 
-	IconWidgetItem *item = (IconWidgetItem *)d->iss_customRoster->firstItem();
-	for ( ; item; item = (IconWidgetItem *)item->next()) {
+	for (int row = 0; row < d->iss_customRoster->count(); row++) {
+		IconWidgetItem *item = (IconWidgetItem *)d->iss_customRoster->item(row);
 		const Iconset *is = item->iconset();
 		if ( is ) {
 			QFileInfo fi ( is->fileName() );
 			if ( fi.fileName() == name ) {
-				d->iss_customRoster->setSelected(item, true);
-				d->iss_customRoster->ensureCurrentVisible();
+				d->iss_customRoster->setItemSelected(item, true);
+				d->iss_customRoster->scrollToItem(item);
 				break;
 			}
 		}
@@ -962,12 +1082,12 @@ void OptionsTabIconsetRoster::isCustom_selectionChanged(QListViewItem *it)
 void OptionsTabIconsetRoster::isCustom_textChanged()
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *item = d->lv_customRoster->selectedItem();
+	QTreeWidgetItem *item = d->tw_customRoster->currentItem();
 	if ( !item )
 		return;
 
 	item->setText( 0, clipCustomText(d->le_customRoster->text()) );
-	item->setText( 2, d->le_customRoster->text() );
+	item->setData( 0, Qt::UserRole, d->le_customRoster->text() );
 }
 
 void OptionsTabIconsetRoster::isCustom_add()
@@ -976,23 +1096,23 @@ void OptionsTabIconsetRoster::isCustom_add()
 
 	QString def;
 	const Iconset *iconset = d->iss_defRoster->iconset();
-	if ( is ) {
+	if ( iconset ) {
 		QFileInfo fi( iconset->fileName() );
 		def = fi.fileName();
 	}
 	else
 		qWarning("OptionsTabIconsetRoster::isCustom_add(): 'def' is null!");
 
-	QListViewItem *item = new QListViewItem(d->lv_customRoster, d->lv_customRoster->lastItem());
-	const Iconset *i = is->roster[def];
+	QTreeWidgetItem *item = new QTreeWidgetItem(d->tw_customRoster);
+	const Iconset *i = PsiIconset::instance()->roster[def];
 	if ( i ) {
 		item->setText(1, i->name());
 
 		QFileInfo fi(i->fileName());
-		item->setText(3, fi.fileName());
+		item->setData(1, Qt::UserRole, fi.fileName());
 	}
 
-	d->lv_customRoster->setSelected(item, true);
+	d->tw_customRoster->setCurrentItem(item);
 	emit dataChanged();
 
 	d->le_customRoster->setFocus();
@@ -1001,12 +1121,13 @@ void OptionsTabIconsetRoster::isCustom_add()
 void OptionsTabIconsetRoster::isCustom_delete()
 {
 	IconsetRosterUI *d = (IconsetRosterUI *)w;
-	QListViewItem *item = d->lv_customRoster->selectedItem();
+	QTreeWidgetItem *item = d->tw_customRoster->currentItem();
 	if ( !item )
 		return;
 
 	delete item;
 	isCustom_selectionChanged(0);
+	d->tw_customRoster->clearSelection();
 	emit dataChanged();
 }
 
