@@ -1,12 +1,15 @@
 #ifndef OPT_SOUND_H
 #define OPT_SOUND_H
 
+#include <QMap>
+
 #include "optionstab.h"
 
 class QWidget;
 struct Options;
 class QLineEdit;
 class QButtonGroup;
+class QAbstractButton;
 
 class OptionsTabSound : public OptionsTab
 {
@@ -20,14 +23,16 @@ public:
 	void restoreOptions(const Options *opt);
 
 private slots:
-	void chooseSoundEvent(int x);
-	void previewSoundEvent(int x);
+	void chooseSoundEvent(QAbstractButton*);
+	void previewSoundEvent(QAbstractButton*);
 	void soundReset();
 	void setData(PsiCon *, QWidget *);
 
 private:
 	QWidget *w, *parentWidget;
-	QLineEdit *le_oe[10];
+	QList<QLineEdit*> sounds_;
+	QMap<QAbstractButton*,QLineEdit*> modify_buttons_;
+	QMap<QAbstractButton*,QLineEdit*> play_buttons_;
 	QButtonGroup *bg_se, *bg_sePlay;
 };
 
