@@ -3,18 +3,22 @@
 #include "common.h"
 #include "psicon.h"
 #include "fancylabel.h"
+#include "iconset.h"
 #include "iconwidget.h"
 
 #include <qlayout.h>
 #include <qlabel.h>
-#include <qlistview.h>
-#include <qwidgetstack.h>
-#include <qheader.h>
-#include <qobjectlist.h>
+#include <q3listview.h>
+#include <QStackedWidget>
 #include <qpen.h>
 #include <qpainter.h>
+#include <Q3Dict>
+#include <QPixmap>
+#include <Q3PtrList>
+#include <QVBoxLayout>
 
 // tabs
+#include "opt_toolbars.h"
 #include "opt_application.h"
 #include "opt_appearance.h"
 #include "opt_chat.h"
@@ -24,32 +28,35 @@
 #include "opt_groupchat.h"
 #include "opt_sound.h"
 #include "opt_advanced.h"
+#include "opt_shortcuts.h"
 
-#include "opt_lookfeel.h"
+#ifdef PSI_PLUGINS
+#include "opt_plugins.h"
+#endif
 
 //----------------------------------------------------------------------------
 // FancyItem
 //----------------------------------------------------------------------------
 
-class FancyItem : public QListViewItem
+class FancyItem : public Q3ListViewItem
 {
 public:
-	FancyItem(QListView *, QListViewItem *after);
+	FancyItem(Q3ListView *, Q3ListViewItem *after);
 
 	void setup();
-	int width(const QFontMetrics &, const QListView *lv, int c) const;
+	int width(const QFontMetrics &, const Q3ListView *lv, int c) const;
 	void paintFocus(QPainter *, const QColorGroup &, const QRect &);
 	void paintCell(QPainter *p, const QColorGroup &, int c, int width, int align);
 };
 
-FancyItem::FancyItem(QListView *lv, QListViewItem *after)
-: QListViewItem(lv, after)
+FancyItem::FancyItem(Q3ListView *lv, Q3ListViewItem *after)
+: Q3ListViewItem(lv, after)
 {
 }
 
 void FancyItem::setup()
 {
-	QListView *lv = listView();
+	Q3ListView *lv = listView();
 	int ph = 0;
 	for(int i = 0; i < lv->columns(); ++i) {
 		if(pixmap(i))
@@ -60,7 +67,7 @@ void FancyItem::setup()
 	setHeight(y);
 }
 
-int FancyItem::width(const QFontMetrics &fm, const QListView *, int c) const
+int FancyItem::width(const QFontMetrics &fm, const Q3ListView *, int c) const
 {
 	int x = 0;
 	const QPixmap *pix = pixmap(c);
@@ -101,7 +108,7 @@ void FancyItem::paintCell(QPainter *p, const QColorGroup &cg, int c, int w, int)
 	p->setPen(isSelected() ? cg.highlightedText() : cg.text());
 	p->drawText(x, y, text(c));
 
-	p->setPen(QPen(QColor(0xE0, 0xE0, 0xE0), 0, DotLine));
+	p->setPen(QPen(QColor(0xE0, 0xE0, 0xE0), 0, Qt::DotLine));
 	p->drawLine(0, h-1, w-1, h-1);
 }
 
@@ -158,10 +165,10 @@ public:
 
 public slots:
 	void doApply();
+	void openTab(QString id);
 
 private slots:
-	void itemSelected(QListViewItem *);
-	void openTab(QString id);
+	void itemSelected(Q3ListViewItem *);
 	void dataChanged();
 	void noDirtySlot(bool);
 	void createTabs();
@@ -175,10 +182,10 @@ public:
 	PsiCon *psi;
 	Options opt;
 	bool dirty, noDirty;
-	QDict<QWidget> id2widget;
-	QPtrList<OptionsTab> tabs;
+	Q3Dict<QWidget> id2widget;
+	Q3PtrList<OptionsTab> tabs;
 
-	QMap<QString, QCString> changedMap;
+	QMap<QString, QByteArray> changedMap;
 };
 
 OptionsDlg::Private::Private(OptionsDlg *d, PsiCon *_psi, const Options &_opt)
@@ -193,13 +200,13 @@ OptionsDlg::Private::Private(OptionsDlg *d, PsiCon *_psi, const Options &_opt)
 
 	dlg->lv_tabs->setSorting( -1 );
 	dlg->lv_tabs->header()->hide();
-	connect(dlg->lv_tabs, SIGNAL(selectionChanged(QListViewItem *)), SLOT(itemSelected(QListViewItem *)));
+	connect(dlg->lv_tabs, SIGNAL(selectionChanged(Q3ListViewItem *)), SLOT(itemSelected(Q3ListViewItem *)));
 
 	createTabs();
 	createChangedMap();
 
 	// fill the QListView
-	QPtrListIterator<OptionsTab> it ( tabs );
+	Q3PtrListIterator<OptionsTab> it ( tabs );
 	OptionsTab *opttab;
 	for ( ; it.current(); ++it) {
 		opttab = it.current();
@@ -211,10 +218,10 @@ OptionsDlg::Private::Private(OptionsDlg *d, PsiCon *_psi, const Options &_opt)
 		connect(opttab, SIGNAL(connectDataChanged(QWidget *)), SLOT(connectDataChanged(QWidget *)));
 
 		// search for parent
-		QListViewItem *parent = 0, *prev = 0;
+		Q3ListViewItem *parent = 0, *prev = 0;
 		QString parentId = opttab->parentId();
 		if ( !parentId.isEmpty() ) {
-			QListViewItemIterator it2( dlg->lv_tabs );
+			Q3ListViewItemIterator it2( dlg->lv_tabs );
 			for ( ; it2.current(); ++it2) {
 				//qWarning("Searching the QListView %s...", it2.current()->text(1).latin1());
 				if ( it2.current()->text(1) == parentId ) {
@@ -222,7 +229,7 @@ OptionsDlg::Private::Private(OptionsDlg *d, PsiCon *_psi, const Options &_opt)
 					parent = it2.current();
 
 					// notify the parent about the child
-					QPtrListIterator<OptionsTab> it3 ( tabs );
+					Q3PtrListIterator<OptionsTab> it3 ( tabs );
 					OptionsTab *opttab2;
 					for ( ; it3.current(); ++it3) {
 						opttab2 = it3.current();
@@ -242,7 +249,7 @@ OptionsDlg::Private::Private(OptionsDlg *d, PsiCon *_psi, const Options &_opt)
 		//qWarning("****************");
 
 		// search for previous item
-		QListViewItem *top;
+		Q3ListViewItem *top;
 		if ( parent )
 			top = parent->firstChild();
 		else
@@ -258,7 +265,7 @@ OptionsDlg::Private::Private(OptionsDlg *d, PsiCon *_psi, const Options &_opt)
 			continue;
 
 		// create tab
-		QListViewItem *item;
+		Q3ListViewItem *item;
 		//if ( parent )
 		//	item = new FancyItem(parent, prev);
 		//else
@@ -278,7 +285,7 @@ OptionsDlg::Private::Private(OptionsDlg *d, PsiCon *_psi, const Options &_opt)
 	// fix the width of the listview based on the largest item
 	int largestWidth = 0;
 	QFontMetrics fm(dlg->lv_tabs->font());
-	for(QListViewItem *i = dlg->lv_tabs->firstChild(); i; i = i->nextSibling())
+	for(Q3ListViewItem *i = dlg->lv_tabs->firstChild(); i; i = i->nextSibling())
 		largestWidth = QMAX(largestWidth, i->width(fm, dlg->lv_tabs, 0));
 	dlg->lv_tabs->setFixedWidth(largestWidth + 32);
 
@@ -292,7 +299,7 @@ void OptionsDlg::Private::createTabs()
 {
 	// tabs - base
 	/*tabs.append( new OptionsTabGeneral(this) );
-	//tabs.append( new OptionsTabBase(this, "general",  "", "psi/psi16",	tr("General"),		tr("General preferences list")) );
+	//tabs.append( new OptionsTabBase(this, "general",  "", "psi/logo_16",	tr("General"),		tr("General preferences list")) );
 	tabs.append( new OptionsTabEvents(this) );
 	//tabs.append( new OptionsTabBase(this, "events",   "", "psi/system",	tr("Events"),		tr("Change the events behaviour")) );
 	tabs.append( new OptionsTabPresence(this) );
@@ -314,10 +321,13 @@ void OptionsDlg::Private::createTabs()
 	//tabs.append( new OptionsTabIconsetEmoticons(this) );
 	tabs.append( new OptionsTabGroupchat(this) );
 	tabs.append( new OptionsTabSound(this) );
+	tabs.append( new OptionsTabToolbars(this) );
+#ifdef PSI_PLUGINS
+	tabs.append( new OptionsTabPlugins(this) );
+#endif
+	tabs.append( new OptionsTabShortcuts(this) );
 	tabs.append( new OptionsTabAdvanced(this) );
 
-	tabs.append( new OptionsTabLookFeelToolbars(this) ); // it needs to be there, otherwise toolbar options would not be saved correctly in all cases
-
 	// tabs - general
 	/*tabs.append( new OptionsTabGeneralRoster(this) );
 	tabs.append( new OptionsTabGeneralDocking(this) );
@@ -355,8 +365,10 @@ void OptionsDlg::Private::createChangedMap()
 	//   Instead, connect the widget's signal to your tab own dataChaged() signal
 	changedMap.insert("QButton", SIGNAL(stateChanged(int)));
 	changedMap.insert("QCheckBox", SIGNAL(stateChanged(int)));
-	changedMap.insert("QPushButton", SIGNAL(stateChanged(int)));
-	changedMap.insert("QRadioButton", SIGNAL(stateChanged(int)));
+	//qt4 port: there are no stateChangedSignals anymore
+	//changedMap.insert("QPushButton", SIGNAL(stateChanged(int)));
+	//changedMap.insert("QRadioButton", SIGNAL(stateChanged(int)));
+	changedMap.insert("QRadioButton",SIGNAL(toggled (bool)));
 	changedMap.insert("QComboBox", SIGNAL(activated (int)));
 	//changedMap.insert("QComboBox", SIGNAL(textChanged(const QString &)));
 	changedMap.insert("QDateEdit", SIGNAL(valueChanged(const QDate &)));
@@ -386,7 +398,7 @@ void OptionsDlg::Private::openTab(QString id)
 	QWidget *tab = id2widget[id];
 	if ( !tab ) {
 		bool found = false;
-		QPtrListIterator<OptionsTab> it ( tabs );
+		Q3PtrListIterator<OptionsTab> it ( tabs );
 		OptionsTab *opttab;
 		for ( ; it.current(); ++it) {
 			opttab = it.current();
@@ -400,6 +412,8 @@ void OptionsDlg::Private::openTab(QString id)
 				// idea: maybe do it only for those, whose sizeHint is bigger than ws_tabs'
 				QWidget *w = new QWidget(dlg->ws_tabs, "QWidgetStack/tab");
 				QVBoxLayout *vbox = new QVBoxLayout(w);
+				vbox->setSpacing(0);
+				vbox->setMargin(0);
 
 				/*FancyLabel *toplbl = new FancyLabel(w, "QWidgetStack/tab/FancyLabel");
 				toplbl->setText( opttab->name() );
@@ -437,7 +451,7 @@ void OptionsDlg::Private::openTab(QString id)
 	}
 
 	{
-		QPtrListIterator<OptionsTab> it ( tabs );
+		Q3PtrListIterator<OptionsTab> it ( tabs );
 		OptionsTab *opttab;
 		for ( ; it.current(); ++it) {
 			opttab = it.current();
@@ -445,7 +459,7 @@ void OptionsDlg::Private::openTab(QString id)
 			if ( opttab->id() == id.latin1() ) {
 				dlg->lb_pageTitle->setText( opttab->name() );
 				dlg->lb_pageTitle->setHelp( opttab->desc() );
-				dlg->lb_pageTitle->setIcon( opttab->icon() );
+				dlg->lb_pageTitle->setPsiIcon( opttab->psiIcon() );
 
 				break;
 			}
@@ -455,7 +469,7 @@ void OptionsDlg::Private::openTab(QString id)
 	dlg->ws_tabs->raiseWidget( tab );
 
 	// and select item in lv_tabs...
-	QListViewItemIterator it( dlg->lv_tabs );
+	Q3ListViewItemIterator it( dlg->lv_tabs );
 	while ( it.current() ) {
 		it.current()->setSelected( it.current()->text(1) == id );
 		++it;
@@ -464,23 +478,18 @@ void OptionsDlg::Private::openTab(QString id)
 
 void OptionsDlg::Private::connectDataChanged(QWidget *widget)
 {
-	QObjectList *l = widget->queryList( "QWidget", 0, false, true ); // search for all QWidget children of widget
-	QObjectListIterator it( *l );
-
-	for ( ; it.current(); ++it) {
-		QWidget *w = (QWidget *)it.current();
-
-		QMap<QString, QCString>::Iterator it2 = changedMap.find( w->className() );
+	QObjectList l = widget->queryList( "QWidget", 0, false, true ); // search for all QWidget children of widget
+	for ( QObjectList::Iterator it = l.begin(); it != l.end(); ++it) {
+		QWidget *w = (QWidget*) (*it);
+		QMap<QString, QByteArray>::Iterator it2 = changedMap.find( w->className() );
 		if ( it2 != changedMap.end() ) {
 			disconnect(w, changedMap[w->className()], this, SLOT(dataChanged()));
 			connect(w, changedMap[w->className()], SLOT(dataChanged()));
 		}
 	}
-
-	delete l;
 }
 
-void OptionsDlg::Private::itemSelected(QListViewItem *item)
+void OptionsDlg::Private::itemSelected(Q3ListViewItem *item)
 {
 	if ( !item )
 		return;
@@ -509,7 +518,7 @@ void OptionsDlg::Private::doApply()
 	if ( !dirty )
 		return;
 
-	QPtrListIterator<OptionsTab> it ( tabs );
+	Q3PtrListIterator<OptionsTab> it ( tabs );
 	OptionsTab *opttab;
 	for ( ; it.current(); ++it) {
 		opttab = it.current();
@@ -527,14 +536,22 @@ void OptionsDlg::Private::doApply()
 // OptionsDlg
 //----------------------------------------------------------------------------
 
-OptionsDlg::OptionsDlg(PsiCon *psi, const Options &opt, QWidget *parent, const char *name)
-: OptionsUI(parent, name, false, WDestructiveClose)
+OptionsDlg::OptionsDlg(PsiCon *psi, const Options &opt, QWidget *parent)
+	: QDialog(parent)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
+	setupUi(this);
 	d = new Private(this, psi, opt);
+	setModal(false);
 	d->psi->dialogRegister(this);
 
-	setCaption(CAP(caption()));
+	setWindowTitle(CAP(caption()));
 	resize(640, 480);
+
+	connect(pb_ok, SIGNAL(clicked()), SLOT(doOk()));
+	connect(pb_apply,SIGNAL(clicked()),SLOT(doApply()));
+	connect(pb_cancel, SIGNAL(clicked()), SLOT(reject()));
+
 }
 
 OptionsDlg::~OptionsDlg()
@@ -543,6 +560,11 @@ OptionsDlg::~OptionsDlg()
 	delete d;
 }
 
+void OptionsDlg::openTab(const QString& id)
+{
+	d->openTab(id);
+}
+
 void OptionsDlg::doOk()
 {
 	doApply();
