@@ -1,18 +1,21 @@
 #ifndef OPTIONSDLG_H
 #define OPTIONSDLG_H
 
-#include "ui_options.h"
+#include "ui_ui_options.h"
+#include <QDialog>
 
 class PsiCon;
 struct Options;
 
-class OptionsDlg : public OptionsUI
+class OptionsDlg : public QDialog, public Ui::OptionsUI
 {
 	Q_OBJECT
 public:
-	OptionsDlg(PsiCon *, const Options &, QWidget *parent = 0, const char *name = 0);
+	OptionsDlg(PsiCon *, const Options &, QWidget *parent = 0);
 	~OptionsDlg();
 
+	void openTab(const QString& id);
+
 signals:
 	void applyOptions(const Options &);
 
