@@ -4,6 +4,7 @@
 #include <qtabwidget.h>
 #include <qlayout.h>
 #include <qmap.h>
+#include <QVBoxLayout>
 
 //----------------------------------------------------------------------------
 // OptionsTab
@@ -14,7 +15,7 @@ OptionsTab::OptionsTab(QObject *parent, const char *name)
 {
 }
 
-OptionsTab::OptionsTab(QObject *parent, QCString _id, QCString _parentId, QString _name, QString _desc, QString _tabIconName, QString _iconName)
+OptionsTab::OptionsTab(QObject *parent, QByteArray _id, QByteArray _parentId, QString _name, QString _desc, QString _tabIconName, QString _iconName)
 : QObject(parent, _name.latin1())
 {
 	v_id = _id;
@@ -29,12 +30,12 @@ OptionsTab::~OptionsTab()
 {
 }
 
-QCString OptionsTab::id() const
+QByteArray OptionsTab::id() const
 {
 	return v_id;
 }
 
-QCString OptionsTab::parentId() const
+QByteArray OptionsTab::parentId() const
 {
 	return v_parentId;
 }
@@ -44,12 +45,12 @@ QString OptionsTab::tabName() const
 	return v_name;
 }
 
-Icon *OptionsTab::tabIcon() const
+PsiIcon *OptionsTab::tabIcon() const
 {
 	if ( v_tabIconName.isEmpty() )
 		return 0;
 
-	return (Icon *)IconsetFactory::iconPtr( v_tabIconName );
+	return (PsiIcon *)IconsetFactory::iconPtr( v_tabIconName );
 }
 
 QString OptionsTab::name() const
@@ -62,16 +63,16 @@ QString OptionsTab::desc() const
 	return v_desc;
 }
 
-Icon *OptionsTab::icon() const
+PsiIcon *OptionsTab::psiIcon() const
 {
 	if ( v_iconName.isEmpty() ) {
 		//if ( tabIcon() )
 		//	return tabIcon();
 
-		return (Icon *)IconsetFactory::iconPtr("psi/psi32");
+		return (PsiIcon *)IconsetFactory::iconPtr("psi/logo_32");
 	}
 
-	return (Icon *)IconsetFactory::iconPtr( v_iconName );
+	return (PsiIcon *)IconsetFactory::iconPtr( v_iconName );
 }
 
 void OptionsTab::applyOptions(Options *)
@@ -137,17 +138,25 @@ void OptionsTabWidget::addTab(OptionsTab *tab)
 	if ( tab->tabName().isEmpty() )
 		return; // skip the dummy tabs
 
-	QWidget *w = new QWidget(this, tab->name().latin1());
+	// the widget will have no parent; it will be reparented
+	// when inserting it with "addTab"
+	QWidget *w = new QWidget(NULL, tab->name().latin1());
 
 	if ( tab->tabIcon() )
-		insertTab(w, tab->tabIcon()->iconSet(), tab->tabName());
+		QTabWidget::addTab(w, tab->tabIcon()->icon(), tab->tabName());
 	else
-		insertTab(w, tab->tabName());
+		QTabWidget::addTab(w, tab->tabName());
 
 	if ( !tab->desc().isEmpty() )
 		setTabToolTip(w, tab->desc());
 
 	w2tab[w] = TabData(tab);
+	
+	//FIXME: this is safe for our current use of addTab, but may
+	//be inconvenient in the future (Qt circa 4.2 had a bug which stopped
+	//setCurrentIndex(0); from working)
+	setCurrentIndex(1);
+	setCurrentIndex(0);
 }
 
 void OptionsTabWidget::updateCurrent(QWidget *w)
@@ -199,7 +208,7 @@ MetaOptionsTab::MetaOptionsTab(QObject *parent, const char *name)
 	init();
 }
 
-MetaOptionsTab::MetaOptionsTab(QObject *parent, QCString id, QCString parentId, QString name, QString desc, QString tabIconName, QString iconName)
+MetaOptionsTab::MetaOptionsTab(QObject *parent, QByteArray id, QByteArray parentId, QString name, QString desc, QString tabIconName, QString iconName)
 : OptionsTab(parent, id, parentId, name, desc, tabIconName, iconName)
 {
 	init();
@@ -238,16 +247,19 @@ QWidget *MetaOptionsTab::widget()
 	connect(w, SIGNAL(connectDataChanged(QWidget *)), SIGNAL(connectDataChanged(QWidget *)));
 	connect(w, SIGNAL(noDirty(bool)), SIGNAL(noDirty(bool)));
 
-	QPtrListIterator<OptionsTab> it(tabs);
+	Q3PtrListIterator<OptionsTab> it(tabs);
 	for ( ; it.current(); ++it)
 		t->addTab(it.current());
 
+	// set the current widget to 0, otherwise qt4 will show no widget
+	t->setCurrentIndex(0);
+
 	return w;
 }
 
 void MetaOptionsTab::applyOptions(Options *opt)
 {
-	QPtrListIterator<OptionsTab> it(tabs);
+	Q3PtrListIterator<OptionsTab> it(tabs);
 	for ( ; it.current(); ++it)
 		it.current()->applyOptions(opt);
 }
@@ -259,14 +271,14 @@ void MetaOptionsTab::restoreOptions(const Options *opt)
 		d->restoreOptions(opt);
 	}
 
-	QPtrListIterator<OptionsTab> it(tabs);
+	Q3PtrListIterator<OptionsTab> it(tabs);
 	for ( ; it.current(); ++it)
 		it.current()->restoreOptions(opt);
 }
 
 void MetaOptionsTab::setData(PsiCon *psi, QWidget *w)
 {
-	QPtrListIterator<OptionsTab> it(tabs);
+	Q3PtrListIterator<OptionsTab> it(tabs);
 	for ( ; it.current(); ++it)
 		it.current()->setData(psi, w);
 }
