@@ -18,48 +18,27 @@
  *
  */
 
-#include"passphrasedlg.h"
+#include <QLineEdit>
+#include <QPushButton>
 
-#include<qlineedit.h>
-#include<qradiobutton.h>
-#include<qpushbutton.h>
-#include"iconwidget.h"
+#include "passphrasedlg.h"
 
-PassphraseDlg::PassphraseDlg(QWidget *parent, const char *name)
-:PassphraseUI(parent, name, true, WDestructiveClose)
+PassphraseDlg::PassphraseDlg(QWidget *parent) : QDialog (parent)
 {
-	connect(pb_ok, SIGNAL(clicked()), SLOT(doOK()));
-	connect(pb_cancel, SIGNAL(clicked()), SLOT(reject()));
+	ui_.setupUi(this);
+	setModal(false);
+	connect(ui_.pb_ok, SIGNAL(clicked()), SLOT(accept()));
+	connect(ui_.pb_cancel, SIGNAL(clicked()), SLOT(reject()));
+}
+
+void PassphraseDlg::promptPassphrase(const QString& name)
+{
+	setWindowTitle(tr("%1: OpenPGP Passphrase").arg(name));
 	resize(minimumSize());
 }
 
-PassphraseDlg::~PassphraseDlg()
+QString PassphraseDlg::getPassphrase() const
 {
+	return ui_.le_pass->text();
 }
 
-QString PassphraseDlg::passphrase() const
-{
-	return le_pass->text();
-}
-
-void PassphraseDlg::reject()
-{
-	rejectPassphrase();
-	QDialog::reject();
-}
-
-void PassphraseDlg::doOK()
-{
-	le_pass->setEnabled(false);
-	pb_ok->setEnabled(false);
-
-	submitPassphrase(le_pass->text());
-}
-
-void PassphraseDlg::unblock()
-{
-	le_pass->setText("");
-	le_pass->setEnabled(true);
-	pb_ok->setEnabled(true);
-	le_pass->setFocus();
-}
