@@ -18,22 +18,25 @@
  *
  */
 
-#include"profiledlg.h"
-#include"common.h"
+#include "profiledlg.h"
+#include "applicationinfo.h"
+#include "iconset.h"
 
-#include<qlabel.h>
-#include<qcombobox.h>
-#include<qcheckbox.h>
-#include<qpushbutton.h>
-#include<qgroupbox.h>
-#include<qlistbox.h>
-#include<qinputdialog.h>
-#include<qfile.h>
-#include<qfileinfo.h>
-#include<qbuttongroup.h>
-#include<qwhatsthis.h>
-#include"profiles.h"
-#include"iconwidget.h"
+#include <QLabel>
+#include <QComboBox>
+#include <QMessageBox>
+#include <QCheckBox>
+#include <QPushButton>
+#include <Q3Button>
+#include <Q3GroupBox>
+#include <Q3ListBox>
+#include <QInputDialog>
+#include <QFile>
+#include <QFileInfo>
+#include <Q3ButtonGroup>
+#include <QPixmap>
+#include "profiles.h"
+#include "iconwidget.h"
 
 #include <qpainter.h>
 class StretchLogoLabel : public QLabel
@@ -47,16 +50,21 @@ public:
 		replaceWidget(label, this);
 	}
 
-	void drawContents(QPainter *p)
+	void paintEvent(QPaintEvent *event)
 	{
+		QPainter *p = new QPainter(this);
 		p->drawTiledPixmap(0, 0, width(), height(), *pixmap());
+		delete p;
+		QLabel::paintEvent(event);
 	}
 };
 
-ProfileOpenDlg::ProfileOpenDlg(const QString &def, const VarList &_langs, const QString &curLang, QWidget *parent, const char *name)
-:ProfileOpenUI(parent, name, TRUE)
+ProfileOpenDlg::ProfileOpenDlg(const QString &def, const VarList &_langs, const QString &curLang, QWidget *parent)
+:QDialog(parent)
 {
-	setCaption(CAP(caption()));
+	setupUi(this);
+	setModal(true);
+	setWindowTitle(CAP(caption()));
 
 	langs = _langs;
 
@@ -94,12 +102,12 @@ ProfileOpenDlg::ProfileOpenDlg(const QString &def, const VarList &_langs, const
 	}
 
 	// QWhatsThis helpers
-	QWhatsThis::add(cb_profile,
+	cb_profile->setWhatsThis(
 		tr("Select a profile to open from this list."));
-	QWhatsThis::add(cb_lang,
+	cb_lang->setWhatsThis(
 		tr("Select a language you would like Psi to use from this "
 		"list.  You can download extra language packs from the Psi homepage."));
-	QWhatsThis::add(ck_auto,
+	ck_auto->setWhatsThis(
 		tr("Automatically open this profile when Psi is started.  Useful if "
 		"you only have one profile."));
 
@@ -159,10 +167,12 @@ void ProfileOpenDlg::langChange(int x)
 }
 
 
-ProfileManageDlg::ProfileManageDlg(const QString &choose, QWidget *parent, const char *name)
-:ProfileManageUI(parent, name, TRUE)
+ProfileManageDlg::ProfileManageDlg(const QString &choose, QWidget *parent)
+:QDialog(parent)
 {
-	setCaption(CAP(caption()));
+	setupUi(this);
+	setModal(true);
+	setWindowTitle(CAP(caption()));
 
 	// setup signals
 	connect(pb_new, SIGNAL(clicked()), SLOT(slotProfileNew()));
@@ -238,7 +248,7 @@ void ProfileManageDlg::slotProfileDelete()
 	if(x == -1)
 		return;
 	QString name = lbx_profiles->text(x);
-	QString path = g.pathProfiles + "/" + name;
+	QString path = ApplicationInfo::profilesDir() + "/" + name;
 
 	// prompt first
 	int r = QMessageBox::warning(this,
@@ -289,12 +299,14 @@ void ProfileManageDlg::updateSelection()
 }
 
 
-ProfileNewDlg::ProfileNewDlg(QWidget *parent, const char *name)
-:ProfileNewUI(parent, name, TRUE)
+ProfileNewDlg::ProfileNewDlg(QWidget *parent)
+:QDialog(parent)
 {
-	setCaption(CAP(caption()));
+	setupUi(this);
+	setModal(true);
+	setWindowTitle(CAP(caption()));
 
-	bg_defAct->setButton(bg_defAct->id((QButton *)rb_chat));
+	bg_defAct->setButton(bg_defAct->id((Q3Button *)rb_chat));
 	le_name->setFocus();
 
 	connect(pb_create, SIGNAL(clicked()), SLOT(slotCreate()));
@@ -320,7 +332,7 @@ void ProfileNewDlg::slotCreate()
 
 	// save config
 	UserProfile p;
-	p.prefs.defaultAction = bg_defAct->selected() == (QButton *)rb_message ? 0: 1;
+	p.prefs.defaultAction = bg_defAct->selected() == (Q3Button *)rb_message ? 0: 1;
 	p.prefs.useEmoticons = ck_useEmoticons->isChecked();
 	p.toFile(pathToProfileConfig(name));
 
