@@ -21,12 +21,17 @@
 #ifndef PROFILES_H
 #define PROFILES_H
 
-#include<qstring.h>
-#include<qstringlist.h>
-#include<qvaluelist.h>
-#include"im.h"
-#include"proxy.h"
-#include"common.h"
+#include <qstring.h>
+#include <qstringlist.h>
+#include <QList>
+#include <QtCrypto>
+
+#include "varlist.h"
+#include "proxy.h"
+#include "common.h"
+#include "xmpp_clientstream.h"
+#include "xmpp_roster.h"
+#include "xmpp_jid.h"
 
 class UserAccount
 {
@@ -40,16 +45,22 @@ public:
 	void fromXml(const QDomElement &);
 
 	QString name;
-	QString jid, pass, host, resource;
+	QString jid, pass, host, resource, authid, realm;
+	bool customAuth;
 	int port, priority;
- 	bool opt_enabled, opt_ssl, opt_pass, opt_host, opt_auto, opt_keepAlive, opt_plain, opt_log, opt_reconn, opt_ignoreSSLWarnings;
+ 	bool opt_enabled, opt_pass, opt_host, opt_auto, opt_keepAlive, opt_log, opt_reconn, opt_ignoreSSLWarnings, opt_compress;
+	XMPP::ClientStream::AllowPlainType allow_plain;
 	bool tog_offline, tog_away, tog_agents, tog_hidden, tog_self;
+	bool req_mutual_auth;
+	bool legacy_ssl_probe;
+	bool opt_automatic_resource;
+	int security_level;
+	enum SSLFlag { SSL_No = 0, SSL_Yes = 1, SSL_Auto = 2, SSL_Legacy = 3 } ssl;
 
 	int proxy_index;
 	int proxy_type, proxy_port;
 	QString proxy_host, proxy_user, proxy_pass;
 
-	QString olr_string;
 	XMPP::Roster roster;
 
 	struct GroupData {
@@ -58,14 +69,14 @@ public:
 	};
 	QMap<QString, GroupData> groupState;
 
-	QString pgpSecretKeyID;
+	QCA::PGPKey pgpSecretKey;
 
 	VarList keybind;
 
 	XMPP::Jid dtProxy;
 };
 
-typedef QValueList<UserAccount> UserAccountList;
+typedef QList<UserAccount> UserAccountList;
 
 
 class UserProfile
@@ -98,4 +109,6 @@ bool profileNew(const QString &);
 bool profileRename(const QString &, const QString &);
 bool profileDelete(const QString &);
 
+extern QString activeProfile;
+
 #endif
