@@ -18,23 +18,63 @@
  *
  */
 
-#include"profiles.h"
-#include"common.h"
-#include<qdir.h>
-#include<qfileinfo.h>
-#include<qdom.h>
+#include "profiles.h"
+#include "common.h"
+#include "applicationinfo.h"
+#include <qdir.h>
+#include <qfileinfo.h>
+#include <qdom.h>
 
-#include<qapplication.h>
-#include"eventdlg.h"
-#include"chatdlg.h"
-#include"contactview.h"
-#include"xmpp_xmlcommon.h"
-#include"fancylabel.h"
-#include"advwidget.h"
+#include <qapplication.h>
+//Added by qt3to4:
+#include <QTextStream>
+#include <QtCrypto>
+#include <QList>
+
+#include "eventdlg.h"
+#include "chatdlg.h"
+#include "pgputil.h"
+#include "xmpp_xmlcommon.h"
+#include "fancylabel.h"
+#include "advwidget.h"
+#include "psioptions.h"
+#include "varlist.h"
 
 using namespace XMPP;
 using namespace XMLHelper;
 
+#define PROXY_NONE       0
+#define PROXY_HTTPS      1
+#define PROXY_SOCKS4     2
+#define PROXY_SOCKS5     3
+
+template<typename T, typename F>
+void migrateEntry(const QDomElement& element, const QString& entry, const QString& option, F f)
+{
+	bool found;
+	findSubTag(element, entry, &found);
+	if (found) {
+		T value;
+		f(element, entry, &value);
+		PsiOptions::instance()->setOption(option, value);
+	}
+}
+
+void migrateIntEntry(const QDomElement& element, const QString& entry, const QString& option)
+{
+	migrateEntry<int>(element, entry, option, readNumEntry);
+}
+
+void migrateBoolEntry(const QDomElement& element, const QString& entry, const QString& option)
+{
+	migrateEntry<bool>(element, entry, option, readBoolEntry);
+}
+
+void migrateSizeEntry(const QDomElement& element, const QString& entry, const QString& option)
+{
+	migrateEntry<QSize>(element, entry, option, readSizeEntry);
+}
+
 UserAccount::UserAccount()
 {
 	reset();
@@ -45,27 +85,31 @@ void UserAccount::reset()
 	name = "Default";
 	opt_enabled = TRUE;
 	opt_auto = FALSE;
-	opt_ssl = FALSE;
 	tog_offline = TRUE;
 	tog_away = TRUE;
-	tog_hidden = TRUE;
+	tog_hidden = FALSE;
 	tog_agents = TRUE;
 	tog_self = FALSE;
+	customAuth = FALSE;
+	req_mutual_auth = FALSE;
+	legacy_ssl_probe = TRUE;
+	security_level = QCA::SL_None;
+	ssl = SSL_Auto;
 	jid = "";
 	pass = "";
 	opt_pass = FALSE;
 	port = 5222;
 	opt_host = FALSE;
 	host = "";
+	opt_automatic_resource = TRUE;
 	resource = "Psi";
 	priority = 5;
-	olr_string = "";
 	opt_keepAlive = TRUE;
-	opt_plain = FALSE;
+	allow_plain = XMPP::ClientStream::AllowPlainOverTLS;
+	opt_compress = FALSE;
 	opt_log = TRUE;
 	opt_reconn = FALSE;
 	opt_ignoreSSLWarnings = false;
-	pgpSecretKeyID = "";
 
 	proxy_index = 0;
 	proxy_type = PROXY_NONE;
@@ -90,14 +134,16 @@ QDomElement UserAccount::toXml(QDomDocument &doc, const QString &tagName)
 
 	setBoolAttribute(a, "enabled", opt_enabled);
 	setBoolAttribute(a, "auto", opt_auto);
-	setBoolAttribute(a, "ssl", opt_ssl);
 	setBoolAttribute(a, "showOffline", tog_offline);
 	setBoolAttribute(a, "showAway", tog_away);
 	setBoolAttribute(a, "showHidden", tog_hidden);
 	setBoolAttribute(a, "showAgents", tog_agents);
 	setBoolAttribute(a, "showSelf", tog_self);
 	setBoolAttribute(a, "keepAlive", opt_keepAlive);
-	setBoolAttribute(a, "plain", opt_plain);
+	setBoolAttribute(a, "compress", opt_compress);
+	setBoolAttribute(a, "require-mutual-auth", req_mutual_auth);
+	setBoolAttribute(a, "legacy-ssl-probe", legacy_ssl_probe);
+	setBoolAttribute(a, "automatic-resource", opt_automatic_resource);
 	setBoolAttribute(a, "log", opt_log);
 	setBoolAttribute(a, "reconn", opt_reconn);
 	setBoolAttribute(a, "ignoreSSLWarnings", opt_ignoreSSLWarnings);
@@ -106,16 +152,29 @@ QDomElement UserAccount::toXml(QDomDocument &doc, const QString &tagName)
 	//QString jid = user + '@' + vhost;
 	a.appendChild(textTag(doc, "name", name));
 	a.appendChild(textTag(doc, "jid", jid));
+	
+	QDomElement customauth = doc.createElement("custom-auth");
+	setBoolAttribute(customauth, "use", customAuth);
+	QDomElement ac = textTag(doc, "authid", authid);
+	QDomElement re = textTag(doc, "realm", realm);
+	customauth.appendChild(ac);
+	customauth.appendChild(re);
+	a.appendChild(customauth);
+
 	if(opt_pass)
 		a.appendChild(textTag(doc, "password", encodePassword(pass, jid) ));
 	a.appendChild(textTag(doc, "useHost", opt_host));
+	a.appendChild(textTag(doc, "security-level", security_level));
+	a.appendChild(textTag(doc, "ssl", ssl));
 	a.appendChild(textTag(doc, "host", host));
 	a.appendChild(textTag(doc, "port", QString::number(port)));
 	a.appendChild(textTag(doc, "resource", resource));
 	a.appendChild(textTag(doc, "priority", QString::number(priority)));
-	a.appendChild(textTag(doc, "OLR", olr_string));
-	a.appendChild(textTag(doc, "pgpSecretKeyID", pgpSecretKeyID));
-
+	if (!pgpSecretKey.isNull()) {
+		a.appendChild(textTag(doc, "pgpSecretKeyID", pgpSecretKey.keyId()));
+	}
+	a.appendChild(textTag(doc, "allow-plain", allow_plain));
+	
 	QDomElement r = doc.createElement("roster");
 	a.appendChild(r);
 	Roster::ConstIterator rit = roster.begin();
@@ -130,8 +189,8 @@ QDomElement UserAccount::toXml(QDomDocument &doc, const QString &tagName)
 	groupList << "/\\/" + name + "\\/\\"; // account name is a very 'special' group
 
 	// special groups that should also have their state remembered
-	groupList << ContactProfile::tr("General");
-	groupList << ContactProfile::tr("Agents/Transports");
+	groupList << qApp->translate("ContactProfile", "General");
+	groupList << qApp->translate("ContactProfile", "Agents/Transports");
 
 	// first, add all groups' names to groupList
 	rit = roster.begin();
@@ -197,19 +256,40 @@ void UserAccount::fromXml(const QDomElement &a)
 	readEntry(a, "name", &name);
 	readBoolAttribute(a, "enabled", &opt_enabled);
 	readBoolAttribute(a, "auto", &opt_auto);
-	readBoolAttribute(a, "ssl", &opt_ssl);
 	readBoolAttribute(a, "showOffline", &tog_offline);
 	readBoolAttribute(a, "showAway", &tog_away);
 	readBoolAttribute(a, "showHidden", &tog_hidden);
 	readBoolAttribute(a, "showAgents", &tog_agents);
 	readBoolAttribute(a, "showSelf", &tog_self);
 	readBoolAttribute(a, "keepAlive", &opt_keepAlive);
-	readBoolAttribute(a, "plain", &opt_plain);
+	readBoolAttribute(a, "compress", &opt_compress);
+	readBoolAttribute(a, "require-mutual-auth", &req_mutual_auth);
+	readBoolAttribute(a, "legacy-ssl-probe", &legacy_ssl_probe);
 	readBoolAttribute(a, "log", &opt_log);
 	readBoolAttribute(a, "reconn", &opt_reconn);
 	readBoolAttribute(a, "ignoreSSLWarnings", &opt_ignoreSSLWarnings);
 	//readBoolAttribute(a, "gpg", &opt_gpg);
+	if (a.hasAttribute("automatic-resource")) {
+		readBoolAttribute(a, "automatic-resource", &opt_automatic_resource);
+	}
+	else {
+		opt_automatic_resource = false;
+	}
+	
+	// Will be overwritten if there is a new option
+	bool opt_plain = false;
+	readBoolAttribute(a, "plain", &opt_plain);
+	allow_plain = (opt_plain ? XMPP::ClientStream::AllowPlain : XMPP::ClientStream::NoAllowPlain);
+	readNumEntry(a, "allow-plain", (int*) &allow_plain);
+	
+	// Will be overwritten if there is a new option
+	bool opt_ssl = true;
+	readBoolAttribute(a, "ssl", &opt_ssl);
+	if (opt_ssl)
+		ssl = UserAccount::SSL_Legacy;
 
+	readNumEntry(a, "security-level", &security_level);
+	readNumEntry(a, "ssl", (int*) &ssl);
 	readEntry(a, "host", &host);
 	readNumEntry(a, "port", &port);
 
@@ -246,10 +326,26 @@ void UserAccount::fromXml(const QDomElement &a)
 		pass = decodePassword(pass, jid);
 	}
 
+	QDomElement ca = findSubTag(a, "custom-auth", &found);
+	if(found) {
+		readBoolAttribute(ca, "use", &customAuth);
+		QDomElement authid_el = findSubTag(ca, "authid", &found);
+		if (found)
+			authid = tagContent(authid_el);
+		QDomElement realm_el = findSubTag(ca, "realm", &found);
+		if (found)
+			realm = tagContent(realm_el);
+	}
+
 	readEntry(a, "resource", &resource);
 	readNumEntry(a, "priority", &priority);
-	readEntry(a, "OLR", &olr_string);
+	QString pgpSecretKeyID;
 	readEntry(a, "pgpSecretKeyID", &pgpSecretKeyID);
+	if (!pgpSecretKeyID.isEmpty()) {
+		QCA::KeyStoreEntry e = PGPUtil::instance().getSecretKeyStoreEntry(pgpSecretKeyID);
+		if (!e.isNull())
+			pgpSecretKey = e.pgpSecretKey();
+	}
 
 	QDomElement r = findSubTag(a, "roster", &found);
 	if(found) {
@@ -322,8 +418,6 @@ void UserProfile::reset()
 	mac = TRUE;
 #endif
 
-	progver = PROG_VERSION;
-
 	// global
 	mwgeom.setRect(64, 64, 150, 360);
 	lastStatusString = "";
@@ -334,24 +428,24 @@ void UserProfile::reset()
 	// prefs
 	prefs.useleft = FALSE;
 	prefs.singleclick = FALSE;
- 	prefs.hideMenubar = TRUE;
+ 	prefs.hideMenubar = FALSE;
 	prefs.defaultAction = 1;
 	prefs.delChats = dcHour;
-	prefs.showJoins = TRUE;
 	prefs.browser = 0;
-	prefs.customBrowser = "";
-	prefs.customMailer = "";
 	prefs.alwaysOnTop = FALSE;
 	prefs.keepSizes = TRUE;
 	prefs.ignoreHeadline = FALSE;
 	prefs.ignoreNonRoster = FALSE;
 	prefs.excludeGroupChatsFromIgnore = TRUE;
-	prefs.useDock = win ? TRUE: FALSE;
+	prefs.useDock = true;
 	prefs.dockDCstyle = win ? TRUE: FALSE;
 	prefs.dockHideMW = FALSE;
 	prefs.dockToolMW = FALSE;
-	prefs.isWMDock = false;
+#ifdef Q_WS_MAC
+	prefs.alertStyle = 0;
+#else
 	prefs.alertStyle = 2;
+#endif
 	prefs.popupMsgs = FALSE;
 	prefs.popupChats = FALSE;
 	prefs.popupHeadlines = FALSE;
@@ -363,7 +457,6 @@ void UserProfile::reset()
 	prefs.askOnline = FALSE;
 	prefs.askOffline = FALSE;
 	prefs.rosterAnim = TRUE;
-	prefs.autoVersion = false;
 	prefs.autoVCardOnLogin = true;
 	prefs.xmlConsoleOnLogin = false;
 	prefs.asAway = 10;
@@ -375,14 +468,14 @@ void UserProfile::reset()
 	prefs.asMessage = QObject::tr("Auto Status (idle)");
 	prefs.scrollTo = TRUE;
 	prefs.useEmoticons = false;
-	prefs.alertOpenChats = false;
+	prefs.alertOpenChats = true;
 	prefs.raiseChatWindow = false;
 	prefs.showSubjects = true;
 	prefs.showCounter = false;
 	prefs.chatSays = false;
-	prefs.chatSoftReturn = true;
 	prefs.showGroupCounts = true;
 	prefs.smallChats = false;
+	prefs.brushedMetal = false;
 	prefs.chatLineEdit = true;
 	prefs.useTabs = false;
 	prefs.usePerTabCloseButton = true;
@@ -393,25 +486,39 @@ void UserProfile::reset()
 	prefs.chatBgImage = "";
 	prefs.rosterBgImage = "";
 	prefs.autoCopy = false;
+	prefs.useCaps = true;
+	prefs.useRC = false;
 
 	prefs.sp.clear();
-	prefs.sp.set(QObject::tr("Away from desk"), QObject::tr("I am away from my desk.  Leave a message."));
-	prefs.sp.set(QObject::tr("Showering"), QObject::tr("I'm in the shower.  You'll have to wait for me to get out."));
-	prefs.sp.set(QObject::tr("Eating"), QObject::tr("Out eating.  Mmmm.. food."));
-	prefs.sp.set(QObject::tr("Sleep"), QObject::tr("Sleep is good.  Zzzzz"));
-	prefs.sp.set(QObject::tr("Work"), QObject::tr("Can't chat.  Gotta work."));
-	prefs.sp.set(QObject::tr("Air"), QObject::tr("Stepping out to get some fresh air."));
-	prefs.sp.set(QObject::tr("Movie"), QObject::tr("Out to a movie.  Is that OK with you?"));
-	prefs.sp.set(QObject::tr("Secret"), QObject::tr("I'm not available right now and that's all you need to know."));
-	prefs.sp.set(QObject::tr("Out for the night"), QObject::tr("Out for the night."));
-	prefs.sp.set(QObject::tr("Greece"), QObject::tr("I have gone to a far away place.  I will be back someday!"));
-	prefs.recentStatus.clear();
+	QString name;
+	name = QObject::tr("Away from desk");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("I am away from my desk.  Leave a message."), STATUS_AWAY);;
+	name = QObject::tr("Showering");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("I'm in the shower.  You'll have to wait for me to get out."), STATUS_AWAY);
+	name = QObject::tr("Eating");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("Out eating.  Mmmm.. food."), STATUS_AWAY);
+	name = QObject::tr("Sleep");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("Sleep is good.  Zzzzz"),STATUS_DND);
+	name = QObject::tr("Work");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("Can't chat.  Gotta work."), STATUS_DND);
+	name = QObject::tr("Air");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("Stepping out to get some fresh air."), STATUS_AWAY);
+	name = QObject::tr("Movie");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("Out to a movie.  Is that OK with you?"), STATUS_AWAY);
+	name = QObject::tr("Secret");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("I'm not available right now and that's all you need to know."), STATUS_XA);
+	name = QObject::tr("Out for the night");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("Out for the night."), STATUS_AWAY);
+	name = QObject::tr("Greece");
+	prefs.sp[name] = StatusPreset(name, QObject::tr("I have gone to a far away place.  I will be back someday!"), STATUS_XA);
+ 	prefs.recentStatus.clear();
 
 	prefs.color[cOnline]    = QColor("#000000");
 	prefs.color[cListBack]  = QColor("#FFFFFF");
 	prefs.color[cAway]      = QColor("#004BB4");
 	prefs.color[cDND]       = QColor("#7E0000");
 	prefs.color[cOffline]   = QColor("#646464");
+	prefs.color[cStatus]    = QColor("#808080");
 	prefs.color[cProfileFore] = QColor("#FFFFFF");
 	prefs.color[cProfileBack] = QColor("#969696");
 	prefs.color[cGroupFore] = QColor("#5A5A5A");
@@ -437,26 +544,24 @@ void UserProfile::reset()
 	FancyLabel::setSmallFontSize( prefs.smallFontSize );
 
 	prefs.clNewHeadings = false;
-	prefs.chatOpacity = 100;
-	prefs.rosterOpacity = 95;
 	prefs.outlineHeadings = false;
 
 	prefs.player = "play";
 	prefs.noAwaySound = FALSE;
 
-	prefs.onevent[eMessage]    = g.pathBase + "/sound/chat2.wav";
-	prefs.onevent[eChat1]      = g.pathBase + "/sound/chat1.wav";
-	prefs.onevent[eChat2]      = g.pathBase + "/sound/chat2.wav";
-	prefs.onevent[eSystem]     = g.pathBase + "/sound/chat2.wav";
-	prefs.onevent[eHeadline]   = g.pathBase + "/sound/chat2.wav";
-	prefs.onevent[eOnline]     = g.pathBase + "/sound/online.wav";
-	prefs.onevent[eOffline]    = g.pathBase + "/sound/offline.wav";
-	prefs.onevent[eSend]       = g.pathBase + "/sound/send.wav";
-	prefs.onevent[eIncomingFT] = g.pathBase + "/sound/ft_incoming.wav";
-	prefs.onevent[eFTComplete] = g.pathBase + "/sound/ft_complete.wav";
+	prefs.onevent[eMessage]    = ApplicationInfo::resourcesDir() + "/sound/chat2.wav";
+	prefs.onevent[eChat1]      = ApplicationInfo::resourcesDir() + "/sound/chat1.wav";
+	prefs.onevent[eChat2]      = ApplicationInfo::resourcesDir() + "/sound/chat2.wav";
+	prefs.onevent[eSystem]     = ApplicationInfo::resourcesDir() + "/sound/chat2.wav";
+	prefs.onevent[eHeadline]   = ApplicationInfo::resourcesDir() + "/sound/chat2.wav";
+	prefs.onevent[eOnline]     = ApplicationInfo::resourcesDir() + "/sound/online.wav";
+	prefs.onevent[eOffline]    = ApplicationInfo::resourcesDir() + "/sound/offline.wav";
+	prefs.onevent[eSend]       = ApplicationInfo::resourcesDir() + "/sound/send.wav";
+	prefs.onevent[eIncomingFT] = ApplicationInfo::resourcesDir() + "/sound/ft_incoming.wav";
+	prefs.onevent[eFTComplete] = ApplicationInfo::resourcesDir() + "/sound/ft_complete.wav";
 
 	// Added by Kiko 020621: sets up the default certificate store
-	prefs.trustCertStoreDir = g.pathBase + "/certs";
+	prefs.trustCertStoreDir = ApplicationInfo::resourcesDir() + "/certs";
 
 	prefs.sizeEventDlg = EventDlg::defaultSize();
 	prefs.sizeChatDlg = ChatDlg::defaultSize();
@@ -466,8 +571,6 @@ void UserProfile::reset()
 	prefs.grabUrls = false;
 	prefs.noGCSound = true;
 
-	prefs.pgp = "";
-
 	// toolbars
 	prefs.toolbars.clear();
 
@@ -508,9 +611,9 @@ void UserProfile::reset()
 		tb[0].dock = Qt::DockBottom;
 
 		tb[1].name = QObject::tr("Show contacts");
-		tb[1].on = defaultEnableToolbars;
+		tb[1].on = false;
 		tb[1].locked = true;
-		tb[1].keys << "show_offline" << "show_away" << "show_agents" << "show_self";
+		tb[1].keys << "show_offline" << "show_hidden" << "show_agents" << "show_self" << "show_statusmsg"; 
 
 		tb[2].name = QObject::tr("Event notifier");
 		tb[2].on = false;
@@ -557,16 +660,13 @@ void UserProfile::reset()
 
 	// lockdown
 	prefs.lockdown.roster   = false;
-	prefs.lockdown.options  = false;
-	prefs.lockdown.profiles = false;
 	prefs.lockdown.services = false;
-	prefs.lockdown.accounts = false;
 
 	prefs.useTransportIconsForContacts = false;
 
 	// iconsets
 	prefs.systemIconset = "default";
-	prefs.emoticons = "default";
+	prefs.emoticons = QStringList("default");
 	prefs.defaultRosterIconset = "default";
 
 	prefs.serviceRosterIconset.clear();
@@ -585,22 +685,19 @@ void UserProfile::reset()
 	prefs.rosterGroupSortStyle   = Options::GroupSortStyle_Alpha;
 	prefs.rosterAccountSortStyle = Options::AccountSortStyle_Alpha;
 
-	// tip of the day
-	prefs.showTips = true;
-	prefs.tipNum   = 0;
-
 	// disco dialog
-	prefs.discoItems = true;
+	prefs.discoItems = false;
 	prefs.discoInfo  = true;
 
 	// auto-auth
 	prefs.autoAuth = false;
 
 	// Notify authorization
-	prefs.notifyAuth = true;
+	prefs.notifyAuth = false;
 
 	// message events
 	prefs.messageEvents = true;
+	prefs.inactiveEvents = false;
 
 	// event priority
 	prefs.eventPriorityHeadline = 0;
@@ -608,6 +705,7 @@ void UserProfile::reset()
 	prefs.eventPriorityMessage  = 1;
 	prefs.eventPriorityAuth     = 2;
 	prefs.eventPriorityFile     = 3;
+	prefs.eventPriorityFile     = 2;
 
 	// Last used path remembering
 	prefs.lastPath = QDir::homeDirPath();
@@ -617,16 +715,6 @@ void UserProfile::reset()
 	prefs.dtPort = 8010;
 	prefs.dtExternal = "";
 
-	// Avatars
-	prefs.avatarsEnabled = true;
-	prefs.avatarsChatdlgEnabled = true;
-	prefs.avatarsSize = 48;
-
-	// global accelerators
-	prefs.globalAccels.clear();
-	prefs.globalAccels.append(QKeySequence()); // process next event
-	prefs.globalAccels.append(QKeySequence()); // show/hide roster
-
 	// advanced widget
 	GAdvancedWidget::setStickEnabled( false ); //until this is bugless
 	GAdvancedWidget::setStickToWindows( false ); //again
@@ -684,7 +772,7 @@ bool UserProfile::toFile(const QString &fname)
 	base.setAttribute("version", "1.0");
 	doc.appendChild(base);
 
-	base.appendChild(textTag(doc, "progver", PROG_VERSION));
+	base.appendChild(textTag(doc, "progver", ApplicationInfo::version()));
 	base.appendChild(textTag(doc, "geom", mwgeom));
 	base.appendChild(stringListToXml(doc, "recentGCList", recentGCList));
 	base.appendChild(stringListToXml(doc, "recentBrowseList", recentBrowseList));
@@ -746,20 +834,13 @@ bool UserProfile::toFile(const QString &fname)
 			}
 		}
 
-		// Avatars
-		{
-			QDomElement p_avatars = doc.createElement("avatars");
-			p_general.appendChild(p_avatars);
-			p_avatars.appendChild(textTag(doc, "use", prefs.avatarsEnabled));
-			p_avatars.appendChild(textTag(doc, "use_chatdlg", prefs.avatarsChatdlgEnabled));
-			p_avatars.appendChild(textTag(doc, "size", prefs.avatarsSize));
-		}
 		{
 			QDomElement p_misc = doc.createElement("misc");
 			p_general.appendChild(p_misc);
 
 			p_misc.appendChild(textTag(doc, "chatBgImage", prefs.chatBgImage));
 			p_misc.appendChild(textTag(doc, "smallChats", prefs.smallChats));
+			p_misc.appendChild(textTag(doc, "brushedMetal", prefs.brushedMetal));
 			p_misc.appendChild(textTag(doc, "chatLineEdit", prefs.chatLineEdit));
 			p_misc.appendChild(textTag(doc, "useTabs", prefs.useTabs));
 			p_misc.appendChild(textTag(doc, "usePerTabCloseButton", prefs.usePerTabCloseButton));
@@ -768,10 +849,7 @@ bool UserProfile::toFile(const QString &fname)
 			p_misc.appendChild(textTag(doc, "autoRosterSizeGrowTop", prefs.autoRosterSizeGrowTop));
 			p_misc.appendChild(textTag(doc, "autoResolveNicksOnAdd", prefs.autoResolveNicksOnAdd));
 			p_misc.appendChild(textTag(doc, "delChats", prefs.delChats));
-			p_misc.appendChild(textTag(doc, "showJoins", prefs.showJoins));
 			p_misc.appendChild(textTag(doc, "browser", prefs.browser));
-			p_misc.appendChild(textTag(doc, "customBrowser", prefs.customBrowser));
-			p_misc.appendChild(textTag(doc, "customMailer", prefs.customMailer));
 			p_misc.appendChild(textTag(doc, "alwaysOnTop", prefs.alwaysOnTop));
 			p_misc.appendChild(textTag(doc, "keepSizes", prefs.keepSizes));
 			p_misc.appendChild(textTag(doc, "ignoreHeadline", prefs.ignoreHeadline));
@@ -784,14 +862,16 @@ bool UserProfile::toFile(const QString &fname)
 			p_misc.appendChild(textTag(doc, "showSubjects", prefs.showSubjects));
 			p_misc.appendChild(textTag(doc, "showCounter", prefs.showCounter));
 			p_misc.appendChild(textTag(doc, "chatSays", prefs.chatSays));
-			p_misc.appendChild(textTag(doc, "chatSoftReturn", prefs.chatSoftReturn));
 			p_misc.appendChild(textTag(doc, "showGroupCounts", prefs.showGroupCounts));
 			p_misc.appendChild(textTag(doc, "jidComplete", prefs.jidComplete));
 			p_misc.appendChild(textTag(doc, "grabUrls", prefs.grabUrls));
 			p_misc.appendChild(textTag(doc, "messageEvents", prefs.messageEvents));
+			p_misc.appendChild(textTag(doc, "inactiveEvents", prefs.inactiveEvents));
 			p_misc.appendChild(textTag(doc, "lastPath", prefs.lastPath));
 			p_misc.appendChild(textTag(doc, "lastSavePath", prefs.lastSavePath));
 			p_misc.appendChild(textTag(doc, "autoCopy", prefs.autoCopy));
+			p_misc.appendChild(textTag(doc, "useCaps", prefs.useCaps));
+			p_misc.appendChild(textTag(doc, "rc", prefs.useRC));
 		}
 		{
 			QDomElement p_dock = doc.createElement("dock");
@@ -801,7 +881,6 @@ bool UserProfile::toFile(const QString &fname)
 			p_dock.appendChild(textTag(doc, "dockDCstyle", prefs.dockDCstyle));
 			p_dock.appendChild(textTag(doc, "dockHideMW", prefs.dockHideMW));
 			p_dock.appendChild(textTag(doc, "dockToolMW", prefs.dockToolMW));
-			p_dock.appendChild(textTag(doc, "isWMDock", prefs.isWMDock));
 		}
 		/*{
 			QDomElement p_sec = doc.createElement("security");
@@ -851,6 +930,7 @@ bool UserProfile::toFile(const QString &fname)
 			p_priority.appendChild(textTag(doc, "headline", prefs.eventPriorityHeadline));
 			p_priority.appendChild(textTag(doc, "auth",     prefs.eventPriorityAuth));
 			p_priority.appendChild(textTag(doc, "file",     prefs.eventPriorityFile));
+			p_priority.appendChild(textTag(doc, "rosterx",     prefs.eventPriorityRosterExchange));
 		}
 	}
 
@@ -865,7 +945,6 @@ bool UserProfile::toFile(const QString &fname)
 			tag.appendChild(textTag(doc, "askOnline", prefs.askOnline));
 			tag.appendChild(textTag(doc, "askOffline", prefs.askOffline));
 			tag.appendChild(textTag(doc, "rosterAnim", prefs.rosterAnim));
-			tag.appendChild(textTag(doc, "autoVersion", prefs.autoVersion));
 			tag.appendChild(textTag(doc, "autoVCardOnLogin", prefs.autoVCardOnLogin));
 			tag.appendChild(textTag(doc, "xmlConsoleOnLogin", prefs.xmlConsoleOnLogin));
 		}
@@ -890,7 +969,12 @@ bool UserProfile::toFile(const QString &fname)
 			tag.appendChild(textTag(doc, "message", prefs.asMessage));
 		}
 		{
-			p_pres.appendChild(prefs.sp.toXml(doc, "statuspresets"));
+			// Status presets
+			QDomElement p_statuspresets = doc.createElement("statuspresets");
+			p_pres.appendChild(p_statuspresets);
+			foreach (StatusPreset p, prefs.sp) {
+				p_statuspresets.appendChild(p.toXml(doc));
+			}
 		}
 		{
 			p_pres.appendChild(stringListToXml(doc, "recentstatus",prefs.recentStatus));
@@ -902,8 +986,6 @@ bool UserProfile::toFile(const QString &fname)
 		p.appendChild(p_lnf);
 
 		p_lnf.appendChild(textTag(doc, "newHeadings", prefs.clNewHeadings));
-		p_lnf.appendChild(textTag(doc, "chat-opacity", prefs.chatOpacity));
-		p_lnf.appendChild(textTag(doc, "roster-opacity", prefs.rosterOpacity));
 		p_lnf.appendChild(textTag(doc, "outline-headings", prefs.outlineHeadings));
 
 		{
@@ -915,6 +997,7 @@ bool UserProfile::toFile(const QString &fname)
 			tag.appendChild(textTag(doc, "away", prefs.color[cAway].name() ));
 			tag.appendChild(textTag(doc, "dnd", prefs.color[cDND].name() ));
 			tag.appendChild(textTag(doc, "offline", prefs.color[cOffline].name() ));
+			tag.appendChild(textTag(doc, "status", prefs.color[cStatus].name() ));
 			tag.appendChild(textTag(doc, "profilefore", prefs.color[cProfileFore].name() ));
 			tag.appendChild(textTag(doc, "profileback", prefs.color[cProfileBack].name() ));
 			tag.appendChild(textTag(doc, "groupfore", prefs.color[cGroupFore].name() ));
@@ -972,13 +1055,13 @@ bool UserProfile::toFile(const QString &fname)
 		QDomElement p_toolbars = doc.createElement("toolbars");
 		p.appendChild(p_toolbars);
 
-		QMap< QString, QValueList<Options::ToolbarPrefs> >::Iterator itBars = prefs.toolbars.begin();
+		QMap< QString, QList<Options::ToolbarPrefs> >::Iterator itBars = prefs.toolbars.begin();
 		for ( ; itBars != prefs.toolbars.end(); ++itBars ) {
-			QValueList<Options::ToolbarPrefs> toolbars = itBars.data();
+			QList<Options::ToolbarPrefs> toolbars = itBars.data();
 			QDomElement p_bars = doc.createElement( itBars.key() );
 			p_toolbars.appendChild( p_bars );
 
-			QValueList<Options::ToolbarPrefs>::Iterator it = toolbars.begin();
+			QList<Options::ToolbarPrefs>::Iterator it = toolbars.begin();
 			for ( ; it != toolbars.end(); ++it) {
 				Options::ToolbarPrefs tb = *it;
 				if ( tb.keys.size() ||
@@ -1073,10 +1156,7 @@ bool UserProfile::toFile(const QString &fname)
 		QDomElement p_lockdown = doc.createElement("lockdown");
 		p.appendChild(p_lockdown);
 		p_lockdown.appendChild(textTag(doc, "roster", prefs.lockdown.roster));
-		p_lockdown.appendChild(textTag(doc, "options", prefs.lockdown.options));
-		p_lockdown.appendChild(textTag(doc, "profiles", prefs.lockdown.profiles));
 		p_lockdown.appendChild(textTag(doc, "services", prefs.lockdown.services));
-		p_lockdown.appendChild(textTag(doc, "accounts", prefs.lockdown.accounts));
 	}
 
 	{
@@ -1096,7 +1176,7 @@ bool UserProfile::toFile(const QString &fname)
 			QDomElement roster_service = doc.createElement("service");
 			is_roster.appendChild(roster_service);
 
-			QMapIterator<QString, QString> it = prefs.serviceRosterIconset.begin();
+			QMap<QString, QString>::iterator it = prefs.serviceRosterIconset.begin();
 			for ( ; it != prefs.serviceRosterIconset.end(); ++it) {
 				QDomElement item = doc.createElement("item");
 				roster_service.appendChild(item);
@@ -1111,7 +1191,7 @@ bool UserProfile::toFile(const QString &fname)
 			QDomElement roster_custom = doc.createElement("custom");
 			is_roster.appendChild(roster_custom);
 
-			QMapIterator<QString, QString> it = prefs.customRosterIconset.begin();
+			QMap<QString, QString>::iterator it = prefs.customRosterIconset.begin();
 			for ( ; it != prefs.customRosterIconset.end(); ++it) {
 				QDomElement item = doc.createElement("item");
 				roster_custom.appendChild(item);
@@ -1132,15 +1212,6 @@ bool UserProfile::toFile(const QString &fname)
 		}
 	}
 
-	{
-		// tip of the day
-		QDomElement p_tip = doc.createElement("tipOfTheDay");
-		p.appendChild(p_tip);
-
-		p_tip.appendChild( textTag(doc, "show", prefs.showTips ) );
-		p_tip.appendChild( textTag(doc, "num",  prefs.tipNum ) );
-	}
-
 	{
 		// disco dialog
 		QDomElement p_disco = doc.createElement("disco");
@@ -1158,26 +1229,6 @@ bool UserProfile::toFile(const QString &fname)
 		p_dt.appendChild( textTag(doc, "external", prefs.dtExternal ) );
 	}
 
-	{
-		// global accelerators
-		QDomElement p_globalAccel = doc.createElement("globalAccel");
-		p.appendChild(p_globalAccel);
-
-		QStringList accelNames;
-		accelNames << "processNextEvent";
-		accelNames << "showHideRoster";
-
-		QStringList::Iterator it1 = accelNames.begin();
-		QValueList<QKeySequence>::iterator it2 = prefs.globalAccels.begin();
-
-		for ( ; it2 != prefs.globalAccels.end(); ++it1, ++it2 ) {
-			QDomElement item = doc.createElement("command");
-			p_globalAccel.appendChild(item);
-			item.setAttribute("type", *it1);
-			item.appendChild( doc.createTextNode(*it2) );
-		}
-	}
-
 	{
 		// advanced widget
 		QDomElement p_advWidget = doc.createElement("advancedWidget");
@@ -1192,13 +1243,12 @@ bool UserProfile::toFile(const QString &fname)
 	}
 
 	QFile f(fname);
-	if(!f.open(IO_WriteOnly))
+	if(!f.open(QIODevice::WriteOnly))
 		return FALSE;
 	QTextStream t;
 	t.setDevice(&f);
 	t.setEncoding(QTextStream::UnicodeUTF8);
 	t << doc.toString();
-	t.unsetDevice();
 	f.close();
 
 	return TRUE;
@@ -1208,9 +1258,10 @@ bool UserProfile::fromFile(const QString &fname)
 {
 	QString confver;
 	QDomDocument doc;
+	QString progver;
 
 	QFile f(fname);
-	if(!f.open(IO_ReadOnly))
+	if(!f.open(QIODevice::ReadOnly))
 		return FALSE;
 	if(!doc.setContent(&f))
 		return FALSE;
@@ -1268,7 +1319,7 @@ bool UserProfile::fromFile(const QString &fname)
 	QDomElement prox = findSubTag(base, "proxies", &found);
 	if(found) {
 		QDomNodeList list = prox.elementsByTagName("proxy");
-		for(uint n = 0; n < list.count(); ++n) {
+		for(int n = 0; n < list.count(); ++n) {
 			QDomElement e = list.item(n).toElement();
 			ProxyItem p;
 			p.name = "";
@@ -1325,22 +1376,11 @@ bool UserProfile::fromFile(const QString &fname)
 				}
 			}
 
-			// Avatars
-			QDomElement avTag = findSubTag(p_general, "avatars", &found);
-			if(found) {
-				readBoolEntry(avTag, "use", &prefs.avatarsEnabled);
-				readBoolEntry(avTag, "use_chatdlg", &prefs.avatarsChatdlgEnabled);
-				readNumEntry(avTag, "size", &prefs.avatarsSize);
-			}
-
 			QDomElement tag = findSubTag(p_general, "misc", &found);
 			if(found) {
 				readEntry(tag, "chatBgImage", &prefs.chatBgImage);
 				readNumEntry(tag, "delChats", &prefs.delChats);
-				readBoolEntry(tag, "showJoins", &prefs.showJoins);
 				readNumEntry(tag, "browser", &prefs.browser);
-				readEntry(tag, "customBrowser", &prefs.customBrowser);
-				readEntry(tag, "customMailer", &prefs.customMailer);
 				readBoolEntry(tag, "alwaysOnTop", &prefs.alwaysOnTop);
 				readBoolEntry(tag, "keepSizes", &prefs.keepSizes);
 				readBoolEntry(tag, "ignoreHeadline", &prefs.ignoreHeadline);
@@ -1351,13 +1391,13 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(tag, "alertOpenChats", &prefs.alertOpenChats);
 				readBoolEntry(tag, "raiseChatWindow", &prefs.raiseChatWindow);
 				readBoolEntry(tag, "showSubjects", &prefs.showSubjects);
-				readBoolEntry(tag, "chatSoftReturn", &prefs.chatSoftReturn);
 				readBoolEntry(tag, "showGroupCounts", &prefs.showGroupCounts);
 				readBoolEntry(tag, "showCounter", &prefs.showCounter);
 				readBoolEntry(tag, "chatSays", &prefs.chatSays);
 				readBoolEntry(tag, "jidComplete", &prefs.jidComplete);
 				readBoolEntry(tag, "grabUrls", &prefs.grabUrls);
 				readBoolEntry(tag, "smallChats", &prefs.smallChats);
+				readBoolEntry(tag, "brushedMetal", &prefs.brushedMetal);
 				readBoolEntry(tag, "chatLineEdit", &prefs.chatLineEdit);
 				readBoolEntry(tag, "useTabs", &prefs.useTabs);
 				readBoolEntry(tag, "usePerTabCloseButton", &prefs.usePerTabCloseButton);
@@ -1366,9 +1406,26 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(tag, "autoRosterSizeGrowTop", &prefs.autoRosterSizeGrowTop);
 				readBoolEntry(tag, "autoResolveNicksOnAdd", &prefs.autoResolveNicksOnAdd);
 				readBoolEntry(tag, "messageEvents", &prefs.messageEvents);
+				readBoolEntry(tag, "inactiveEvents", &prefs.inactiveEvents);
 				readEntry(tag, "lastPath", &prefs.lastPath);
 				readEntry(tag, "lastSavePath", &prefs.lastSavePath);
 				readBoolEntry(tag, "autoCopy", &prefs.autoCopy);
+				readBoolEntry(tag, "useCaps", &prefs.useCaps);
+				readBoolEntry(tag, "rc", &prefs.useRC);
+				
+				// Migrating for soft return option
+				bool found;
+				findSubTag(tag, "chatSoftReturn", &found);
+				if (found) {
+					bool soft;
+					readBoolEntry(tag, "chatSoftReturn", &soft);
+					QVariantList vl;
+					if (soft) 
+						vl << qVariantFromValue(QKeySequence(Qt::Key_Enter)) << qVariantFromValue(QKeySequence(Qt::Key_Return));
+					else 
+						vl << qVariantFromValue(QKeySequence(Qt::Key_Enter+Qt::CTRL)) << qVariantFromValue(QKeySequence(Qt::CTRL+Qt::Key_Return));
+					PsiOptions::instance()->setOption("options.shortcuts.chat.send",vl);
+				}
 			}
 
 			tag = findSubTag(p_general, "dock", &found);
@@ -1377,7 +1434,6 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(tag, "dockDCstyle", &prefs.dockDCstyle);
 				readBoolEntry(tag, "dockHideMW", &prefs.dockHideMW);
 				readBoolEntry(tag, "dockToolMW", &prefs.dockToolMW);
-				readBoolEntry(tag, "isWMDock", &prefs.isWMDock);
 			}
 
 			/*tag = findSubTag(p_general, "security", &found);
@@ -1419,6 +1475,7 @@ bool UserProfile::fromFile(const QString &fname)
 				readNumEntry(tag, "headline", &prefs.eventPriorityHeadline);
 				readNumEntry(tag, "auth",     &prefs.eventPriorityAuth);
 				readNumEntry(tag, "file",     &prefs.eventPriorityFile);
+				readNumEntry(tag, "rosterx",  &prefs.eventPriorityRosterExchange);
  			}
 		}
 
@@ -1431,7 +1488,6 @@ bool UserProfile::fromFile(const QString &fname)
 				readBoolEntry(tag, "askOnline", &prefs.askOnline);
 				readBoolEntry(tag, "askOffline", &prefs.askOffline);
 				readBoolEntry(tag, "rosterAnim", &prefs.rosterAnim);
-				readBoolEntry(tag, "autoVersion", &prefs.autoVersion);
 				readBoolEntry(tag, "autoVCardOnLogin", &prefs.autoVCardOnLogin);
 				readBoolEntry(tag, "xmlConsoleOnLogin", &prefs.xmlConsoleOnLogin);
 			}
@@ -1470,8 +1526,14 @@ bool UserProfile::fromFile(const QString &fname)
 			}
 
 			tag = findSubTag(p_pres, "statuspresets", &found);
-			if(found)
-				prefs.sp.fromXml(tag);
+			if(found) {
+				prefs.sp.clear();
+				for(QDomNode n = tag.firstChild(); !n.isNull(); n = n.nextSibling()) {
+					StatusPreset preset(n.toElement());
+					if (!preset.name().isEmpty()) 
+						prefs.sp[preset.name()] = preset;
+				}
+			}
 			xmlToStringList(p_pres, "recentstatus", &prefs.recentStatus);
 		}
 
@@ -1481,8 +1543,8 @@ bool UserProfile::fromFile(const QString &fname)
 
 			readBoolEntry(p_lnf, "newHeadings", &prefs.clNewHeadings);
 			readBoolEntry(p_lnf, "outline-headings", &prefs.outlineHeadings);
-			readNumEntry(p_lnf, "chat-opacity", &prefs.chatOpacity);
-			readNumEntry(p_lnf, "roster-opacity", &prefs.rosterOpacity);
+			migrateIntEntry(p_lnf, "chat-opacity", "options.ui.chat.opacity");
+			migrateIntEntry(p_lnf, "roster-opacity", "options.ui.contactlist.opacity");
 
 			QDomElement tag = findSubTag(p_lnf, "colors", &found);
 			if(found) {
@@ -1491,6 +1553,7 @@ bool UserProfile::fromFile(const QString &fname)
 				readColorEntry(tag, "away", &prefs.color[cAway]);
 				readColorEntry(tag, "dnd", &prefs.color[cDND]);
 				readColorEntry(tag, "offline", &prefs.color[cOffline]);
+				readColorEntry(tag, "status", &prefs.color[cStatus]);
 				readColorEntry(tag, "groupfore", &prefs.color[cGroupFore]);
 				readColorEntry(tag, "groupback", &prefs.color[cGroupBack]);
 				readColorEntry(tag, "profilefore", &prefs.color[cProfileFore]);
@@ -1534,7 +1597,7 @@ bool UserProfile::fromFile(const QString &fname)
 		QDomElement p_sizes = findSubTag(p, "sizes", &found);
 		if(found) {
 			readSizeEntry(p_sizes, "eventdlg", &prefs.sizeEventDlg);
-			readSizeEntry(p_sizes, "chatdlg", &prefs.sizeChatDlg);
+			migrateSizeEntry(p_sizes, "chatdlg", "options.ui.chat.size");
 			readSizeEntry(p_sizes, "tabdlg", &prefs.sizeTabDlg);
 		}
 
@@ -1601,7 +1664,7 @@ bool UserProfile::fromFile(const QString &fname)
 			if ( progver == "0.9" || progver == "0.9-CVS" ) {
 				// at first, we need to scan the options, to determine, whether event_notifier already available
 				bool found = false;
-				QValueList<Options::ToolbarPrefs>::Iterator it = prefs.toolbars["mainWin"].begin();
+				QList<Options::ToolbarPrefs>::Iterator it = prefs.toolbars["mainWin"].begin();
 				for ( ; it != prefs.toolbars["mainWin"].end(); ++it) {
 					QStringList::Iterator it2 = (*it).keys.begin();
 					for ( ; it2 != (*it).keys.end(); ++it2) {
@@ -1669,10 +1732,7 @@ bool UserProfile::fromFile(const QString &fname)
 		QDomElement p_lockdown = findSubTag(p, "lockdown", &found);
 		if(found) {
 			readBoolEntry(p_lockdown, "roster", &prefs.lockdown.roster);
-			readBoolEntry(p_lockdown, "options", &prefs.lockdown.options);
-			readBoolEntry(p_lockdown, "profiles", &prefs.lockdown.profiles);
 			readBoolEntry(p_lockdown, "services", &prefs.lockdown.services);
-			readBoolEntry(p_lockdown, "accounts", &prefs.lockdown.accounts);
 		}
 
 		QDomElement p_iconset = findSubTag(p, "iconset", &found);
@@ -1726,8 +1786,8 @@ bool UserProfile::fromFile(const QString &fname)
 
 		QDomElement p_tip = findSubTag(p, "tipOfTheDay", &found);
 		if (found) {
-			readBoolEntry(p_tip, "show", &prefs.showTips);
-			readNumEntry(p_tip, "num", &prefs.tipNum);
+			migrateIntEntry(p_tip, "num", "options.ui.tip.number");
+			migrateBoolEntry(p_tip, "show", "options.ui.tip.show");
 		}
 
 		QDomElement p_disco = findSubTag(p, "disco", &found);
@@ -1744,18 +1804,19 @@ bool UserProfile::fromFile(const QString &fname)
 
 		QDomElement p_globalAccel = findSubTag(p, "globalAccel", &found);
 		if (found) {
-			QMap<QString, int> accelNames;
-			accelNames["processNextEvent"] = 0;
-			accelNames["showHideRoster"] = 1;
-
 			for (QDomNode n = p_globalAccel.firstChild(); !n.isNull(); n = n.nextSibling()) {
 				QDomElement i = n.toElement();
 				if ( i.isNull() )
 					continue;
 
 				if ( i.tagName() == "command" && i.hasAttribute("type") ) {
-					int n = accelNames[i.attribute("type")];
-					prefs.globalAccels[n] = QKeySequence(i.text());
+					QVariant k = qVariantFromValue(QKeySequence(i.text()));
+					QString shortcut;
+					if (i.attribute("type") == "processNextEvent")
+						shortcut = "event";
+					else
+						shortcut = "toggle-visibility";
+					PsiOptions::instance()->setOption(QString("options.shortcuts.global.") + shortcut, k);
 				}
 			}
 		}
@@ -1784,7 +1845,7 @@ bool UserProfile::fromFile(const QString &fname)
 
 QString pathToProfile(const QString &name)
 {
-	return g.pathProfiles + "/" + name;
+	return ApplicationInfo::profilesDir() + "/" + name;
 }
 
 QString pathToProfileConfig(const QString &name)
@@ -1796,7 +1857,7 @@ QStringList getProfilesList()
 {
 	QStringList list;
 
-	QDir d(g.pathProfiles);
+	QDir d(ApplicationInfo::profilesDir());
 	if(!d.exists())
 		return list;
 
@@ -1840,13 +1901,12 @@ bool profileNew(const QString &name)
 	}
 
 	// make it
-	QDir d(g.pathProfiles);
+	QDir d(ApplicationInfo::profilesDir());
 	if(!d.exists())
 		return FALSE;
-	QDir p(g.pathProfiles + "/" + name);
+	QDir p(ApplicationInfo::profilesDir() + "/" + name);
 	if(!p.exists()) {
-		d.mkdir(name);
-		if(!p.exists())
+	        if (!d.mkdir(name))
 			return FALSE;
 	}
 
@@ -1865,7 +1925,7 @@ bool profileRename(const QString &oldname, const QString &name)
 	}
 
 	// locate the folder
-	QDir d(g.pathProfiles);
+	QDir d(ApplicationInfo::profilesDir());
 	if(!d.exists())
 		return FALSE;
 	if(!d.rename(oldname, name))
@@ -1909,3 +1969,5 @@ bool profileDelete(const QString &path)
 
 	return folderRemove(QDir(path));
 }
+
+QString activeProfile;
