@@ -21,11 +21,15 @@
 #ifndef PSIAPPLICATION_H
 #define PSIAPPLICATION_H
 
-#include <qapplication.h>
-
+#include <QApplication>
 #ifdef Q_WS_MAC
 #include <Carbon/Carbon.h>
 #endif
+#ifdef Q_WS_WIN
+#include <qt_windows.h>
+#endif
+
+class QEvent;
 
 class PsiApplication : public QApplication
 {
@@ -41,6 +45,9 @@ public:
 #ifdef Q_WS_MAC
 	bool macEventFilter(EventHandlerCallRef, EventRef);
 #endif
+#ifdef Q_WS_WIN
+	bool winEventFilter(MSG* msg, long* result);
+#endif
 
 signals:
 	void dockActivated();
