@@ -21,60 +21,40 @@
 #ifndef PSICON_H
 #define PSICON_H
 
-#include<qcombobox.h>
-#include"profiles.h"
-#include"psievent.h"
-#include"im.h"
+#include <Q3PtrList>
+#include <QList>
+
+#include "profiles.h"
 
 using namespace XMPP;
 
 class PsiCon;
 class PsiAccount;
+class PsiEvent;
 class ContactView;
 class EventDlg;
 class UserListItem;
 class EDB;
 class EDBItem;
 class ProxyManager;
-class MainWin;
+class QMenuBar;
 class FileTransDlg;
 class IconSelectPopup;
 class QThread;
 class PsiActionList;
 class PsiToolBar;
-class	TabDlg;
+class TabDlg;
+class AccountsComboBox;
 class ChatDlg;
-
-namespace OpenPGP
-{
+class TuneController;
+class PsiContactList;
+class Q3DockWindow;
+namespace OpenPGP {
 	class Engine;
 }
-
-typedef QPtrList<PsiAccount> PsiAccountList;
-typedef QPtrListIterator<PsiAccount> PsiAccountListIt;
-
-class AccountsComboBox : public QComboBox
-{
-	Q_OBJECT
-public:
-	AccountsComboBox(PsiCon *, QWidget *parent=0, bool online_only = false);
-	~AccountsComboBox();
-
-	void setAccount(PsiAccount *);
-
-signals:
-	void activated(PsiAccount *);
-
-private slots:
-	void changeAccount();
-	void updateAccounts();
-	void deleteMe();
-
-private:
-	PsiCon *psi;
-	PsiAccount *pa;
-	bool onlineOnly;
-};
+namespace XMPP {
+	class Jid;
+}
 
 class PsiCon : public QObject
 {
@@ -87,10 +67,10 @@ public:
 	bool init();
 	void deinit();
 
+	PsiContactList* contactList() const;
 	ContactView *contactView() const;
-	const PsiAccountList & accountList(bool enabledOnly=FALSE) const;
 	EDB *edb() const;
-	OpenPGP::Engine *pgp() const;
+	TuneController* tuneController() const;
 	ProxyManager *proxy() const;
 	FileTransDlg *ftdlg() const;
 
@@ -100,26 +80,22 @@ public:
 	bool isChatActiveWindow(ChatDlg*);
 	ChatDlg* getChatInTabs(QString);
 	TabDlg* getManagingTabs(ChatDlg*);
-	QPtrList<TabDlg>* getTabSets();
+	Q3PtrList<TabDlg>* getTabSets();
 
 	QWidget *dialogFind(const char *className);
 	void dialogRegister(QWidget *w);
 	void dialogUnregister(QWidget *w);
 
-	bool isValid(PsiAccount *);
-	void createAccount(const QString &name, const Jid &j="", const QString &pass="", bool opt_host=false, const QString &host="", int port=5222, bool ssl=false, int proxy=0);
+	QMenuBar* defaultMenuBar() const;
+
+	void createAccount(const QString &name, const Jid &j="", const QString &pass="", bool opt_host=false, const QString &host="", int port=5222, bool legacy_ssl_probe = true, UserAccount::SSLFlag ssl=UserAccount::SSL_Auto, int proxy=0);
+	PsiAccount *createAccount(const UserAccount &);
 	//void createAccount(const QString &, const QString &host="", int port=5222, bool ssl=false, const QString &user="", const QString &pass="");
-	void modifyAccount(PsiAccount *);
 	void removeAccount(PsiAccount *);
-	void enableAccount(PsiAccount *, bool e=TRUE);
 
 	void playSound(const QString &);
 	void raiseMainwin();
 
-	// global event handling
-	int queueCount();
-	PsiAccount *queueLowestEventId();
-
 	AccountsComboBox *accountsComboBox(QWidget *parent=0, bool online_only = false);
 
 	const QStringList & recentGCList() const;
@@ -132,11 +108,13 @@ public:
 	EventDlg *createEventDlg(const QString &, PsiAccount *);
 	void updateContactGlobal(PsiAccount *, const Jid &);
 
-	QPtrList<PsiToolBar> *toolbarList() const;
+	QList<PsiToolBar*> toolbarList() const;
 	PsiToolBar *findToolBar(QString group, int index);
 	PsiActionList *actionList() const;
 
-	MainWin *mainWin() const;
+	void buildToolbars();
+	bool getToolbarLocation(Q3DockWindow* dw, Qt::Dock& dock, int& index, bool& nl, int& extraOffset) const;
+
 	IconSelectPopup *iconSelectPopup() const;
 	void processEvent(PsiEvent *);
 
@@ -148,14 +126,13 @@ signals:
 	void accountCountChanged();
 	void accountActivityChanged();
 	void emitOptionsUpdate();
-	void pgpToggled(bool);
-	void pgpKeysUpdated();
 
 public slots:
-	void setGlobalStatus(const Status &);
-	QWidget *doToolbars();
+	void setGlobalStatus(const Status &, bool withPriority = false);
+	void doToolbars();
+	void checkAccountsEmpty();
 
-private slots:
+public slots:
 	void doSleep();
 	void doWakeup();
 	void closeProgram();
@@ -171,34 +148,30 @@ private slots:
 	void slotApplyOptions(const Options &);
 	void queueChanged();
 	void recvNextEvent();
-	void setStatusFromDialog(const Status &);
-	void pgp_initFinished(bool, const QString &);
-	void pgp_keysUpdated();
+	void setStatusFromDialog(const XMPP::Status &, bool withPriority);
 	void proxy_settingsChanged();
-	void accel_activated(int);
 	void updateMainwinStatus();
 	void tabDying(TabDlg*);
 
-	void mainWinGeomChanged(int x, int y, int w, int h);
+	void mainWinGeomChanged(QRect saveableGeometry);
+
+private slots:
+	void saveAccounts();
+	void optionsUpdate();
 
 private:
 	class Private;
 	Private *d;
 
-	PsiAccount *loadAccount(const UserAccount &);
-	void loadAccounts(const UserAccountList &);
-	UserAccountList unloadAccounts();
 	void deleteAllDialogs();
-	void saveAccounts();
-	void pgp_init(const QString &);
 	void s5b_init();
 	void updateS5BServerAddresses();
+	void setShortcuts();
 
-	friend class PsiAccount;
-	void link(PsiAccount *);
-	void unlink(PsiAccount *);
+	friend class PsiAccount; // FIXME
 	void setToggles(bool tog_offline, bool tog_away, bool tog_agents, bool tog_hidden, bool tog_self);
 	void getToggles(bool *tog_offline, bool *tog_away, bool *tog_agents, bool *tog_hidden, bool *tog_self);
+	void promptUserToCreateAccount();
 
 	friend class EventQueue;
 	int getId();
