@@ -21,12 +21,17 @@
 #include "psievent.h"
 
 #include <qdom.h>
+#include <QTextStream>
+#include <QList>
 
 #include "psicon.h"
 #include "psiaccount.h"
 #include "xmpp_xmlcommon.h"
 #include "filetransfer.h"
+#include "applicationinfo.h"
+#include "psicontactlist.h"
 
+using namespace XMPP;
 using namespace XMLHelper;
 
 static PsiEvent *copyPsiEvent(PsiEvent *fe)
@@ -145,7 +150,7 @@ QDomElement PsiEvent::toXml(QDomDocument *doc) const
 
 	e.appendChild( textTag(*doc, "originLocal",	v_originLocal) );
 	e.appendChild( textTag(*doc, "late",		v_late) );
-	e.appendChild( textTag(*doc, "ts",		v_ts.toString( ISODate )) );
+	e.appendChild( textTag(*doc, "ts",		v_ts.toString( Qt::ISODate )) );
 	if ( !v_jid.full().isEmpty() )
 		e.appendChild( textTag(*doc, "jid",		v_jid.full()) );
 
@@ -164,19 +169,17 @@ bool PsiEvent::fromXml(PsiCon *psi, PsiAccount *account, const QDomElement *e)
 
 	readBoolEntry(*e, "originLocal", &v_originLocal);
 	readBoolEntry(*e, "late", &v_late);
-	v_ts  = QDateTime::fromString(subTagText(*e, "ts"), ISODate);
+	v_ts  = QDateTime::fromString(subTagText(*e, "ts"), Qt::ISODate);
 	v_jid = Jid( subTagText(*e, "jid") );
 
 	if ( account ) {
 		v_account = account;
 	}
 	else if ( hasSubTag(*e, "account") ) {
-		PsiAccountList list = psi->accountList();
-		PsiAccountListIt it(list);
 		QString accName = subTagText(*e, "account");
-		for ( ; it.current(); ++it) {
-			if ( it.current()->name() == accName ) {
-				v_account = it.current();
+		foreach(PsiAccount* account, psi->contactList()->accounts()) {
+			if ( account->name() == accName ) {
+				v_account = account;
 				break;
 			}
 		}
@@ -236,6 +239,16 @@ void MessageEvent::setFrom(const Jid &j)
 	v_m.setFrom(j);
 }
 
+const QString& MessageEvent::nick() const
+{
+	return v_m.nick();
+}
+
+void MessageEvent::setNick(const QString &nick)
+{
+	v_m.setNick(nick);
+}
+
 bool MessageEvent::sentToChatWindow() const
 {
 	return v_sentToChatWindow;
@@ -336,6 +349,16 @@ void AuthEvent::setFrom(const Jid &j)
 	v_from = j;
 }
 
+const QString& AuthEvent::nick() const
+{
+	return v_nick;
+}
+
+void AuthEvent::setNick(const QString &nick)
+{
+	v_nick = nick;
+}
+
 QString AuthEvent::authType() const
 {
 	return v_at;
@@ -347,6 +370,7 @@ QDomElement AuthEvent::toXml(QDomDocument *doc) const
 
 	e.appendChild( textTag(*doc, "from",	 v_from.full()) );
 	e.appendChild( textTag(*doc, "authType", v_at) );
+	e.appendChild( textTag(*doc, "nick", v_nick) );
 
 	return e;
 }
@@ -358,6 +382,7 @@ bool AuthEvent::fromXml(PsiCon *psi, PsiAccount *account, const QDomElement *e)
 
 	v_from = Jid( subTagText(*e, "from") );
 	v_at   = subTagText(*e, "authType");
+	v_nick   = subTagText(*e, "nick");
 
 	return false;
 }
@@ -409,6 +434,113 @@ FileTransfer *FileEvent::takeFileTransfer()
 	return _ft;
 }
 
+//----------------------------------------------------------------------------
+// HttpAuthEvent
+//----------------------------------------------------------------------------
+
+HttpAuthEvent::HttpAuthEvent(const PsiHttpAuthRequest &req, PsiAccount *acc)
+:MessageEvent(acc), v_req(req)
+{
+	const XMPP::Stanza &s = req.stanza();
+
+	XMPP::Message m;
+
+	if ( s.kind() == XMPP::Stanza::Message ) {
+		m.fromStanza(s, acc->client()->timeZoneOffset());
+	}
+	else {
+		m.setFrom(s.from());
+		m.setTimeStamp(QDateTime::currentDateTime());
+		m.setHttpAuthRequest(HttpAuthRequest(s.element().elementsByTagNameNS("http://jabber.org/protocol/http-auth", "confirm").item(0).toElement()));
+	}
+
+	setMessage(m);
+}
+
+HttpAuthEvent::~HttpAuthEvent()
+{
+}
+//----------------------------------------------------------------------------
+// RosterExchangeEvent
+//----------------------------------------------------------------------------
+RosterExchangeEvent::RosterExchangeEvent(const Jid &j, const RosterExchangeItems& i, const QString& text, PsiAccount *acc)
+:PsiEvent(acc)
+{
+	v_from = j;
+	v_items = i;
+	v_text = text;
+}
+
+int RosterExchangeEvent::priority() const
+{
+	return option.eventPriorityRosterExchange;
+}
+
+Jid RosterExchangeEvent::from() const
+{
+	return v_from;
+}
+
+void RosterExchangeEvent::setFrom(const Jid &j)
+{
+	v_from = j;
+}
+
+const RosterExchangeItems& RosterExchangeEvent::rosterExchangeItems() const
+{
+	return v_items;
+}
+
+void RosterExchangeEvent::setRosterExchangeItems(const RosterExchangeItems& i)
+{
+	v_items = i;
+}
+
+const QString& RosterExchangeEvent::text() const
+{
+	return v_text;
+}
+
+void RosterExchangeEvent::setText(const QString& text)
+{
+	v_text = text;
+}
+
+//----------------------------------------------------------------------------
+// Status
+//----------------------------------------------------------------------------
+/*StatusEvent::StatusEvent(const Jid &j, const XMPP::Status& s, PsiAccount *acc)
+:PsiEvent(acc)
+{
+	v_from = j;
+	v_status = s;
+}
+
+int StatusEvent::priority() const
+{
+	return option.eventPriorityChat;
+}
+
+Jid StatusEvent::from() const
+{
+	return v_from;
+}
+
+void StatusEvent::setFrom(const Jid &j)
+{
+	v_from = j;
+}
+
+const XMPP::Status& StatusEvent::status() const
+{
+	return v_status;
+}
+
+void StatusEvent::setStatus(const XMPP::Status& s)
+{
+	v_status = s;
+}*/
+
 //----------------------------------------------------------------------------
 // EventQueue
 //----------------------------------------------------------------------------
@@ -450,11 +582,9 @@ private:
 class EventQueue::Private
 {
 public:
-	Private() {
-		list.setAutoDelete(true);
-	}
+	Private() { }
 
-	QPtrList<EventItem> list;
+	QList<EventItem*> list;
 	PsiCon *psi;
 	PsiAccount *account;
 };
@@ -482,8 +612,10 @@ EventQueue::~EventQueue()
 
 int EventQueue::nextId() const
 {
-	QPtrListIterator<EventItem> it(d->list);
-	EventItem *i = it.current();
+	if (d->list.isEmpty())
+		return -1;
+
+	EventItem *i = d->list.first();
 	if(!i)
 		return -1;
 	return i->id();
@@ -497,14 +629,11 @@ int EventQueue::count() const
 int EventQueue::count(const Jid &j, bool compareRes) const
 {
 	int total = 0;
-
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++it) {
+	foreach(EventItem *i, d->list) {
 		Jid j2(i->event()->jid());
 		if(j.compare(j2, compareRes))
 			++total;
 	}
-
 	return total;
 }
 
@@ -516,8 +645,8 @@ void EventQueue::enqueue(PsiEvent *e)
 	bool found = false;
 
 	// skip all with higher or equal priority
-	for ( EventItem *ei = d->list.first(); ei; ei = d->list.next() ) {
-		if ( ei && ei->event()->priority() < prior ) {
+	foreach(EventItem *ei, d->list) {
+		if (ei && ei->event()->priority() < prior ) {
 			d->list.insert(d->list.find(ei), i);
 			found = true;
 			break;
@@ -536,10 +665,10 @@ void EventQueue::dequeue(PsiEvent *e)
 	if ( !e )
 		return;
 
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++it) {
+	foreach(EventItem *i, d->list) {
 		if ( e == i->event() ) {
 			d->list.remove(i);
+			delete i;
 			return;
 		}
 	}
@@ -549,13 +678,13 @@ void EventQueue::dequeue(PsiEvent *e)
 
 PsiEvent *EventQueue::dequeue(const Jid &j, bool compareRes)
 {
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++it) {
+	foreach(EventItem *i, d->list) {
 		PsiEvent *e = i->event();
 		Jid j2(e->jid());
 		if(j.compare(j2, compareRes)) {
-			d->list.removeRef(i);
+			d->list.remove(i);
 			emit queueChanged();
+			delete i;
 			return e;
 		}
 	}
@@ -565,8 +694,7 @@ PsiEvent *EventQueue::dequeue(const Jid &j, bool compareRes)
 
 PsiEvent *EventQueue::peek(const Jid &j, bool compareRes) const
 {
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++it) {
+	foreach(EventItem *i, d->list) {
 		PsiEvent *e = i->event();
 		Jid j2(e->jid());
 		if(j.compare(j2, compareRes)) {
@@ -579,20 +707,25 @@ PsiEvent *EventQueue::peek(const Jid &j, bool compareRes) const
 
 PsiEvent *EventQueue::dequeueNext()
 {
-	QPtrListIterator<EventItem> it(d->list);
-	EventItem *i = it.current();
+	if (d->list.isEmpty())
+		return 0;
+
+	EventItem *i = d->list.first();
 	if(!i)
 		return 0;
 	PsiEvent *e = i->event();
-	d->list.remove(it);
+	d->list.remove(i);
 	emit queueChanged();
+	delete i;
 	return e;
 }
 
 PsiEvent *EventQueue::peekNext() const
 {
-	QPtrListIterator<EventItem> it(d->list);
-	EventItem *i = it.current();
+	if (d->list.isEmpty())
+		return 0;
+
+	EventItem *i = d->list.first();
 	if(!i)
 		return 0;
 	return i->event();
@@ -600,8 +733,7 @@ PsiEvent *EventQueue::peekNext() const
 
 PsiEvent *EventQueue::peekFirstChat(const Jid &j, bool compareRes) const
 {
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++it) {
+	foreach(EventItem *i, d->list) {
 		PsiEvent *e = i->event();
 		if(e->type() == PsiEvent::Message) {
 			MessageEvent *me = (MessageEvent *)e;
@@ -619,18 +751,19 @@ bool EventQueue::hasChats(const Jid &j, bool compareRes) const
 }
 
 // this function extracts all chats from the queue, and returns a list of queue positions
-void EventQueue::extractChats(QPtrList<PsiEvent> *el, const Jid &j, bool compareRes)
+void EventQueue::extractChats(QList<PsiEvent*> *el, const Jid &j, bool compareRes)
 {
 	bool changed = false;
 
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current());) {
-		PsiEvent *e = i->event();
+	for(QList<EventItem*>::Iterator it = d->list.begin(); it != d->list.end();) {
+		PsiEvent *e = (*it)->event();
 		if(e->type() == PsiEvent::Message) {
 			MessageEvent *me = (MessageEvent *)e;
 			if(j.compare(me->from(), compareRes) && me->message().type() == "chat") {
 				el->append(me);
-				d->list.remove(it);
+				EventItem* ei = *it;
+				it = d->list.erase(it);
+				delete ei;
 				changed = true;
 				continue;
 			}
@@ -642,10 +775,31 @@ void EventQueue::extractChats(QPtrList<PsiEvent> *el, const Jid &j, bool compare
 		emit queueChanged();
 }
 
+// this function extracts all messages from the queue, and returns a list of them
+void EventQueue::extractMessages(QList<PsiEvent*> *el)
+{
+	bool changed = false;
+
+	for(QList<EventItem*>::Iterator it = d->list.begin(); it != d->list.end();) {
+		PsiEvent *e = (*it)->event();
+		if(e->type() == PsiEvent::Message) {
+			el->append(e);
+			EventItem* ei = *it;
+			it = d->list.erase(it);
+			delete ei;
+			changed = true;
+			continue;
+		}
+		++it;
+	}
+
+	if ( changed )
+		emit queueChanged();
+}
+
 void EventQueue::printContent() const
 {
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++it) {
+	foreach(EventItem *i, d->list) {
 		PsiEvent *e = i->event();
 		printf("  %d: (%d) from=[%s] jid=[%s]\n", i->id(), e->type(), e->from().full().latin1(), e->jid().full().latin1());
 	}
@@ -653,7 +807,8 @@ void EventQueue::printContent() const
 
 void EventQueue::clear()
 {
-	d->list.clear();
+	while(!d->list.isEmpty()) 
+		delete d->list.takeFirst();
 
 	emit queueChanged();
 }
@@ -663,12 +818,13 @@ void EventQueue::clear(const Jid &j, bool compareRes)
 {
 	bool changed = false;
 
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current());) {
-		PsiEvent *e = i->event();
+	for(QList<EventItem*>::Iterator it = d->list.begin(); it != d->list.end();) {
+		PsiEvent *e = (*it)->event();
 		Jid j2(e->jid());
 		if(j.compare(j2, compareRes)) {
-			d->list.removeRef(i);
+			EventItem* ei = *it;
+			it = d->list.erase(it);
+			delete ei;
 			changed = true;
 		}
 		else
@@ -681,13 +837,13 @@ void EventQueue::clear(const Jid &j, bool compareRes)
 
 EventQueue &EventQueue::operator= (const EventQueue &from)
 {
-	d->list.clear();
+	while(!d->list.isEmpty()) 
+		delete d->list.takeFirst();
+		
 	d->psi = from.d->psi;
 
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++i) {
+	foreach(EventItem *i, from.d->list) {
 		PsiEvent *e = i->event();
-
 		enqueue( copyPsiEvent(e) );
 	}
 
@@ -698,11 +854,10 @@ QDomElement EventQueue::toXml(QDomDocument *doc) const
 {
 	QDomElement e = doc->createElement("eventQueue");
 	e.setAttribute("version", "1.0");
-	e.appendChild(textTag(doc, "progver", PROG_VERSION));
+	e.appendChild(textTag(doc, "progver", ApplicationInfo::version()));
 
-	QPtrListIterator<EventItem> it(d->list);
-	for(EventItem *i; (i = it.current()); ++it) {
-		QDomElement event = (*it)->event()->toXml(doc);
+	foreach(EventItem *i, d->list) {
+		QDomElement event = i->event()->toXml(doc);
 		e.appendChild( event );
 	}
 
@@ -762,13 +917,12 @@ bool EventQueue::toFile(const QString &fname)
 	doc.appendChild(element);
 
 	QFile f( fname );
-	if( !f.open(IO_WriteOnly) )
+	if( !f.open(QIODevice::WriteOnly) )
 		return FALSE;
 	QTextStream t;
 	t.setDevice( &f );
 	t.setEncoding( QTextStream::UnicodeUTF8 );
 	t << doc.toString(4);
-	t.unsetDevice();
 	f.close();
 
 	return TRUE;
@@ -780,7 +934,7 @@ bool EventQueue::fromFile(const QString &fname)
 	QDomDocument doc;
 
 	QFile f(fname);
-	if(!f.open(IO_ReadOnly))
+	if(!f.open(QIODevice::ReadOnly))
 		return FALSE;
 	if(!doc.setContent(&f, true))
 		return FALSE;
