@@ -21,23 +21,25 @@
 #ifndef PSIEVENT_H
 #define PSIEVENT_H
 
-#include <qdatetime.h>
-#include <qobject.h>
-#include <qptrlist.h>
-#include <qdom.h>
-#include <qguardedptr.h>
+#include <QList>
+#include <QDateTime>
+#include <QObject>
+#include <QDomDocument>
+#include <QDomElement>
+#include <QPointer>
 
-#include"im.h"
+#include "xmpp_jid.h"
+#include "xmpp_rosterx.h"
+#include "xmpp_message.h"
+#include "psihttpauthrequest.h"
 
-namespace XMPP
-{
+namespace XMPP {
 	class FileTransfer;
 };
 
-using namespace XMPP;
-
 class PsiCon;
 class PsiAccount;
+class QDomElement;
 
 
 class PsiEvent : public QObject
@@ -52,15 +54,18 @@ public:
 		Message,
 		Auth,
 		PGP,
-		File
+		File,
+		RosterExchange,
+		//Status
+		HttpAuth
 	};
 	virtual int type() const = 0;
 
-	virtual Jid from() const = 0;
-	virtual void setFrom(const Jid &j) = 0;
+	virtual XMPP::Jid from() const = 0;
+	virtual void setFrom(const XMPP::Jid &j) = 0;
 
-	Jid jid() const;
-	void setJid(const Jid &);
+	XMPP::Jid jid() const;
+	void setJid(const XMPP::Jid &);
 
 	bool originLocal() const;
 	bool late() const;
@@ -82,7 +87,7 @@ public:
 private:
 	bool v_originLocal, v_late;
 	QDateTime v_ts;
-	Jid v_jid;
+	XMPP::Jid v_jid;
 	PsiAccount *v_account;
 };
 
@@ -97,8 +102,11 @@ public:
 	~MessageEvent();
 
 	int type() const;
-	Jid from() const;
-	void setFrom(const Jid &j);
+	XMPP::Jid from() const;
+	void setFrom(const XMPP::Jid &j);
+
+	const QString& nick() const;
+	void setNick(const QString&);
 
 	bool sentToChatWindow() const;
 	const XMPP::Message & message() const;
@@ -123,13 +131,16 @@ class AuthEvent : public PsiEvent
 {
 	Q_OBJECT
 public:
-	AuthEvent(const Jid &j, const QString &authType, PsiAccount *acc);
+	AuthEvent(const XMPP::Jid &j, const QString &authType, PsiAccount *acc);
 	AuthEvent(const AuthEvent &from);
 	~AuthEvent();
 
 	int type() const;
-	Jid from() const;
-	void setFrom(const Jid &j);
+	XMPP::Jid from() const;
+	void setFrom(const XMPP::Jid &j);
+
+	const QString& nick() const;
+	void setNick(const QString&);
 
 	QString authType() const;
 
@@ -141,7 +152,8 @@ public:
 	virtual PsiEvent *copy() const;
 
 private:
-	Jid v_from;
+	XMPP::Jid v_from;
+	QString v_nick;
 	QString v_at;
 };
 
@@ -155,8 +167,8 @@ public:
 	: PsiEvent(from) {}
 	~PGPEvent() {}
 	int type() const { return PGP; }
-	Jid from() const { return QString(); }
-	void setFrom(const Jid &) {}
+	XMPP::Jid from() const { return QString(); }
+	void setFrom(const XMPP::Jid &) {}
 };
 
 // incoming file transfer
@@ -164,19 +176,78 @@ class FileEvent : public PsiEvent
 {
 	Q_OBJECT
 public:
-	FileEvent(const Jid &j, FileTransfer *ft, PsiAccount *acc);
+	FileEvent(const XMPP::Jid &j, XMPP::FileTransfer *ft, PsiAccount *acc);
 	~FileEvent();
 
 	int type() const { return File; }
-	Jid from() const;
-	void setFrom(const Jid &);
-	FileTransfer *takeFileTransfer();
+	XMPP::Jid from() const;
+	void setFrom(const XMPP::Jid &);
+	XMPP::FileTransfer *takeFileTransfer();
 
 	virtual int priority() const;
 
 private:
-	Jid v_from;
-	QGuardedPtr<FileTransfer> ft;
+	XMPP::Jid v_from;
+	QPointer<XMPP::FileTransfer> ft;
+};
+
+// roster item exchange event
+class RosterExchangeEvent : public PsiEvent
+{
+	Q_OBJECT
+public:
+	RosterExchangeEvent(const XMPP::Jid &j, const XMPP::RosterExchangeItems& i, const QString& body, PsiAccount *acc);
+
+	int type() const { return RosterExchange; }
+	XMPP::Jid from() const;
+	void setFrom(const XMPP::Jid &);
+	const XMPP::RosterExchangeItems& rosterExchangeItems() const;
+	void setRosterExchangeItems(const XMPP::RosterExchangeItems&);
+	const QString& text() const;
+	void setText(const QString& text);
+	
+	virtual int priority() const;
+
+private:
+	XMPP::Jid v_from;
+	XMPP::RosterExchangeItems v_items;
+	QString v_text;
+};
+
+/*class StatusEvent : public PsiEvent
+{
+	Q_OBJECT
+public:
+	StatusEvent(const XMPP::Jid &j, const XMPP::Status& i, PsiAccount *acc);
+
+	int type() const { return Status; }
+	XMPP::Jid from() const;
+	void setFrom(const XMPP::Jid &);
+	const XMPP::Status& status() const;
+	void setStatus(const XMPP::Status&);
+	
+	virtual int priority() const;
+
+private:
+	XMPP::Jid v_from;
+	XMPP::Status v_status;
+};*/
+
+// http auth
+class HttpAuthEvent : public MessageEvent
+{
+	Q_OBJECT
+public:
+	HttpAuthEvent(const PsiHttpAuthRequest &req, PsiAccount *acc);
+	~HttpAuthEvent();
+
+	int type() const { return HttpAuth; }
+
+	const PsiHttpAuthRequest & request() { return v_req; }
+
+private:
+	PsiHttpAuthRequest v_req;
+
 };
 
 // event queue
@@ -192,19 +263,20 @@ public:
 
 	int nextId() const;
 	int count() const;
-	int count(const Jid &, bool compareRes=true) const;
+	int count(const XMPP::Jid &, bool compareRes=true) const;
 	void enqueue(PsiEvent *);
 	void dequeue(PsiEvent *);
-	PsiEvent *dequeue(const Jid &, bool compareRes=true);
-	PsiEvent *peek(const Jid &, bool compareRes=true) const;
+	PsiEvent *dequeue(const XMPP::Jid &, bool compareRes=true);
+	PsiEvent *peek(const XMPP::Jid &, bool compareRes=true) const;
 	PsiEvent *dequeueNext();
 	PsiEvent *peekNext() const;
-	bool hasChats(const Jid &, bool compareRes=true) const;
-	PsiEvent *peekFirstChat(const Jid &, bool compareRes=true) const;
-	void extractChats(QPtrList<PsiEvent> *list, const Jid &, bool compareRes=true);
+	bool hasChats(const XMPP::Jid &, bool compareRes=true) const;
+	PsiEvent *peekFirstChat(const XMPP::Jid &, bool compareRes=true) const;
+	void extractMessages(QList<PsiEvent*> *list);
+	void extractChats(QList<PsiEvent*> *list, const XMPP::Jid &, bool compareRes=true);
 	void printContent() const;
 	void clear();
-	void clear(const Jid &, bool compareRes=true);
+	void clear(const XMPP::Jid &, bool compareRes=true);
 
 	QDomElement toXml(QDomDocument *) const; // these work with pointers, to save inclusion of qdom.h, which is pretty large
 	bool fromXml(const QDomElement *);
