@@ -23,10 +23,14 @@
 #include "common.h"
 #include "userlist.h"
 #include "anim.h"
+#include "applicationinfo.h"
 
-#include <qfileinfo.h>
-#include <qdict.h>
-#include <qptrlist.h>
+#include <QFileInfo>
+#include <Q3Dict>
+#include <Q3PtrList>
+#include <QCoreApplication>
+
+using namespace XMPP;
 
 //----------------------------------------------------------------------------
 // PsiIconset
@@ -47,9 +51,10 @@ public:
 
 	QString iconsetPath(QString name) {
 		QStringList dirs;
+		dirs << ":";
 		dirs << ".";
-		dirs << g.pathHome;
-		dirs << g.pathBase;
+		dirs << ApplicationInfo::homeDir();
+		dirs << ApplicationInfo::resourcesDir();
 
 		QStringList::Iterator it = dirs.begin();
 		for ( ; it != dirs.end(); ++it) {
@@ -64,9 +69,9 @@ public:
 	}
 
 	void stripFirstAnimFrame(Iconset &is) {
-		QPtrListIterator<Icon> it = is.iterator();
-		for ( ; it.current(); ++it) {
-			it.current()->stripFirstAnimFrame();
+		QListIterator<PsiIcon*> it = is.iterator();
+		while (it.hasNext()) {
+			it.next()->stripFirstAnimFrame();
 		}
 	}
 
@@ -84,12 +89,12 @@ public:
 			return;
 		}
 
-		QPtrListIterator<Icon> it = from->iterator();
-		for ( ; it.current(); ++it) {
-			Icon *icon = it.current();
+		QListIterator<PsiIcon*> it = from->iterator();
+		while( it.hasNext()) {
+			PsiIcon *icon = it.next();
 
 			if ( icon && !icon->name().isEmpty() ) {
-				Icon *toIcon = (Icon *)to->icon(icon->name());
+				PsiIcon *toIcon = (PsiIcon *)to->icon(icon->name());
 				if ( toIcon ) {
 					if ( icon->anim() ) {
 						toIcon->setAnim  ( *icon->anim(), false );
@@ -108,10 +113,10 @@ public:
 		to->setInformation(*from);
 	}
 
-	Icon *jid2icon(const Jid &jid, const QString &iconName)
+	PsiIcon *jid2icon(const Jid &jid, const QString &iconName)
 	{
 		// first level -- global default icon
-		Icon *icon = (Icon *)IconsetFactory::iconPtr(iconName);
+		PsiIcon *icon = (PsiIcon *)IconsetFactory::iconPtr(iconName);
 
 		// second level -- transport icon
 		if ( jid.user().isEmpty() || option.useTransportIconsForContacts ) {
@@ -125,16 +130,16 @@ public:
 
 			bool found = false;
 
-			QMapIterator<QString, QRegExp> it = services.begin();
+			QMap<QString, QRegExp>::Iterator it = services.begin();
 			for ( ; it != services.end(); ++it) {
 				QRegExp rx = it.data();
 				if ( rx.search(jid.host()) != -1 ) {
 					// get the iconset name of the current service
-					QMapIterator<QString, QString> it2 = option.serviceRosterIconset.find(it.key());
+					QMap<QString, QString>::Iterator it2 = option.serviceRosterIconset.find(it.key());
 					if ( it2 != option.serviceRosterIconset.end() ) {
 						Iconset *is = psi->roster.find(it2.data());
 						if ( is ) {
-							Icon *i = (Icon *)is->icon(iconName);
+							PsiIcon *i = (PsiIcon *)is->icon(iconName);
 							if ( i ) {
 								icon = i;
 								found = true;
@@ -149,7 +154,7 @@ public:
 			if ( !found && jid.user().isEmpty() ) {
 				Iconset *is = psi->roster.find(option.serviceRosterIconset["transport"]);
 				if ( is ) {
-					Icon *i = (Icon *)is->icon(iconName);
+					PsiIcon *i = (PsiIcon *)is->icon(iconName);
 					if ( i )
 						icon = i;
 				}
@@ -157,15 +162,15 @@ public:
 		}
 
 		// third level -- custom icons
-		QMapIterator<QString, QString> it = option.customRosterIconset.begin();
+		QMap<QString, QString>::Iterator it = option.customRosterIconset.begin();
 		for ( ; it != option.customRosterIconset.end(); ++it) {
 			QRegExp rx = QRegExp(it.key());
 			if ( rx.search(jid.userHost()) != -1 ) {
 				Iconset *is = psi->roster.find(it.data());
 				if ( is ) {
-					Icon *i = (Icon *)is->icon(iconName);
+					PsiIcon *i = (PsiIcon *)is->icon(iconName);
 					if ( i )
-						icon = (Icon *)is->icon(iconName);
+						icon = (PsiIcon *)is->icon(iconName);
 				}
 			}
 		}
@@ -176,7 +181,7 @@ public:
 	Iconset systemIconset(bool *ok)
 	{
 		Iconset def;
-		*ok = def.load ( iconsetPath("system/default") );
+		*ok = def.load(":/iconsets/system/default");
 
 		if ( option.systemIconset != "default" ) {
 			Iconset is;
@@ -193,7 +198,7 @@ public:
 	Iconset *defaultRosterIconset(bool *ok)
 	{
 		Iconset *def = new Iconset;
-		*ok = def->load ( iconsetPath("roster/default") );
+		*ok = def->load (":/iconsets/roster/default");
 
 		if ( option.defaultRosterIconset != "default" ) {
 			Iconset is;
@@ -207,14 +212,13 @@ public:
 		return def;
 	}
 
-	QPtrList<Iconset> emoticons()
+	Q3PtrList<Iconset> emoticons()
 	{
-		QPtrList<Iconset> emo;
+		Q3PtrList<Iconset> emo;
 
-		QStringList::Iterator it3 = option.emoticons.begin();
-		for ( ; it3 != option.emoticons.end(); ++it3) {
+		foreach(QString name, option.emoticons) {
 			Iconset *is = new Iconset;
-			if ( is->load ( iconsetPath("emoticons/" + *it3) ) ) {
+			if ( is->load ( iconsetPath("emoticons/" + name) ) ) {
 				PsiIconset::removeAnimation(is);
 				is->addToFactory();
 				emo.append( is );
@@ -228,6 +232,7 @@ public:
 };
 
 PsiIconset::PsiIconset()
+	: QObject(QCoreApplication::instance())
 {
 	d = new Private(this);
 }
@@ -318,7 +323,7 @@ bool PsiIconset::optionsChanged(const Options *old)
 	}
 
 	// service&custom roster iconsets
-	if ( old->serviceRosterIconset != option.serviceRosterIconset || old->customRosterIconset != option.customRosterIconset ) {
+	if (  operator!=(old->serviceRosterIconset,option.serviceRosterIconset) || operator!=(old->customRosterIconset,option.customRosterIconset)) {
 		QStringList rosterIconsets;
 
 		QMap<QString, QString>::Iterator it = option.serviceRosterIconset.begin();
@@ -356,7 +361,7 @@ bool PsiIconset::optionsChanged(const Options *old)
 		while ( !clear ) {
 			clear = true;
 
-			QDictIterator<Iconset> it3 ( roster );
+			Q3DictIterator<Iconset> it3 ( roster );
 			for ( ; it3.current(); ++it3) {
 				QString name = it3.currentKey();
 				if ( name == option.defaultRosterIconset )
@@ -382,7 +387,7 @@ bool PsiIconset::optionsChanged(const Options *old)
 	return old->defaultRosterIconset != option.defaultRosterIconset;
 }
 
-Icon *PsiIconset::event2icon(PsiEvent *e)
+PsiIcon *PsiIconset::event2icon(PsiEvent *e)
 {
 	QString icon;
 	if(e->type() == PsiEvent::Message) {
@@ -452,39 +457,39 @@ static QString status2name(int s)
 	return name;
 }
 
-Icon *PsiIconset::statusPtr(int s)
+PsiIcon *PsiIconset::statusPtr(int s)
 {
-	return (Icon *)IconsetFactory::iconPtr(status2name(s));
+	return (PsiIcon *)IconsetFactory::iconPtr(status2name(s));
 }
 
-Icon PsiIconset::status(int s)
+PsiIcon PsiIconset::status(int s)
 {
-	Icon *icon = statusPtr(s);
+	PsiIcon *icon = statusPtr(s);
 	if ( icon )
 		return *icon;
-	return Icon();
+	return PsiIcon();
 }
 
-Icon *PsiIconset::statusPtr(const XMPP::Status &s)
+PsiIcon *PsiIconset::statusPtr(const XMPP::Status &s)
 {
 	return statusPtr(makeSTATUS(s));
 }
 
-Icon PsiIconset::status(const XMPP::Status &s)
+PsiIcon PsiIconset::status(const XMPP::Status &s)
 {
 	return status(makeSTATUS(s));
 }
 
-Icon *PsiIconset::transportStatusPtr(QString name, int s)
+PsiIcon *PsiIconset::transportStatusPtr(QString name, int s)
 {
-	Icon *icon = 0;
+	PsiIcon *icon = 0;
 
-	QMapIterator<QString, QString> it = option.serviceRosterIconset.begin();
+	QMap<QString, QString>::Iterator it = option.serviceRosterIconset.begin();
 	for ( ; it != option.serviceRosterIconset.end(); ++it) {
 		if (name == it.key()) {
 			Iconset *is = roster.find(it.data());
 			if ( is ) {
-				icon = (Icon *)is->icon(status2name(s));
+				icon = (PsiIcon *)is->icon(status2name(s));
 				if ( icon )
 					break;
 			}
@@ -497,54 +502,54 @@ Icon *PsiIconset::transportStatusPtr(QString name, int s)
 	return icon;
 }
 
-Icon *PsiIconset::transportStatusPtr(QString name, const XMPP::Status &s)
+PsiIcon *PsiIconset::transportStatusPtr(QString name, const XMPP::Status &s)
 {
 	return transportStatusPtr(name, makeSTATUS(s));
 }
 
-Icon PsiIconset::transportStatus(QString name, int s)
+PsiIcon PsiIconset::transportStatus(QString name, int s)
 {
-	Icon *icon = transportStatusPtr(name, s);
+	PsiIcon *icon = transportStatusPtr(name, s);
 	if ( icon )
 		return *icon;
-	return Icon();
+	return PsiIcon();
 }
 
-Icon PsiIconset::transportStatus(QString name, const XMPP::Status &s)
+PsiIcon PsiIconset::transportStatus(QString name, const XMPP::Status &s)
 {
-	Icon *icon = transportStatusPtr(name, s);
+	PsiIcon *icon = transportStatusPtr(name, s);
 	if ( icon )
 		return *icon;
-	return Icon();
+	return PsiIcon();
 }
 
-Icon *PsiIconset::statusPtr(const XMPP::Jid &jid, int s)
+PsiIcon *PsiIconset::statusPtr(const XMPP::Jid &jid, int s)
 {
 	return d->jid2icon(jid, status2name(s));
 }
 
-Icon *PsiIconset::statusPtr(const XMPP::Jid &jid, const XMPP::Status &s)
+PsiIcon *PsiIconset::statusPtr(const XMPP::Jid &jid, const XMPP::Status &s)
 {
 	return statusPtr(jid, makeSTATUS(s));
 }
 
-Icon PsiIconset::status(const XMPP::Jid &jid, int s)
+PsiIcon PsiIconset::status(const XMPP::Jid &jid, int s)
 {
-	Icon *icon = statusPtr(jid, s);
+	PsiIcon *icon = statusPtr(jid, s);
 	if ( icon )
 		return *icon;
-	return Icon();
+	return PsiIcon();
 }
 
-Icon PsiIconset::status(const XMPP::Jid &jid, const XMPP::Status &s)
+PsiIcon PsiIconset::status(const XMPP::Jid &jid, const XMPP::Status &s)
 {
-	Icon *icon = statusPtr(jid, s);
+	PsiIcon *icon = statusPtr(jid, s);
 	if ( icon )
 		return *icon;
-	return Icon();
+	return PsiIcon();
 }
 
-Icon *PsiIconset::statusPtr(UserListItem *u)
+PsiIcon *PsiIconset::statusPtr(UserListItem *u)
 {
 	if ( !u )
 		return 0;
@@ -570,12 +575,12 @@ Icon *PsiIconset::statusPtr(UserListItem *u)
 	return statusPtr(u->jid(), s);
 }
 
-Icon PsiIconset::status(UserListItem *u)
+PsiIcon PsiIconset::status(UserListItem *u)
 {
-	Icon *icon = statusPtr(u);
+	PsiIcon *icon = statusPtr(u);
 	if ( icon )
 		return *icon;
-	return Icon();
+	return PsiIcon();
 }
 
 const Iconset &PsiIconset::system() const
@@ -592,9 +597,18 @@ void PsiIconset::stripFirstAnimFrame(Iconset *is)
 void PsiIconset::removeAnimation(Iconset *is)
 {
 	if ( is ) {
-		QPtrListIterator<Icon> it = is->iterator();
-		for ( ; it.current(); ++it) {
-			it.current()->removeAnim(false);
+		QListIterator<PsiIcon*> it = is->iterator();
+		while (it.hasNext()) {
+			it.next()->removeAnim(false);
 		}
 	}
 }
+
+PsiIconset* PsiIconset::instance()
+{
+	if (!instance_) 
+		instance_ = new PsiIconset();
+	return instance_;
+}
+
+PsiIconset* PsiIconset::instance_ = NULL;
