@@ -21,23 +21,26 @@
 #ifndef PSIPOPUP_H
 #define PSIPOPUP_H
 
-#include <qobject.h>
-
-#include "im.h"
-using namespace XMPP;
+#include <QObject>
 
 class PsiCon;
 class PsiAccount;
 class UserListItem;
 class FancyPopup;
-class Icon;
+class PsiIcon;
 class PsiEvent;
+namespace XMPP {
+	class Jid;
+	class Resource;
+}
+using namespace XMPP;
+
 
 class PsiPopup : public QObject
 {
 	Q_OBJECT
 public:
-	PsiPopup(const Icon *titleIcon, QString titleText, PsiAccount *acc);
+	PsiPopup(const PsiIcon *titleIcon, QString titleText, PsiAccount *acc);
 	~PsiPopup();
 
 	enum PopupType {
@@ -54,7 +57,7 @@ public:
 	};
 	PsiPopup(PopupType type, PsiAccount *acc);
 
-	void setData(const Icon *icon, QString text);
+	void setData(const PsiIcon *icon, QString text);
 	void setData(const Jid &, const Resource &, const UserListItem * = 0, const PsiEvent * = 0);
 
 	void show();
