@@ -20,14 +20,17 @@
 
 #include "common.h"
 #include "psitoolbar.h"
-#include <qpopupmenu.h>
+#include <q3popupmenu.h>
 #include <qlabel.h>
 #include <qaction.h>
+//Added by qt3to4:
+#include <QContextMenuEvent>
+#include <Q3PtrList>
 #include "psicon.h"
-#include "mainwin.h"
+#include "iconset.h"
 #include "psicon.h"
 #include "iconaction.h"
-#include "options/toolbardlg.h"
+#include "options/opt_toolbars.h"
 
 //----------------------------------------------------------------------------
 // PsiToolBar::Private
@@ -43,7 +46,7 @@ public:
 	QString group;
 	int groupIndex;
 	PsiCon *psi;
-	QPtrList<IconAction> uniqueActions;
+	Q3PtrList<IconAction> uniqueActions;
 };
 
 PsiToolBar::Private::Private()
@@ -58,16 +61,11 @@ PsiToolBar::Private::Private()
 // PsiToolBar
 //----------------------------------------------------------------------------
 
-PsiToolBar::PsiToolBar(const QString &label, QMainWindow *mainWindow, QWidget *parent, bool newLine, const char *name, WFlags f)
-: QToolBar(label, mainWindow, parent, newLine, name, f)
-{
-	d = new Private();
-}
-
-PsiToolBar::PsiToolBar(QMainWindow *parent, const char *name)
-: QToolBar(parent, name)
+PsiToolBar::PsiToolBar(const QString& label, Q3MainWindow* mainWindow, PsiCon* psi)
+: Q3ToolBar(label, mainWindow, (QWidget*)mainWindow)
 {
 	d = new Private();
+	d->psi = psi;
 }
 
 PsiToolBar::~PsiToolBar()
@@ -98,26 +96,17 @@ void PsiToolBar::setMoveable( bool b )
 void PsiToolBar::contextMenuEvent(QContextMenuEvent *e)
 {
 	e->accept();
-	if(option.lockdown.options)
-		return;
 
 	if ( !d->customizeable )
 		return;
 
-	MainWin *mainWin = (MainWin *)mainWindow();
-	QPopupMenu pm;
-
-	pm.insertItem(IconsetFactory::icon("psi/toolbars"), tr("Configure &Toolbar..."), 0);
+	Q3PopupMenu pm;
+	pm.insertItem(IconsetFactory::icon("psi/toolbars").icon(), tr("Configure &Toolbar..."), 0);
 
 	int ret = pm.exec( e->globalPos() );
 
 	if ( ret == 0 ) {
-		QWidget *w = mainWin->psiCon()->doToolbars();
-		if ( !w->inherits("ToolbarDlg") )
-			return;
-
-		ToolbarDlg *dlg = (ToolbarDlg *)w;
-		dlg->setCurrentToolbar(this);
+		d->psi->doToolbars();
 	}
 }
 
@@ -159,7 +148,7 @@ void PsiToolBar::initialize( Options::ToolbarPrefs &tbPref, bool createUniqueAct
 	if ( d->psi ) {
 		ActionList actions = d->psi->actionList()->suitableActions( d->type );
 		QStringList keys = tbPref.keys;
-		for (uint j = 0; j < keys.size(); j++) {
+		for (int j = 0; j < keys.size(); j++) {
 			IconAction *action = actions.action( keys[j] );
 
 			if ( action ) {
@@ -185,9 +174,3 @@ void PsiToolBar::initialize( Options::ToolbarPrefs &tbPref, bool createUniqueAct
 
 	tbPref.dirty = false;
 }
-
-void PsiToolBar::setPsiCon( PsiCon *psi )
-{
-	d->psi = psi;
-}
-
