@@ -18,19 +18,28 @@
  *
  */
 
-#include"statusdlg.h"
+#include "statusdlg.h"
 
-#include<qpushbutton.h>
-#include<qlayout.h>
-#include<qlabel.h>
-#include<qcombobox.h>
-#include<qinputdialog.h>
-#include<qcheckbox.h>
-#include"psicon.h"
-#include"psiaccount.h"
-#include"userlist.h"
-#include"common.h"
-#include"msgmle.h"
+#include <qpushbutton.h>
+#include <QMessageBox>
+#include <qlayout.h>
+#include <qlabel.h>
+#include <qcombobox.h>
+#include <qinputdialog.h>
+#include <qcheckbox.h>
+#include <QVBoxLayout>
+#include <QHBoxLayout>
+#include <QKeyEvent>
+
+#include "jidutil.h"
+#include "psicon.h"
+#include "psioptions.h"
+#include "psiaccount.h"
+#include "userlist.h"
+#include "common.h"
+#include "msgmle.h"
+#include "statuspreset.h"
+#include "shortcutmanager.h"
 
 
 //----------------------------------------------------------------------------
@@ -38,8 +47,9 @@
 // FIXME: Will no longer be needed once it is out of the groupchat contactview
 //----------------------------------------------------------------------------
 StatusShowDlg::StatusShowDlg(const UserListItem &u)
-:QDialog(0, 0, false, WDestructiveClose)
+	: QDialog(0, 0, false)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
 	// build the dialog
 	QVBoxLayout *vb = new QVBoxLayout(this, 8);
 	PsiTextView *te = new PsiTextView(this);
@@ -53,10 +63,10 @@ StatusShowDlg::StatusShowDlg(const UserListItem &u)
 
 	// set the rest up
 	te->setReadOnly(true);
-	te->setTextFormat(RichText);
+	te->setTextFormat(Qt::RichText);
 	te->setText(u.makeDesc());
 
-	setCaption(tr("Status for %1").arg(jidnick(u.jid().full(), u.name())));
+	setWindowTitle(tr("Status for %1").arg(JIDUtil::nickOrJid(u.name(), u.jid().full())));
 	resize(400,240);
 
 	pb->setFocus();
@@ -66,7 +76,6 @@ StatusShowDlg::StatusShowDlg(const UserListItem &u)
 //----------------------------------------------------------------------------
 // StatusSetDlg
 //----------------------------------------------------------------------------
-static int combomap[7] = { STATUS_CHAT, STATUS_ONLINE, STATUS_AWAY, STATUS_XA, STATUS_DND, STATUS_INVISIBLE, STATUS_OFFLINE };
 
 class StatusSetDlg::Private
 {
@@ -78,32 +87,35 @@ public:
 	Status s;
 	ChatView *te;
 	QComboBox *cb_type, *cb_preset;
+	QLineEdit *le_priority;
 	QCheckBox *save;
 };
 
 StatusSetDlg::StatusSetDlg(PsiCon *psi, const Status &s)
-:QDialog(0, 0, false, WDestructiveClose)
+	: QDialog(0, 0, false)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
 	d = new Private;
 	d->psi = psi;
 	d->pa = 0;
 	d->psi->dialogRegister(this);
 	d->s = s;
 
-	setCaption(CAP(tr("Set Status: All accounts")));
+	setWindowTitle(CAP(tr("Set Status: All accounts")));
 	init();
 }
 
 StatusSetDlg::StatusSetDlg(PsiAccount *pa, const Status &s)
-:QDialog(0, 0, false, WDestructiveClose)
+	: QDialog(0, 0, false)
 {
+	setAttribute(Qt::WA_DeleteOnClose);
 	d = new Private;
 	d->psi = 0;
 	d->pa = pa;
 	d->pa->dialogRegister(this);
 	d->s = s;
 
-	setCaption(CAP(tr("Set Status: %1").arg(d->pa->name())));
+	setWindowTitle(CAP(tr("Set Status: %1").arg(d->pa->name())));
 	init();
 }
 
@@ -115,33 +127,56 @@ void StatusSetDlg::init()
 	QVBoxLayout *vb = new QVBoxLayout(this, 8);
 	QHBoxLayout *hb1 = new QHBoxLayout(vb);
 
+	// Status
 	QLabel *l;
 	l = new QLabel(tr("Status:"), this);
 	hb1->addWidget(l);
 	d->cb_type = new QComboBox(this);
-	int n;
-	for(n = 0; n < 7; ++n)
-		d->cb_type->insertItem(status2txt(combomap[n]));
-	for(n = 0; n < 7; ++n) {
-		if(type == combomap[n]) {
-			d->cb_type->setCurrentItem(n);
+
+	QList<XMPP::Status::Type> statuses;
+	statuses << STATUS_CHAT << STATUS_ONLINE << STATUS_AWAY << STATUS_XA << STATUS_DND;
+	if (PsiOptions::instance()->getOption("options.ui.menu.status.invisible").toBool()) {
+		statuses << STATUS_INVISIBLE;
+	}
+	statuses << STATUS_OFFLINE;
+	
+	foreach(XMPP::Status::Type status, statuses) {
+		d->cb_type->addItem(status2txt(status), status);
+	}
+	for(int i = 0; i < d->cb_type->count(); i++) {
+		if (d->cb_type->itemData(i).toInt() == type) {
+			d->cb_type->setCurrentItem(i);
 			break;
 		}
 	}
-	hb1->addWidget(d->cb_type,1);
+	hb1->addWidget(d->cb_type,3);
+
+	// Priority
+	l = new QLabel(tr("Priority:"), this);
+	hb1->addWidget(l);
+	d->le_priority = new QLineEdit(this);
+	d->le_priority->setMinimumWidth(30);
+	hb1->addWidget(d->le_priority,1);
+	
+	// Status preset
 	l = new QLabel(tr("Preset:"), this);
 	hb1->addWidget(l);
 	d->cb_preset = new QComboBox(this);
 	d->cb_preset->insertItem(tr("<None>"));
-	QStringList presets=option.sp.varsToStringList();
+	QStringList presets;
+	foreach(StatusPreset p, option.sp) {
+		presets += p.name();
+	}
 	presets.sort();
 	d->cb_preset->insertStringList(presets);
 	connect(d->cb_preset, SIGNAL(highlighted(int)), SLOT(chooseStatusPreset(int)));
-	hb1->addWidget(d->cb_preset,1);
+	hb1->addWidget(d->cb_preset,3);
 
 	d->te = new ChatView(this);
+	d->te->setDialog(this);
 	d->te->setReadOnly(false);
-	d->te->setTextFormat(PlainText);
+	d->te->setTextFormat(Qt::PlainText);
+	d->te->setAcceptRichText(false);
 	d->te->setMinimumHeight(50);
 	vb->addWidget(d->te);
 	QHBoxLayout *hb = new QHBoxLayout(vb);
@@ -157,12 +192,15 @@ void StatusSetDlg::init()
 	hb->addWidget(pb2);
 
 	// set the rest up
-	d->te->setTextFormat(PlainText);
+	d->te->setTextFormat(Qt::PlainText);
 	d->te->setText(d->s.status());
 	d->te->selectAll();
 	connect(pb1, SIGNAL(clicked()), SLOT(doButton()));
 	connect(pb2, SIGNAL(clicked()), SLOT(cancel()));
 	d->te->setFocus();
+	
+	ShortcutManager::connect("common.close", this, SLOT(close()));
+	ShortcutManager::connect("status.set", this, SLOT(doButton()));
 
 	resize(400,240);
 }
@@ -176,16 +214,6 @@ StatusSetDlg::~StatusSetDlg()
 	delete d;
 }
 
-void StatusSetDlg::keyPressEvent(QKeyEvent *e)
-{
-	if(e->key() == Key_Escape)
-		close();
-	else if(e->key() == Key_Return && ((e->state() & ControlButton) || (e->state() & AltButton)) )
-		doButton();
-	else
-		e->ignore();
-}
-
 void StatusSetDlg::doButton()
 {
 	// Save preset
@@ -206,7 +234,7 @@ void StatusSetDlg::doButton()
 				QMessageBox::information(this, tr("Error"), 
 					tr("Can't create a blank preset!"));
 			}
-			else if(option.sp.findByKey(text) != option.sp.end()) {
+			else if(option.sp.contains(text)) {
 				QMessageBox::information(this, tr("Error"), 
 					tr("You already have a preset with that name!"));
 			}
@@ -214,14 +242,21 @@ void StatusSetDlg::doButton()
 				break;
 		}
 		// Store preset
-		option.sp.set(text,d->te->text());
+ 		option.sp[text] = StatusPreset(text,d->te->text());
+ 		if (!d->le_priority->text().isEmpty()) 
+ 			option.sp[text].setPriority(d->le_priority->text().toInt());
+ 		option.sp[text].setStatus((XMPP::Status::Type) d->cb_type->itemData(d->cb_type->currentIndex()).toInt());
 	} 
 
 	// Set status
-	int type = combomap[d->cb_type->currentItem()];
+	int type = d->cb_type->itemData(d->cb_type->currentIndex()).toInt();
 	QString str = d->te->text();
 
-	set(makeStatus(type, str));
+ 	if (d->le_priority->text().isEmpty())
+ 		emit set(makeStatus(type, str), false);
+ 	else 
+ 		emit set(makeStatus(type, str, d->le_priority->text().toInt()), true);
+
 	close();
 }
 
@@ -229,8 +264,21 @@ void StatusSetDlg::chooseStatusPreset(int x)
 {
 	if(x < 1)
 		return;
+	
+	StatusPreset preset = option.sp[d->cb_preset->text(x)];
+	d->te->setText(preset.message());
+	if (preset.priority().hasValue()) 
+		d->le_priority->setText(QString::number(preset.priority().value()));
+	else
+		d->le_priority->clear();
 
-	d->te->setText(option.sp.get(d->cb_preset->text(x)));
+	int n;
+	for(n = 0; n < d->cb_type->count(); ++n) {
+		if(preset.status() == d->cb_type->itemData(n).toInt()) {
+			d->cb_type->setCurrentItem(n);
+			break;
+		}
+	}
 }
 
 void StatusSetDlg::cancel()
